@@ -23,49 +23,62 @@ typedef struct localctx_struct {
 	// Newicons-specific data
 	int has_newicons;
 	dbuf *newicons_data[2];
-	u8 pending_data;
-	int pending_data_bits_used;
-	int newicons_bits_per_pixel;
-	int newicons_line_count;
+	UI newicons_bits_per_pixel;
+	u8 newicons_finished_pal;
+	u8 newicons_largepalentry_flag;
+	struct de_bitbuf_lowlevel newicons_bbll;
 
 	// Glowicons-specific data
 	int has_glowicons;
 	i64 glowicons_pos;
 	i64 glowicons_width, glowicons_height;
-	u32 glowicons_palette[256];
+	de_color glowicons_palette[256];
+#define NEWICONS_MAX_PALENTRIES 512
+	de_color tmppal[NEWICONS_MAX_PALENTRIES];
 } lctx;
 
-static const u32 rev1pal[4] = { 0x55aaff,0x000000,0xffffff,0xff8800 }; // http://krashan.ppa.pl/articles/amigaicons/
-//static const u32 rev1pal[4] = { 0x0055aa,0x000020,0xffffff,0xff8a00 }; // Netpbm
+static const de_color rev1pal[4] = { 0x55aaff,0x000000,0xffffff,0xff8800 }; // http://krashan.ppa.pl/articles/amigaicons/
+//static const de_color rev1pal[4] = { 0x0055aa,0x000020,0xffffff,0xff8a00 }; // Netpbm
 
-static const u32 rev2pal[4] = { 0x959595,0xffffff,0x000000,0x3b67a2 }; // http://krashan.ppa.pl/articles/amigaicons/
-//static const u32 rev2pal[4] = { 0xaaaaaa,0xffffff,0x000000,0x556699 }; // XnView
+static const de_color rev2pal[4] = { 0x959595,0xffffff,0x000000,0x3b67a2 }; // http://krashan.ppa.pl/articles/amigaicons/
+//static const de_color rev2pal[4] = { 0xaaaaaa,0xffffff,0x000000,0x556699 }; // XnView
 
-static const u32 magicwbpal[8] = {
+static const de_color magicwbpal[8] = {
 	0x959595,0x7b7b7b,0xffffff,0xaa907c,0x000000,0xafafaf,0x3b67a2,0xffa997 // http://krashan.ppa.pl/articles/amigaicons/ fixed? (& Wikipedia)
 	//0xaaaaaa,0x999999,0xffffff,0xbbaa99,0x000000,0xbbbbbb,0x556699,0xffbbaa // XnView
 };
 
-static void do_newicons_append_bit(deark *c, lctx *d, dbuf *f, u8 b)
+static void do_newicons_process_bit(deark *c, lctx *d, dbuf *f, u8 b)
 {
-	if(d->pending_data_bits_used==0) {
-		d->pending_data = 0;
-	}
-	d->pending_data = (d->pending_data<<1) | (b & 0x1);
-	d->pending_data_bits_used++;
+	UI nbits_needed;
 
-	if(d->newicons_line_count==0) {
-		// We're still reading palette samples, which are always 8 bits.
-		if(d->pending_data_bits_used==8) {
-			dbuf_writebyte(f, d->pending_data);
-			d->pending_data_bits_used=0;
+	de_bitbuf_lowlevel_add_bits(&d->newicons_bbll, (u64)b, 1);
+
+	// If line_count==0, we're still reading palette samples, which are always 8 bits.
+	nbits_needed = (d->newicons_finished_pal==0) ? 8 : d->newicons_bits_per_pixel;
+
+	if(d->newicons_bbll.nbits_in_bitbuf >= nbits_needed) {
+		UI x;
+
+		x = (UI)de_bitbuf_lowlevel_get_bits(&d->newicons_bbll, nbits_needed);
+		if(x>255 && d->newicons_finished_pal) {
+			// For now at least, we only support 9-bit images if they don't
+			// actually use the 9th bit.
+			d->newicons_largepalentry_flag = 1;
 		}
-		return;
+		dbuf_writebyte(f, (u8)x);
+		d->newicons_bbll.nbits_in_bitbuf = 0;
 	}
+}
 
-	if(d->pending_data_bits_used >= d->newicons_bits_per_pixel) {
-		dbuf_writebyte(f, d->pending_data);
-		d->pending_data_bits_used=0;
+static void do_newicons_process_bits(deark *c, lctx *d, dbuf *f, u8 b, UI nbits)
+{
+	UI i;
+	u8 tmpb;
+
+	for(i=0; i<nbits; i++) {
+		tmpb = (b>>(nbits-1-i))&0x01;
+		do_newicons_process_bit(c, d, f, tmpb);
 	}
 }
 
@@ -76,7 +89,7 @@ static void do_decode_newicons(deark *c, lctx *d,
 	dbuf *f, int newicons_num)
 {
 	u8 trns_code, width_code, height_code;
-	u8 b0, b1, tmpb;
+	u8 b0, b1;
 	de_bitmap *img = NULL;
 	int has_trns;
 	i64 width, height;
@@ -87,12 +100,12 @@ static void do_decode_newicons(deark *c, lctx *d,
 	i64 i;
 	i64 rle_len;
 	int saved_indent_level;
-	u32 pal[256];
 
 	de_dbg_indent_save(c, &saved_indent_level);
 	de_dbg(c, "decoding NewIcons[%d], len=%"I64_FMT, newicons_num, f->len);
 	de_dbg_indent(c, 1);
 
+	d->newicons_largepalentry_flag = 0;
 	trns_code = dbuf_getbyte(f, 0);
 	has_trns = (trns_code=='B');
 	width_code = dbuf_getbyte(f, 1);
@@ -100,9 +113,15 @@ static void do_decode_newicons(deark *c, lctx *d,
 
 	b0 = dbuf_getbyte(f, 3);
 	b1 = dbuf_getbyte(f, 4);
-	ncolors = ((((i64)b0)-0x21)<<6) + (((i64)b1)-0x21);
-	if(ncolors<1) ncolors=1;
-	if(ncolors>256) ncolors=256;
+	if(width_code<0x21 || height_code<0x21 || b0<0x21 || b1<0x21) {
+		de_err(c, "Bad NewIcons data");
+		goto done;
+	}
+	ncolors = (i64)((((UI)b0-(UI)0x21)<<6) + ((UI)b1-(UI)0x21));
+	if(ncolors<1 || ncolors>NEWICONS_MAX_PALENTRIES) {
+		de_err(c, "Unsupported number of colors: %d", (int)ncolors);
+		goto done;
+	}
 
 	width = (i64)width_code - 0x21;
 	height = (i64)height_code - 0x21;
@@ -114,47 +133,41 @@ static void do_decode_newicons(deark *c, lctx *d,
 
 	decoded = dbuf_create_membuf(c, 2048, 0);
 
-	d->pending_data = 0;
-	d->pending_data_bits_used = 0;
+	d->newicons_bbll.is_lsb = 0;
+	de_bitbuf_lowlevel_empty(&d->newicons_bbll);
 
-	d->newicons_bits_per_pixel = (int)de_log2_rounded_up(ncolors);
+	d->newicons_bits_per_pixel = (UI)de_log2_rounded_up(ncolors);
 
 	// We decode both the palette and the bitmap into the same buffer, and
 	// keep track of where in the buffer the bitmap starts.
 
-	// Count the number of lines (EOL represented by 0x00 byte).
-	// This is only needed because the bitmap starts on the second line.
-	d->newicons_line_count=0;
+	d->newicons_finished_pal = 0;
 
 	for(srcpos=5; srcpos<f->len; srcpos++) {
 		b0 = dbuf_getbyte(f, srcpos);
 		if((b0>=0x20 && b0<=0x6f) || (b0>=0xa1 && b0<=0xd0)) {
 			if(b0<=0x6f) b1 = b0-0x20;
 			else b1 = 0x50 + (b0-0xa1);
-
-			for(i=0; i<7; i++) {
-				tmpb = (b1>>(6-i))&0x01;
-				do_newicons_append_bit(c, d, decoded, tmpb);
-			}
+			do_newicons_process_bits(c, d, decoded, b1, 7);
 		}
 		else if(b0>=0xd1) {
 			// RLE compression for "0" bits
-			tmpb = 0;
 			rle_len = 7*(i64)(b0-0xd0);
 			for(i=0; i<rle_len; i++) {
-				do_newicons_append_bit(c, d, decoded, tmpb);
+				do_newicons_process_bit(c, d, decoded, 0);
 			}
 		}
 		else if(b0==0x00) {
 			// End of a line.
 			// Throw away any bits we've decoded that haven't been used yet.
-			d->pending_data_bits_used = 0;
+			d->newicons_bbll.nbits_in_bitbuf = 0;
 
-			if(d->newicons_line_count==0) {
+			// Presumably, the bitmap always starts at the beginning of a line.
+			if(d->newicons_finished_pal==0 && decoded->len>=ncolors*3) {
 				// The bitmap will start at this position. Remember that.
 				bitmap_start_pos = decoded->len;
+				d->newicons_finished_pal = 1;
 			}
-			d->newicons_line_count++;
 		}
 	}
 
@@ -163,22 +176,27 @@ static void do_decode_newicons(deark *c, lctx *d,
 	// The first ncolors*3 bytes are the palette
 	de_dbg2(c, "NewIcons palette");
 	de_dbg_indent(c, 1);
+	de_zeromem(d->tmppal, sizeof(d->tmppal));
 	for(i=0; i<ncolors; i++) {
-		if(i>255) break;
-		pal[i] = dbuf_getRGB(decoded, i*3, 0);
+		d->tmppal[i] = dbuf_getRGB(decoded, i*3, 0);
 
 		// Educated guess: If the transparency flag is set, it means
 		// palette entry 0 is transparent.
 		if(i==0 && has_trns)
-			pal[i] = DE_SET_ALPHA(pal[i], 0x00);
+			d->tmppal[i] = DE_SET_ALPHA(d->tmppal[i], 0x00);
 
-		de_dbg_pal_entry(c, i, pal[i]);
+		de_dbg_pal_entry(c, i, d->tmppal[i]);
 	}
 	de_dbg_indent(c, -1);
 
+	if(d->newicons_largepalentry_flag) {
+		de_err(c, "Image has more than 256 colors; not supported");
+		goto done;
+	}
+
 	img = de_bitmap_create(c, width, height, 4);
 	de_convert_image_paletted(decoded, bitmap_start_pos,
-		8, width, pal, img, 0);
+		8, width, d->tmppal, img, 0);
 	de_bitmap_write_to_file(img, c->filenames_from_file?"n":NULL,
 		d->has_glowicons?DE_CREATEFLAG_IS_AUX:0);
 
@@ -211,11 +229,10 @@ static int do_read_main_icon(deark *c, lctx *d,
 	i64 pdwidth;
 	i64 depth;
 	i64 src_rowspan, src_planespan;
-	i64 i, j, plane;
+	i64 i;
 	int retval = 0;
 	de_bitmap *img = NULL;
-	u8 b, b1;
-	u32 pal[256];
+	de_color pal[256];
 
 	de_dbg(c, "main icon #%d at %"I64_FMT, (int)icon_index, pos);
 	de_dbg_indent(c, 1);
@@ -228,7 +245,7 @@ static int do_read_main_icon(deark *c, lctx *d,
 	de_dbg(c, "depth: %d", (int)depth);
 
 	if(depth<1 || depth>8) {
-		de_err(c, "Unsupported bit depth (%d)", (int)depth);
+		de_err(c, "Unsupported bit depth: %d", (int)depth);
 		goto done;
 	}
 
@@ -261,31 +278,17 @@ static int do_read_main_icon(deark *c, lctx *d,
 	else if(depth==3) {
 		for(i=0; i<8; i++) pal[i] = magicwbpal[i];
 	}
-	else if(depth==4) {
-		// ???
-		for(i=0; i<16; i++) pal[i] = magicwbpal[i>>1];
-	}
-	else if(depth==8) {
-		// Don't ask me. Just doing what other apps seem to do.
-		for(i=0; i<256; i++) pal[i] = magicwbpal[i>>5];
-	}
-	else {
-		de_warn(c, "Don't know how to handle images with bit depth %d", (int)depth);
-	}
-
-	pos += 20;
-
-	for(j=0; j<height; j++) {
-		for(i=0; i<pdwidth; i++) {
-			b = 0x00;
-			for(plane=0; plane<depth; plane++) {
-				b1 = de_get_bits_symbol(c->infile, 1, pos+plane*src_planespan + j*src_rowspan, i);
-				b = (b<<1) | (b1 & 0x1);
-			}
-			de_bitmap_setpixel_rgb(img, i, j, pal[b]);
+	else { // depth 4..8
+		for(i=0; i<(1LL<<depth); i++) {
+			// I don't know the logic behind this, and it might not be correct in
+			// all cases.
+			pal[i] = magicwbpal[i>>(depth-3)];
 		}
 	}
 
+	pos += 20;
+	de_convert_image_paletted_planar(c->infile, pos, depth, src_rowspan, src_planespan,
+		pal, img, 0);
 	de_bitmap_write_to_file(img, NULL, (d->has_newicons||d->has_glowicons)?DE_CREATEFLAG_IS_AUX:0);
 
 	retval = 1;
@@ -392,38 +395,40 @@ done:
 	return retval;
 }
 
-// Uncompress a slice of f, and append to outf.
+// Decompress a slice of f, and append to outf.
 // The algorithm is the same as PackBits, except that the data elements may
 // be less than 8 bits.
-static void glowdata_uncompress(dbuf *f, i64 pos, i64 len,
-	dbuf *outf, int bits_per_pixel)
+static void glowdata_decompress(dbuf *f, i64 pos, i64 len,
+	dbuf *outf, UI bits_per_pixel)
 {
 	i64 x;
 	i64 i;
 	u8 b, b2;
-	i64 bitpos;
+	struct de_bitreader bitrd;
 
-	bitpos = 0;
+	de_zeromem(&bitrd, sizeof(struct de_bitreader));
+	bitrd.f = f;
+	bitrd.curpos = pos;
+	bitrd.endpos = pos + len;
+	bitrd.bbll.is_lsb = 0;
+	de_bitbuf_lowlevel_empty(&bitrd.bbll);
 
-	// Continue as long as at least 8 bits remain.
-	while(bitpos <= (len-1)*8) {
-		b = de_get_bits_symbol2(f, 8, pos, bitpos);
-		bitpos+=8;
+	while(1) {
+		b = (u8)de_bitreader_getbits(&bitrd, 8);
+		if(bitrd.eof_flag) break;
 
 		if(b<=127) {
 			// 1+b literal pixels
 			x = 1+(i64)b;
 			for(i=0; i<x; i++) {
-				b2 = de_get_bits_symbol2(f, bits_per_pixel, pos, bitpos);
-				bitpos += bits_per_pixel;
+				b2 = (u8)de_bitreader_getbits(&bitrd, bits_per_pixel);
 				dbuf_writebyte(outf, b2);
 			}
 		}
 		else if(b>=129) {
 			// 257-b repeated pixels
 			x = 257 - (i64)b;
-			b2 = de_get_bits_symbol2(f, bits_per_pixel, pos, bitpos);
-			bitpos += bits_per_pixel;
+			b2 = (u8)de_bitreader_getbits(&bitrd, bits_per_pixel);
 			for(i=0; i<x; i++) {
 				dbuf_writebyte(outf, b2);
 			}
@@ -473,13 +478,6 @@ static void do_glowicons_IMAG(deark *c, lctx *d,
 		goto done;
 	}
 
-	if(cmpr_type!=1) {
-		// TODO uncompressed images (Need sample files. I don't know how
-		// they are structured.)
-		de_err(c, "Uncompressed images are not supported");
-		goto done;
-	}
-
 	if(has_palette) {
 		pal_cmpr_type = de_getbyte(pos+4);
 		de_dbg(c, "palette compression type: %d", pal_cmpr_type);
@@ -502,6 +500,7 @@ static void do_glowicons_IMAG(deark *c, lctx *d,
 	pos+=10;
 
 	tmpbuf = dbuf_create_membuf(c, 10240, 0);
+	dbuf_set_length_limit(tmpbuf, 1048576);
 
 	image_pos = pos;
 	pal_pos = image_pos+image_size_in_bytes;
@@ -512,7 +511,7 @@ static void do_glowicons_IMAG(deark *c, lctx *d,
 		de_dbg_indent(c, 1);
 
 		if(pal_cmpr_type==1) {
-			glowdata_uncompress(c->infile, pal_pos, pal_size_in_bytes, tmpbuf, 8);
+			glowdata_decompress(c->infile, pal_pos, pal_size_in_bytes, tmpbuf, 8);
 		}
 		else {
 			dbuf_copy(c->infile, pal_pos, pal_size_in_bytes, tmpbuf);
@@ -534,9 +533,14 @@ static void do_glowicons_IMAG(deark *c, lctx *d,
 		de_dbg_indent(c, -1);
 	}
 
-	// Uncompress the pixels
+	// Decompress the pixels
 	dbuf_empty(tmpbuf);
-	glowdata_uncompress(c->infile, image_pos, image_size_in_bytes, tmpbuf, (int)bits_per_pixel);
+	if(cmpr_type==0) {
+		dbuf_copy(c->infile, image_pos, image_size_in_bytes, tmpbuf);
+	}
+	else {
+		glowdata_decompress(c->infile, image_pos, image_size_in_bytes, tmpbuf, (UI)bits_per_pixel);
+	}
 
 	img = de_bitmap_create(c, d->glowicons_width, d->glowicons_height, has_trns?4:3);
 
