@@ -16,6 +16,7 @@ DE_DECLARE_MODULE(de_module_arcmac);
 #define FMT_ARC 1
 #define FMT_SPARK 2
 #define FMT_ARCMAC 3
+#define FMT_PAK16SFX 4
 
 #define MAX_NESTING_LEVEL 24
 
@@ -78,9 +79,11 @@ struct localctx_struct {
 	u8 has_arc_extensions;
 	i64 prescan_pos_after_eoa;
 	i64 num_top_level_members; // Not including EOA marker
+	i64 end_of_known_data;
 	struct de_crcobj *crco;
 	struct de_strarray *curpath;
 	struct persistent_member_data *persistent_md; // optional array[num_top_level_members]
+	dbuf *pak16sfx_outf;
 };
 
 struct member_parser_data {
@@ -96,6 +99,78 @@ struct member_parser_data {
 
 typedef void (*member_cb_type)(deark *c, lctx *d, struct member_parser_data *mpd);
 
+// Look for any known ARC cmpr meth, including end-of-archive marker.
+// Not for Spark format
+static int is_known_cmpr_meth(u8 m)
+{
+	if(m<=11 || m==20 || m==21 || m==22 || m==30) {
+		return 1;
+	}
+	return 0;
+}
+
+// Is this a plausible cmpr meth for the first archive member
+// (including end-of-archive marker)?
+// (Not for Spark format.)
+static int is_known_first_cmpr_meth(u8 m)
+{
+	if((m<=11) || m==20) {
+		return 1;
+	}
+	return 0;
+}
+
+// For ARC, not for Spark or ArcMac.
+// It's assumed that the byte at pos1 is known to be 0x1a.
+// Will validate the first member, and (unless it is the end marker,
+// or strictness==0), the first two bytes of the second member.
+static int is_valid_file_at(dbuf *f, i64 pos1, i64 endpos, UI strictness)
+{
+	u8 marker;
+	u8 cmpr_meth;
+	i64 cmpr_len;
+	i64 pos = pos1;
+
+	pos++;
+
+	cmpr_meth = dbuf_getbyte_p(f, &pos);
+	if(!is_known_first_cmpr_meth(cmpr_meth)) {
+		return 0;
+	}
+	if(cmpr_meth==0) return 1; // End marker
+
+	if(cmpr_meth<20 && strictness>=2) {
+		u8 f1;
+
+		// test 1st char of filename
+		f1 = dbuf_getbyte(f, pos);
+		if(f1<0x20) return 0;
+	}
+	pos += 13;
+
+	if(strictness==0) return 1;
+
+	cmpr_len = dbuf_getu32le_p(f, &pos);
+	pos += 6;
+	if(cmpr_meth!=1) pos += 4;
+	pos += cmpr_len;
+	if(pos==endpos) return 1;
+	if(pos>endpos-2) return 0;
+
+	marker = dbuf_getbyte_p(f, &pos);
+	if(marker!=0x1a) return 0;
+	cmpr_meth = dbuf_getbyte_p(f, &pos);
+	if(!is_known_cmpr_meth(cmpr_meth)) return 0;
+	return 1;
+}
+
+static void mark_end_of_known_data(lctx *d, i64 pos)
+{
+	if(pos > d->end_of_known_data) {
+		d->end_of_known_data = pos;
+	}
+}
+
 // Calls the supplied callback function for each ARC member found.
 // Also called for end-of-archive/directory markers.
 // Also called if unexpected data is encountered (with mpd->magic != 0x1a).
@@ -116,7 +191,18 @@ static void parse_member_sequence(deark *c, lctx *d, i64 pos1, i64 len, int nest
 		mpd->member_pos = pos;
 
 		mpd->magic = de_getbyte_p(&pos);
-		if(mpd->magic!=d->sig_byte) {
+
+		if(mpd->magic==0xfe && d->fmt==FMT_PAK16SFX) {
+			// Some hacks here. We're not setting all the fields correctly,
+			// just the ones we need.
+			mpd->cmpr_data_len = (i64)de_getbyte_p(&pos);
+			mpd->cmpr_data_pos = pos;
+			mpd->member_len = 2+mpd->cmpr_data_len;
+			member_cbfn(c, d, mpd);
+			pos = mpd->member_pos + mpd->member_len;
+			continue;
+		}
+		else if(mpd->magic!=d->sig_byte) {
 			mpd->member_len = 1;
 			mpd->cmpr_data_pos = mpd->member_pos; // dummy value
 			member_cbfn(c, d, mpd);
@@ -393,8 +479,28 @@ static void init_trailer_data(deark *c, lctx *d)
 	}
 }
 
+static int is_pkcomment_sig_at(deark *c, lctx *d, i64 pos)
+{
+	i64 comments_descr_pos;
+
+	if(!d->prescan_found_eoa) return 0;
+	if(pos < d->prescan_pos_after_eoa) return 0;
+	if(pos > c->infile->len-8) return 0;
+	if((UI)de_getu32be(pos) != 0x504baa55U) {
+		return 0;
+	}
+	comments_descr_pos = de_getu32le(pos+4);
+	if((comments_descr_pos < d->prescan_pos_after_eoa) ||
+		(comments_descr_pos > pos-4))
+	{
+		return 0;
+	}
+	return 1;
+}
+
 static void do_pk_comments(deark *c, lctx *d)
 {
+	int found = 0;
 	i64 sig_pos;
 	i64 comments_descr_pos;
 	int has_file_comments = 0;
@@ -405,18 +511,44 @@ static void do_pk_comments(deark *c, lctx *d)
 
 	if(!d->prescan_found_eoa) return;
 	sig_pos = c->infile->len-8;
-	if(sig_pos < d->prescan_pos_after_eoa) return;
-	if(de_getu32be(sig_pos) != 0x504baa55) {
-		return;
+	if(is_pkcomment_sig_at(c, d, sig_pos)){
+		found = 1;
 	}
+
+	if(!found) {
+		i64 count;
+
+		// The PK signature normally appears 8 bytes from the end of the file,
+		// but we'll scan for it, in case the file has padding appended to it.
+		// The assumption is that it appears after a whole number of 32-byte
+		// records. In case of multiple signatures, we use the earliest one that
+		// looks valid.
+		sig_pos = d->prescan_pos_after_eoa+32;
+		count = 0;
+
+		while(1) {
+			if(sig_pos >= c->infile->len-8) break;
+			if(count > d->num_top_level_members+10) break;
+			if(is_pkcomment_sig_at(c, d, sig_pos)){
+				found = 1;
+				break;
+			}
+			sig_pos += 32;
+			count++;
+		}
+	}
+
+	if(!found) return;
+
+	de_dbg(c, "PKARC/PKPAK comment block found, ID at %"I64_FMT, sig_pos);
+	de_dbg_indent(c, 1);
+	mark_end_of_known_data(d, sig_pos+8);
 	init_trailer_data(c, d);
 
-	de_dbg(c, "PKARC/PKPAK comment block found");
-	de_dbg_indent(c, 1);
 	// Note: This logic is based on reverse engineering, and could be wrong.
-	comments_descr_pos = de_getu32le(c->infile->len-4);
+	comments_descr_pos = de_getu32le(sig_pos+4);
 	de_dbg(c, "descriptor pos: %"I64_FMT, comments_descr_pos);
-	if(comments_descr_pos >= sig_pos) goto done;
+	// comments_descr_pos has already been validated, by is_pkcomment_sig_at().
 
 	de_read(dscr, comments_descr_pos, 4);
 	if(dscr[0]==0x20 && dscr[1]==0x20 && dscr[2]==0x20 && dscr[3]==0x00) {
@@ -464,11 +596,12 @@ static void do_pk_comments(deark *c, lctx *d)
 		}
 	}
 
-done:
 	ucstring_destroy(archive_comment);
 	de_dbg_indent(c, -1);
 }
 
+// Always sets *pbytes_consumed.
+// Returns 0 if there are no more records after this.
 static int do_pak_ext_record(deark *c, lctx *d, i64 pos1, i64 *pbytes_consumed)
 {
 	i64 pos = pos1;
@@ -483,6 +616,7 @@ static int do_pak_ext_record(deark *c, lctx *d, i64 pos1, i64 *pbytes_consumed)
 	int saved_indent_level;
 
 	de_dbg_indent_save(c, &saved_indent_level);
+	*pbytes_consumed = 0;
 	if(de_getbyte_p(&pos) != 0xfe) goto done;
 	de_dbg(c, "record at %"I64_FMT, pos1);
 	de_dbg_indent(c, 1);
@@ -495,7 +629,10 @@ static int do_pak_ext_record(deark *c, lctx *d, i64 pos1, i64 *pbytes_consumed)
 	case 3: rtname = "security envelope"; break;
 	}
 	de_dbg(c, "rectype: %d (%s)", (int)rectype, rtname);
-	if(rectype==0) goto done;
+	if(rectype==0) {
+		*pbytes_consumed = 2;
+		goto done;
+	}
 
 	filenum = de_getu16le_p(&pos);
 	de_dbg(c, "file num: %d", (int)filenum);
@@ -565,12 +702,14 @@ static void do_pak_trailer(deark *c, lctx *d)
 	init_trailer_data(c, d);
 
 	while(1) {
+		int ret;
 		i64 bytes_consumed = 0;
 
 		if(pos > c->infile->len-2) break;
-		if(!do_pak_ext_record(c, d, pos, &bytes_consumed)) break;
-		if(bytes_consumed<8) break;
+		ret = do_pak_ext_record(c, d, pos, &bytes_consumed);
 		pos += bytes_consumed;
+		mark_end_of_known_data(d, pos);
+		if(!ret || bytes_consumed<8) break;
 	}
 
 	de_dbg_indent(c, -1);
@@ -751,8 +890,15 @@ static void do_extract_member_file(deark *c, lctx *d, struct member_data *md,
 	de_dbg_indent(c, 1);
 
 	if(!md->cmi || !md->cmi->decompressor) {
+		const char *mname;
+
+		// A quick hack because we don't want the error message claiming that
+		// "trimmed or crushed" isn't supported. We do support trimmed, so
+		// if we got here, we must have decided it's crushed.
+		mname = md->cmpr_meth==10 ? "crushed" : md->cmpr_meth_name;
+
 		de_err(c, "%s: Compression type 0x%02x (%s) is not supported.",
-			ucstring_getpsz_d(md->fn), (unsigned int)md->cmpr_meth, md->cmpr_meth_name);
+			ucstring_getpsz_d(md->fn), (UI)md->cmpr_meth, mname);
 		goto done;
 	}
 
@@ -1222,6 +1368,9 @@ static void destroy_lctx(deark *c, lctx *d)
 	if(!d) return;
 	de_crcobj_destroy(d->crco);
 	de_strarray_destroy(d->curpath);
+	if(d->pak16sfx_outf) {
+		dbuf_close(d->pak16sfx_outf);
+	}
 	if(d->persistent_md) {
 		i64 i;
 
@@ -1234,11 +1383,28 @@ static void destroy_lctx(deark *c, lctx *d)
 	de_free(c, d);
 }
 
+static int is_btspk(deark *c, lctx *d)
+{
+	if(d->fmt!=FMT_ARC && d->fmt!=FMT_ARCMAC) return 0;
+
+	if(dbuf_memcmp(c->infile, c->infile->len-31,
+		(const void*)"\x1b\x03" "MASTR\xff" "END!", 12))
+	{
+		return 0;
+	}
+	return 1;
+}
+
 static void do_run_arc_spark_internal(deark *c, lctx *d)
 {
 	i64 members_endpos;
+	i64 num_extra_bytes;
 	i64 pos = 0;
 
+	if(is_btspk(c, d)) {
+		de_warn(c, "This looks like BTSPK format, which is not correctly supported.");
+	}
+
 	d->sig_byte = (d->fmt==FMT_ARCMAC) ? 0x1b : 0x1a;
 
 	if(d->sig_byte==0x1a) {
@@ -1263,6 +1429,7 @@ static void do_run_arc_spark_internal(deark *c, lctx *d)
 	else {
 		members_endpos = c->infile->len;
 	}
+	mark_end_of_known_data(d, members_endpos);
 
 	if(d->fmt==FMT_ARC) {
 		do_pk_comments(c, d);
@@ -1271,20 +1438,46 @@ static void do_run_arc_spark_internal(deark *c, lctx *d)
 
 	do_sequence_of_members(c, d, pos, members_endpos, 0);
 
-	if(d->prescan_found_eoa && !d->has_trailer_data) {
-		i64 num_extra_bytes;
-
-		num_extra_bytes = c->infile->len - d->prescan_pos_after_eoa;
-		if(num_extra_bytes>0) {
-			de_dbg(c, "extra bytes at end of archive: %"I64_FMT" (at %"I64_FMT")",
-				num_extra_bytes, d->prescan_pos_after_eoa);
-		}
+	num_extra_bytes = c->infile->len - d->end_of_known_data;
+	if(num_extra_bytes>0) {
+		de_dbg(c, "extra bytes at end of archive: %"I64_FMT" (at %"I64_FMT")",
+			num_extra_bytes, d->end_of_known_data);
 	}
 
 done:
 	;
 }
 
+////////// Special converter for PAK v1.6 SFX archives
+
+static void member_cb_for_pak16sfx(deark *c, lctx *d, struct member_parser_data *mpd)
+{
+	de_dbg2(c, "pak16sfx member at %"I64_FMT", type=0x%02x", mpd->member_pos,
+		(UI)mpd->magic);
+	de_dbg_indent(c, 1);
+
+	// This is not perfect. We throw away the extended info, instead of converting
+	// it to extended records.
+	if(mpd->magic==0x1a) {
+		if(!d->pak16sfx_outf) {
+			d->pak16sfx_outf = dbuf_create_output_file(c, "pak", NULL, 0);
+		}
+		dbuf_copy(c->infile, mpd->member_pos, mpd->member_len, d->pak16sfx_outf);
+	}
+	de_dbg_indent(c, -1);
+}
+
+static void do_convert_pak16sfx(deark *c, lctx *d)
+{
+	d->fmt = FMT_PAK16SFX;
+	d->recurse_subdirs = 0;
+	d->sig_byte = 0x1a;
+	parse_member_sequence(c, d, 0, c->infile->len, 0,member_cb_for_pak16sfx);
+	if(d->pak16sfx_outf) {
+		dbuf_writeu16be(d->pak16sfx_outf, 0xfe00);
+	}
+}
+
 /////////////////////// ARC (core ARC-only functions)
 
 static void de_run_arc(deark *c, de_module_params *mparams)
@@ -1317,14 +1510,23 @@ static void de_run_arc(deark *c, de_module_params *mparams)
 		}
 	}
 
+	if(de_havemodcode(c, mparams, '6')) {
+		do_convert_pak16sfx(c, d);
+		goto done;
+	}
+
 	do_run_arc_spark_internal(c, d);
+
+done:
 	destroy_lctx(c, d);
 }
 
 static int de_identify_arc(deark *c)
 {
 	static const char *exts[] = {"arc", "ark", "pak", "spk", "sdn", "com"};
-	int has_ext = 0;
+	u8 has_ext = 0;
+	UI ext_idx = 0;
+	UI strictness;
 	int ends_with_trailer = 0;
 	int ends_with_comments = 0;
 	int starts_with_trailer = 0;
@@ -1342,20 +1544,26 @@ static int de_identify_arc(deark *c)
 	}
 
 	cmpr_meth = buf[arc_start+1];
-	if(cmpr_meth>11 && cmpr_meth!=20 && cmpr_meth!=21 && cmpr_meth!=22 && cmpr_meth!=30) {
-		return 0;
-	}
-	if(cmpr_meth==0) starts_with_trailer = 1;
 
+	if(cmpr_meth==0) {
+		// Don't tolerate empty archives that don't start at the beginning of file.
+		if(arc_start!=0) return 0;
+
+		starts_with_trailer = 1;
+	}
+
+	// Get info about file extension
 	for(k=0; k<DE_ARRAYCOUNT(exts); k++) {
 		if(de_input_file_has_ext(c, exts[k])) {
-			has_ext = (int)(k+1);
+			has_ext = 1;
+			ext_idx = (UI)k;
 			break;
 		}
 	}
 
+	// Only tolerate leading junk for a few file extensions
 	if(arc_start>0) {
-		if(has_ext==1 || has_ext==2 || has_ext==6) { // .arc, .ark, .com
+		if(ext_idx==0 || ext_idx==1 || ext_idx==5) { // .arc, .ark, .com
 			;
 		}
 		else {
@@ -1363,14 +1571,33 @@ static int de_identify_arc(deark *c)
 		}
 	}
 
-	if(starts_with_trailer && c->infile->len==2) {
-		if(has_ext>=1 && has_ext<=4) return 15; // Empty archive, 2-byte file
+	// Look at some of the file, to see if it seems ok.
+	if(has_ext && ext_idx<=1 && arc_start==0) {
+		strictness = 0;
+	}
+	else if(has_ext && ext_idx<=3 && arc_start==0) {
+		strictness = 1;
+	}
+	else {
+		strictness = 2;
+	}
+	if(!is_valid_file_at(c->infile, arc_start, c->infile->len, strictness)) {
 		return 0;
 	}
 
-	if((!starts_with_trailer) && (de_getu16be(c->infile->len-2) == 0x1a00)) {
+	// Handle 2-byte files
+	if(starts_with_trailer && c->infile->len==2) {
+		if(!has_ext) return 0;
+		if(ext_idx==0) return 100;
+		if(ext_idx<=3) return 15;
+		return 0;
+	}
+
+	if(de_getu16be(c->infile->len-2) == 0x1a00) {
+		// Standard ARC trailer
 		ends_with_trailer = 1;
 	}
+
 	if(de_getu32be(c->infile->len-8) == 0x504baa55) {
 		// PKARC trailer, for files with comments
 		ends_with_comments = 1;
@@ -1388,8 +1615,9 @@ static int de_identify_arc(deark *c)
 		else return 0;
 	}
 	if(has_ext && (ends_with_trailer || ends_with_comments)) return 90;
-	if(ends_with_trailer || ends_with_comments) return 25;
-	if(has_ext) return 15;
+	if(ends_with_trailer || ends_with_comments) return 35;
+	if(has_ext) return 24;
+	if(arc_start==0) return 19;
 	return 0;
 }
 
