@@ -19,7 +19,7 @@ typedef struct localctx_struct {
 	u8 is_jp2_jpx_jpm, is_jpx, is_jpm;
 	u8 is_mj2;
 	u8 is_heif;
-	u8 is_jpegxt;
+	u8 is_jumbf;
 	i64 max_entries_to_print;
 
 	u8 exif_item_id_known;
@@ -36,7 +36,7 @@ struct box_type_info {
 	// 0x00000001 = Generic BMFF (isom brand, etc.)
 	// 0x00000008 = MJ2
 	// 0x00010000 = JP2/JPX/JPM
-	// 0x00040000 = JPEG XT
+	// 0x00040000 = JUMBF or JPEG XT
 	// 0x00080000 = HEIF
 	// 0x01000000 = Used in ilst boxes
 	u32 flags1;
@@ -83,6 +83,7 @@ struct box_type_info {
 #define BOX_ispe 0x69737065U
 #define BOX_jP   0x6a502020U
 #define BOX_jp2c 0x6a703263U
+#define BOX_jumb 0x6a756d62U
 #define BOX_load 0x6c6f6164U
 #define BOX_mdat 0x6d646174U
 #define BOX_mdhd 0x6d646864U
@@ -1457,12 +1458,14 @@ static void do_box_iloc(deark *c, lctx *d, struct de_boxesctx *bctx)
 			if(offset_size>0) {
 				xoffs = dbuf_getint_ext(bctx->f, pos, offset_size, 0, 0);
 				de_dbg(c, "offset: %"I64_FMT, xoffs);
+				de_sanitize_offset(&xoffs);
 			}
 			pos += offset_size;
 
 			if(length_size>0) {
 				xlen = dbuf_getint_ext(bctx->f, pos, length_size, 0, 0);
 				de_dbg(c, "length: %"I64_FMT, xlen);
+				de_sanitize_length(&xlen);
 			}
 			pos += length_size;
 
@@ -1696,6 +1699,7 @@ static const struct box_type_info box_type_info_arr[] = {
 	{BOX_uinf, 0x00010000, 0x00000001, "UUID info", NULL},
 	{BOX_ulst, 0x00010000, 0x00000000, "UUID list", do_box_ulst},
 	{BOX_xml , 0x00010008, 0x00000000, "XML", do_box_xml},
+	{BOX_jumb, 0x00040000, 0x00000001, NULL, NULL},
 	{BOX_LCHK, 0x00040000, 0x00000000, "checksum", NULL},
 	{BOX_RESI, 0x00040000, 0x00000000, "residual codestream", NULL},
 	{BOX_SPEC, 0x00040000, 0x00000001, NULL, NULL},
@@ -1737,7 +1741,7 @@ static const struct box_type_info *find_box_type_info(deark *c, lctx *d,
 	if(d->is_bmff) mask |= 0x00000001;
 	if(d->is_mj2) mask |= 0x000000008;
 	if(d->is_jp2_jpx_jpm) mask |= 0x00010000;
-	if(d->is_jpegxt) mask |= 0x00040000;
+	if(d->is_jumbf) mask |= 0x00040000;
 	if(d->is_heif) mask |= 0x00080000;
 
 	for(k=0; k<DE_ARRAYCOUNT(box_type_info_arr); k++) {
@@ -1853,7 +1857,7 @@ static void de_run_bmff(deark *c, de_module_params *mparams)
 	bctx = de_malloc(c, sizeof(struct de_boxesctx));
 
 	if(de_havemodcode(c, mparams, 'T')) {
-		d->is_jpegxt = 1;
+		d->is_jumbf = 1;
 		skip_autodetect = 1;
 	}
 	if(de_havemodcode(c, mparams, 'X')) {
