@@ -8,20 +8,20 @@
 // public domain(-ish) software by Andre Rodrigues de la Rocha.
 
 // Note:
-// DMS does a thing I call "persistent decompression state".
-// Roughly speaking, the state of some decompressors is reset only after
-// processing a track for which the low bit of the track_flags field is 0.
-// Otherwise it persists, and will most likely be used with a future track.
+// DMS has a feature, which I guess is its version of a "solid" archive, whereby
+// part of the state of some decompressors is reset only after processing a
+// track for which the low bit of the track_flags field is 0. Otherwise it
+// persists, to be used by the next suitable track.
 // I don't understand the fine details of how this works, especially when it
 // comes to "extra" tracks that are not part of the main track sequence.
 // xDMS behaves as if the state from an extra track never persists into a
 // "real" track, or vice versa, even if its flag says it does. I have a file
 // that seems to confirm that that is the case.
-// But that still leaves a lot of open questions. (What's the precise
+// But that still leaves a lot of open questions. What's the precise
 // definition of an extra track? What if there are multiple compressed extra
 // tracks? Or an extra track in the middle of the real tracks? To what extent
 // can multiple compression methods be used in the same file, and how do they
-// interact?)
+// interact?
 
 #include <deark-private.h>
 #include <deark-fmtutil.h>
@@ -82,6 +82,8 @@ struct dmsctx {
 	UI cmpr_type;
 	i64 first_track, last_track;
 	i64 num_tracks_in_file;
+	u8 have_first_last_track_info;
+	struct de_crcobj *crco_cksum;
 
 	// Entries in use: 0 <= n < .num_tracks_in_file
 	struct dms_tracks_by_file_order_entry tracks_by_file_order[DMS_MAX_TRACKS];
@@ -89,6 +91,7 @@ struct dmsctx {
 	// Entries potentially in use: .first_track <= n <= .last_track
 	struct dms_tracks_by_track_num_entry tracks_by_track_num[DMS_MAX_TRACKS];
 
+	struct std_saved_state_wrapper saved_quick_state;
 	struct std_saved_state_wrapper saved_medium_state;
 	struct std_saved_state_wrapper saved_deep_cmpr_state;
 	struct dmsheavy_cmpr_state *saved_heavy_state;
@@ -789,21 +792,131 @@ static void dmsmediumlz77_codec(struct de_dfilter_ctx *dfctx, void *codec_privat
 	mctx->ringbuf->writebyte_cb = medium_lz77buf_writebytecb;
 }
 
-///////////////// "Medium" and "Deep" decompression //////////////
+///////////////// Codec for the LZ77 part of "Quick" decompression //////////////
 
-// Note: The Medium and Deep decompressors have a different design than Heavy.
-// Both the codecs are "pushable", which is a good thing for DMS's segmented
-// format.
+// This was derived from the Medium codec. Some of the symbol names are
+// misleading, because of that.
+
+static void quicklz77_codec_addbuf(struct de_dfilter_ctx *dfctx,
+	const u8 *buf, i64 buf_len)
+{
+	struct medium_ctx *mctx = (struct medium_ctx *)dfctx->codec_private;
+	struct medium_state_machine *mdst = &mctx->mdst;
+	i64 bufpos = 0;
+	UI n;
+	UI length;
+	UI ocode, lcode;
+
+	if(dfctx->finished_flag || mctx->errflag) {
+		goto done;
+	}
+
+	while(1) {
+		if(medium_have_enough_output(mctx)) {
+			dfctx->finished_flag = 1;
+			goto done;
+		}
+
+		// Top off the bitbuf, if possible. 32 is an arbitrary number that's
+		// large enough for this format.
+		while(bufpos<buf_len && mctx->bbll.nbits_in_bitbuf<32) {
+			de_bitbuf_lowlevel_add_byte(&mctx->bbll, buf[bufpos++]);
+		}
+
+		if(mdst->state==MEDLZST_WAITING_FOR_OCODE1) goto read_ocode1;
+
+		// Minimum useful number of bits is 9.
+		if(mctx->bbll.nbits_in_bitbuf < 9) goto done;
+
+		n = (UI)de_bitbuf_lowlevel_get_bits(&mctx->bbll, 1);
+		if(n) { // literal byte
+			u8 b;
+
+			b = (u8)de_bitbuf_lowlevel_get_bits(&mctx->bbll, 8);
+			de_lz77buffer_add_literal_byte(mctx->ringbuf, (u8)b);
+			continue;
+		}
+
+	read_ocode1:
+		if(mctx->bbll.nbits_in_bitbuf < 10) {
+			mdst->state = MEDLZST_WAITING_FOR_OCODE1;
+			goto done;
+		}
+
+		lcode = (UI)de_bitbuf_lowlevel_get_bits(&mctx->bbll, 2);
+		ocode = (UI)de_bitbuf_lowlevel_get_bits(&mctx->bbll, 8);
+		length = lcode + 2;
+		de_lz77buffer_copy_from_hist(mctx->ringbuf, mctx->ringbuf->curpos - 1 - ocode, length);
+		mdst->state = MEDLZST_NEUTRAL;
+	}
+
+done:
+	if(!dfctx->finished_flag && !mctx->errflag) {
+		if(bufpos<buf_len) {
+			// It shouldn't be possible to get here. If we do, it means some input
+			// bytes will be lost.
+			de_dfilter_set_generic_error(dfctx->c, mctx->dres, mctx->modname);
+			mctx->errflag = 1;
+		}
+	}
+}
+
+static void quicklz77_codec_command(struct de_dfilter_ctx *dfctx, int cmd, UI flags)
+{
+	struct medium_ctx *mctx = (struct medium_ctx*)dfctx->codec_private;
+
+	if(cmd==DE_DFILTER_COMMAND_FINISH_BLOCK) {
+		de_bitbuf_lowlevel_empty(&mctx->bbll);
+		mctx->mdst.state = MEDLZST_NEUTRAL;
+		de_lz77buffer_set_curpos(mctx->ringbuf, mctx->ringbuf->curpos + 5);
+	}
+	else if(cmd==DE_DFILTER_COMMAND_RESET_COUNTERS) {
+		mctx->nbytes_written = 0;
+		mctx->errflag = 0;
+		dfctx->finished_flag = 0;
+	}
+}
+
+// codec_private_params: unused
+static void dmsquicklz77_codec(struct de_dfilter_ctx *dfctx, void *codec_private_params)
+{
+	struct medium_ctx *mctx = NULL;
+
+	mctx = de_malloc(dfctx->c, sizeof(struct medium_ctx));
+	mctx->c = dfctx->c;
+	mctx->modname = "dmsquick_lz77";
+	mctx->dcmpro = dfctx->dcmpro;
+	mctx->dres = dfctx->dres;
+
+	dfctx->codec_private = (void*)mctx;
+	dfctx->codec_addbuf_fn = quicklz77_codec_addbuf;
+	dfctx->codec_command_fn = quicklz77_codec_command;
+	dfctx->codec_destroy_fn = mediumlz77_codec_destroy;
+
+	mctx->ringbuf = de_lz77buffer_create(dfctx->c, 256);
+	de_lz77buffer_set_curpos(mctx->ringbuf, 251);
+
+	mctx->ringbuf->userdata = (void*)mctx;
+	mctx->ringbuf->writebyte_cb = medium_lz77buf_writebytecb;
+}
+
+///////////////// "Quick", "Medium", "Deep" decompression //////////////
+
+// Note: These decompressors have a different design than Heavy.
+// The codecs are "pushable", which is a good thing for DMS's segmented format.
 // It may look more complicated, but ultimately it's cleaner and less hacky.
 
-static void do_decompress_track_medium_or_deep(deark *c, struct dmsctx *d, struct dms_track_info *tri,
+static void do_decompress_track_q_m_d(deark *c, struct dmsctx *d, struct dms_track_info *tri,
 	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
 	struct de_dfilter_results *dres)
 {
 	struct dms_std_cmpr_state *cst = NULL;
 	struct std_saved_state_wrapper *ssw;
 
-	if(tri->cmpr_type==DMSCMPR_DEEP) {
+	if(tri->cmpr_type==DMSCMPR_QUICK) {
+		ssw = &d->saved_quick_state;
+	}
+	else if(tri->cmpr_type==DMSCMPR_DEEP) {
 		ssw = &d->saved_deep_cmpr_state;
 	}
 	else {
@@ -830,7 +943,10 @@ static void do_decompress_track_medium_or_deep(deark *c, struct dmsctx *d, struc
 	if(!cst) {
 		cst = de_malloc(c, sizeof(struct dms_std_cmpr_state));
 		cst->cmpr_meth = tri->cmpr_type;
-		if(cst->cmpr_meth==DMSCMPR_DEEP) {
+		if(cst->cmpr_meth==DMSCMPR_QUICK) {
+			cst->cmpr_meth_name = "quickcmpr";
+		}
+		else if(cst->cmpr_meth==DMSCMPR_DEEP) {
 			cst->cmpr_meth_name = "deepcmpr";
 		}
 		else {
@@ -864,6 +980,10 @@ static void do_decompress_track_medium_or_deep(deark *c, struct dmsctx *d, struc
 			cst->dfctx_codec1 = de_dfilter_create(c, dfilter_lh1_codec,
 				(void*)&lh1p, &cst->dcmpro1, &cst->dres1);
 		}
+		else if(cst->cmpr_meth==DMSCMPR_QUICK) {
+			cst->dfctx_codec1 = de_dfilter_create(c, dmsquicklz77_codec,
+				NULL, &cst->dcmpro1, &cst->dres1);
+		}
 		else {
 			cst->dfctx_codec1 = de_dfilter_create(c, dmsmediumlz77_codec,
 				NULL, &cst->dcmpro1, &cst->dres1);
@@ -942,6 +1062,10 @@ static void do_decompress_heavy(deark *c, struct dmsctx *d, struct dms_track_inf
 
 static void destroy_saved_dcrmpr_state(deark *c, struct dmsctx *d)
 {
+	if(d->saved_quick_state.saved_state) {
+		destroy_std_cmpr_state(c, d->saved_quick_state.saved_state);
+		d->saved_quick_state.saved_state = NULL;
+	}
 	if(d->saved_medium_state.saved_state) {
 		destroy_std_cmpr_state(c, d->saved_medium_state.saved_state);
 		d->saved_medium_state.saved_state = NULL;
@@ -989,11 +1113,10 @@ static int dms_decompress_track(deark *c, struct dmsctx *d, struct dms_track_inf
 		de_dfilter_decompress_oneshot(c, dmsrle_codec, NULL,
 			&dcmpri, &dcmpro, &dres);
 	}
-	else if(tri->cmpr_type==DMSCMPR_MEDIUM) {
-		do_decompress_track_medium_or_deep(c, d, tri, &dcmpri, &dcmpro, &dres);
-	}
-	else if(tri->cmpr_type==DMSCMPR_DEEP) {
-		do_decompress_track_medium_or_deep(c, d, tri, &dcmpri, &dcmpro, &dres);
+	else if(tri->cmpr_type==DMSCMPR_QUICK || tri->cmpr_type==DMSCMPR_MEDIUM ||
+		tri->cmpr_type==DMSCMPR_DEEP)
+	{
+		do_decompress_track_q_m_d(c, d, tri, &dcmpri, &dcmpro, &dres);
 	}
 	else if(tri->cmpr_type==DMSCMPR_HEAVY1 || tri->cmpr_type==DMSCMPR_HEAVY2) {
 		do_decompress_heavy(c, d, tri, &dcmpri, &dcmpro, &dres);
@@ -1035,24 +1158,14 @@ done:
 	return retval;
 }
 
-static int dms_checksum_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
-	i64 buf_len)
+// f is presumed to be membuf containing one track, and nothing else.
+static u32 dms_calc_checksum(deark *c, struct dmsctx *d, dbuf *f)
 {
-	u32 *cksum = (u32*)brctx->userdata;
-	i64 i;
+	u32 cksum;
 
-	for(i=0; i<buf_len; i++) {
-		*cksum += (u32)buf[i];
-	}
-	return 1;
-}
-
-// outf is presumed to be membuf containing one track, and nothing else.
-static u32 dms_calc_checksum(deark *c, dbuf *outf)
-{
-	u32 cksum = 0;
-
-	dbuf_buffered_read(outf, 0, outf->len, dms_checksum_cbfn, (void*)&cksum);
+	de_crcobj_reset(d->crco_cksum);
+	de_crcobj_addslice(d->crco_cksum, f, 0, f->len);
+	cksum = de_crcobj_getval(d->crco_cksum);
 	cksum &= 0xffff;
 	return cksum;
 }
@@ -1072,7 +1185,7 @@ static void get_trackflags_descr(deark *c, de_ucstring *s, UI tflags1, UI cmpr)
 		}
 	}
 	if(tflags & 0x1) {
-		ucstring_append_flags_item(s, "persist decompr. state");
+		ucstring_append_flags_item(s, "solid");
 		tflags -= 0x1;
 	}
 	if(tflags>0) ucstring_append_flags_itemf(s, "0x%02x", tflags);
@@ -1133,7 +1246,7 @@ static int dms_read_and_decompress_track(deark *c, struct dmsctx *d,
 	if(!dms_decompress_track(c, d, tri, outf)) goto done;
 	de_dbg_indent(c, -1);
 
-	tri->cksum_calc = dms_calc_checksum(c, outf);
+	tri->cksum_calc = dms_calc_checksum(c, d, outf);
 	de_dbg(c, "checksum (calculated): 0x%04x", (UI)tri->cksum_calc);
 	if(tri->cksum_calc != tri->cksum_reported) {
 		de_err(c, "[%s] Checksum check failed", tri->shortname);
@@ -1206,6 +1319,7 @@ static int do_dms_header(deark *c, struct dmsctx *d, i64 pos1)
 {
 	i64 n;
 	i64 pos = pos1;
+	i64 cmpr_len, orig_len;
 	struct de_timestamp cr_time;
 	int retval = 0;
 
@@ -1234,11 +1348,18 @@ static int do_dms_header(deark *c, struct dmsctx *d, i64 pos1)
 	if(d->last_track < d->first_track) goto done;
 	if(d->last_track >= DMS_MAX_TRACKS) goto done;
 
-	n = de_getu32be_p(&pos); // [20..23] = packed len
-	de_dbg(c, "compressed len: %"I64_FMT, n);
+	cmpr_len = de_getu32be_p(&pos); // [20..23] = packed len
+	de_dbg(c, "compressed len: %"I64_FMT, cmpr_len);
 
-	n = de_getu32be_p(&pos); // [24..27] = unpacked len
-	de_dbg(c, "decompressed len: %"I64_FMT, n);
+	orig_len = de_getu32be_p(&pos); // [24..27] = unpacked len
+	de_dbg(c, "decompressed len: %"I64_FMT, orig_len);
+
+	if(d->first_track==0 && d->last_track==0 && (cmpr_len==0 || orig_len==0)) {
+		de_warn(c, "File lacks info about which tracks are present");
+	}
+	else {
+		d->have_first_last_track_info = 1;
+	}
 
 	// [46..47] = creating software version
 	pos = pos1+50;
@@ -1263,6 +1384,8 @@ static int dms_scan_file(deark *c, struct dmsctx *d, i64 pos1)
 {
 	i64 pos = pos1;
 	i64 i;
+	i64 lowest_real_track_num = DMS_MAX_TRACKS;
+	i64 highest_real_track_num = 0;
 	u32 next_real_tracknum_expected;
 	int retval = 0;
 
@@ -1302,15 +1425,42 @@ static int dms_scan_file(deark *c, struct dmsctx *d, i64 pos1)
 		d->tracks_by_file_order[d->num_tracks_in_file].file_pos = pos;
 		d->tracks_by_file_order[d->num_tracks_in_file].track_num = (u32)track_num_reported;
 
-		if(track_num_reported>=d->first_track && track_num_reported<=d->last_track) {
-			d->tracks_by_track_num[track_num_reported].order_in_file = (u32)d->num_tracks_in_file;
-			d->tracks_by_track_num[track_num_reported].in_use = 1;
+		if(d->have_first_last_track_info) {
+			if(track_num_reported>=d->first_track && track_num_reported<=d->last_track) {
+				d->tracks_by_track_num[track_num_reported].order_in_file = (u32)d->num_tracks_in_file;
+				d->tracks_by_track_num[track_num_reported].in_use = 1;
+			}
+		}
+		else {
+			if(track_num_reported<DMS_MAX_TRACKS) {
+				d->tracks_by_track_num[track_num_reported].order_in_file = (u32)d->num_tracks_in_file;
+				d->tracks_by_track_num[track_num_reported].in_use = 1;
+			}
+		}
+
+		if(track_num_reported<DMS_MAX_TRACKS) {
+			if(track_num_reported<lowest_real_track_num) {
+				lowest_real_track_num = track_num_reported;
+			}
+			if(track_num_reported>highest_real_track_num) {
+				highest_real_track_num = track_num_reported;
+			}
 		}
 
 		d->num_tracks_in_file++;
 		pos += DMS_TRACK_HDR_LEN + cmpr_len;
 	}
 
+	if(d->num_tracks_in_file==0) {
+		de_err(c, "No tracks found");
+		goto done;
+	}
+
+	if(!d->have_first_last_track_info) {
+		d->first_track = lowest_real_track_num;
+		d->last_track = highest_real_track_num;
+	}
+
 	// Make sure all expected tracks are present, and mark the "real" tracks in
 	// tracks_by_file_order[].
 	// One reason for doing it this way is that there may be two tracks numbered 0,
@@ -1325,7 +1475,6 @@ static int dms_scan_file(deark *c, struct dmsctx *d, i64 pos1)
 		d->tracks_by_file_order[d->tracks_by_track_num[i].order_in_file].is_real = 1;
 	}
 
-
 	next_real_tracknum_expected = (u32)d->first_track;
 	for(i=0; i<d->num_tracks_in_file; i++) {
 		if(d->tracks_by_file_order[i].is_real) {
@@ -1350,6 +1499,7 @@ static void de_run_amiga_dms(deark *c, de_module_params *mparams)
 	struct dmsctx *d = NULL;
 
 	d = de_malloc(c, sizeof(struct dmsctx));
+	d->crco_cksum = de_crcobj_create(c, DE_CRCOBJ_SUM_BYTES);
 	if(!do_dms_header(c, d, 0)) goto done;
 	if(!dms_scan_file(c, d, DMS_FILE_HDR_LEN)) goto done;
 	do_dms_main(c, d);
@@ -1357,6 +1507,7 @@ static void de_run_amiga_dms(deark *c, de_module_params *mparams)
 done:
 	if(d) {
 		destroy_saved_dcrmpr_state(c, d);
+		de_crcobj_destroy(d->crco_cksum);
 		de_free(c, d);
 	}
 }
