@@ -12,30 +12,51 @@ DE_DECLARE_MODULE(de_module_epocimage);
 #define DE_PFMT_SKETCH        3
 #define DE_PFMT_AIF           4
 
-static const u32 supplpal[40] = {
-	0x111111,0x222222,0x444444,0x555555,0x777777,
-	0x110000,0x220000,0x440000,0x550000,0x770000,
-	0x001100,0x002200,0x004400,0x005500,0x007700,
-	0x000011,0x000022,0x000044,0x000055,0x000077,
-	0x000088,0x0000aa,0x0000bb,0x0000dd,0x0000ee,
-	0x008800,0x00aa00,0x00bb00,0x00dd00,0x00ee00,
-	0x880000,0xaa0000,0xbb0000,0xdd0000,0xee0000,
-	0x888888,0xaaaaaa,0xbbbbbb,0xdddddd,0xeeeeee
+// Corresponds to a Paint Data Section
+struct phys_image_ctx {
+	i64 width, height;
+	i64 color_type;
+	i64 bits_per_pixel;
+	int is_mask;
+	i64 src_rowspan;
+	de_bitmap *img;
+	i64 paint_data_section_size;
+	de_color pal[256];
 };
 
-static u32 getpal256(int k)
-{
-	int x;
-	u8 r, g, b;
+typedef struct localctx_struct {
+	int fmt;
+	i64 last_paint_data_section_size;
+	int warned_exp;
 
-	if(k<0 || k>255) return DE_STOCKCOLOR_BLACK;
+	i64 jumptable_offset;
+	i64 section_table_offset;
+	u8 have_stdpal256;
+	de_color stdpal256[256];
+} lctx;
+
+static const de_color supplpal[40] = {
+	0xff111111U,0xff222222U,0xff444444U,0xff555555U,0xff777777U,
+	0xff110000U,0xff220000U,0xff440000U,0xff550000U,0xff770000U,
+	0xff001100U,0xff002200U,0xff004400U,0xff005500U,0xff007700U,
+	0xff000011U,0xff000022U,0xff000044U,0xff000055U,0xff000077U,
+	0xff000088U,0xff0000aaU,0xff0000bbU,0xff0000ddU,0xff0000eeU,
+	0xff008800U,0xff00aa00U,0xff00bb00U,0xff00dd00U,0xff00ee00U,
+	0xff880000U,0xffaa0000U,0xffbb0000U,0xffdd0000U,0xffee0000U,
+	0xff888888U,0xffaaaaaaU,0xffbbbbbbU,0xffddddddU,0xffeeeeeeU
+};
+
+static de_color getpal256(UI k)
+{
+	UI x;
+	u8 r, g, b;
 
 	// The first and last 108 entries together make up the simple palette once
 	// known as the "web safe" palette. The middle 40 entries are
 	// supplementary grayscale and red/green/blue shades.
 
 	if(k>=108 && k<148) {
-		return DE_MAKE_OPAQUE(supplpal[k-108]);
+		return supplpal[k-108];
 	}
 
 	x = k<108 ? k : k-40;
@@ -46,117 +67,121 @@ static u32 getpal256(int k)
 	return DE_MAKE_RGB(r,g,b);
 }
 
+static void populate_stdpal256(lctx *d)
+{
+	UI k;
+
+	if(d->have_stdpal256) return;
+	d->have_stdpal256 = 1;
+
+	for(k=0; k<256; k++) {
+		d->stdpal256[k] = getpal256(k);
+	}
+}
+
 // I believe this is the correct palette (or at least *a* correct palette),
 // though it has some differences from the one in the Psiconv documentation.
-static const u32 pal16[16] = {
-	0x000000,0x555555,0x800000,0x808000,0x008000,0xff0000,0xffff00,0x00ff00,
-	0xff00ff,0x0000ff,0x00ffff,0x800080,0x000080,0x008080,0xaaaaaa,0xffffff
+static const de_color g_stdpal16[16] = {
+	0xff000000U,0xff555555U,0xff800000U,0xff808000U,0xff008000U,0xffff0000U,0xffffff00U,0xff00ff00U,
+	0xffff00ffU,0xff0000ffU,0xff00ffffU,0xff800080U,0xff000080U,0xff008080U,0xffaaaaaaU,0xffffffffU
 };
 
-struct page_ctx {
-	i64 width, height;
-	i64 color_type;
-	i64 bits_per_pixel;
-};
+static void destroy_phys_image(deark *c, struct phys_image_ctx *pi)
+{
+	if(!pi) return;
+	if(pi->img) {
+		de_bitmap_destroy(pi->img);
+	}
+	de_free(c, pi);
+}
 
-typedef struct localctx_struct {
-	int fmt;
-	i64 paint_data_section_size;
-	int warned_exp;
+static void decode_image_16bit(deark *c, lctx *d, struct phys_image_ctx *pi,
+	dbuf *unc_pixels, de_bitmap *img)
+{
+	i64 i, j;
 
-	i64 jumptable_offset;
-	i64 section_table_offset;
-} lctx;
+	for(j=0; j<pi->height; j++) {
+		for(i=0; i<pi->width; i++) {
+			u8 cr;
+			u32 n;
+			de_color clr;
 
-static de_bitmap *do_create_image(deark *c, lctx *d, struct page_ctx *pg,
-	dbuf *unc_pixels, int is_mask)
+			n = (u32)dbuf_getu16le(unc_pixels, j*pi->src_rowspan + i*2);
+			if(pi->is_mask) {
+				cr = (u8)(n>>8);
+				clr = DE_MAKE_RGB(cr, cr, cr);
+			}
+			else {
+				clr = de_rgb565_to_888(n);
+			}
+			de_bitmap_setpixel_rgb(img, i, j, clr);
+		}
+	}
+}
+
+// Returns an image in pi->img
+static void do_read_phys_image(deark *c, lctx *d, struct phys_image_ctx *pi,
+	dbuf *unc_pixels)
 {
 	de_bitmap *img = NULL;
-	i64 i, j;
-	i64 src_rowspan;
 	i64 pdwidth;
-	u8 b;
-	u8 cr;
-	u32 n;
-	u32 clr;
 	int bypp;
 
-	pdwidth = pg->width;
+	if(pi->img) goto done;
+	pdwidth = pi->width;
 
-	if(pg->bits_per_pixel==24) {
+	if(pi->bits_per_pixel==24) {
 		// 24-bit images seem to be 12-byte aligned
-		src_rowspan = ((pg->bits_per_pixel*pg->width +95)/96)*12;
+		pi->src_rowspan = ((pi->bits_per_pixel*pi->width +95)/96)*12;
 	}
-	else if(pg->bits_per_pixel==12) {
+	else if(pi->bits_per_pixel==12) {
 		// Our decompression algorithm expands RLE12 to an RGB24 format.
 		// Apparently, rows with an odd number of pixels have one pixel of
 		// padding, which at this stage is 3 bytes.
-		src_rowspan = 3*pg->width;
-		if(pg->width%2) src_rowspan += 3;
+		pi->src_rowspan = 3*pi->width;
+		if(pi->width%2) pi->src_rowspan += 3;
 	}
 	else {
 		i64 bits_per_row;
 		// Rows are 4-byte aligned
 
-		bits_per_row = de_pad_to_n(pg->bits_per_pixel*pg->width, 32);
-		src_rowspan = bits_per_row / 8;
-		pdwidth = bits_per_row / pg->bits_per_pixel;
+		bits_per_row = de_pad_to_n(pi->bits_per_pixel*pi->width, 32);
+		pi->src_rowspan = bits_per_row / 8;
+		pdwidth = bits_per_row / pi->bits_per_pixel;
 	}
 
-	bypp = pg->color_type ? 3 : 1;
-	img = de_bitmap_create2(c, pg->width, pdwidth, pg->height, bypp);
-	img->orig_colortype = (int)pg->color_type;
-	img->orig_bitdepth = (int)pg->bits_per_pixel;
+	bypp = pi->color_type ? 3 : 1;
+	img = de_bitmap_create2(c, pi->width, pdwidth, pi->height, bypp);
+	pi->img = img;
 
-	for(j=0; j<pg->height; j++) {
-		for(i=0; i<pg->width; i++) {
-			switch(pg->bits_per_pixel) {
-			case 1:
-				b = de_get_bits_symbol_lsb(unc_pixels, pg->bits_per_pixel, j*src_rowspan, i);
-				de_bitmap_setpixel_gray(img, i, j, b*255);
-				break;
-			case 2:
-				b = de_get_bits_symbol_lsb(unc_pixels, pg->bits_per_pixel, j*src_rowspan, i);
-				de_bitmap_setpixel_gray(img, i, j, b*85);
-				break;
-			case 4:
-				b = de_get_bits_symbol_lsb(unc_pixels, pg->bits_per_pixel, j*src_rowspan, i);
-				if(pg->color_type)
-					de_bitmap_setpixel_rgb(img, i, j, DE_MAKE_OPAQUE(pal16[(unsigned int)b]));
-				else
-					de_bitmap_setpixel_gray(img, i, j, b*17);
-				break;
-			case 8:
-				b = dbuf_getbyte(unc_pixels, j*src_rowspan + i);
-				if(pg->color_type) {
-					de_bitmap_setpixel_rgb(img, i, j, getpal256((unsigned int)b));
-				}
-				else {
-					// I have no 8-bit grayscale samples, so I don't know if this is
-					// correct, or valid.
-					de_bitmap_setpixel_gray(img, i, j, b);
-				}
-				break;
-			case 16:
-				n = (u32)dbuf_getu16le(unc_pixels, j*src_rowspan + i*2);
-				if(is_mask) {
-					cr = (u8)(n>>8);
-					clr = DE_MAKE_RGB(cr, cr, cr);
-				}
-				else {
-					clr = de_rgb565_to_888(n);
-				}
-				de_bitmap_setpixel_rgb(img, i, j, clr);
-				break;
-			case 12:
-			case 24:
-				clr = dbuf_getRGB(unc_pixels, j*src_rowspan + i*3, 0);
-				de_bitmap_setpixel_rgb(img, i, j, clr);
-				break;
-			}
+	if(pi->bits_per_pixel==1 || pi->bits_per_pixel==2 || pi->bits_per_pixel==4 ||
+		pi->bits_per_pixel==8)
+	{
+		if(pi->bits_per_pixel==4 && pi->color_type) {
+			de_memcpy(pi->pal, g_stdpal16, sizeof(g_stdpal16));
 		}
+		else if(pi->bits_per_pixel==8 && pi->color_type) {
+			populate_stdpal256(d);
+			de_memcpy(pi->pal, d->stdpal256, sizeof(d->stdpal256));
+		}
+		else {
+			// I have no 8-bit grayscale sample files, so I don't know if this is
+			// correct for them.
+			de_make_grayscale_palette(pi->pal, 1LL<<pi->bits_per_pixel, 0);
+		}
+
+		de_convert_image_paletted(unc_pixels, 0, pi->bits_per_pixel, pi->src_rowspan,
+			pi->pal, img, 0x1);
 	}
-	return img;
+	else if(pi->bits_per_pixel==12 || pi->bits_per_pixel==24) {
+		de_convert_image_rgb(unc_pixels, 0, pi->src_rowspan, 3, img, 0);
+	}
+	else if(pi->bits_per_pixel==16) {
+		decode_image_16bit(c, d, pi, unc_pixels, img);
+	}
+
+done:
+	;
 }
 
 static void do_rle8(deark *c, lctx *d, dbuf *unc_pixels,
@@ -259,74 +284,72 @@ static const char *get_cmpr_type_name(i64 t)
 }
 
 // Sets d->paint_data_section_size.
-// Returns a bitmap.
-static de_bitmap *do_read_paint_data_section(deark *c, lctx *d,
-	i64 pos1, int is_mask)
+// Caller creates pi, and sets pi->is_mask.
+// Returns a bitmap in pi->img, if successful
+static void do_read_paint_data_section(deark *c, lctx *d,
+	struct phys_image_ctx *pi, i64 pos1)
 {
 	i64 pixel_data_offset;
 	i64 pos;
 	dbuf *unc_pixels = NULL;
 	i64 compression_type;
 	i64 cmpr_pixels_size;
-	de_bitmap *img = NULL;
-	struct page_ctx *pg = NULL;
 
-	pg = de_malloc(c, sizeof(struct page_ctx));
 	pos = pos1;
-	de_dbg(c, "paint data section at %d", (int)pos1);
+	de_dbg(c, "paint data section at %"I64_FMT, pos1);
 	de_dbg_indent(c, 1);
 
-	d->paint_data_section_size = de_getu32le(pos);
-	de_dbg(c, "paint data section size: %d", (int)d->paint_data_section_size);
+	pi->paint_data_section_size = de_getu32le(pos);
+	de_dbg(c, "paint data section size: %"I64_FMT, pi->paint_data_section_size);
 
 	// offset within "paint data section"
 	pixel_data_offset = de_getu32le(pos+4);
-	de_dbg(c, "pixel data offset: %d", (int)pixel_data_offset);
+	de_dbg(c, "pixel data offset: %"I64_FMT, pixel_data_offset);
 
-	pg->width = de_getu16le(pos+8);
-	pg->height = de_getu16le(pos+12);
-	de_dbg(c, "picture dimensions: %d"DE_CHAR_TIMES"%d", (int)pg->width, (int)pg->height);
+	pi->width = de_getu16le(pos+8);
+	pi->height = de_getu16le(pos+12);
+	de_dbg(c, "picture dimensions: %d"DE_CHAR_TIMES"%d", (int)pi->width, (int)pi->height);
 
-	pg->bits_per_pixel = de_getu32le(pos+24);
-	de_dbg(c, "bits/pixel: %d", (int)pg->bits_per_pixel);
+	pi->bits_per_pixel = de_getu32le(pos+24);
+	de_dbg(c, "bits/pixel: %d", (int)pi->bits_per_pixel);
 
-	pg->color_type = de_getu32le(pos+28);
+	pi->color_type = de_getu32le(pos+28);
 	// 0=grayscale  1=color
-	de_dbg(c, "color type: %d", (int)pg->color_type);
+	de_dbg(c, "color type: %d", (int)pi->color_type);
 
 	compression_type = de_getu32le(pos+36);
 	// 0=uncompressed  1=8-bit RLE  2=12-bit RLE  3=16-bit RLE  4=24-bit RLE
 	de_dbg(c, "compression type: %d (%s)", (int)compression_type,
 		get_cmpr_type_name(compression_type));
 
-	if(pg->color_type==0) {
-		if(pg->bits_per_pixel!=1 && pg->bits_per_pixel!=2 && pg->bits_per_pixel!=4 &&
-			pg->bits_per_pixel!=8)
+	if(pi->color_type==0) {
+		if(pi->bits_per_pixel!=1 && pi->bits_per_pixel!=2 && pi->bits_per_pixel!=4 &&
+			pi->bits_per_pixel!=8)
 		{
-			de_err(c, "Unsupported bits/pixel (%d) for grayscale image", (int)pg->bits_per_pixel);
+			de_err(c, "Unsupported bits/pixel (%d) for grayscale image", (int)pi->bits_per_pixel);
 			goto done;
 		}
 	}
 	else {
-		if(pg->bits_per_pixel!=4 && pg->bits_per_pixel!=8 && pg->bits_per_pixel!=12 &&
-			pg->bits_per_pixel!=16 && pg->bits_per_pixel!=24)
+		if(pi->bits_per_pixel!=4 && pi->bits_per_pixel!=8 && pi->bits_per_pixel!=12 &&
+			pi->bits_per_pixel!=16 && pi->bits_per_pixel!=24)
 		{
-			de_err(c, "Unsupported bits/pixel (%d) for color image", (int)pg->bits_per_pixel);
+			de_err(c, "Unsupported bits/pixel (%d) for color image", (int)pi->bits_per_pixel);
 			goto done;
 		}
-		if(pg->bits_per_pixel==12 && compression_type!=2) {
+		if(pi->bits_per_pixel==12 && compression_type!=2) {
 			de_err(c, "12 bits/pixel images are not supported with this compression type (%d)",
 				(int)compression_type);
 		}
-		if(pg->bits_per_pixel==16 && !d->warned_exp) {
+		if(pi->bits_per_pixel==16 && !d->warned_exp) {
 			de_warn(c, "Support for this type of 16-bit image is experimental, and may not be correct.");
 			d->warned_exp = 1;
 		}
 	}
 
 	pos += 40;
-	cmpr_pixels_size = d->paint_data_section_size-40;
-	de_dbg(c, "pixel data at %d", (int)pos);
+	cmpr_pixels_size = pi->paint_data_section_size-40;
+	de_dbg(c, "pixel data at %"I64_FMT, pos);
 
 	switch(compression_type) {
 	case 0: // uncompressed
@@ -353,38 +376,46 @@ static de_bitmap *do_read_paint_data_section(deark *c, lctx *d,
 		goto done;
 	}
 
-	img = do_create_image(c, d, pg, unc_pixels, is_mask);
+	do_read_phys_image(c, d, pi, unc_pixels);
 
 done:
 	if(unc_pixels) dbuf_close(unc_pixels);
 	de_dbg_indent(c, -1);
-	de_free(c, pg);
-	return img;
 }
 
 // Writes the image to a file.
-// Sets d->paint_data_section_size.
+// Sets d->last_paint_data_section_size.
 static void do_read_and_write_paint_data_section(deark *c, lctx *d, i64 pos1)
 {
-	de_bitmap *img = NULL;
+	struct phys_image_ctx *pi = NULL;
 
-	img = do_read_paint_data_section(c, d, pos1, 0);
-	de_bitmap_write_to_file(img, NULL, 0);
-	de_bitmap_destroy(img);
+	pi = de_malloc(c, sizeof(struct phys_image_ctx));
+	pi->is_mask = 0;
+	do_read_paint_data_section(c, d, pi, pos1);
+	if(pi->img) {
+		de_bitmap_write_to_file(pi->img, NULL, 0);
+	}
+	d->last_paint_data_section_size = pi->paint_data_section_size;
+	destroy_phys_image(c, pi);
 }
 
 static void do_combine_and_write_images(deark *c, lctx *d,
-	de_bitmap *fg_img, de_bitmap *mask_img)
+	struct phys_image_ctx *pi_fg, struct phys_image_ctx *pi_mask)
 {
+	de_bitmap *fg_img;
+	de_bitmap *mask_img;
 	de_bitmap *img = NULL; // The combined image
 	i64 i, j;
 
-	if(!fg_img) goto done;
-	if(!mask_img) {
-		de_bitmap_write_to_file(fg_img, NULL, 0);
+	if(!pi_fg || !pi_fg->img) goto done;
+	if(!pi_mask || !pi_mask->img) {
+		de_bitmap_write_to_file(pi_fg->img, NULL, 0);
 		goto done;
 	}
 
+	fg_img = pi_fg->img;
+	mask_img = pi_mask->img;
+
 	// Create a new image (which supports transparency).
 	img = de_bitmap_create2(c, fg_img->unpadded_width, fg_img->width, fg_img->height,
 		(fg_img->bytes_per_pixel<=2 ? 2 : 4));
@@ -411,7 +442,7 @@ static void do_combine_and_write_images(deark *c, lctx *d,
 				a1 = (i64)DE_COLOR_R(clrm) + (i64)DE_COLOR_G(clrm) + (i64)DE_COLOR_B(clrm);
 				a = de_scale_n_to_255(255*3, a1);
 
-				if(mask_img->orig_colortype==0 && mask_img->orig_bitdepth==8) {
+				if(pi_mask->color_type==0 && pi_mask->bits_per_pixel==8) {
 					a = 255-a;
 				}
 			}
@@ -440,7 +471,7 @@ static void do_sketch_section(deark *c, lctx *d, i64 pos1)
 	pos = pos1;
 
 	// 18-byte header
-	de_dbg(c, "sketch section at %d", (int)pos);
+	de_dbg(c, "sketch section at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	s_s_w = de_getu16le(pos);
@@ -457,16 +488,16 @@ static void do_sketch_section(deark *c, lctx *d, i64 pos1)
 	// so we don't have to read it before calling
 	// do_read_and_write_paint_data_section() to convert the image.
 
-	pos = paint_data_section_start + d->paint_data_section_size;
+	pos = paint_data_section_start + d->last_paint_data_section_size;
 	x1 = de_getu16le(pos);
 	x2 = de_getu16le(pos+2);
-	de_dbg(c, "magnification: %d"DE_CHAR_TIMES"%d", (int)x1, (int)x2);
+	de_dbg(c, "magnification: %u"DE_CHAR_TIMES"%u", (UI)x1, (UI)x2);
 	x1 = de_getu32le(pos+4);
 	x2 = de_getu32le(pos+8);
-	de_dbg(c, "left, right cut: %d, %d", (int)x1, (int)x2);
+	de_dbg(c, "left, right cut: %u, %u", (UI)x1, (UI)x2);
 	x1 = de_getu32le(pos+12);
 	x2 = de_getu32le(pos+16);
-	de_dbg(c, "top, bottom cut: %d, %d", (int)x1, (int)x2);
+	de_dbg(c, "top, bottom cut: %u, %u", (UI)x1, (UI)x2);
 
 	de_dbg_indent(c, -1);
 }
@@ -479,8 +510,8 @@ static void do_epocsketch_section_table_entry(deark *c, lctx *d,
 
 	section_id = de_getu32le(pos);
 	section_loc = de_getu32le(pos+4);
-	de_dbg(c, "section #%d: id=0x%08x, pos=%d", (int)entry_index,
-		(unsigned int)section_id, (int)section_loc);
+	de_dbg(c, "section #%d: id=0x%08x, pos=%"I64_FMT, (int)entry_index,
+		(UI)section_id, section_loc);
 	de_dbg_indent(c, 1);
 	if(section_id==0x1000007d) {
 		do_sketch_section(c, d, section_loc);
@@ -495,7 +526,7 @@ static void do_epocsketch_section_table(deark *c, lctx *d, i64 pos)
 	i64 i;
 
 	// Section table section
-	de_dbg(c, "section table at %d", (int)pos);
+	de_dbg(c, "section table at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	section_table_size_code = de_getbyte(pos);
@@ -504,7 +535,7 @@ static void do_epocsketch_section_table(deark *c, lctx *d, i64 pos)
 	// bytes, so divide by 2 to get the number of entries.
 	num_sections = ((int)section_table_size_code)/2;
 
-	de_dbg(c, "section table size: %d (%d entries)", (int)section_table_size_code,
+	de_dbg(c, "section table size: %u (%d entries)", (UI)section_table_size_code,
 		(int)num_sections);
 	pos++;
 
@@ -516,10 +547,10 @@ static void do_epocsketch_section_table(deark *c, lctx *d, i64 pos)
 
 static void do_epocsketch_header(deark *c, lctx *d, i64 pos)
 {
-	de_dbg(c, "header section at %d", (int)pos);
+	de_dbg(c, "header section at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 	d->section_table_offset = de_getu32le(pos+16);
-	de_dbg(c, "section table offset: %d", (int)d->section_table_offset);
+	de_dbg(c, "section table offset: %"I64_FMT, d->section_table_offset);
 	de_dbg_indent(c, -1);
 }
 
@@ -538,17 +569,17 @@ static void de_run_epocaif(deark *c, lctx *d)
 	i64 num_images;
 	i64 first_image_pos;
 	i64 img_pos;
-	de_bitmap *fg_img = NULL;
-	de_bitmap *mask_img = NULL;
+	struct phys_image_ctx *pi_fg = NULL;
+	struct phys_image_ctx *pi_mask = NULL;
 
 	de_dbg(c, "header at %d", 0);
 	de_dbg_indent(c, 1);
 	table_offset = de_getu32le(16);
-	de_dbg(c, "table offset: %d", (int)table_offset);
+	de_dbg(c, "table offset: %"I64_FMT, table_offset);
 	de_dbg_indent(c, -1);
 
 	pos = table_offset;
-	de_dbg(c, "table at %d", (int)pos);
+	de_dbg(c, "table at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 	// The first byte seems to be 2 times the number of captions.
 	caption_count_code = de_getbyte(pos);
@@ -564,7 +595,7 @@ static void de_run_epocaif(deark *c, lctx *d)
 	pos++;
 
 	first_image_pos = de_getu32le(pos);
-	de_dbg(c, "offset of first bitmap: %d", (int)first_image_pos);
+	de_dbg(c, "offset of first bitmap: %"I64_FMT, first_image_pos);
 
 	de_dbg_indent(c, -1);
 
@@ -577,37 +608,46 @@ static void de_run_epocaif(deark *c, lctx *d)
 	img_pos = first_image_pos;
 	i = 0;
 	while(i<num_images) {
+		if(pi_fg) {
+			destroy_phys_image(c, pi_fg);
+			pi_fg = NULL;
+		}
+		if(pi_mask) {
+			destroy_phys_image(c, pi_mask);
+			pi_mask = NULL;
+		}
+
 		de_dbg(c, "image #%d", (int)(i/2));
 		de_dbg_indent(c, 1);
 
-		de_dbg(c, "foreground bitmap at %d", (int)img_pos);
+		de_dbg(c, "foreground bitmap at %"I64_FMT, img_pos);
 		de_dbg_indent(c, 1);
-		fg_img = do_read_paint_data_section(c, d, img_pos, 0);
-		if(d->paint_data_section_size<=0) break;
-		img_pos += d->paint_data_section_size;
+		pi_fg = de_malloc(c, sizeof(struct phys_image_ctx));
+		pi_fg->is_mask = 0;
+		do_read_paint_data_section(c, d, pi_fg, img_pos);
+		if(pi_fg->paint_data_section_size<=0) break;
+		img_pos += pi_fg->paint_data_section_size;
 		i++;
 		de_dbg_indent(c, -1);
 
 		if(i<num_images) {
-			de_dbg(c, "mask bitmap at %d", (int)img_pos);
+			de_dbg(c, "mask bitmap at %"I64_FMT, img_pos);
 			de_dbg_indent(c, 1);
-			mask_img = do_read_paint_data_section(c, d, img_pos, 1);
-			if(d->paint_data_section_size<=0) break;
-			img_pos += d->paint_data_section_size;
+			pi_mask = de_malloc(c, sizeof(struct phys_image_ctx));
+			pi_mask->is_mask = 1;
+			do_read_paint_data_section(c, d, pi_mask, img_pos);
+			if(pi_mask->paint_data_section_size<=0) break;
+			img_pos += pi_mask->paint_data_section_size;
 			i++;
 			de_dbg_indent(c, -1);
 		}
 
-		do_combine_and_write_images(c, d, fg_img, mask_img);
-		de_bitmap_destroy(fg_img);
-		fg_img = NULL;
-		de_bitmap_destroy(mask_img);
-		mask_img = NULL;
+		do_combine_and_write_images(c, d, pi_fg, pi_mask);
 		de_dbg_indent(c, -1);
 	}
 
-	de_bitmap_destroy(fg_img);
-	de_bitmap_destroy(mask_img);
+	destroy_phys_image(c, pi_fg);
+	destroy_phys_image(c, pi_mask);
 }
 
 static void do_epocmbm_jumptable_entry(deark *c, lctx *d, i64 entry_index,
@@ -616,7 +656,7 @@ static void do_epocmbm_jumptable_entry(deark *c, lctx *d, i64 entry_index,
 	i64 img_pos;
 
 	img_pos = de_getu32le(pos);
-	de_dbg(c, "image #%d, pos=%d", (int)entry_index, (int)pos);
+	de_dbg(c, "image #%d, pos=%"I64_FMT, (int)entry_index, pos);
 	de_dbg_indent(c, 1);
 	do_read_and_write_paint_data_section(c, d, img_pos);
 	de_dbg_indent(c, -1);
@@ -627,11 +667,11 @@ static void do_epocmbm_jumptable(deark *c, lctx *d, i64 pos)
 	i64 num_images;
 	i64 i;
 
-	de_dbg(c, "MBM jumptable at %d", (int)pos);
+	de_dbg(c, "MBM jumptable at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	num_images = de_getu32le(pos);
-	de_dbg(c, "number of images: %d", (int)num_images);
+	de_dbg(c, "number of images: %"I64_FMT, num_images);
 	if(!de_good_image_count(c, num_images)) {
 		de_err(c, "Too many images");
 		goto done;
@@ -647,10 +687,10 @@ done:
 
 static void do_epocmbm_header(deark *c, lctx *d, i64 pos)
 {
-	de_dbg(c, "header section at %d", (int)pos);
+	de_dbg(c, "header section at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 	d->jumptable_offset = de_getu32le(pos+16);
-	de_dbg(c, "MBM jumptable offset: %d", (int)d->jumptable_offset);
+	de_dbg(c, "MBM jumptable offset: %"I64_FMT, d->jumptable_offset);
 	de_dbg_indent(c, -1);
 }
 
