@@ -10,14 +10,13 @@
 DE_DECLARE_MODULE(de_module_fat);
 DE_DECLARE_MODULE(de_module_loaddskf);
 
-#include "../foreign/dskdcmps.h"
-
 #define MAX_NESTING_LEVEL 16
 
 struct member_data {
 	u8 fn_base[8];
 	u8 fn_ext[3];
 	u8 is_subdir;
+	u8 is_volume_label;
 	u8 is_special;
 	UI attribs;
 	UI ea_handle;
@@ -81,39 +80,6 @@ typedef struct localctx_struct {
 	dbuf *ea_data; // NULL if not available
 } lctx;
 
-static void ucstring_append_hexbytes(de_ucstring *s, const u8 *buf, i64 buflen)
-{
-	i64 i;
-
-	for(i = 0; i<buflen; i++) {
-		if(i>0) ucstring_append_char(s, ' ');
-		ucstring_append_char(s, (de_rune)de_get_hexchar((int)(buf[i]/16)));
-		ucstring_append_char(s, (de_rune)de_get_hexchar((int)(buf[i]%16)));
-	}
-}
-
-static void dbg_hexbytes_oneline_mem(deark *c, const u8 *buf, i64 buflen, const char *label)
-{
-	de_ucstring *s = NULL;
-
-	if(buflen<0) buflen = 0;
-	if(buflen>64) buflen = 64;
-	s = ucstring_create(c);
-	ucstring_append_hexbytes(s, buf, buflen);
-	de_dbg(c, "%s: %s", label, ucstring_getpsz_d(s));
-	ucstring_destroy(s);
-}
-
-static void dbg_hexbytes_oneline(deark *c, dbuf *f, i64 pos, i64 len, const char *label)
-{
-	u8 buf[64];
-
-	if(len<0) len = 0;
-	if(len>(i64)sizeof(buf)) len = (i64)sizeof(buf);
-	dbuf_read(f, buf, pos, len);
-	dbg_hexbytes_oneline_mem(c, buf, len, label);
-}
-
 static void fat_save_cluster_use_flags(deark *c, lctx *d)
 {
 	if(!d->cluster_used_flags) return;
@@ -237,6 +203,9 @@ static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 	if(md->is_subdir) {
 		fi->is_directory = 1;
 	}
+	else if(md->is_volume_label) {
+		fi->is_volume_label = 1;
+	}
 	if(md->mod_time.is_valid) {
 		fi->timestamp[DE_TIMESTAMPIDX_MODIFY] = md->mod_time;
 	}
@@ -398,6 +367,9 @@ static void do_fat_eadata_item(deark *c, lctx *d, struct member_data *md)
 	de_dbg(c, "reading OS/2 extended attributes");
 	de_dbg_indent(c, 1);
 	// TODO: Better filenames for icons that may be extracted.
+	// The ea_data module will use the filename contained in the EA data stream,
+	// but it would be better to use the "fullfn" that will be constructed in
+	// do_extract_file(). Some refactoring will be needed.
 	de_run_module_by_id_on_slice(c, "ea_data", mparams, d->ea_data, 0, d->ea_data->len);
 	de_dbg_indent(c, -1);
 
@@ -432,7 +404,6 @@ static int do_dir_entry(deark *c, lctx *d, struct dirctx *dctx,
 	i64 ddate, dtime;
 	int retval = 0;
 	int is_deleted = 0;
-	int is_volume_label = 0;
 	int need_curpath_pop = 0;
 	de_ucstring *descr = NULL;
 	struct member_data *md = NULL;
@@ -465,8 +436,7 @@ static int do_dir_entry(deark *c, lctx *d, struct dirctx *dctx,
 		; // Normal file
 	}
 	else if((md->attribs & 0x18) == 0x08) {
-		is_volume_label = 1;
-		md->is_special = 1;
+		md->is_volume_label = 1;
 	}
 	else if((md->attribs & 0x18) == 0x10) {
 		md->is_subdir = 1;
@@ -501,7 +471,7 @@ static int do_dir_entry(deark *c, lctx *d, struct dirctx *dctx,
 	}
 
 	md->short_fn = ucstring_create(c);
-	if(is_volume_label) {
+	if(md->is_volume_label) {
 		decode_volume_label_name(c, d, md);
 	}
 	else {
@@ -536,6 +506,7 @@ static int do_dir_entry(deark *c, lctx *d, struct dirctx *dctx,
 
 	md->filesize = de_getu32le(pos1+28);
 	de_dbg(c, "file size: %"I64_FMT, md->filesize);
+	if(md->is_volume_label) md->filesize = 0;
 
 	// (Done reading dir entry)
 
@@ -736,14 +707,10 @@ static int root_dir_seems_valid(deark *c, lctx *d)
 
 static void do_atarist_boot_checksum(deark *c, lctx *d, i64 pos1)
 {
-	i64 i;
-	UI ck = 0;
-
-	for(i=0; i<256; i++) {
-		ck += (UI)de_getu16be(pos1+i*2);
-		ck &= 0xffff;
-	}
+	UI ck;
 
+	ck = de_calccrc_oneshot(c->infile, pos1, 512, DE_CRCOBJ_SUM_U16BE);
+	ck &= 0xffff;
 	de_dbg(c, "Atari ST checksum: 0x%04x", ck);
 	if(ck==0x1234) {
 		d->has_atarist_checksum = 1;
@@ -781,21 +748,26 @@ static int do_boot_sector(deark *c, lctx *d, i64 pos1)
 	i64 num_sectors32 = 0;
 	i64 num_sectors_per_track;
 	i64 num_heads;
+	i64 num_sectors_per_cylinder;
+	i64 num_cylinders = 0;
 	i64 jmpinstrlen;
 	u8 b;
 	u8 cksum_sig[2];
 	int retval = 0;
+	char tmpbuf[16];
 
 	de_dbg(c, "boot sector at %"I64_FMT, pos1);
 	de_dbg_indent(c, 1);
 
 	// BIOS parameter block
 	jmpinstrlen = (d->subfmt==FAT_SUBFMT_ATARIST)?2:3;
-	dbg_hexbytes_oneline(c, c->infile, pos1, jmpinstrlen, "jump instr");
+	de_dbg(c, "jump instr: %s",
+		de_render_hexbytes_from_dbuf(c->infile, pos1, jmpinstrlen, tmpbuf, sizeof(tmpbuf)));
 
 	if(d->subfmt==FAT_SUBFMT_ATARIST) {
 		do_oem_name(c, d, pos1+2, 6);
-		dbg_hexbytes_oneline(c, c->infile, pos1+8, 3, "serial num");
+		de_dbg(c, "serial num: %s",
+			de_render_hexbytes_from_dbuf(c->infile, pos1+8, 3, tmpbuf, sizeof(tmpbuf)));
 	}
 	else {
 		do_oem_name(c, d, pos1+3, 8);
@@ -835,7 +807,8 @@ static int do_boot_sector(deark *c, lctx *d, i64 pos1)
 
 	pos = pos1+0x1fe;
 	de_read(cksum_sig, pos, 2);
-	dbg_hexbytes_oneline_mem(c, cksum_sig, 2, "boot sector signature");
+	de_dbg(c, "boot sector signature: %s",
+		de_render_hexbytes_from_mem(cksum_sig, 2, tmpbuf, sizeof(tmpbuf)));
 
 	do_atarist_boot_checksum(c, d, pos1);
 	if(d->has_atarist_checksum) {
@@ -871,6 +844,12 @@ static int do_boot_sector(deark *c, lctx *d, i64 pos1)
 		d->num_sectors = num_sectors16;
 	}
 
+	num_sectors_per_cylinder = num_heads * num_sectors_per_track;
+	if(num_sectors_per_cylinder > 0) {
+		num_cylinders = de_pad_to_n(d->num_sectors, num_sectors_per_cylinder) / num_sectors_per_cylinder;
+	}
+	de_dbg(c, "number of cylinders (calculated): %"I64_FMT, num_cylinders);
+
 	if(d->sectors_per_cluster<1) goto done;
 	if(d->bytes_per_sector<32) goto done;
 	d->bytes_per_cluster = d->bytes_per_sector * d->sectors_per_cluster;
@@ -884,7 +863,7 @@ static int do_boot_sector(deark *c, lctx *d, i64 pos1)
 	num_data_region_sectors = d->num_sectors - (d->root_dir_sector + num_root_dir_sectors);
 	if(num_data_region_sectors<0) goto done;
 	d->num_data_region_clusters = num_data_region_sectors / d->sectors_per_cluster;
-	de_dbg(c, "num clusters (calculated): %"I64_FMT, d->num_data_region_clusters);
+	de_dbg(c, "number of clusters (calculated): %"I64_FMT, d->num_data_region_clusters);
 
 	d->data_region_sector = d->root_dir_sector + num_root_dir_sectors;
 	d->data_region_pos = d->data_region_sector * d->bytes_per_sector;
@@ -1130,11 +1109,37 @@ struct skf_ctx {
 	int new_fmt;
 	int is_compressed;
 	u32 checksum_reported;
+	u32 checksum_calc;
 	i64 hdr_size;
 	i64 expected_dcmpr_size; // 0 if unknown
 	i64 padded_size; // 0 if unknown
+	struct de_crcobj *crco;
 };
 
+// Get, report, and compare calculated checksum
+static void loaddskf_finish_checksum(deark *c, struct skf_ctx *d)
+{
+	d->checksum_calc = de_crcobj_getval(d->crco);
+	de_dbg(c, "checksum (calculated): 0x%08x", (UI)d->checksum_calc);
+	if(d->checksum_calc != d->checksum_reported) {
+		de_warn(c, "Checksum mismatch (reported 0x%08x, calculated 0x%08x). "
+			"Something may have gone wrong.", (UI)d->checksum_reported,
+			(UI)d->checksum_calc);
+	}
+}
+
+static void loaddskf_calc_checksum_noncmpr(deark *c, struct skf_ctx *d)
+{
+	i64 nbytes_to_checksum;
+
+	nbytes_to_checksum = c->infile->len - d->hdr_size;
+	if(d->expected_dcmpr_size && d->expected_dcmpr_size < nbytes_to_checksum) {
+		nbytes_to_checksum = d->expected_dcmpr_size;
+	}
+
+	de_crcobj_addslice(d->crco, c->infile, d->hdr_size, nbytes_to_checksum);
+}
+
 static void loaddskf_pad_ima_file(deark *c, struct skf_ctx *d, dbuf *outf)
 {
 	i64 num_padding_bytes;
@@ -1166,8 +1171,9 @@ static void loaddskf_convert_noncmpr_to_ima(deark *c, struct skf_ctx *d)
 
 static void loaddskf_decode_as_fat(deark *c, struct skf_ctx *d)
 {
-	i64 dlen = c->infile->len - d->hdr_size;
+	i64 dlen;
 
+	dlen = de_max_int(c->infile->len - d->hdr_size, d->padded_size);
 	de_dbg(c, "decoding as FAT, pos=%"I64_FMT", len=%"I64_FMT, d->hdr_size, dlen);
 	if(dlen<=0) goto done;
 
@@ -1202,14 +1208,19 @@ static void loaddskf_decompress(deark *c, struct skf_ctx *d)
 	dcmpri.len = c->infile->len - dcmpri.pos;
 	dcmpro.f = outf;
 
+	dbuf_set_writelistener(outf, de_writelistener_for_crc, (void*)d->crco);
+
 	de_dbg(c, "[decompressing]");
-	dskdcmps_run(c, &dcmpri, &dcmpro, &dres);
+	fmtutil_ibmlzw_codectype1(c, &dcmpri, &dcmpro, &dres, NULL);
 	dbuf_flush(dcmpro.f);
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
 		goto done;
 	}
 
+	d->checksum_calc = de_crcobj_getval(d->crco);
+	loaddskf_finish_checksum(c, d);
+
 	if(d->to_raw) {
 		loaddskf_pad_ima_file(c, d, outf);
 	}
@@ -1286,6 +1297,7 @@ static void de_run_loaddskf(deark *c, de_module_params *mparams)
 
 	d = de_malloc(c, sizeof(struct skf_ctx));
 	d->to_raw = de_get_ext_option_bool(c, "loaddskf:toraw", 0);
+	d->crco = de_crcobj_create(c, DE_CRCOBJ_SUM_U16LE);
 
 	sig = (UI)de_getu16be(0);
 	switch(sig) {
@@ -1312,6 +1324,8 @@ static void de_run_loaddskf(deark *c, de_module_params *mparams)
 		loaddskf_decompress(c, d);
 	}
 	else {
+		loaddskf_calc_checksum_noncmpr(c, d);
+		loaddskf_finish_checksum(c, d);
 		if(d->to_raw) {
 			loaddskf_convert_noncmpr_to_ima(c, d);
 		}
@@ -1321,7 +1335,10 @@ static void de_run_loaddskf(deark *c, de_module_params *mparams)
 	}
 
 done:
-	de_free(c, d);
+	if(d) {
+		de_crcobj_destroy(d->crco);
+		de_free(c, d);
+	}
 }
 
 static int de_identify_loaddskf(deark *c)
