@@ -367,6 +367,7 @@ struct comment_ext_ctx {
 	dbuf *outf;
 };
 
+// Used for comments and frame names
 static void callback_for_comment_ext(deark *c, lctx *d, struct subblock_reader_data *sbrd)
 {
 	struct comment_ext_ctx *ctx = (struct comment_ext_ctx*)sbrd->userdata;
@@ -402,6 +403,21 @@ static void do_comment_extension(deark *c, lctx *d, i64 pos1)
 	ucstring_destroy(ctx.s);
 }
 
+// Used by Gifsicle
+static void do_framename_extension(deark *c, lctx *d, i64 pos1)
+{
+	struct comment_ext_ctx ctx;
+	i64 pos = pos1;
+
+	de_zeromem(&ctx, sizeof(struct comment_ext_ctx));
+	ctx.s = ucstring_create(c);
+
+	do_read_subblocks_p(c, d, c->infile, callback_for_comment_ext, (void*)&ctx, &pos);
+
+	de_dbg(c, "frame name: \"%s\"", ucstring_getpsz_d(ctx.s));
+	ucstring_destroy(ctx.s);
+}
+
 static void decode_text_color(deark *c, lctx *d, const char *name, u8 clr_idx,
 	de_color *pclr)
 {
@@ -811,6 +827,7 @@ static int do_read_extension(deark *c, lctx *d, i64 pos1, i64 *bytesused)
 	switch(ext_type) {
 	case 0x01: ext_name="plain text"; break;
 	case 0xf9: ext_name="graphic control"; break;
+	case 0xce: ext_name="frame name"; break;
 	case 0xfe: ext_name="comment"; break;
 	case 0xff: ext_name="application"; break;
 	default: ext_name="?";
@@ -833,6 +850,9 @@ static int do_read_extension(deark *c, lctx *d, i64 pos1, i64 *bytesused)
 	case 0xff:
 		do_application_extension(c, d, pos);
 		break;
+	case 0xce:
+		do_framename_extension(c, d, pos);
+		break;
 	}
 	de_dbg_indent(c, -1);
 
@@ -1025,6 +1045,7 @@ static int do_image_internal(deark *c, lctx *d,
 	de_zeromem(&delzwp, sizeof(struct de_lzw_params));
 	de_zeromem(&u, sizeof(struct my_giflzw_userdata));
 	custom_outf = dbuf_create_custom_dbuf(c, 0, 0);
+	dbuf_enable_wbuffer(custom_outf);
 	u.c = c;
 	u.d = d;
 	u.gi = gi;
@@ -1047,6 +1068,7 @@ static int do_image_internal(deark *c, lctx *d,
 	}
 
 	de_dfilter_finish(gi->dfctx);
+	dbuf_flush(custom_outf);
 
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
@@ -1143,9 +1165,7 @@ done:
 static void do_after_trailer(deark *c, lctx *d, i64 pos1)
 {
 	i64 extra_bytes_at_eof;
-	i64 i;
 	u8 first_byte;
-	u8 flag = 0;
 
 	extra_bytes_at_eof = c->infile->len - pos1;
 	if(extra_bytes_at_eof<=0) return;
@@ -1153,14 +1173,17 @@ static void do_after_trailer(deark *c, lctx *d, i64 pos1)
 	// If all extra bytes are 0x00, or all are 0x1a, don't report it.
 	first_byte = de_getbyte(pos1);
 	if(first_byte==0x00 || first_byte==0x1a) {
+		i64 i;
+		u8 flag = 0;
+
 		for(i=1; i<extra_bytes_at_eof; i++) {
 			if(de_getbyte(pos1+i)!=first_byte) {
 				flag = 1;
 				break;
 			}
 		}
+		if(!flag) return;
 	}
-	if(!flag) return;
 
 	de_info(c, "Note: %"I64_FMT" bytes of unidentified data found at end "
 		"of file (starting at %"I64_FMT").", extra_bytes_at_eof, pos1);
