@@ -28,10 +28,10 @@ struct XYZ_type {
 	double v[3];
 };
 
-typedef struct localctx_struct {
-	unsigned int profile_ver_major;
-	unsigned int profile_ver_minor;
-	unsigned int profile_ver_bugfix;
+typedef struct localctx_iccprofile {
+	UI profile_ver_major;
+	UI profile_ver_minor;
+	UI profile_ver_bugfix;
 	i64 profile_size;
 	char tmpbuf1[80];
 	char tmpbuf2[80];
@@ -61,7 +61,6 @@ struct taginfo {
 	u32 flags;
 
 	const char *name;
-	void *reserved2;
 };
 
 static double read_s15Fixed16Number(dbuf *f, i64 pos)
@@ -116,7 +115,7 @@ static void destroy_tagset(deark *c, struct tagset_type *tgs)
 // flag 0x2: Interpret 0 as (none)
 // Returns a copy of the buf pointer.
 static const char *format_4cc_dbgstr(const struct de_fourcc *tmp4cc,
-	char *buf, size_t buflen, unsigned int flags)
+	char *buf, size_t buflen, UI flags)
 {
 	char str[40];
 
@@ -126,7 +125,7 @@ static const char *format_4cc_dbgstr(const struct de_fourcc *tmp4cc,
 		de_snprintf(str, sizeof(str), "'%s'", tmp4cc->id_dbgstr);
 
 	if(flags&0x1)
-		de_snprintf(buf, buflen, "0x%08x=%s", (unsigned int)tmp4cc->id, str);
+		de_snprintf(buf, buflen, "0x%08x=%s", (UI)tmp4cc->id, str);
 	else
 		de_strlcpy(buf, str, buflen);
 
@@ -287,8 +286,8 @@ static void do_mluc_record(deark *c, lctx *d, i64 tagstartpos,
 
 	string_len = de_getu32be(pos+4);
 	string_offset = de_getu32be(pos+8);
-	de_dbg(c, "string offset=%d+%d, len=%d bytes", (int)tagstartpos,
-		(int)string_offset, (int)string_len);
+	de_dbg(c, "string offset=%"I64_FMT"+%"I64_FMT", len=%"I64_FMT" bytes", tagstartpos,
+		string_offset, string_len);
 
 	dbuf_read_to_ucstring_n(c->infile, tagstartpos+string_offset, string_len, DE_DBG_MAX_STRLEN*2,
 		s, 0, DE_ENCODING_UTF16BE);
@@ -310,11 +309,11 @@ static void typedec_mluc(deark *c, struct typedec_params *p)
 	pos += 8;
 
 	num_recs = de_getu32be(pos);
-	de_dbg(c, "number of records: %d", (int)num_recs);
+	de_dbg(c, "number of records: %u", (UI)num_recs);
 	pos += 4;
 
 	recsize = de_getu32be(pos);
-	de_dbg(c, "record size: %d", (int)recsize);
+	de_dbg(c, "record size: %u", (UI)recsize);
 	if(recsize<12) goto done;
 	pos += 4;
 
@@ -323,7 +322,7 @@ static void typedec_mluc(deark *c, struct typedec_params *p)
 	for(rec=0; rec<num_recs; rec++) {
 		if(rec_array_startpos+rec*recsize > p->pos1+p->len) break;
 
-		de_dbg(c, "record #%d at %d", (int)rec, (int)pos);
+		de_dbg(c, "record #%u at %"I64_FMT, (UI)rec, pos);
 		de_dbg_indent(c, 1);
 		do_mluc_record(c, p->d, p->pos1, pos, recsize);
 		de_dbg_indent(c, -1);
@@ -532,81 +531,81 @@ static const struct datatypeinfo datatypeinfo_arr[] = {
 };
 
 static const struct taginfo taginfo_arr[] = {
-	{ 0x41324230U, 0, "AToB0", NULL }, // A2B0
-	{ 0x41324231U, 0, "AToB1", NULL }, // A2B1
-	{ 0x41324232U, 0, "AToB2", NULL }, // A2B2
-	{ 0x41324233U, 0, "AToB3", NULL }, // A2B3
-	{ 0x42324130U, 0, "BToA0", NULL }, // B2A0
-	{ 0x42324131U, 0, "BToA1", NULL }, // B2A1
-	{ 0x42324132U, 0, "BToA2", NULL }, // B2A2
-	{ 0x42324133U, 0, "BToA3", NULL }, // B2A3
-	{ 0x42324430U, 0, "BToD0", NULL }, // B2D0
-	{ 0x42324431U, 0, "BToD1", NULL }, // B2D1
-	{ 0x42324432U, 0, "BToD2", NULL }, // B2D2
-	{ 0x42324433U, 0, "BToD3", NULL }, // B2D3
-	{ 0x44324230U, 0, "DToB0", NULL }, // D2B0
-	{ 0x44324231U, 0, "DToB1", NULL }, // D2B1
-	{ 0x44324232U, 0, "DToB2", NULL }, // D2B2
-	{ 0x44324233U, 0, "DToB3", NULL }, // D2B3
-	{ 0x62545243U, 0, "blueTRC", NULL }, // bTRC
-	{ 0x6258595aU, 0x1, "blueColorant", NULL }, // bXYZ
-	{ 0x6258595aU, 0x2, "blueMatrixColumn", NULL }, // bXYZ
-	{ 0x62666420U, 0, "ucrbg", NULL }, // bfd
-	{ 0x626b7074U, 0, "mediaBlackPoint", NULL }, // bkpt
-	{ 0x63327370U, 0, "customToStandardPcc", NULL }, // c2sp
-	{ 0x63616c74U, 0, "calibrationDateTime", NULL }, // calt
-	{ 0x63657074U, 0, "colorEncodingParams", NULL }, // cept
-	{ 0x63686164U, 0, "chromaticAdaptation", NULL }, // chad
-	{ 0x6368726dU, 0, "chromaticity", NULL }, // chrm
-	{ 0x63696973U, 0, "colorimetricIntentImageState", NULL }, // ciis
-	{ 0x636c6f74U, 0, "colorantTableOut", NULL }, // clot
-	{ 0x636c726fU, 0, "colorantOrder", NULL }, // clro
-	{ 0x636c7274U, 0, "colorantTable", NULL }, // clrt
-	{ 0x63707274U, 0, "copyright", NULL }, // cprt
-	{ 0x63726469U, 0, "crdInfo", NULL }, // crdi
-	{ 0x63736e6dU, 0, "colorSpaceName", NULL }, // csnm
-	{ 0x64657363U, 0, "profileDescription", NULL }, // desc
-	{ 0x64657673U, 0, "deviceSettings", NULL }, // devs
-	{ 0x646d6464U, 0, "deviceModelDesc", NULL }, // dmdd
-	{ 0x646d6e64U, 0, "deviceMfgDesc", NULL }, // dmnd
-	{ 0x67616d74U, 0, "gamut", NULL }, // gamt
-	{ 0x67626431U, 0, "gamutBoundaryDescription1", NULL }, // gbd1
-	{ 0x67545243U, 0, "greenTRC", NULL }, // gTRC
-	{ 0x6758595aU, 0x1, "greenColorant", NULL }, // gXYZ
-	{ 0x6758595aU, 0x2, "greenMatrixColumn", NULL }, // gXYZ
-	{ 0x6b545243U, 0, "grayTRC", NULL }, // kTRC
-	{ 0x6c756d69U, 0, "luminance", NULL }, // lumi
-	{ 0x6d656173U, 0, "measurement", NULL }, // meas
-	{ 0x6e636c32U, 0, "namedColor2", NULL }, // ncl2
-	{ 0x6e636f6cU, 0, "namedColor", NULL }, // ncol
-	{ 0x70726530U, 0, "preview0", NULL }, // pre0
-	{ 0x70726531U, 0, "preview1", NULL }, // pre1
-	{ 0x70726532U, 0, "preview2", NULL }, // pre2
-	{ 0x70733269U, 0, "ps2RenderingIntent", NULL }, // ps2i
-	{ 0x70733273U, 0, "ps2CSA", NULL }, // ps2s
-	{ 0x70736430U, 0, "ps2CRD0", NULL }, // psd0
-	{ 0x70736431U, 0, "ps2CRD1", NULL }, // psd1
-	{ 0x70736432U, 0, "ps2CRD2", NULL }, // psd2
-	{ 0x70736433U, 0, "ps2CRD3", NULL }, // psd3
-	{ 0x70736571U, 0, "profileSequenceDesc", NULL }, // pseq
-	{ 0x70736964U, 0, "profileSequenceIdentifier", NULL }, // psid
-	{ 0x72545243U, 0, "redTRC", NULL }, // rTRC
-	{ 0x7258595aU, 0x1, "redColorant", NULL }, // rXYZ
-	{ 0x7258595aU, 0x2, "redMatrixColumn", NULL }, // rXYZ
-	{ 0x72657370U, 0, "outputResponse", NULL }, // resp
-	{ 0x72666e6dU, 0, "referenceName", NULL }, // rfnm
-	{ 0x72696730U, 0, "perceptualRenderingIntentGamut", NULL }, // rig0
-	{ 0x72696732U, 0, "saturationRenderingIntentGamut", NULL }, // rig2
-	{ 0x73326370U, 0, "standardToCustomPcc", NULL }, // s2cp
-	{ 0x73637264U, 0, "screeningDesc", NULL }, // scrd
-	{ 0x7363726eU, 0, "screening", NULL }, // scrn
-	{ 0x7376636eU, 0, "spectralViewingConditions", NULL }, // svcn
-	{ 0x74617267U, 0, "charTarget", NULL }, // targ
-	{ 0x74656368U, 0, "technology", NULL }, // tech
-	{ 0x76636774U, 0, "Video Card Gamma Type", NULL }, // vcgt (Apple)
-	{ 0x76696577U, 0, "viewingConditions", NULL }, // view
-	{ 0x76756564U, 0, "viewingCondDesc", NULL }, // vued
-	{ 0x77747074U, 0, "mediaWhitePoint", NULL } // wtpt
+	{ 0x41324230U, 0, "AToB0" }, // A2B0
+	{ 0x41324231U, 0, "AToB1" }, // A2B1
+	{ 0x41324232U, 0, "AToB2" }, // A2B2
+	{ 0x41324233U, 0, "AToB3" }, // A2B3
+	{ 0x42324130U, 0, "BToA0" }, // B2A0
+	{ 0x42324131U, 0, "BToA1" }, // B2A1
+	{ 0x42324132U, 0, "BToA2" }, // B2A2
+	{ 0x42324133U, 0, "BToA3" }, // B2A3
+	{ 0x42324430U, 0, "BToD0" }, // B2D0
+	{ 0x42324431U, 0, "BToD1" }, // B2D1
+	{ 0x42324432U, 0, "BToD2" }, // B2D2
+	{ 0x42324433U, 0, "BToD3" }, // B2D3
+	{ 0x44324230U, 0, "DToB0" }, // D2B0
+	{ 0x44324231U, 0, "DToB1" }, // D2B1
+	{ 0x44324232U, 0, "DToB2" }, // D2B2
+	{ 0x44324233U, 0, "DToB3" }, // D2B3
+	{ 0x62545243U, 0, "blueTRC" }, // bTRC
+	{ 0x6258595aU, 0x1, "blueColorant" }, // bXYZ
+	{ 0x6258595aU, 0x2, "blueMatrixColumn" }, // bXYZ
+	{ 0x62666420U, 0, "ucrbg" }, // bfd
+	{ 0x626b7074U, 0, "mediaBlackPoint" }, // bkpt
+	{ 0x63327370U, 0, "customToStandardPcc" }, // c2sp
+	{ 0x63616c74U, 0, "calibrationDateTime" }, // calt
+	{ 0x63657074U, 0, "colorEncodingParams" }, // cept
+	{ 0x63686164U, 0, "chromaticAdaptation" }, // chad
+	{ 0x6368726dU, 0, "chromaticity" }, // chrm
+	{ 0x63696973U, 0, "colorimetricIntentImageState" }, // ciis
+	{ 0x636c6f74U, 0, "colorantTableOut" }, // clot
+	{ 0x636c726fU, 0, "colorantOrder" }, // clro
+	{ 0x636c7274U, 0, "colorantTable" }, // clrt
+	{ 0x63707274U, 0, "copyright" }, // cprt
+	{ 0x63726469U, 0, "crdInfo" }, // crdi
+	{ 0x63736e6dU, 0, "colorSpaceName" }, // csnm
+	{ 0x64657363U, 0, "profileDescription" }, // desc
+	{ 0x64657673U, 0, "deviceSettings" }, // devs
+	{ 0x646d6464U, 0, "deviceModelDesc" }, // dmdd
+	{ 0x646d6e64U, 0, "deviceMfgDesc" }, // dmnd
+	{ 0x67616d74U, 0, "gamut" }, // gamt
+	{ 0x67626431U, 0, "gamutBoundaryDescription1" }, // gbd1
+	{ 0x67545243U, 0, "greenTRC" }, // gTRC
+	{ 0x6758595aU, 0x1, "greenColorant" }, // gXYZ
+	{ 0x6758595aU, 0x2, "greenMatrixColumn" }, // gXYZ
+	{ 0x6b545243U, 0, "grayTRC" }, // kTRC
+	{ 0x6c756d69U, 0, "luminance" }, // lumi
+	{ 0x6d656173U, 0, "measurement" }, // meas
+	{ 0x6e636c32U, 0, "namedColor2" }, // ncl2
+	{ 0x6e636f6cU, 0, "namedColor" }, // ncol
+	{ 0x70726530U, 0, "preview0" }, // pre0
+	{ 0x70726531U, 0, "preview1" }, // pre1
+	{ 0x70726532U, 0, "preview2" }, // pre2
+	{ 0x70733269U, 0, "ps2RenderingIntent" }, // ps2i
+	{ 0x70733273U, 0, "ps2CSA" }, // ps2s
+	{ 0x70736430U, 0, "ps2CRD0" }, // psd0
+	{ 0x70736431U, 0, "ps2CRD1" }, // psd1
+	{ 0x70736432U, 0, "ps2CRD2" }, // psd2
+	{ 0x70736433U, 0, "ps2CRD3" }, // psd3
+	{ 0x70736571U, 0, "profileSequenceDesc" }, // pseq
+	{ 0x70736964U, 0, "profileSequenceIdentifier" }, // psid
+	{ 0x72545243U, 0, "redTRC" }, // rTRC
+	{ 0x7258595aU, 0x1, "redColorant" }, // rXYZ
+	{ 0x7258595aU, 0x2, "redMatrixColumn" }, // rXYZ
+	{ 0x72657370U, 0, "outputResponse" }, // resp
+	{ 0x72666e6dU, 0, "referenceName" }, // rfnm
+	{ 0x72696730U, 0, "perceptualRenderingIntentGamut" }, // rig0
+	{ 0x72696732U, 0, "saturationRenderingIntentGamut" }, // rig2
+	{ 0x73326370U, 0, "standardToCustomPcc" }, // s2cp
+	{ 0x73637264U, 0, "screeningDesc" }, // scrd
+	{ 0x7363726eU, 0, "screening" }, // scrn
+	{ 0x7376636eU, 0, "spectralViewingConditions" }, // svcn
+	{ 0x74617267U, 0, "charTarget" }, // targ
+	{ 0x74656368U, 0, "technology" }, // tech
+	{ 0x76636774U, 0, "Video Card Gamma Type" }, // vcgt (Apple)
+	{ 0x76696577U, 0, "viewingConditions" }, // view
+	{ 0x76756564U, 0, "viewingCondDesc" }, // vued
+	{ 0x77747074U, 0, "mediaWhitePoint" } // wtpt
 };
 
 static void do_read_header(deark *c, lctx *d, i64 pos)
@@ -619,7 +618,7 @@ static void do_read_header(deark *c, lctx *d, i64 pos)
 	const char *name;
 	struct XYZ_type xyz;
 
-	de_dbg(c, "header at %d", (int)pos);
+	de_dbg(c, "header at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	d->profile_size = de_getu32be(pos+0);
@@ -836,12 +835,12 @@ static void do_read_main_tags(deark *c, lctx *d, i64 pos1)
 		tgs->data_area_len = c->infile->len;
 	}
 	tgs->num_tags = de_getu32be(pos1);
-	de_dbg(c, "number of tags: %d", (int)tgs->num_tags);
+	de_dbg(c, "number of tags: %u", (UI)tgs->num_tags);
 	if(tgs->num_tags>MAX_TAGS_PER_TAGSET) {
-		de_err(c, "Invalid or excessive number of tags: %d", (int)tgs->num_tags);
+		de_err(c, "Invalid or excessive number of tags: %u", (UI)tgs->num_tags);
 		goto done;
 	}
-	de_dbg(c, "expected start of data segment: %d", (int)(pos1+4+12*tgs->num_tags));
+	de_dbg(c, "expected start of data segment: %"I64_FMT, (i64)(pos1+4+12*tgs->num_tags));
 
 	// Make a place to record some information about each tag we encounter in the table.
 	tgs->tags_seen = de_mallocarray(c, tgs->num_tags, sizeof(struct tag_seen_type));
