@@ -4,8 +4,8 @@
 
 // ID3 metadata
 
-#include <deark-config.h>
 #include <deark-private.h>
+#include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_id3);
 
 // **************************************************************************
@@ -479,15 +479,16 @@ static const struct apic_type_info *get_apic_type_info(u8 t)
 static void extract_pic_apic(deark *c, id3v2ctx *d, dbuf *f,
 	 i64 pos, i64 len, const struct apic_type_info *ptinfo)
 {
-	const char *ext;
 	char fullext[32];
-	u8 sig[2];
 	const char *token = NULL;
+	struct fmtutil_fmtid_ctx *idctx = NULL;
 
-	dbuf_read(f, sig, pos, 2);
-	if(sig[0]==0x89 && sig[1]==0x50) ext="png";
-	else if(sig[0]==0xff && sig[1]==0xd8) ext="jpg";
-	else ext="bin";
+	idctx = de_malloc(c, sizeof(struct fmtutil_fmtid_ctx));
+	idctx->inf = f;
+	idctx->inf_pos = pos;
+	idctx->inf_len = len;
+	idctx->mode = FMTUTIL_FMTIDMODE_ALL_IMG; // expecting PNG or JPEG
+	fmtutil_fmtid(c, idctx);
 
 	if(ptinfo && ptinfo->token) token = ptinfo->token;
 	if(!token) {
@@ -495,9 +496,11 @@ static void extract_pic_apic(deark *c, id3v2ctx *d, dbuf *f,
 	}
 	if(!token) token = "id3pic";
 
-	de_snprintf(fullext, sizeof(fullext), "%s.%s", token, ext);
+	de_snprintf(fullext, sizeof(fullext), "%s.%s", token, idctx->ext_sz);
 
 	dbuf_create_file_from_slice(f, pos, len, fullext, NULL, DE_CREATEFLAG_IS_AUX);
+
+	de_free(c, idctx);
 }
 
 // Similar to decode_id3v2_frame_pic_apic()
@@ -892,7 +895,7 @@ static const char *get_id3v2_frame_name(id3v2ctx *d, u32 id)
 		{0x54494dU, 0x54494d45U, "Time"},
 		{0x545432U, 0x54495432U, "Title"},
 		{0x54524bU, 0x5452434bU, "Track number"},
-		{0x554c54U, 0x55534c54U, "Unsychronized lyric transcription"},
+		{0x554c54U, 0x55534c54U, "Unsynchronized lyric transcription"},
 		{CODE_TXX,  CODE_TXXX,   "User defined text information"},
 		{CODE_WXX,  CODE_WXXX,   "User defined URL link"},
 		{0x545945U, 0x54594552U, "Year"}
@@ -1115,7 +1118,7 @@ static const char *get_id3v1_genre_name(u8 g)
 		{25, "Euro-Techno"}, {26, "Ambient"}, {27, "Trip-Hop"}, {28, "Vocal"}, {29, "Jazz+Funk"},
 		{30, "Fusion"}, {31, "Trance"}, {32, "Classical"}, {33, "Instrumental"}, {34, "Acid"},
 		{35, "House"}, {36, "Game"}, {37, "Sound Clip"}, {38, "Gospel"}, {39, "Noise"},
-		{40, "AlternRock"}, {41, "Bass"}, {42, "Soul"}, {43, "Punk"}, {44, "Space"},
+		{40, "Altern. Rock"}, {41, "Bass"}, {42, "Soul"}, {43, "Punk"}, {44, "Space"},
 		{45, "Meditative"}, {46, "Instrumental Pop"}, {47, "Instrumental Rock"}, {48, "Ethnic"}, {49, "Gothic"},
 		{50, "Darkwave"}, {51, "Techno-Industrial"}, {52, "Electronic"}, {53, "Pop-Folk"}, {54, "Eurodance"},
 		{55, "Dream"}, {56, "Southern Rock"}, {57, "Comedy"}, {58, "Cult"}, {59, "Gangsta"},
@@ -1130,9 +1133,10 @@ static const char *get_id3v1_genre_name(u8 g)
 		{100, "Humour"}, {101, "Speech"}, {102, "Chanson"}, {103, "Opera"}, {104, "Chamber Music"},
 		{105, "Sonata"}, {106, "Symphony"}, {107, "Booty Brass"}, {108, "Primus"}, {109, "Porn Groove"},
 		{110, "Satire"}, {111, "Slow Jam"}, {112, "Club"}, {113, "Tango"}, {114, "Samba"},
-		{115, "Folklore"}, {116, "Ballad"}, {117, "Poweer Ballad"}, {118, "Rhytmic Soul"}, {119, "Freestyle"},
-		{120, "Duet"}, {121, "Punk Rock"}, {122, "Drum Solo"}, {123, "A Capela"}, {124, "Euro-House"},
+		{115, "Folklore"}, {116, "Ballad"}, {117, "Power Ballad"}, {118, "Rhythmic Soul"}, {119, "Freestyle"},
+		{120, "Duet"}, {121, "Punk Rock"}, {122, "Drum Solo"}, {123, "A Cappella"}, {124, "Euro-House"},
 		{125, "Dance Hall"},
+		// TODO?: More IDs have been defined, by various versions of Winamp.
 		{255, "unspecified"} };
 	size_t k;
 
