@@ -10,6 +10,7 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_ilbm);
 DE_DECLARE_MODULE(de_module_anim);
+DE_DECLARE_MODULE(de_module_deep);
 
 #define ANIM_MAX_FRAMES 10000
 
@@ -37,10 +38,13 @@ DE_DECLARE_MODULE(de_module_anim);
 #define CODE_ILBM 0x494c424dU
 #define CODE_PBM  0x50424d20U
 #define CODE_PCHG 0x50434847U
+#define CODE_RGB8 0x52474238U
+#define CODE_RGBN 0x5247424eU
 #define CODE_SBDY 0x53424459U
 #define CODE_SHAM 0x5348414dU
 #define CODE_TINY 0x54494e59U
 #define CODE_VDAT 0x56444154U
+#define CODE_XS24 0x58533234U
 
 #define ANIM_OP_XOR 1
 
@@ -71,7 +75,8 @@ struct imgbody_info {
 	i64 frame_buffer_size;
 	enum colortype_enum colortype;
 	int is_thumb;
-	int is_pbm; // frame buffer is PBM pixel format
+	u8 is_pbm; // frame buffer is PBM pixel format
+	u8 is_rgb24; // frame buffer is RGB24 pixel format
 };
 
 struct frame_ctx {
@@ -85,7 +90,7 @@ struct frame_ctx {
 	dbuf *frame_buffer;
 };
 
-typedef struct localctx_struct {
+typedef struct localctx_ilbm {
 	int is_anim;
 	u32 formtype;
 	char formtype_sanitized_sz[8];
@@ -95,14 +100,20 @@ typedef struct localctx_struct {
 	int num_frames_started;
 	int num_frames_finished;
 	int debug_frame_buffer;
-	u8 opt_notrans;
-	u8 opt_fixpal;
+#define TRANS_REMOVE   0
+#define TRANS_RESPECT  1
+#define TRANS_AUTO     2
+	u8 trans_setting;
+	u8 fixpal_setting;
 	u8 opt_allowsham;
+	u8 opt_allowdctv;
+	u8 opt_allowhame;
 	u8 opt_anim_includedups;
 	u8 found_bmhd;
 	u8 found_cmap;
 	u8 cmap_changed_flag;
 	u8 bmhd_changed_flag;
+	u8 camg_changed_flag;
 	u8 has_camg;
 	u8 ham_flag; // "hold and modify"
 	u8 is_ham6;
@@ -189,6 +200,11 @@ static const char *get_cmprtype_name(lctx *d, u8 n)
 	case 0: name = "uncompressed"; break;
 	case 1: name = "PackBits"; break;
 	case 2: name = "VDAT"; break;
+	case 3: case 4:
+		if(d->formtype==CODE_RGBN || d->formtype==CODE_RGB8) {
+			name = "RGBN/RGB8";
+		}
+		break;
 	}
 	return name?name:"?";
 }
@@ -212,7 +228,7 @@ static void on_multipalette_enabled(deark *c, lctx *d)
 	}
 	else {
 		de_err(c, "Multi-palette ILBM images are not supported. "
-			"(\"-opt ilbm:allowsham\" to decode anyway)");
+			"(\"-opt ilbm:allowspecial\" to decode anyway)");
 	}
 }
 
@@ -417,6 +433,7 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 	i64 col;
 	i64 dststride = ibi->frame_buffer_rowspan;
 	UI unc_threshold;
+	u8 last_col_quirk = 0;
 	int baddata_flag = 0;
 
 	if(separate_data_stream) {
@@ -450,12 +467,41 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 		num_columns = (ibi->width+31)/32;
 	}
 
+	if(frctx->op==8 && dataelem_size==4) {
+		i64 num_columns_if_size2;
+
+		num_columns_if_size2 = (ibi->width+15)/16;
+		if(num_columns*2 != num_columns_if_size2) {
+			// A dumb quirk of ANIM-8: The last column may use "word" mode,
+			// even when when all other columns use "long" mode.
+			last_col_quirk = 1;
+		}
+	}
+
 	for(col=0; col<num_columns; col++) {
 		i64 opcount;
 		i64 opidx;
 		i64 elem_bytes_to_write;
 		i64 ypos = 0;
 		i64 col_start_dstpos;
+		i64 dataelem_size_thiscol;
+		i64 code_size_thiscol;
+		UI unc_threshold_thiscol;
+		u8 is_last_col;
+
+		if(c->debug_level>=3) {
+			de_dbg3(c, "col %u at %"I64_FMT, (UI)col, pos);
+		}
+
+		is_last_col = (col+1 == num_columns);
+		code_size_thiscol = code_size;
+		dataelem_size_thiscol = dataelem_size;
+		unc_threshold_thiscol = unc_threshold;
+		if(is_last_col && last_col_quirk) {
+			code_size_thiscol = 2;
+			dataelem_size_thiscol = 2;
+			unc_threshold_thiscol = 0x8000;
+		}
 
 		if(pos>=endpos) {
 			baddata_flag = 1;
@@ -463,16 +509,16 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 		}
 
 		// Defend against writing beyond the right edge of this plane
-		if((dataelem_size==4) && (col+1 == num_columns) && (ibi->bytes_per_row_per_plane%4)) {
+		if(is_last_col && (dataelem_size_thiscol==4) && (ibi->bytes_per_row_per_plane%4)) {
 			elem_bytes_to_write = 2;
 		}
 		else {
-			elem_bytes_to_write = dataelem_size;
+			elem_bytes_to_write = dataelem_size_thiscol;
 		}
 
 		col_start_dstpos = plane_idx * ibi->bytes_per_row_per_plane + dataelem_size*col;
 
-		opcount = get_elem_as_int_p(inf, &pos, code_size);
+		opcount = get_elem_as_int_p(inf, &pos, code_size_thiscol);
 		if(c->debug_level>=3) {
 			de_dbg3(c, "col %d op count: %"I64_FMT, (int)col, opcount);
 		}
@@ -488,11 +534,12 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 				baddata_flag = 1;
 				goto done;
 			}
-			op = (UI)get_elem_as_int_p(inf, &pos, code_size);
+			op = (UI)get_elem_as_int_p(inf, &pos, code_size_thiscol);
 
 			if(op==0) { // RLE
-				count = get_elem_as_int_p(inf, &pos, code_size);
+				count = get_elem_as_int_p(inf, &pos, code_size_thiscol);
 				if(ypos+count > ibi->height) {
+					// TODO: Should we tolerate this, and set count = 0?
 					baddata_flag = 1;
 					goto done;
 				}
@@ -506,12 +553,12 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 						baddata_flag = 1;
 						goto done;
 					}
-					dbuf_read(inf, valbuf, datapos, dataelem_size);
-					datapos += dataelem_size;
+					dbuf_read(inf, valbuf, datapos, dataelem_size_thiscol);
+					datapos += dataelem_size_thiscol;
 				}
 				else {
-					dbuf_read(inf, valbuf, pos, dataelem_size);
-					pos += dataelem_size;
+					dbuf_read(inf, valbuf, pos, dataelem_size_thiscol);
+					pos += dataelem_size_thiscol;
 				}
 
 				for(k=0; k<count; k++) {
@@ -528,11 +575,11 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 					ypos++;
 				}
 			}
-			else if(op < unc_threshold) { // skip
+			else if(op < unc_threshold_thiscol) { // skip
 				ypos += (i64)op;
 			}
 			else { // uncompressed run
-				count = (i64)(op - unc_threshold);
+				count = (i64)(op - unc_threshold_thiscol);
 				if(ypos+count > ibi->height) {
 					baddata_flag = 1;
 					goto done;
@@ -548,12 +595,12 @@ static void decompress_plane_vdelta(deark *c, lctx *d, struct imgbody_info *ibi,
 							baddata_flag = 1;
 							goto done;
 						}
-						dbuf_read(inf, valbuf, datapos, dataelem_size);
-						datapos += dataelem_size;
+						dbuf_read(inf, valbuf, datapos, dataelem_size_thiscol);
+						datapos += dataelem_size_thiscol;
 					}
 					else {
-						dbuf_read(inf, valbuf, pos, dataelem_size);
-						pos += dataelem_size;
+						dbuf_read(inf, valbuf, pos, dataelem_size_thiscol);
+						pos += dataelem_size_thiscol;
 					}
 
 					dstpos = col_start_dstpos + ypos*dststride;
@@ -835,7 +882,7 @@ static void decompress_delta_op74(deark *c, lctx *d, struct imgbody_info *ibi,
 			break;
 		}
 		else {
-			de_warn(c, "Bad or unsupported ANIM-J compression code (%u)", code);
+			de_warn(c, "Bad or unsupported ANIM-J compression code: %u", code);
 			goto done;
 		}
 	}
@@ -1042,6 +1089,7 @@ static void fixup_palette(deark *c, lctx *d)
 	u8 cr, cg, cb;
 
 	if(d->pal_ncolors<1) return;
+	if(d->formtype==CODE_RGBN || d->formtype==CODE_RGB8) return;
 
 	if(d->is_ham8) {
 		pal_fixup6(c, d);
@@ -1067,7 +1115,19 @@ static void fixup_palette(deark *c, lctx *d)
 // Called when we encounter a BODY or DLTA or TINY chunk
 static void do_before_image_chunk(deark *c, lctx *d)
 {
-	if(d->bmhd_changed_flag) {
+	if(d->bmhd_changed_flag || d->camg_changed_flag) {
+		if(d->ham_flag) {
+			if(d->planes_raw==6 || d->planes_raw==5) {
+				d->is_ham6 = 1;
+			}
+			else if(d->planes_raw==8 || d->planes_raw==7) {
+				d->is_ham8 = 1;
+			}
+			else {
+				de_warn(c, "Invalid bit depth (%d) for HAM image.", (int)d->planes_raw);
+			}
+		}
+
 		if(!d->found_cmap && d->planes_raw<=8) {
 			de_make_grayscale_palette(d->pal, (i64)1<<(UI)d->planes_raw, 0);
 		}
@@ -1097,8 +1157,7 @@ static void do_before_image_chunk(deark *c, lctx *d)
 		}
 	}
 
-
-	if(d->opt_fixpal && !d->is_anim && d->cmap_changed_flag) {
+	if(d->fixpal_setting && d->cmap_changed_flag) {
 		fixup_palette(c, d);
 	}
 
@@ -1108,6 +1167,7 @@ static void do_before_image_chunk(deark *c, lctx *d)
 
 	d->cmap_changed_flag = 0;
 	d->bmhd_changed_flag = 0;
+	d->camg_changed_flag = 0;
 }
 
 static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is_thumb)
@@ -1120,6 +1180,22 @@ static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is
 	// frame buffer format (and back). So we support a special frame buffer format
 	// just for PBM.
 	ibi->is_pbm = (d->formtype==CODE_PBM);
+	// We also have a special nonplanar RGB24 format (is_rgb24).
+
+	if(d->formtype==CODE_RGBN || d->formtype==CODE_RGB8) {
+		ibi->is_rgb24 = 1;
+		if(((d->formtype==CODE_RGBN && d->planes_raw==13) ||
+			(d->formtype==CODE_RGB8 && d->planes_raw==25)) &&
+			(d->compression==3 || d->compression==4) &&
+			d->masking_code==MASKINGTYPE_NONE)
+		{
+			;
+		}
+		else {
+			de_err(c, "Unsupported RGBN/RGB8 format");
+			goto done;
+		}
+	}
 
 	if(is_thumb) {
 		ibi->width = d->thumb_width;
@@ -1154,7 +1230,7 @@ static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is
 		ibi->colortype = COLORTYPE_DEFAULT;
 	}
 
-	if(ibi->is_pbm) {
+	if(ibi->is_pbm || ibi->is_rgb24) {
 		ibi->planes_fg = 1;
 		ibi->planes_total = 1;
 	}
@@ -1184,6 +1260,10 @@ static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is
 		ibi->bits_per_row_per_plane = ibi->bytes_per_row_per_plane * 8;
 		// Note: The PBM row size might be adjusted later, after decompression.
 	}
+	else if(ibi->is_rgb24) {
+		ibi->bytes_per_row_per_plane = ibi->width*3;
+		ibi->bits_per_row_per_plane = ibi->bytes_per_row_per_plane * 8;
+	}
 	else {
 		ibi->bits_per_row_per_plane = de_pad_to_n(ibi->width, 16);
 		ibi->bytes_per_row_per_plane = ibi->bits_per_row_per_plane/8;
@@ -1195,7 +1275,7 @@ static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is
 		;
 	}
 	else if(ibi->masking_code==MASKINGTYPE_COLORKEY) {
-		if(!d->opt_notrans && ibi->planes_fg<=8 && !d->ham_flag) {
+		if(ibi->planes_fg<=8 && !d->ham_flag) {
 			ibi->use_colorkey_transparency = 1;
 		}
 	}
@@ -1214,7 +1294,7 @@ static int init_imgbody_info(deark *c, lctx *d, struct imgbody_info *ibi, int is
 		;
 	}
 	else if(ibi->planes_fg<1 || ibi->planes_fg>8) {
-		de_err(c, "Bad or unsupported number of planes (%d)", (int)ibi->planes_fg);
+		de_err(c, "Bad or unsupported number of planes: %d", (int)ibi->planes_fg);
 		goto done;
 	}
 	retval = 1;
@@ -1328,23 +1408,97 @@ static int decompress_method0(deark *c, lctx *d, i64 pos, i64 len, dbuf *unc_pix
 	return 1;
 }
 
-static int decompress_method1(deark *c, lctx *d, i64 pos, i64 len, dbuf *unc_pixels,
-	i64 expected_len)
+static int decompress_rgbn8(deark *c, lctx *d, struct imgbody_info *ibi,
+	i64 pos1, i64 len, dbuf *unc_pixels)
+{
+	int retval = 0;
+	i64 pos = pos1;
+	i64 endpos = pos1+len;
+	i64 npixels = 0;
+	i64 npixels_expected;
+	u8 is_rgb8 = (d->formtype==CODE_RGB8);
+
+	npixels_expected = ibi->width * ibi->height;
+
+	while(1) {
+		i64 i;
+		u32 n;
+		i64 count;
+		u8 cbuf[3];
+
+		if(npixels >= npixels_expected) {
+			retval = 1;
+			goto done;
+		}
+		if(pos >= endpos) {
+			goto done;
+		}
+
+
+		if(is_rgb8) {
+			n = (u32)de_getu32be_p(&pos);
+			cbuf[0] = (u8)(n>>24);
+			cbuf[1] = (u8)((n&0x00ff0000U)>>16);
+			cbuf[2] = (u8)((n&0x0000ff00U)>>8);
+			// TODO?: There's also a "genlock" bit that's sort of a transparency mask.
+			count = (i64)(n & 0x7f);
+		}
+		else {
+			n = (u32)de_getu16be_p(&pos);
+			cbuf[0] = (u8)(n>>12);
+			cbuf[1] = (u8)((n&0x0f00)>>8);
+			cbuf[2] = (u8)((n&0x00f0)>>4);
+			for(i=0; i<3; i++) {
+				cbuf[i] *= 17;
+			}
+			count = (i64)(n & 0x07);
+		}
+
+		if(count==0) {
+			u8 c2raw;
+
+			c2raw = de_getbyte_p(&pos);
+			if(c2raw!=0) {
+				count = (i64)c2raw;
+			}
+			else {
+				count = de_getu16be_p(&pos);
+			}
+		}
+
+		for(i=0; i<count; i++) {
+			dbuf_write(unc_pixels, cbuf, 3);
+		}
+		npixels += count;
+	}
+
+done:
+	if(!retval) {
+		de_err(c, "RGBN/RGB8 decompression failed");
+	}
+	return retval;
+}
+
+static int decompress_ilbm_packbits(deark *c, dbuf *inf, i64 pos, i64 len,
+	dbuf *unc_pixels, i64 expected_len, UI nbytes_per_unit)
 {
 	int retval = 0;
 	struct de_dfilter_in_params dcmpri;
 	struct de_dfilter_out_params dcmpro;
 	struct de_dfilter_results dres;
+	struct de_packbits_params pbparams;
 
+	de_zeromem(&pbparams, sizeof(struct de_packbits_params));
+	pbparams.nbytes_per_unit = nbytes_per_unit;
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
-	dcmpri.f = c->infile;
+	dcmpri.f = inf;
 	dcmpri.pos = pos;
 	dcmpri.len = len;
 	dcmpro.f = unc_pixels;
 	dcmpro.len_known = 1;
 	dcmpro.expected_len = expected_len;
 
-	fmtutil_decompress_packbits_ex(c, &dcmpri, &dcmpro, &dres, NULL);
+	fmtutil_decompress_packbits_ex(c, &dcmpri, &dcmpro, &dres, &pbparams);
 	dbuf_flush(dcmpro.f);
 	if(dres.errcode) {
 		de_err(c, "Decompression failed: %s", dres.errmsg);
@@ -1561,23 +1715,47 @@ static void detect_hame(deark *c, lctx *d, struct imgbody_info *ibi,
 
 	if(de_memcmp(pixelval, sig, 15)) return;
 	if(pixelval[15]!=0x4 && pixelval[15]!=0x8) return;
-	de_warn(c, "This is probably a HAM-E image, which is not supported correctly.");
+
+	if(d->opt_allowhame) {
+		de_warn(c, "This is probably a HAM-E image, which is not supported correctly.");
+	}
+	else {
+		de_err(c, "HAM-E images are not supported. "
+			"(\"-opt ilbm:allowspecial\" to decode anyway)");
+		d->errflag = 1;
+	}
+
 	d->is_hame = 1;
 }
 
+static int is_dctv_sig_at(dbuf *f, i64 pos)
+{
+	u8 x;
+
+	static const u8 sig[31] = {
+		0x49, 0x87, 0x28, 0xde, 0x11, 0x0b, 0xef, 0xd2, 0x0c, 0x8e, 0x8b, 0x35, 0x5b, 0x75, 0xec, 0xb8,
+		0x29, 0x6b, 0x03, 0xf9, 0x2b, 0xb4, 0x34, 0xee, 0x67, 0x1e, 0x7c, 0x4f, 0x53, 0x63, 0x15 };
+
+	x = dbuf_getbyte(f, pos);
+	if(x != sig[0]) return 0;
+	if(dbuf_memcmp(f, pos, sig, 31)) return 0;
+	return 1;
+}
+
 // Detect and warn about DCTV, which we don't support.
 static void detect_dctv(deark *c, lctx *d, struct imgbody_info *ibi,
 	struct frame_ctx *frctx)
 {
-	static const u8 sig[31] = {
-		0x49, 0x87, 0x28, 0xde, 0x11, 0x0b, 0xef, 0xd2, 0x0c, 0x8e, 0x8b, 0x35, 0x5b, 0x75, 0xec, 0xb8,
-		0x29, 0x6b, 0x03, 0xf9, 0x2b, 0xb4, 0x34, 0xee, 0x67, 0x1e, 0x7c, 0x4f, 0x53, 0x63, 0x15 };
 	i64 pos;
+	int result;
 
 	// As far as I can tell, in DCTV images, the last plane of the first row is
 	// as follows:
 	//   <00> <31-byte signature> <00 fill> <31-byte signature> <00>
 	// (Sometimes, the last plane of the *second* row is the same.)
+	// (But I have a 2-plane image in which it's the *first* plane of the first
+	// two rows that are special. TODO: Figure this out.)
+	//
 	// Unknowns:
 	// * Is DCTV possible if there are fewer than 64 bytes per row per plane (i.e. width < 512)?
 	// * Can a DCTV image have transparency?
@@ -1588,10 +1766,28 @@ static void detect_dctv(deark *c, lctx *d, struct imgbody_info *ibi,
 	if(ibi->is_thumb) return;
 	if(frctx->formtype!=CODE_ILBM && frctx->formtype!=CODE_ACBM) return;
 	if(ibi->bytes_per_row_per_plane<64) return;
+
+	// Test end of last plane of first row
 	pos = d->planes_raw * ibi->bytes_per_row_per_plane - 32;
-	if(dbuf_getbyte(frctx->frame_buffer, pos) != sig[0]) return;
-	if(dbuf_memcmp(frctx->frame_buffer, pos, sig, 31)) return;
-	de_warn(c, "This is probably a DCTV image, which is not supported correctly.");
+	result = is_dctv_sig_at(frctx->frame_buffer, pos);
+
+	if(!result) {
+		// Test end of first plane of first row
+		pos = ibi->bytes_per_row_per_plane - 32;
+		result = is_dctv_sig_at(frctx->frame_buffer, pos);
+	}
+
+	if(!result) return;
+
+	if(d->opt_allowdctv) {
+		de_warn(c, "This is probably a DCTV image, which is not supported correctly.");
+	}
+	else {
+		de_err(c, "DCTV images are not supported. "
+			"(\"-opt ilbm:allowspecial\" to decode anyway)");
+		d->errflag = 1;
+	}
+
 	d->is_dctv = 1;
 }
 
@@ -1624,12 +1820,16 @@ static int do_image_chunk_internal(deark *c, lctx *d, struct frame_ctx *frctx, i
 		// Note: I don't think ABIT images are ever compressed.
 		if(!convert_abit(c, d, ibi, pos1, len, frctx->frame_buffer)) goto done;
 	}
+	else if(d->formtype==CODE_RGBN || d->formtype==CODE_RGB8) {
+		if(!decompress_rgbn8(c, d, ibi, pos1, len, frctx->frame_buffer)) goto done;
+	}
 	else if(ibi->compression==0) {
 		if(!decompress_method0(c, d, pos1, len, frctx->frame_buffer, ibi->frame_buffer_size)) goto done;
 	}
 	else if(ibi->compression==1) {
 		dbuf_enable_wbuffer(frctx->frame_buffer);
-		ret = decompress_method1(c, d, pos1, len, frctx->frame_buffer, ibi->frame_buffer_size);
+		ret = decompress_ilbm_packbits(c, c->infile, pos1, len,
+			frctx->frame_buffer, ibi->frame_buffer_size, 1);
 		dbuf_disable_wbuffer(frctx->frame_buffer);
 		if(!ret) goto done;
 	}
@@ -1637,7 +1837,7 @@ static int do_image_chunk_internal(deark *c, lctx *d, struct frame_ctx *frctx, i
 		if(!decompress_method2(c, d, ibi, pos1, len, frctx->frame_buffer, ibi->frame_buffer_size)) goto done;
 	}
 	else {
-		de_err(c, "Unsupported compression method (%d)", (int)ibi->compression);
+		de_err(c, "Unsupported compression method: %d", (int)ibi->compression);
 		goto done;
 	}
 
@@ -1705,6 +1905,29 @@ done:
 	destroy_frame(c, d, frctx);
 }
 
+// A few ILBM files with extension ".iff24" have this chunk, as
+// do some DEEP files.
+static void do_xs24(deark *c, dbuf *f, i64 pos1, i64 len)
+{
+	i64 w, h, rowspan;
+	i64 pos = pos1;
+	de_bitmap *img = NULL;
+
+	w = de_getu16be_p(&pos);
+	rowspan = w*3;
+	h = de_getu16be_p(&pos);
+	de_dbg(c, "24-bit thumbnail image, dimensions: %u"DE_CHAR_TIMES"%u", (UI)w, (UI)h);
+	if(!de_good_image_dimensions_noerr(c, w, h)) goto done;
+	if(rowspan*h+6 != len) goto done;
+	pos += 2; // unknown field
+
+	img = de_bitmap_create(c, w, h, 3);
+	de_convert_image_rgb(f, pos, rowspan, 3, img, 0);
+	de_bitmap_write_to_file(img, "thumb", DE_CREATEFLAG_IS_AUX|DE_CREATEFLAG_OPT_IMAGE);
+done:
+	de_bitmap_destroy(img);
+}
+
 static void get_bits_descr(deark *c, lctx *d, struct frame_ctx *frctx, de_ucstring *s)
 {
 	UI bits = frctx->bits;
@@ -1800,6 +2023,7 @@ static void do_camg(deark *c, lctx *d, i64 pos, i64 len)
 {
 	if(len<4) return;
 	d->has_camg = 1;
+	d->camg_changed_flag = 1;
 
 	d->ham_flag = 0;
 	d->is_ham6 = 0;
@@ -1814,22 +2038,15 @@ static void do_camg(deark *c, lctx *d, i64 pos, i64 len)
 	if(d->camg_mode & 0x0080)
 		d->ehb_flag = 1;
 
+	if(d->formtype==CODE_RGBN || d->formtype==CODE_RGB8) {
+		d->ham_flag = 0;
+		d->ehb_flag = 0;
+	}
+
 	de_dbg_indent(c, 1);
 	de_dbg(c, "HAM: %d", (int)d->ham_flag);
 	de_dbg(c, "EHB: %d", (int)d->ehb_flag);
 	de_dbg_indent(c, -1);
-
-	if(d->ham_flag) {
-		if(d->planes_raw==6 || d->planes_raw==5) {
-			d->is_ham6 = 1;
-		}
-		else if(d->planes_raw==8 || d->planes_raw==7) {
-			d->is_ham8 = 1;
-		}
-		else {
-			de_warn(c, "Invalid bit depth (%d) for HAM image.", (int)d->planes_raw);
-		}
-	}
 }
 
 static void do_dpi(deark *c, lctx *d, i64 pos, i64 len)
@@ -2097,7 +2314,10 @@ static void write_frame(deark *c, lctx *d, struct imgbody_info *ibi, struct fram
 	rowbuf = de_mallocarray(c, rowbuf_size, sizeof(rowbuf[0]));
 	rowbuf_trns = de_mallocarray(c, rowbuf_size, sizeof(rowbuf_trns[0]));
 
-	if(d->found_cmap && d->pal_is_grayscale && d->planes_raw<=8 && !d->is_ham6 && !d->is_ham8) {
+	if(ibi->is_rgb24) {
+		bypp = 3;
+	}
+	else if(d->found_cmap && d->pal_is_grayscale && d->planes_raw<=8 && !d->is_ham6 && !d->is_ham8) {
 		bypp = 1;
 	}
 	else {
@@ -2105,9 +2325,7 @@ static void write_frame(deark *c, lctx *d, struct imgbody_info *ibi, struct fram
 	}
 
 	if(ibi->use_colorkey_transparency || ibi->masking_code==MASKINGTYPE_1BITMASK) {
-		if(!d->opt_notrans) {
-			bypp++;
-		}
+		bypp++;
 	}
 
 	img = de_bitmap_create(c, ibi->width, ibi->height, bypp);
@@ -2117,6 +2335,11 @@ static void write_frame(deark *c, lctx *d, struct imgbody_info *ibi, struct fram
 			d->pal, img, 0);
 		goto after_render;
 	}
+	else if(ibi->is_rgb24) {
+		de_convert_image_rgb(frctx->frame_buffer, 0, ibi->bytes_per_row_per_plane, 3,
+			img, 0);
+		goto after_render;
+	}
 
 	for(j=0; j<ibi->height; j++) {
 		i64 z;
@@ -2175,7 +2398,7 @@ static void write_frame(deark *c, lctx *d, struct imgbody_info *ibi, struct fram
 		}
 
 		// Handle 1-bit transparency masks here, for all color types.
-		if(ibi->masking_code==MASKINGTYPE_1BITMASK && !d->opt_notrans) {
+		if(ibi->masking_code==MASKINGTYPE_1BITMASK) {
 			i64 i;
 
 			for(i=0; i<rowbuf_size; i++) {
@@ -2191,12 +2414,22 @@ static void write_frame(deark *c, lctx *d, struct imgbody_info *ibi, struct fram
 	}
 
 after_render:
+	if(d->trans_setting==TRANS_AUTO) {
+		if(ibi->masking_code==MASKINGTYPE_1BITMASK) {
+			de_bitmap_optimize_alpha(img, 0x2|0x4);
+		}
+	}
+	else if(d->trans_setting==TRANS_REMOVE) {
+		de_bitmap_remove_alpha(img);
+	}
+
 	fi = de_finfo_create(c);
 	set_finfo_data(c, d, ibi, fi);
 	if(ibi->is_thumb) {
 		createflags |= DE_CREATEFLAG_IS_AUX;
 	}
 	if(!d->is_anim) {
+		// TODO: Enable this for ANIM?
 		createflags |= DE_CREATEFLAG_OPT_IMAGE;
 	}
 
@@ -2303,6 +2536,10 @@ static int my_iff_chunk_handler(struct de_iffctx *ictx)
 		do_tiny(c, d, ictx->chunkctx->dpos, ictx->chunkctx->dlen);
 		ictx->handled = 1;
 		break;
+	case CODE_XS24:
+		do_xs24(c, ictx->f, ictx->chunkctx->dpos, ictx->chunkctx->dlen);
+		ictx->handled = 1;
+		break;
 
 	case CODE_DPI:
 		do_dpi(c, d, ictx->chunkctx->dpos, ictx->chunkctx->dlen);
@@ -2405,7 +2642,9 @@ static int my_on_std_container_start_fn(struct de_iffctx *ictx)
 		}
 		if((ictx->curr_container_contentstype4cc.id == CODE_ILBM) ||
 			(ictx->curr_container_contentstype4cc.id == CODE_PBM) ||
-			(ictx->curr_container_contentstype4cc.id == CODE_ACBM))
+			(ictx->curr_container_contentstype4cc.id == CODE_ACBM) ||
+			(ictx->curr_container_contentstype4cc.id == CODE_RGBN) ||
+			(ictx->curr_container_contentstype4cc.id == CODE_RGB8))
 		{
 			on_frame_begin(c, d, ictx->curr_container_contentstype4cc.id);
 		}
@@ -2509,7 +2748,7 @@ static void print_summary(deark *c, lctx *d)
 	if(d->ehb_flag) summary_append(s, "EHB");
 	if(d->is_sham) summary_append(s, "SHAM");
 	if(d->is_pchg) summary_append(s, "PCHG");
-	if(d->is_ctbl) summary_append(s, "CBTL");
+	if(d->is_ctbl) summary_append(s, "CTBL");
 	if(d->is_beam) summary_append(s, "BEAM");
 	if(d->is_hame) summary_append(s, "HAM-E");
 	if(d->is_dctv) summary_append(s, "DCTV");
@@ -2538,18 +2777,34 @@ static void strip_trailing_space_sz(char *sz)
 static void de_run_ilbm_or_anim(deark *c, de_module_params *mparams)
 {
 	u32 id;
+	u8 opt_notrans = 0;
+	u8 opt_fixpal;
+	int opt;
+	const char *opt_trans_str;
 	lctx *d = NULL;
 	struct de_iffctx *ictx = NULL;
 	struct de_fourcc formtype_4cc;
 
 	d = de_malloc(c, sizeof(lctx));
-	d->opt_fixpal = (u8)de_get_ext_option_bool(c, "ilbm:fixpal", 1);
-	if(de_get_ext_option(c, "ilbm:notrans")) {
-		d->opt_notrans = 1;
+	opt_fixpal = (u8)de_get_ext_option_bool(c, "ilbm:fixpal", 0xff);
+
+	opt_trans_str = de_get_ext_option(c, "ilbm:trans");
+
+	if(!opt_trans_str) {
+		if(de_get_ext_option(c, "ilbm:notrans")) { // Deprecated option
+			opt_notrans = 1;
+		}
 	}
-	if(de_get_ext_option(c, "ilbm:allowsham")) {
+
+	if(de_get_ext_option_bool(c, "ilbm:allowspecial", 0)) {
 		d->opt_allowsham = 1;
+		d->opt_allowdctv = 1;
+		d->opt_allowhame = 1;
 	}
+	// allowsham is deprecated
+	opt = de_get_ext_option_bool(c, "ilbm:allowsham", -1);
+	if(opt==0) d->opt_allowsham = 0;
+	else if(opt>0) d->opt_allowsham = 1;
 
 	id = (u32)de_getu32be(0);
 	if(id!=CODE_FORM) {
@@ -2572,6 +2827,8 @@ static void de_run_ilbm_or_anim(deark *c, de_module_params *mparams)
 	case CODE_ILBM:
 	case CODE_ACBM:
 	case CODE_PBM:
+	case CODE_RGBN:
+	case CODE_RGB8:
 		break;
 	default:
 		de_err(c, "Not a supported ILBM-like format (%s)", d->formtype_sanitized_sz);
@@ -2583,6 +2840,48 @@ static void de_run_ilbm_or_anim(deark *c, de_module_params *mparams)
 		d->opt_anim_includedups = (u8)de_get_ext_option_bool(c, "anim:includedups", 0);
 	}
 
+	if(opt_fixpal==0xff) {
+		// Fixpal option defaults to No for ANIM format, othersize Yes.
+		// (The concern with ANIM is that we don't want frame 1's colors
+		// to fail to match frame 2's colors.)
+		d->fixpal_setting = (d->is_anim ? 0 : 1);
+	}
+	else {
+		d->fixpal_setting = opt_fixpal;
+	}
+
+	// Default for d->trans_setting
+	if(d->is_anim) {
+		// I don't think AUTO makes sense for ANIM.
+		d->trans_setting = TRANS_RESPECT;
+	}
+	else {
+		d->trans_setting = TRANS_AUTO;
+	}
+
+	if(opt_trans_str) {
+		if(!de_strcmp(opt_trans_str, "auto")) {
+			d->trans_setting = TRANS_AUTO;
+		}
+		else if(!de_strcmp(opt_trans_str, "")) {
+			d->trans_setting = TRANS_RESPECT;
+		}
+		else {
+			int opt_trans_n;
+
+			opt_trans_n = de_atoi(opt_trans_str);
+			if(opt_trans_n==0) {
+				d->trans_setting = TRANS_REMOVE;
+			}
+			else if(opt_trans_n>0) {
+				d->trans_setting = TRANS_RESPECT;
+			}
+		}
+	}
+	else if(opt_notrans) {
+		d->trans_setting = TRANS_REMOVE;
+	}
+
 	d->FORM_level = d->is_anim ? 1 : 0;
 
 	ictx = fmtutil_create_iff_decoder(c);
@@ -2633,6 +2932,8 @@ static int de_identify_ilbm(deark *c)
 	case CODE_ILBM:
 	case CODE_PBM:
 	case CODE_ACBM:
+	case CODE_RGBN:
+	case CODE_RGB8:
 		return 100;
 	}
 	return 0;
@@ -2651,12 +2952,11 @@ static int de_identify_anim(deark *c)
 
 static void do_help_ilbm_anim(deark *c, int is_anim)
 {
-	de_msg(c, "-opt ilbm:notrans : Disable support for transparency");
-	if(!is_anim) {
-		de_msg(c, "-opt ilbm:fixpal=<0|1> : Don't/Do try to fix palettes that are "
-			"slightly too dark");
-	}
-	de_msg(c, "-opt ilbm:allowsham : Suppress an error on some images");
+	de_msg(c, "-opt ilbm:trans=<0|1|auto> : Always remove (0) or respect (1) "
+		"transparency");
+	de_msg(c, "-opt ilbm:fixpal=<0|1> : Don't/Do try to fix palettes that are "
+		"slightly too dark");
+	de_msg(c, "-opt ilbm:allowspecial : Suppress an error on some images");
 	if(is_anim) {
 		de_msg(c, "-opt anim:includedups : Do not suppress duplicate frames");
 	}
@@ -2689,3 +2989,521 @@ void de_module_anim(deark *c, struct deark_module_info *mi)
 	mi->identify_fn = de_identify_anim;
 	mi->help_fn = de_help_anim;
 }
+
+///// IFF-DEEP
+
+#define CODE_DBOD 0x44424f44U
+#define CODE_DEEP 0x44454550U
+#define CODE_DGBL 0x4447424cU
+#define CODE_DLOC 0x444c4f43U
+#define CODE_DPEL 0x4450454cU
+#define CODE_MPIC 0x4d504943U
+#define CODE_TVDC 0x54564443U
+#define CODE_TVPP 0x54565050U
+
+#define DEEP_MAX_COMPONENTS 8
+
+struct deep_ctx {
+	UI cmpr_meth;
+	int DBOD_count;
+	u8 trans_setting; // TRANS_*
+	u8 found_DLOC;
+	u8 found_TVDC;
+	u8 found_DPEL;
+	i64 dspw, dsph;
+	i64 width, height;
+	i64 x_aspect, y_aspect;
+	i64 expected_image_size;
+	i64 num_components;
+	u8 have_alpha;
+	u8 have_alpha_hint; // Preliminary, set while reading DPEL
+	u8 is_assoc_alpha;
+	UI cmp_nbits[DEEP_MAX_COMPONENTS];
+	UI cmp_type[DEEP_MAX_COMPONENTS];
+	UI sample_map[DEEP_MAX_COMPONENTS];
+	u8 tvdc_data[32];
+};
+
+static void do_deep_DGBL(deark *c, struct deep_ctx *d, struct de_iffctx *ictx)
+{
+	i64 pos = ictx->chunkctx->dpos;
+
+	d->dspw = dbuf_getu16be_p(ictx->f, &pos);
+	d->dsph = dbuf_getu16be_p(ictx->f, &pos);
+	de_dbg(c, "display size: %"I64_FMT DE_CHAR_TIMES "%"I64_FMT, d->dspw, d->dsph);
+	if(!d->found_DLOC) {
+		d->width = d->dspw;
+		d->height = d->dsph;
+	}
+	d->cmpr_meth = (UI)dbuf_getu16be_p(ictx->f, &pos);
+	de_dbg(c, "cmpr meth: %u", d->cmpr_meth);
+	d->x_aspect = (i64)dbuf_getbyte_p(ictx->f, &pos);
+	d->y_aspect = (i64)dbuf_getbyte_p(ictx->f, &pos);
+	de_dbg(c, "aspect ratio: %d, %d", (int)d->x_aspect, (int)d->y_aspect);
+}
+
+static void do_deep_DLOC(deark *c, struct deep_ctx *d, struct de_iffctx *ictx)
+{
+	i64 pos = ictx->chunkctx->dpos;
+	i64 pixpos_x, pixpos_y;
+
+	d->found_DLOC = 1;
+	d->width = dbuf_getu16be_p(ictx->f, &pos);
+	d->height = dbuf_getu16be_p(ictx->f, &pos);
+	de_dbg_dimensions(c, d->width, d->height);
+	pixpos_x = dbuf_geti16be_p(ictx->f, &pos);
+	pixpos_y = dbuf_geti16be_p(ictx->f, &pos);
+	de_dbg(c, "position: %"I64_FMT", %"I64_FMT, pixpos_x, pixpos_y);
+}
+
+static void do_deep_DPEL(deark *c, struct deep_ctx *d, struct de_iffctx *ictx)
+{
+	i64 pos = ictx->chunkctx->dpos;
+	i64 endpos = ictx->chunkctx->dpos + ictx->chunkctx->dlen;
+	UI i;
+
+	d->found_DPEL = 1;
+	d->num_components = (UI)dbuf_getu32be_p(ictx->f, &pos);
+	de_dbg(c, "num cmpts: %u", (UI)d->num_components);
+	if(d->num_components>DEEP_MAX_COMPONENTS) {
+		d->num_components = 0;
+		goto done;
+	}
+	for(i=0; i<d->num_components; i++) {
+		if(pos+4 > endpos) break;
+		d->cmp_type[i] = (UI)dbuf_getu16be_p(ictx->f, &pos);
+		if(d->cmp_type[i]==4 || d->cmp_type[i]==17) {
+			d->have_alpha_hint = 1;
+		}
+		d->cmp_nbits[i] = (UI)dbuf_getu16be_p(ictx->f, &pos);
+		de_dbg(c, "cmpt[%u]: type=%u depth=%u", i, d->cmp_type[i], d->cmp_nbits[i]);
+	}
+done:
+	;
+}
+
+// Sets d->sample_map[], d->have_alpha...
+// If unsupported image type, reports an error and returns 0.
+static int deep_make_sample_map(deark *c, struct deep_ctx *d)
+{
+	UI k;
+	u8 flags[5] = {0,0,0,0,0};
+	int retval = 0;
+
+	de_zeromem(d->sample_map, sizeof(d->sample_map));
+	d->have_alpha = 0;
+	d->is_assoc_alpha = 0;
+
+	if(d->num_components<3 || d->num_components>DEEP_MAX_COMPONENTS) {
+		goto done;
+	}
+
+	for(k=0; k<(UI)d->num_components; k++) {
+		switch(d->cmp_type[k]) {
+		case 1:
+			d->sample_map[k] = 0;
+			break;
+		case 2:
+			d->sample_map[k] = 1;
+			break;
+		case 3:
+			d->sample_map[k] = 2;
+			break;
+		case 4:
+			d->sample_map[k] = 3;
+			d->have_alpha = 1;
+			d->is_assoc_alpha = 0;
+			break;
+		case 17: // Undocumented type 17 used in Video Toaster "Brushes" files.
+			d->sample_map[k] = 3;
+			d->have_alpha = 1;
+			d->is_assoc_alpha = 1;
+			break;
+		default:
+			d->sample_map[k] = 4;
+		}
+
+		flags[d->sample_map[k]] = 1;
+	}
+
+	if(!flags[0] || !flags[1] || !flags[2]) { // Need R, G, B
+		goto done;
+	}
+	if(flags[4]) { // Found an unsupported sample type
+		de_warn(c, "Image type not fully supported");
+	}
+	retval = 1;
+
+done:
+	if(!retval) {
+		de_err(c, "Unsupported image type");
+	}
+	return retval;
+}
+
+static void deep_decode_image_unc(deark *c, struct deep_ctx *d,
+	dbuf *f, i64 pos1, i64 len)
+{
+	i64 i, j;
+	i64 pos = pos1;
+	UI k;
+	UI dst_samples_per_pixel;
+	de_bitmap *img = NULL;
+	de_finfo *fi = NULL;
+	u8 s[5] = {0,0,0,0,0};
+
+	fi = de_finfo_create(c);
+	if(d->x_aspect && d->y_aspect) {
+		fi->density.code = DE_DENSITY_UNK_UNITS;
+		fi->density.ydens = (double)d->x_aspect;
+		fi->density.xdens = (double)d->y_aspect;
+	}
+
+	dst_samples_per_pixel = d->have_alpha ? 4 : 3;
+	img = de_bitmap_create(c, d->width, d->height, dst_samples_per_pixel);
+
+	for(j=0; j<d->height; j++) {
+		for(i=0; i<d->width; i++) {
+			de_color clr;
+
+			if(pos > pos1+len) {
+				de_warn(c, "Premature end of image");
+				goto after_image;
+			}
+
+			for(k=0; k<(UI)d->num_components; k++) {
+				s[d->sample_map[k]] =  dbuf_getbyte_p(f, &pos);
+			}
+			if(d->have_alpha) {
+				clr = DE_MAKE_RGBA(s[0], s[1], s[2], s[3]);
+				if(d->is_assoc_alpha) {
+					clr = de_unpremultiply_alpha_clr(clr);
+				}
+			}
+			else {
+				clr = DE_MAKE_RGB(s[0], s[1], s[2]);
+			}
+			de_bitmap_setpixel_rgba(img, i, j, clr);
+		}
+	}
+
+after_image:
+	if(d->trans_setting==TRANS_AUTO) {
+		de_bitmap_optimize_alpha(img, (0x4|0x02));
+	}
+	else if(d->trans_setting==TRANS_REMOVE) {
+		de_bitmap_remove_alpha(img);
+	}
+	de_bitmap_write_to_file_finfo(img, fi, DE_CREATEFLAG_OPT_IMAGE);
+
+	de_bitmap_destroy(img);
+	de_finfo_destroy(c, fi);
+}
+
+static int decompress_deep_5(deark *c, struct deep_ctx *d, dbuf *inf,
+	i64 pos1, i64 len, dbuf *unc_pixels)
+{
+	i64 xpos = 0;
+	i64 ypos = 0;
+	i64 output_pos = 0;
+	i64 curr_component = 0;
+	u8 v = 0;
+	int retval = 0;
+	struct de_bitreader bitrd;
+
+	de_zeromem(&bitrd, sizeof(struct de_bitreader));
+	bitrd.f = inf;
+	bitrd.curpos = pos1;
+	bitrd.endpos = pos1+len;
+	bitrd.bbll.is_lsb = 0;
+	dbuf_disable_wbuffer(unc_pixels);
+
+	if(!d->found_TVDC) {
+		de_err(c, "Missing TVDC chunk");
+		goto done;
+	}
+	retval = 1;
+
+	while(1) {
+		UI n;
+		UI k;
+		UI count;
+
+		n = (UI)de_bitreader_getbits(&bitrd, 4);
+		if(bitrd.eof_flag) goto done;
+
+		if(d->tvdc_data[n*2+1]!=0 || d->tvdc_data[n*2]!=0) {
+			v += (u8)(d->tvdc_data[n*2+1]);
+			count = 1;
+		}
+		else {
+			n = (UI)de_bitreader_getbits(&bitrd, 4);
+			count = n+1;
+		}
+
+		for(k=0; k<count; k++) {
+			dbuf_writebyte_at(unc_pixels, output_pos, v);
+			output_pos += d->num_components;
+			xpos++;
+			if(xpos >= d->width) break;
+		}
+
+		if(xpos >= d->width) {
+			de_bitreader_skip_to_byte_boundary(&bitrd);
+			xpos = 0;
+			v = 0;
+			curr_component++;
+			if(curr_component >= d->num_components) {
+				ypos++;
+				if(ypos >= d->height) goto done;
+				curr_component = 0;
+			}
+			output_pos = ypos*(d->width * d->num_components) + curr_component;
+		}
+	}
+
+done:
+	return retval;
+}
+
+static void deep_decompress_and_decode_image(deark *c, struct deep_ctx *d,
+	dbuf *f, i64 pos1, i64 len)
+{
+	int ret;
+	dbuf *unc_pixels = NULL;
+
+	unc_pixels = dbuf_create_membuf(c, d->expected_image_size, 0x1);
+	if(d->cmpr_meth!=5) {
+		dbuf_enable_wbuffer(unc_pixels);
+	}
+	if(d->cmpr_meth==5) {
+		ret = decompress_deep_5(c, d, f, pos1, len, unc_pixels);
+	}
+	else {
+		ret = decompress_ilbm_packbits(c, f, pos1, len, unc_pixels, d->expected_image_size,
+			(UI)d->num_components);
+	}
+	dbuf_flush(unc_pixels);
+	if(!ret) goto done;
+
+	deep_decode_image_unc(c, d, unc_pixels, 0, unc_pixels->len);
+done:
+	dbuf_close(unc_pixels);
+}
+
+static void do_deep_DBOD(deark *c, struct deep_ctx *d, struct de_iffctx *ictx)
+{
+	if(d->cmpr_meth!=0 && d->cmpr_meth!=1 && d->cmpr_meth!=5) {
+		de_err(c, "Unsupported compression type: %u", d->cmpr_meth);
+		goto done;
+	}
+
+	if(!deep_make_sample_map(c, d)) goto done;
+
+	if(!de_good_image_dimensions(c, d->width, d->height)) {
+		goto done;
+	}
+
+	d->expected_image_size = d->width * d->height * d->num_components;
+
+	if(d->cmpr_meth==0) {
+		deep_decode_image_unc(c, d, ictx->f, ictx->chunkctx->dpos,
+			ictx->chunkctx->dlen);
+	}
+	else {
+		deep_decompress_and_decode_image(c, d, ictx->f, ictx->chunkctx->dpos,
+			ictx->chunkctx->dlen);
+	}
+
+done:
+	;
+}
+
+static void convert_image_rgba(dbuf *f, i64 fpos,
+	de_bitmap *img)
+{
+	i64 i, j;
+	de_color clr;
+	i64 pos;
+	u8 buf[4];
+
+	pos = fpos;
+	for(j=0; j<img->height; j++) {
+		for(i=0; i<img->width; i++) {
+			dbuf_read(f, buf, pos, 4);
+			pos += 4;
+			clr = DE_MAKE_RGBA(buf[0], buf[1], buf[2], buf[3]);
+			de_bitmap_setpixel_rgba(img, i, j, clr);
+		}
+	}
+}
+
+static void do_deep_mpic(deark *c, struct deep_ctx *d, dbuf *f, i64 pos1, i64 len)
+{
+	i64 w, h, rowspan;
+	i64 pos = pos1;
+	UI n;
+	u8 ok = 0;
+	de_bitmap *img = NULL;
+
+	w = de_getu16be_p(&pos);
+	rowspan = w*4;
+	h = de_getu16be_p(&pos);
+	de_dbg(c, "32-bit thumbnail image, dimensions: %u"DE_CHAR_TIMES"%u", (UI)w, (UI)h);
+
+	// I have no documentation for MPIC.
+	// I'm guessing that it's always RGBA or RGBx, and doesn't depend on DPEL.
+	// I have some images where DBOD has 3 components, and MPIC has 4 components.
+	// So I'm guessing that MPIC is *not* defined to be "the same format as DBOD".
+
+	// Not sure what these fields are. First one might be sample count.
+	n = (UI)de_getu32be_p(&pos);
+	if(n != 0x00040002) goto done;
+
+	if(pos + rowspan*h > pos1+len) goto done;
+
+	img = de_bitmap_create(c, w, h, 4);
+	convert_image_rgba(f, pos, img);
+
+	// If the main image doesn't have an alpha channel, we assume the MPIC
+	// thumbnail doesn't have transparency.
+	if(d->trans_setting==TRANS_REMOVE || (d->found_DPEL && d->have_alpha_hint==0)) {
+		de_bitmap_remove_alpha(img);
+	}
+	else if(d->trans_setting==TRANS_AUTO) {
+		de_bitmap_optimize_alpha(img, 0x1);
+	}
+
+	de_bitmap_write_to_file(img, "thumb", DE_CREATEFLAG_IS_AUX|DE_CREATEFLAG_OPT_IMAGE);
+	ok = 1;
+
+done:
+	if(!ok) {
+		de_warn(c, "Unsupported MPIC thumbnail");
+	}
+	de_bitmap_destroy(img);
+}
+
+
+static int my_deep_chunk_handler(struct de_iffctx *ictx)
+{
+	deark *c = ictx->c;
+	struct deep_ctx *d = (struct deep_ctx*)ictx->userdata;
+
+	switch(ictx->chunkctx->chunk4cc.id) {
+	case CODE_FORM:
+		ictx->is_std_container = 1;
+		goto done;
+	}
+
+	if(ictx->level != 1) goto done;
+
+	switch(ictx->chunkctx->chunk4cc.id) {
+	case CODE_DGBL:
+		do_deep_DGBL(c, d, ictx);
+		ictx->handled = 1;
+		break;
+	case CODE_DLOC:
+		do_deep_DLOC(c, d, ictx);
+		ictx->handled = 1;
+		break;
+	case CODE_DPEL:
+		do_deep_DPEL(c, d, ictx);
+		ictx->handled = 1;
+		break;
+	case CODE_DBOD:
+		do_deep_DBOD(c, d, ictx);
+		d->DBOD_count++;
+		ictx->handled = 1;
+		break;
+	case CODE_TVDC:
+		d->found_TVDC = 1;
+		dbuf_read(ictx->f, d->tvdc_data, ictx->chunkctx->dpos, sizeof(d->tvdc_data));
+		// Don't set ictx->handled, since we don't emit dbg info.
+		break;
+	case CODE_MPIC:
+		do_deep_mpic(c, d, ictx->f, ictx->chunkctx->dpos, ictx->chunkctx->dlen);
+		ictx->handled = 1;
+		break;
+	case CODE_XS24:
+		do_xs24(c, ictx->f, ictx->chunkctx->dpos, ictx->chunkctx->dlen);
+		ictx->handled = 1;
+		break;
+	}
+
+done:
+	return 1;
+}
+
+static void de_run_deep(deark *c, de_module_params *mparams)
+{
+	struct deep_ctx *d = NULL;
+	struct de_iffctx *ictx = NULL;
+	const char *opt_trans_str;
+
+	d = de_malloc(c, sizeof(struct deep_ctx));
+
+	d->trans_setting = TRANS_AUTO; // default
+	opt_trans_str = de_get_ext_option(c, "deep:trans");
+	if(opt_trans_str) {
+		if(!de_strcmp(opt_trans_str, "auto")) {
+			d->trans_setting = TRANS_AUTO;
+		}
+		else if(!de_strcmp(opt_trans_str, "")) {
+			d->trans_setting = TRANS_RESPECT;
+		}
+		else {
+			int opt_trans_n;
+
+			opt_trans_n = de_atoi(opt_trans_str);
+			if(opt_trans_n==0) {
+				d->trans_setting = TRANS_REMOVE;
+			}
+			else if(opt_trans_n>0) {
+				d->trans_setting = TRANS_RESPECT;
+			}
+		}
+	}
+
+	de_declare_fmt(c, "IFF-DEEP");
+
+	ictx = fmtutil_create_iff_decoder(c);
+	ictx->has_standard_iff_chunks = 1;
+	ictx->alignment = 2;
+	ictx->userdata = (void*)d;
+	ictx->handle_chunk_fn = my_deep_chunk_handler;
+	ictx->f = c->infile;
+
+	fmtutil_read_iff_format(ictx, 0, c->infile->len);
+	de_dbg(c, "image count: %d", d->DBOD_count);
+
+	fmtutil_destroy_iff_decoder(ictx);
+	if(d) {
+		de_free(c, d);
+	}
+}
+
+static int de_identify_deep(deark *c)
+{
+	UI n;
+
+	if((UI)de_getu32be(0)!=CODE_FORM) return 0;
+	n = (UI)de_getu32be(8);
+	if(n==CODE_DEEP || n==CODE_TVPP) return 100;
+	return 0;
+}
+
+static void de_help_deep(deark *c)
+{
+	de_msg(c, "-opt deep:trans=<0|1|auto> : Always remove (0) or respect (1) "
+		"transparency");
+}
+
+void de_module_deep(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "deep";
+	mi->desc = "IFF-DEEP";
+	mi->run_fn = de_run_deep;
+	mi->identify_fn = de_identify_deep;
+	mi->help_fn = de_help_deep;
+}
