@@ -10,6 +10,9 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_jpeg);
 DE_DECLARE_MODULE(de_module_jpegscan);
+DE_DECLARE_MODULE(de_module_syberia_syj);
+DE_DECLARE_MODULE(de_module_esm_pix);
+DE_DECLARE_MODULE(de_module_pegasus_pic2);
 
 struct fpxr_entity_struct {
 	size_t index;
@@ -25,13 +28,15 @@ struct fpxr_data_struct {
 	struct fpxr_entity_struct *entities;
 };
 
-typedef struct localctx_struct {
+typedef struct localctx_JPEG {
 	u8 is_jpegls;
+	u8 is_pegasus_pic2;
 
 	u8 has_jfif_seg, has_jfif_thumb, has_jfxx_seg;
 	u8 has_exif_seg, has_exif_gps, has_spiff_seg, has_mpf_seg, has_afcp;
 	u8 exif_before_jfif;
 	u8 has_psd, has_iptc, has_xmp, has_xmp_ext, has_iccprofile, has_flashpix;
+	u8 has_jumbf;
 	u8 is_baseline, is_progressive, is_lossless, is_arithmetic, is_hierarchical;
 	u8 is_jpeghdr, is_jpegxt, is_mpo, is_jps;
 	u8 has_restart_markers;
@@ -72,6 +77,7 @@ typedef void (*handler_fn_type)(deark *c, lctx *d,
 
 #define FLAG_JPEG_COMPAT   0x0001
 #define FLAG_JPEGLS_COMPAT 0x0002
+#define FLAG_PIC2_ONLY     0x0004
 #define FLAG_NO_DATA       0x0100
 #define FLAG_IS_SOF        0x0200
 
@@ -161,17 +167,44 @@ static void do_jpeghdr_segment(deark *c, lctx *d, i64 pos1,
 	dbuf_copy(c->infile, pos, data_size, d->hdr_residual_file);
 }
 
-static void do_jpegxt_segment(deark *c, lctx *d, i64 pos,
+static void do_boxformat_segment(deark *c, lctx *d, i64 pos,
 	i64 data_size)
 {
 	i64 n;
+	i64 seq;
+	i64 bdata_pos;
+	i64 bdata_avail;
+	i64 blen;
+
 	if(data_size<14) return;
 	n = de_getu16be(pos);
 	de_dbg(c, "enumerator: %u", (unsigned int)n);
-	n = de_getu32be(pos+2);
-	de_dbg(c, "seq number: %u", (unsigned int)n);
+	seq = de_getu32be(pos+2);
+	de_dbg(c, "seq number: %u", (UI)seq);
+
+	// TODO: JUMBF / JPEG XT parsing needs a lot of work.
+	// We'll try to parse it if seems like there's only one segment, but
+	// we're probably not even doing that correctly.
+	bdata_pos = pos+6;
+	bdata_avail = data_size-6;
+	if(seq > 1) return;
+	blen = de_getu32be(bdata_pos);
+
+	if(bdata_avail>=12) {
+		if(!dbuf_memcmp(c->infile, bdata_pos+4, (const void*)"jumb", 4)) {
+			de_dbg(c, "JUMBF detected");
+			d->has_jumbf = 1;
+		}
+		else if(!dbuf_memcmp(c->infile, bdata_pos+4, (const void*)"ftypjpxt", 8)) {
+			d->is_jpegxt = 1;
+			de_dbg(c, "JPEG XT detected");
+		}
+	}
+
+	if(blen > bdata_avail) return;
+
 	de_dbg_indent(c, 1);
-	de_run_module_by_id_on_slice2(c, "bmff", "T", c->infile, pos+6, data_size-6);
+	de_run_module_by_id_on_slice2(c, "bmff", "T", c->infile, bdata_pos, bdata_avail);
 	de_dbg_indent(c, -1);
 }
 
@@ -212,38 +245,46 @@ static void extract_unc_jfif_thumbnail(deark *c, lctx *d,
 }
 
 static void do_jfif_segment(deark *c, lctx *d,
-	i64 pos, i64 data_size)
+	i64 pos1, i64 data_size)
 {
 	u8 units;
 	const char *units_name;
 	i64 xdens, ydens;
+	i64 tn_size;
 	i64 tn_w, tn_h;
+	i64 pos;
 
 	d->has_jfif_seg = 1;
-	if(data_size<9) return;
-	d->jfif_ver_h = de_getbyte(pos);
-	d->jfif_ver_l = de_getbyte(pos+1);
+	if(data_size<9) goto done;
+	pos = pos1;
+	d->jfif_ver_h = de_getbyte_p(&pos);
+	d->jfif_ver_l = de_getbyte_p(&pos);
 	de_dbg(c, "JFIF version: %d.%02d", (int)d->jfif_ver_h, (int)d->jfif_ver_l);
-	units = de_getbyte(pos+2);
-	xdens = de_getu16be(pos+3);
-	ydens = de_getu16be(pos+5);
+	units = de_getbyte_p(&pos);
+	xdens = de_getu16be_p(&pos);
+	ydens = de_getu16be_p(&pos);
 	if(units==1) units_name="dpi";
 	else if(units==2) units_name="dots/cm";
 	else units_name="(unspecified)";
 	de_dbg(c, "density: %d"DE_CHAR_TIMES"%d, units=%s", (int)xdens, (int)ydens, units_name);
 
-	tn_w = (i64)de_getbyte(pos+7);
-	tn_h = (i64)de_getbyte(pos+8);
+	tn_w = (i64)de_getbyte_p(&pos);
+	tn_h = (i64)de_getbyte_p(&pos);
 	de_dbg(c, "thumbnail dimensions: %d"DE_CHAR_TIMES"%d", (int)tn_w, (int)tn_h);
-	if(tn_w>0 && tn_h>0 && data_size>9) {
+	tn_size = pos1 + data_size - pos;
+	if(tn_w>0 && tn_h>0 && tn_size>0) {
+		de_dbg(c, "thunbnail at %"I64_FMT", len=%"I64_FMT, pos, tn_size);
 		d->has_jfif_thumb = 1;
-		if(tn_w*tn_h*3 != data_size-9) {
-			de_warn(c, "Expected %d bytes of JFIF thumbnail image data at %d, found %d",
-				(int)(tn_w*tn_h*3), (int)(pos+9), (int)(data_size-9));
+		if(tn_w*tn_h*3 != tn_size) {
+			de_warn(c, "Expected %"I64_FMT" bytes of JFIF thumbnail image data, "
+				"found %"I64_FMT, (i64)(tn_w*tn_h*3), tn_size);
 		}
-		extract_unc_jfif_thumbnail(c, d, pos+9, data_size-9, tn_w, tn_h,
+		extract_unc_jfif_thumbnail(c, d, pos, tn_size, tn_w, tn_h,
 			0, "jfifthumb");
 	}
+
+done:
+	;
 }
 
 static void do_jfxx_segment(deark *c, lctx *d,
@@ -880,7 +921,7 @@ static void normalize_app_id(const char *app_id_orig, char *app_id_normalized,
 #define APPSEGTYPE_DUCKY          12
 #define APPSEGTYPE_XMP            14
 #define APPSEGTYPE_XMP_EXTENSION  15
-#define APPSEGTYPE_JPEGXT         20
+#define APPSEGTYPE_BOXFORMAT      20
 #define APPSEGTYPE_MPF            21
 #define APPSEGTYPE_JPS            22
 #define APPSEGTYPE_HDR_RI_VER     24
@@ -889,6 +930,7 @@ static void normalize_app_id(const char *app_id_orig, char *app_id_normalized,
 #define APPSEGTYPE_AROT           27
 #define APPSEGTYPE_MSRGBA         100
 #define APPSEGTYPE_RBSWAP         101
+#define APPSEGTYPE_PEGASUSPIC     110
 
 struct app_id_info_struct {
 	int app_id_found;
@@ -1034,7 +1076,8 @@ static void detect_app_seg_type(deark *c, lctx *d, const struct marker_info *mi,
 		app_id_info->appsegtype = APPSEGTYPE_ICC_PROFILE;
 		app_id_info->app_type_name = "ICC profile";
 	}
-	else if(seg_type==0xe2 && !de_strcmp(ad.app_id_normalized, "FPXR")) {
+	else if((seg_type==0xe2 || seg_type==0xe4) && !de_strcmp(ad.app_id_normalized, "FPXR")) {
+		// Normally APP2. APP4 exists, but I don't know why.
 		app_id_info->appsegtype = APPSEGTYPE_FPXR;
 		app_id_info->app_type_name = "Exif Flashpix Ready";
 	}
@@ -1069,8 +1112,9 @@ static void detect_app_seg_type(deark *c, lctx *d, const struct marker_info *mi,
 		app_id_info->app_type_name = "JPEG-HDR Ext";
 	}
 	else if(seg_type==0xeb && ad.nraw_bytes>=2 && !de_strncmp((const char*)ad.raw_bytes, "JP", 2)) {
-		app_id_info->appsegtype = APPSEGTYPE_JPEGXT;
-		app_id_info->app_type_name = "JPEG XT";
+		// Presumably the format defined in ISO/IEC 18477-3
+		app_id_info->appsegtype = APPSEGTYPE_BOXFORMAT;
+		app_id_info->app_type_name = "Box format container";
 		sig_size = 2;
 	}
 	else if(seg_type==0xe2 && !de_strcmp(ad.app_id_normalized, "MPF")) {
@@ -1097,6 +1141,10 @@ static void detect_app_seg_type(deark *c, lctx *d, const struct marker_info *mi,
 		app_id_info->appsegtype = APPSEGTYPE_RBSWAP;
 		app_id_info->app_type_name = "Flag for swapped red/blue";
 	}
+	else if(seg_type==0xe1 && d->is_pegasus_pic2 && !de_strcmp(ad.app_id_orig, "PIC")) {
+		app_id_info->appsegtype = APPSEGTYPE_PEGASUSPIC;
+		app_id_info->app_type_name = "Pegasus PIC params";
+	}
 
 done:
 	app_id_info->payload_pos = seg_data_pos + sig_size;
@@ -1175,9 +1223,8 @@ static void handler_app(deark *c, lctx *d,
 	case APPSEGTYPE_HDR_RI_EXT:
 		do_jpeghdr_segment(c, d, seg_data_pos, seg_data_size, 1);
 		break;
-	case APPSEGTYPE_JPEGXT:
-		d->is_jpegxt = 1;
-		do_jpegxt_segment(c, d, payload_pos, payload_size);
+	case APPSEGTYPE_BOXFORMAT:
+		do_boxformat_segment(c, d, payload_pos, payload_size);
 		break;
 	case APPSEGTYPE_MPF:
 		do_mpf_segment(c, d, payload_pos, payload_size);
@@ -1227,7 +1274,7 @@ static void declare_jpeg_fmt(deark *c, lctx *d, u8 seg_type)
 	if(d->is_jpegls) { name = "JPEG-LS"; }
 	else if(d->is_mpo) { name = "JPEG/MPO"; }
 	else if(d->is_jps) { name = "JPEG/JPS"; }
-	else if(d->is_jpegxt) { name = "JPEG/JPEG_XT"; }
+	else if(d->is_jps) { name = "JPEG XT"; }
 	else if(d->is_jpeghdr) { name = "JPEG-HDR"; }
 	else if(d->is_lossless) { name = "JPEG/lossless"; }
 	else if(d->has_jfif_seg && d->has_exif_seg) { name = "JPEG/JFIF+Exif"; }
@@ -1440,11 +1487,12 @@ static void handler_dac(deark *c, lctx *d,
 	}
 }
 
-static void dump_qtable_data(deark *c, lctx *d, i64 pos, u8 precision_code)
+static void dump_qtable_data(deark *c, lctx *d, i64 pos1, u8 precision_code)
 {
-	u8 qbuf[64];
+	UI qbuf[64];
 	i64 k;
 	de_ucstring *s = NULL;
+	const char *cfmt;
 	static const u8 zigzag[64] = {
 		 0, 1, 5, 6,14,15,27,28,
 		 2, 4, 7,13,16,26,29,42,
@@ -1457,13 +1505,29 @@ static void dump_qtable_data(deark *c, lctx *d, i64 pos, u8 precision_code)
 	};
 
 	if(c->debug_level<2) return;
-	if(precision_code!=0) return;
+	if(precision_code>1) return;
+
+	if(precision_code==1) {
+		i64 pos = pos1;
+
+		cfmt = " %5u";
+		for(k=0; k<64; k++) {
+			qbuf[k] = (UI)de_getu16be_p(&pos);
+		}
+	}
+	else {
+		u8 qbuf8[64];
+
+		cfmt = " %3u";
+		de_read(qbuf8, pos1, 64);
+		for(k=0; k<64; k++) {
+			qbuf[k] = (UI)qbuf8[k];
+		}
+	}
 
-	de_read(qbuf, pos, 64);
 	s = ucstring_create(c);
 	for(k=0; k<64; k++) {
-		ucstring_printf(s, DE_ENCODING_LATIN1, " %3u",
-			(unsigned int)qbuf[(unsigned int)zigzag[k]]);
+		ucstring_printf(s, DE_ENCODING_LATIN1, cfmt, qbuf[(UI)zigzag[k]]);
 		if(k%8==7) { // end of a debug line
 			de_dbg(c, "data:%s", ucstring_getpsz(s));
 			ucstring_empty(s);
@@ -1616,6 +1680,7 @@ done:
 
 static const struct marker_info1 marker_info1_arr[] = {
 	{0x01, 0x0101, "TEM", NULL, NULL},
+	{0xb9, 0x0205, "SOFPIC2", NULL, NULL},
 	{0xc4, 0x0001, "DHT", "Define Huffman table", handler_dht},
 	{0xc8, 0x0201, "JPG", NULL, handler_sof},
 	{0xcc, 0x0001, "DAC", "Define arithmetic coding conditioning", handler_dac},
@@ -1648,6 +1713,7 @@ static int get_marker_info(deark *c, lctx *d, u8 seg_type,
 
 		if(!d->is_jpegls && !(mi1->flags&FLAG_JPEG_COMPAT)) continue;
 		if(d->is_jpegls && !(mi1->flags&FLAG_JPEGLS_COMPAT)) continue;
+		if(!d->is_pegasus_pic2 && (mi1->flags&FLAG_PIC2_ONLY)) continue;
 
 		if(mi1->seg_type == seg_type) {
 			mi->flags = mi1->flags;
@@ -1787,6 +1853,7 @@ static void print_summary(deark *c, lctx *d)
 	de_ucstring *summary = NULL;
 
 	if(d->is_jpegls) goto done;
+	if(d->is_pegasus_pic2) goto done;
 	if(!d->found_sof) goto done;
 	if(!d->found_soi) goto done;
 
@@ -1830,6 +1897,7 @@ static void print_summary(deark *c, lctx *d)
 	if(d->has_flashpix) ucstring_append_sz(summary, " FlashPix", DE_ENCODING_LATIN1);
 	if(d->is_jpeghdr) ucstring_append_sz(summary, " HDR", DE_ENCODING_LATIN1);
 	if(d->is_jpegxt) ucstring_append_sz(summary, " XT", DE_ENCODING_LATIN1);
+	if(d->has_jumbf && !d->is_jpegxt) ucstring_append_sz(summary, " JUMBF", DE_ENCODING_LATIN1);
 	if(d->has_mpf_seg) ucstring_append_sz(summary, " MPO", DE_ENCODING_LATIN1);
 	if(d->is_jps) ucstring_append_sz(summary, " JPS", DE_ENCODING_LATIN1);
 	if(d->has_iccprofile) ucstring_append_sz(summary, " ICC", DE_ENCODING_LATIN1);
@@ -2018,6 +2086,8 @@ static void de_run_jpeg(deark *c, de_module_params *mparams)
 	lctx *d = NULL;
 
 	d = de_malloc(c, sizeof(lctx));
+	d->is_pegasus_pic2 = (u8)de_havemodcode(c, mparams, '2');
+
 	pos = 0;
 	bytes_consumed = 0;
 	retval_stream = do_jpeg_stream(c, d, pos, &bytes_consumed);
@@ -2215,3 +2285,180 @@ void de_module_jpegscan(deark *c, struct deark_module_info *mi)
 	mi->desc = "Extract embedded JPEG images from arbitrary files";
 	mi->run_fn = de_run_jpegscan;
 }
+
+// **************************************************************************
+// Syberia texture (.syj)
+// **************************************************************************
+
+static void de_run_syberia_syj(deark *c, de_module_params *mparams)
+{
+	dbuf *outf = NULL;
+
+	outf = dbuf_create_output_file(c, "jpg", NULL, 0);
+	dbuf_write(outf, (const u8*)"\xff\xd8\xff\xe0\x00\x10\x4a\x46\x49\x46", 10);
+	dbuf_copy(c->infile, 0, c->infile->len, outf);
+	dbuf_close(outf);
+}
+
+static int de_identify_syberia_syj(deark *c)
+{
+	if(de_getu16be(0) != 0x0001) return 0;
+	if(!de_input_file_has_ext(c, "syj")) return 0;
+	if(dbuf_memcmp(c->infile, 8, (const void*)"\0\0\xff", 3)) {
+		return 0;
+	}
+	if(de_getu16be(c->infile->len-2) != 0xffd9) return 0;
+	return 80;
+}
+
+void de_module_syberia_syj(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "syberia_syj";
+	mi->desc = "Syberia texture";
+	mi->run_fn = de_run_syberia_syj;
+	mi->identify_fn = de_identify_syberia_syj;
+}
+
+// **************************************************************************
+// ESM Software .pix
+// **************************************************************************
+
+static void de_run_esm_pix(deark *c, de_module_params *mparams)
+{
+	dbuf *outf = NULL;
+
+	outf = dbuf_create_output_file(c, "jpg", NULL, 0);
+	dbuf_copy(c->infile, 21, c->infile->len - 21, outf);
+	dbuf_close(outf);
+}
+
+static int de_identify_esm_pix(deark *c)
+{
+	if(dbuf_memcmp(c->infile, 0, (const void*)"Esm ", 4)) {
+		return 0;
+	}
+	if(dbuf_memcmp(c->infile, 13, (const void*)"PIX ", 4)) {
+		return 0;
+	}
+	if((UI)de_getu16be(21) != 0xffd8) return 0;
+	return 100;
+}
+
+void de_module_esm_pix(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "esm_pix";
+	mi->desc = "Esm Software PIX";
+	mi->run_fn = de_run_esm_pix;
+	mi->identify_fn = de_identify_esm_pix;
+}
+
+// **************************************************************************
+// Pegasus PIC2
+// **************************************************************************
+
+struct pic2_ctx {
+	u8 is_jpg;
+};
+
+static void pic2_image_segment(deark *c, struct pic2_ctx *d, i64 pos, i64 len)
+{
+	if(!d->is_jpg) goto done;
+	de_dbg(c, "JPEG-like data at %"I64_FMT, pos);
+	de_dbg_indent(c, 1);
+	de_run_module_by_id_on_slice2(c, "jpeg", "2", c->infile, pos, len);
+	de_dbg_indent(c, -1);
+done:
+	;
+}
+
+static const char *get_pic2_segname(u8 t)
+{
+	const char *name = NULL;
+
+	switch(t) {
+	case 0: name = "eof"; break;
+	case 5: name = "cmpr type?"; break;
+	case 6: name = "image data"; break;
+	case 22: name = "comment?"; break;
+	}
+	return name?name:"?";
+}
+
+static void pic2_main(deark *c, struct pic2_ctx *d)
+{
+	i64 pos = 4;
+	int saved_indent_level;
+
+	de_dbg_indent_save(c, &saved_indent_level);
+
+	while(1) {
+		i64 segpos;
+		i64 segdlen;
+		i64 segdpos;
+		u8 segtype;
+
+		if(pos >= c->infile->len-4) goto done;
+		segpos = pos;
+		de_dbg(c, "segment at %"I64_FMT, segpos);
+		de_dbg_indent(c, 1);
+		segtype = de_getbyte_p(&pos);
+		de_dbg(c, "type: %u (%s)", (UI)segtype, get_pic2_segname(segtype));
+		if(segtype==0) goto done;
+		segdlen = de_getu32le_p(&pos);
+		segdpos = pos;
+		de_dbg(c, "dlen: %"I64_FMT, segdlen);
+		if(segdpos+segdlen>c->infile->len) goto done;
+
+		if(segtype==5) {
+			// Wild guess. If this segment starts with "JPG", we'll assume the
+			// compressed data is in a JPEG-like format.
+			if(!dbuf_memcmp(c->infile, segdpos, (const void*)"JPG", 3)) {
+				d->is_jpg = 1;
+			}
+		}
+
+		if(segtype==6) {
+			pic2_image_segment(c, d, segdpos, segdlen);
+		}
+		else {
+			de_dbg_hexdump(c, c->infile, segdpos, segdlen, 256, NULL, 0x1);
+		}
+		pos += segdlen;
+		de_dbg_indent(c, -1);
+	}
+done:
+	;
+}
+
+static void de_run_pegasus_pic2(deark *c, de_module_params *mparams)
+{
+	struct pic2_ctx *d = NULL;
+
+	d = de_malloc(c, sizeof(struct pic2_ctx));
+	de_declare_fmt(c, "Pegasus PIC2");
+	pic2_main(c, d);
+	de_free(c, d);
+}
+
+static int de_identify_pegasus_pic2(deark *c)
+{
+	UI n;
+	u8 b;
+
+	n = (UI)de_getu32be(0);
+#define CODE_PIC2 0x50494332U
+	if(n!=CODE_PIC2) return 0;
+	b = de_getbyte(4); // segment type, expecting 1
+	if(b!=1) return 0;
+	n = (UI)de_getu32le(5); // segment len, expecting 8
+	if(n<4 || n>32) return 0;
+	return 87;
+}
+
+void de_module_pegasus_pic2(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "pegasus_pic2";
+	mi->desc = "Pegasus PIC2";
+	mi->run_fn = de_run_pegasus_pic2;
+	mi->identify_fn = de_identify_pegasus_pic2;
+}
