@@ -9,6 +9,8 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_lha);
 DE_DECLARE_MODULE(de_module_swg);
+DE_DECLARE_MODULE(de_module_tpk);
+DE_DECLARE_MODULE(de_module_pakleo);
 DE_DECLARE_MODULE(de_module_car_lha);
 DE_DECLARE_MODULE(de_module_arx);
 DE_DECLARE_MODULE(de_module_ar001);
@@ -18,6 +20,8 @@ DE_DECLARE_MODULE(de_module_lharc_sfx_com);
 
 #define CODE_S_LH0 0x204c4830 // SAR
 #define CODE_S_LH5 0x204c4835 // SAR
+#define CODE_TK0 0x2d544b30U // TPK
+#define CODE_TK1 0x2d544b31U // TPK
 #define CODE_ah0 0x2d616830U // MAR
 #define CODE_ari 0x2d617269U // MAR
 #define CODE_hf0 0x2d686630U // MAR
@@ -40,6 +44,8 @@ DE_DECLARE_MODULE(de_module_lharc_sfx_com);
 #define CODE_lhd 0x2d6c6864U
 #define CODE_lhe 0x2d6c6865U
 #define CODE_lhx 0x2d6c6878U
+#define CODE_ll0 0x2d6c6c30U
+#define CODE_ll1 0x2d6c6c31U
 #define CODE_lx1 0x2d6c7831U
 #define CODE_lz2 0x2d6c7a32U
 #define CODE_lz3 0x2d6c7a33U
@@ -48,13 +54,19 @@ DE_DECLARE_MODULE(de_module_lharc_sfx_com);
 #define CODE_lz7 0x2d6c7a37U
 #define CODE_lz8 0x2d6c7a38U
 #define CODE_lzs 0x2d6c7a73U
-#define CODE_pc1 0x2d706331U
 #define CODE_pm0 0x2d706d30U
 #define CODE_pm1 0x2d706d31U
 #define CODE_pm2 0x2d706d32U
 #define CODE_sw0 0x2d737730U
 #define CODE_sw1 0x2d737731U
 
+enum lha_basefmt_enum {
+	BASEFMT_LHA = 0,  // LHarc/LHA and other formats that are parsed the same
+	BASEFMT_SWG,
+	BASEFMT_TPK,
+	BASEFMT_PAKLEO
+};
+
 #define TIMESTAMPIDX_INVALID (-1)
 struct timestamp_data {
 	struct de_timestamp ts; // The best timestamp of this type found so far
@@ -74,14 +86,15 @@ struct member_data {
 	u8 is_nonexecutable;
 	u8 is_executable;
 	i64 orig_size;
-	UI hdr_checksum_calc;
+	u32 hdr_checksum_calc;
 
 	u8 have_hdr_crc_reported;
 	u32 hdr_crc_reported;
 	u32 hdr_crc_calc;
 	i64 hdr_crc_field_pos;
 
-	u32 crc16;
+	u8 have_crc_reported;
+	u32 crc_reported;
 	u8 os_id;
 	i64 compressed_data_pos; // relative to beginning of file
 	i64 compressed_data_len;
@@ -95,14 +108,16 @@ typedef struct localctx_struct {
 	de_encoding input_encoding;
 	int lhark_policy; // -1=detect, 0=no, 1=yes
 	int lhark_req;
+	enum lha_basefmt_enum basefmt;
+	const char *basefmt_name;
 	u8 hlev_of_first_member;
-	u8 swg_fmt;
 	u8 lh7_success_flag; // currently unused
 	u8 lh7_failed_flag; // currently unused
 	u8 trailer_found;
 	int member_count;
 	i64 trailer_pos;
 	struct de_crcobj *crco;
+	struct de_crcobj *crco_cksum;
 } lctx;
 
 typedef void (*decompressor_fn)(deark *c, lctx *d, struct member_data *md,
@@ -314,6 +329,51 @@ static void read_filename_hlev1_or_exthdr(deark *c, lctx *d, struct member_data
 	}
 }
 
+// Convert backslashes to slashes, remove trailing backslash.
+static void fixup_tpk_path(de_ucstring *s)
+{
+	i64 i;
+
+	for(i=0; i<s->len; i++) {
+		if(s->str[i]=='\\') {
+			s->str[i] = '/';
+		}
+	}
+
+	if(s->len>0 && s->str[s->len-1]=='/') {
+		ucstring_truncate(s, s->len-1);
+	}
+}
+
+static void read_filename_tpk(deark *c, lctx *d, struct member_data *md,
+	i64 pos1, i64 len1)
+{
+	i64 pos = pos1;
+	i64 len = len1;
+	struct de_stringreaderdata *fn_srd = NULL;
+
+	if(!md->filename) {
+		md->filename = ucstring_create(c);
+	}
+	if(!md->dirname) {
+		md->dirname = ucstring_create(c);
+	}
+
+	fn_srd = dbuf_read_string(c->infile, pos, len, 256, DE_CONVFLAG_STOP_AT_NUL,
+		md->encoding);
+	de_dbg(c, "filename: \"%s\"", ucstring_getpsz_d(fn_srd->str));
+	ucstring_append_ucstring(md->filename, fn_srd->str);
+	pos += fn_srd->bytes_consumed;
+	len -= fn_srd->bytes_consumed;
+
+	dbuf_read_to_ucstring(c->infile, pos, len, md->dirname, DE_CONVFLAG_STOP_AT_NUL,
+		md->encoding);
+	de_dbg(c, "dir name: \"%s\"", ucstring_getpsz_d(md->dirname));
+	fixup_tpk_path(md->dirname);
+
+	de_destroy_stringreaderdata(c, fn_srd);
+}
+
 static void exthdr_common(deark *c, lctx *d, struct member_data *md,
 	u8 id, const struct exthdr_type_info_struct *e,
 	i64 pos, i64 dlen)
@@ -322,7 +382,7 @@ static void exthdr_common(deark *c, lctx *d, struct member_data *md,
 	md->hdr_crc_reported = (u32)de_getu16le(pos);
 	md->have_hdr_crc_reported = 1;
 	md->hdr_crc_field_pos = pos;
-	de_dbg(c, "header crc (reported): 0x%04x", (unsigned int)md->hdr_crc_reported);
+	de_dbg(c, "header crc (reported): 0x%04x", (UI)md->hdr_crc_reported);
 	// TODO: Additional information
 }
 
@@ -391,7 +451,7 @@ static void exthdr_windowstimestamp(deark *c, lctx *d, struct member_data *md,
 	read_windows_FILETIME(c, d, md, pos+16, DE_TIMESTAMPIDX_ACCESS, "access time");
 }
 
-static void interpret_unix_perms(deark *c, lctx *d, struct member_data *md, unsigned int mode)
+static void interpret_unix_perms(deark *c, lctx *d, struct member_data *md, UI mode)
 {
 	if(mode & 0100000) { // regular file
 		if(mode & 0111) { // executable
@@ -411,10 +471,10 @@ static void exthdr_unixperms(deark *c, lctx *d, struct member_data *md,
 	u8 id, const struct exthdr_type_info_struct *e,
 	i64 pos, i64 dlen)
 {
-	unsigned int mode;
+	UI mode;
 
 	if(dlen<2) return;
-	mode = (unsigned int)de_getu16le(pos);
+	mode = (UI)de_getu16le(pos);
 	de_dbg(c, "mode: octal(%06o)", mode);
 	interpret_unix_perms(c, d, md, mode);
 }
@@ -499,6 +559,15 @@ static const struct exthdr_type_info_struct exthdr_type_info_arr[] = {
 	{ 0xff, 0, "level 3 new attribs type-2", exthdr_lev3newattribs2 }
 };
 
+static struct member_data *create_member_data(deark *c)
+{
+	struct member_data *md;
+
+	md = de_malloc(c, sizeof(struct member_data));
+	md->have_crc_reported = 1; // default
+	return md;
+}
+
 static void destroy_member_data(deark *c, struct member_data *md)
 {
 	if(!md) return;
@@ -534,21 +603,21 @@ static void do_read_ext_header(deark *c, lctx *d, struct member_data *md,
 	}
 	name = e ? e->name : "?";
 
-	de_dbg(c, "ext header at %d, len=%d (1+%d+%d), id=0x%02x (%s)", (int)pos1, (int)len,
-		(int)(dlen-1), (int)(len-dlen), (unsigned int)id, name);
+	de_dbg(c, "ext header at %"I64_FMT", len=%"I64_FMT" (1+%"I64_FMT"+%"I64_FMT"), id=0x%02x (%s)",
+		pos1, len, dlen-1, len-dlen, (UI)id, name);
 
 	if(dlen<1) return; // Invalid header, too short to even have an id field
 
+	de_dbg_indent(c, 1);
 	if(e && e->decoder_fn) {
-		de_dbg_indent(c, 1);
 		e->decoder_fn(c, d, md, id, e, pos1+1, dlen-1);
-		de_dbg_indent(c, -1);
 	}
 	else {
 		if(c->debug_level>=2) {
 			de_dbg_hexdump(c, c->infile, pos1+1, dlen-1, 256, NULL, 0x1);
 		}
 	}
+	de_dbg_indent(c, -1);
 }
 
 static const char *get_os_name(u8 id)
@@ -587,14 +656,14 @@ static void do_lev0_ext_area(deark *c, lctx *d, struct member_data *md,
 
 	// TODO: Finish this
 	if(md->os_id=='U') {
-		unsigned int mode;
+		UI mode;
 		i64 uid, gid;
 
 		if(len<12) goto done;
 
 		read_unix_timestamp(c, d, md, pos1+2, DE_TIMESTAMPIDX_MODIFY, "last-modified");
 
-		mode = (unsigned int)de_getu16le(pos1+6);
+		mode = (UI)de_getu16le(pos1+6);
 		de_dbg(c, "mode: octal(%06o)", mode);
 		interpret_unix_perms(c, d, md, mode);
 
@@ -635,7 +704,7 @@ static int do_read_ext_headers(deark *c, lctx *d, struct member_data *md,
 		return 1;
 	}
 
-	de_dbg(c, "ext headers section at %d", (int)pos);
+	de_dbg(c, "ext headers section at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	size_of_size_field = (md->hlev==3) ? 4 : 2;
@@ -667,7 +736,7 @@ static int do_read_ext_headers(deark *c, lctx *d, struct member_data *md,
 
 done:
 	if(retval) {
-		de_dbg(c, "size of ext headers section: %d", (int)*tot_bytes_consumed);
+		de_dbg(c, "size of ext headers section: %"I64_FMT, (i64)*tot_bytes_consumed);
 	}
 	else {
 		de_dbg(c, "failed to parse all extended headers");
@@ -685,7 +754,7 @@ static void make_fullfilename(deark *c, lctx *d, struct member_data *md)
 	}
 	md->fullfilename = ucstring_create(c);
 
-	if(md->hlev==0) {
+	if(md->hlev==0 && d->basefmt!=BASEFMT_TPK) {
 		ucstring_append_ucstring(md->fullfilename, md->filename);
 	}
 	else {
@@ -721,6 +790,36 @@ static void decompress_lh1(deark *c, lctx *d, struct member_data *md,
 	fmtutil_lh1_codectype1(c, dcmpri, dcmpro, dres, NULL);
 }
 
+static void decompress_tk1(deark *c, lctx *d, struct member_data *md,
+	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
+	struct de_dfilter_results *dres)
+{
+	u8 need_errmsg = 0;
+	i64 lzhuf_orig_len;
+
+	if(dcmpri->len < 4) {
+		need_errmsg = 1;
+		goto done;
+	}
+
+
+	lzhuf_orig_len = dbuf_getu32le(dcmpri->f, dcmpri->pos);
+	if(lzhuf_orig_len != md->orig_size) {
+		need_errmsg = 1;
+		goto done;
+	}
+
+	dcmpri->len -= 4;
+	dcmpri->pos += 4;
+
+	fmtutil_lh1_codectype1(c, dcmpri, dcmpro, dres, NULL);
+
+done:
+	if(need_errmsg) {
+		de_dfilter_set_generic_error(c, dres, NULL);
+	}
+}
+
 // Caller supplies fmt (DE_LH5X_FMT_*).
 static void decompress_lh5x_internal(deark *c, lctx *d,
 	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
@@ -770,7 +869,7 @@ static int decompress_lh5x_dry_run(deark *c, lctx *d, struct member_data *md,
 	// Note: Another possible test would be if
 	//  (dres.bytes_consumed == md->compressed_data_len).
 	crc_calc = de_crcobj_getval(crco);
-	if(crc_calc != md->crc16) goto done;
+	if(crc_calc != md->crc_reported) goto done;
 	retval = 1;
 
 done:
@@ -798,7 +897,7 @@ static void detect_lhark(deark *c, lctx *d, struct member_data *md,
 	de_dbg(c, "[detecting lh7 format]");
 	de_dbg_indent(c, 1);
 
-	if(md->hlev != 1) {
+	if(md->hlev!=1 || md->os_id!=0x20) {
 		d->lhark_policy = 0;
 		ok = 1;
 		goto done;
@@ -884,8 +983,20 @@ static void decompress_lz5(deark *c, lctx *d, struct member_data *md,
 	fmtutil_decompress_lzss1(c, dcmpri, dcmpro, dres, 0x2);
 }
 
+static void decompress_pakleo(deark *c, lctx *d, struct member_data *md,
+	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
+	struct de_dfilter_results *dres)
+{
+	struct de_lzw_params delzwp;
+
+	de_zeromem(&delzwp, sizeof(struct de_lzw_params));
+	delzwp.fmt = DE_LZWFMT_PAKLEO;
+	fmtutil_decompress_lzw(c, dcmpri, dcmpro, dres, &delzwp);
+}
+
 struct cmpr_meth_array_item {
-	unsigned int flags;
+	enum lha_basefmt_enum basefmt;
+	u8 flags;
 	u32 uniq_id;
 	const char *descr;
 	decompressor_fn decompressor;
@@ -895,36 +1006,42 @@ struct cmpr_meth_array_item {
 // listed here, but note that it is also possible for get_cmpr_meth_info()
 // to handle them procedurally.
 static const struct cmpr_meth_array_item cmpr_meth_arr[] = {
-	{ 0x00, CODE_lhd, "directory", NULL },
-	{ 0x00, CODE_lh0, "uncompressed", decompress_uncompressed },
-	{ 0x00, CODE_lh1, "LZ77-4K, adaptive Huffman", decompress_lh1 },
-	{ 0x00, CODE_lh4, "LZ77-4K, static Huffman", decompress_lh5x_auto },
-	{ 0x00, CODE_lh5, "LZ77-8K, static Huffman", decompress_lh5 },
-	{ 0x00, CODE_lh6, "LZ77-32K, static Huffman", decompress_lh5x_auto },
-	{ 0x00, CODE_lh7, NULL, decompress_lh5x_auto },
-	{ 0x00, CODE_lh8, NULL, decompress_lh5x_auto },
-	{ 0x00, CODE_lz4, "uncompressed (LArc)", decompress_uncompressed },
-	{ 0x00, CODE_lz5, "LZSS-4K (LArc)", decompress_lz5 },
-	{ 0x00, CODE_pm0, "uncompressed (PMArc)", decompress_uncompressed },
-	{ 0x00, CODE_lZ0, "uncompressed (MicroFox PUT)", decompress_uncompressed },
-	{ 0x00, CODE_lZ1, "MicroFox PUT lZ1", decompress_lh1 },
-	{ 0x00, CODE_lZ5, "MicroFox PUT lZ5", decompress_lh5 },
-	{ 0x00, CODE_S_LH0, "uncompressed (SAR)", decompress_uncompressed },
-	{ 0x00, CODE_S_LH5, "SAR LH5", decompress_lh5 },
-	{ 0x00, CODE_sw0, NULL, decompress_uncompressed },
-	{ 0x00, CODE_sw1, NULL, NULL }
+	{ BASEFMT_LHA, 0x00, CODE_lhd, "directory", NULL },
+	{ BASEFMT_LHA, 0x00, CODE_lh0, "uncompressed", decompress_uncompressed },
+	{ BASEFMT_LHA, 0x00, CODE_lh1, "LZ77-4K, adaptive Huffman", decompress_lh1 },
+	{ BASEFMT_LHA, 0x00, CODE_lh4, "LZ77-4K, static Huffman", decompress_lh5x_auto },
+	{ BASEFMT_LHA, 0x00, CODE_lh5, "LZ77-8K, static Huffman", decompress_lh5 },
+	{ BASEFMT_LHA, 0x00, CODE_lh6, "LZ77-32K, static Huffman", decompress_lh5x_auto },
+	{ BASEFMT_LHA, 0x00, CODE_lh7, NULL, decompress_lh5x_auto },
+	{ BASEFMT_LHA, 0x00, CODE_lh8, NULL, decompress_lh5x_auto },
+	{ BASEFMT_LHA, 0x00, CODE_lz4, "uncompressed (LArc)", decompress_uncompressed },
+	{ BASEFMT_LHA, 0x00, CODE_lz5, "LZSS-4K (LArc)", decompress_lz5 },
+	{ BASEFMT_LHA, 0x00, CODE_pm0, "uncompressed (PMArc)", decompress_uncompressed },
+	{ BASEFMT_LHA, 0x00, CODE_lZ0, "uncompressed (MicroFox PUT)", decompress_uncompressed },
+	{ BASEFMT_LHA, 0x00, CODE_lZ1, "MicroFox PUT lZ1", decompress_lh1 },
+	{ BASEFMT_LHA, 0x00, CODE_lZ5, "MicroFox PUT lZ5", decompress_lh5 },
+	{ BASEFMT_LHA, 0x00, CODE_S_LH0, "uncompressed (SAR)", decompress_uncompressed },
+	{ BASEFMT_LHA, 0x00, CODE_S_LH5, "SAR LH5", decompress_lh5 },
+	{ BASEFMT_SWG, 0x00, CODE_sw0, "uncompressed", decompress_uncompressed },
+	{ BASEFMT_SWG, 0x00, CODE_sw1, NULL, NULL },
+	{ BASEFMT_TPK, 0x00, CODE_TK0, "uncompressed", decompress_uncompressed },
+	{ BASEFMT_TPK, 0x00, CODE_TK1, "LZHUF", decompress_tk1 },
+	{ BASEFMT_PAKLEO, 0x00, CODE_ll0, "uncompressed", decompress_uncompressed },
+	{ BASEFMT_PAKLEO, 0x00, CODE_ll1, "LZW", decompress_pakleo }
 };
 
+// For basefmt==BASEFMT_LHA only
 static const u32 other_known_cmpr_methods[] = {
 	CODE_ah0, CODE_ari, CODE_hf0,
-	CODE_lh2, CODE_lh3, CODE_lh4, CODE_lh7, CODE_lh8, CODE_lh9,
+	CODE_lh2, CODE_lh3, CODE_lh9,
 	CODE_lha, CODE_lhb, CODE_lhc, CODE_lhe, CODE_lhx, CODE_lx1,
 	CODE_lz2, CODE_lz3, CODE_lz7, CODE_lz8, CODE_lzs,
-	CODE_pc1, CODE_pm1, CODE_pm2 };
+	CODE_pm1, CODE_pm2 };
 
 // Only call this after is_possible_cmpr_meth() return nonzero.
 // Caller allocates cmi, and initializes to zeroes.
-static void get_cmpr_meth_info(const u8 idbuf[5], struct cmpr_meth_info *cmi)
+static void get_cmpr_meth_info(const u8 idbuf[5], enum lha_basefmt_enum basefmt,
+	struct cmpr_meth_info *cmi)
 {
 	size_t k;
 	const struct cmpr_meth_array_item *cmai = NULL;
@@ -939,6 +1056,7 @@ static void get_cmpr_meth_info(const u8 idbuf[5], struct cmpr_meth_info *cmi)
 	cmi->id_printable_sz[5] = '\0';
 
 	for(k=0; k<DE_ARRAYCOUNT(cmpr_meth_arr); k++) {
+		if(cmpr_meth_arr[k].basefmt != basefmt) continue;
 		if(cmpr_meth_arr[k].uniq_id == cmi->uniq_id) {
 			cmai = &cmpr_meth_arr[k];
 			break;
@@ -949,7 +1067,7 @@ static void get_cmpr_meth_info(const u8 idbuf[5], struct cmpr_meth_info *cmi)
 		cmi->is_recognized = 1;
 		cmi->decompressor = cmai->decompressor;
 	}
-	else {
+	else if(basefmt==BASEFMT_LHA) {
 		for(k=0; k<DE_ARRAYCOUNT(other_known_cmpr_methods); k++) {
 			if(other_known_cmpr_methods[k] == cmi->uniq_id) {
 				cmi->is_recognized = 1;
@@ -973,7 +1091,6 @@ static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 {
 	de_finfo *fi = NULL;
 	dbuf *outf = NULL;
-	u32 crc_calc;
 	int tsidx;
 	u8 dcmpr_disabled = 0;
 	u8 dcmpr_attempted = 0;
@@ -1050,11 +1167,20 @@ static void do_extract_file(deark *c, lctx *d, struct member_data *md)
 		goto done;
 	}
 
-	crc_calc = de_crcobj_getval(d->crco);
-	de_dbg(c, "crc (calculated): 0x%04x", (unsigned int)crc_calc);
-	if(crc_calc != md->crc16) {
-		de_err(c, "%s: CRC check failed", ucstring_getpsz_d(md->fullfilename));
-		goto done;
+	if(md->have_crc_reported) {
+		u32 crc_calc;
+
+		crc_calc = de_crcobj_getval(d->crco);
+		if(d->basefmt==BASEFMT_PAKLEO) {
+			de_dbg(c, "crc (calculated): 0x%08x", (UI)crc_calc);
+		}
+		else {
+			de_dbg(c, "crc (calculated): 0x%04x", (UI)crc_calc);
+		}
+		if(crc_calc != md->crc_reported) {
+			de_err(c, "%s: CRC check failed", ucstring_getpsz_d(md->fullfilename));
+			goto done;
+		}
 	}
 
 	dcmpr_ok = 1;
@@ -1070,15 +1196,16 @@ done:
 	de_finfo_destroy(c, fi);
 }
 
-static int cksum_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf, i64 buf_len)
+// Simple checksum used by some header formats.
+// Caller supplies a crcobj to use.
+static u32 lha_calc_checksum(dbuf *f, i64 pos, i64 len, struct de_crcobj *crco_cksum)
 {
-	UI *pcksum = (UI*)brctx->userdata;
-	i64 i;
+	u32 v;
 
-	for(i=0; i<buf_len; i++) {
-		*pcksum = (*pcksum + buf[i]) & 0xff;
-	}
-	return 1;
+	de_crcobj_reset(crco_cksum);
+	de_crcobj_addslice(crco_cksum, f, pos, len);
+	v = de_crcobj_getval(crco_cksum);
+	return v & 0xff;
 }
 
 static void do_check_header_crc(deark *c, lctx *d, struct member_data *md)
@@ -1116,17 +1243,31 @@ enum lha_whats_next_enum {
 	LHA_WN_NOTHING
 };
 
+static enum lha_whats_next_enum pakleo_classify_whats_next(deark *c, lctx *d,
+	i64 pos, i64 len)
+{
+	u8 b[7];
+	if(len<=0) return LHA_WN_NOTHING;
+	de_read(b, pos, sizeof(b));
+	if(is_possible_cmpr_meth(&b[2])) return LHA_WN_MEMBER;
+	return LHA_WN_JUNK;
+}
+
 static enum lha_whats_next_enum lha_classify_whats_next(deark *c, lctx *d, i64 pos, i64 len)
 {
 	u8 b[21];
 	u8 hlev;
 
+	if(d->basefmt==BASEFMT_PAKLEO) {
+		return pakleo_classify_whats_next(c, d, pos, len);
+	}
+
 	if(len<=0) return LHA_WN_NOTHING;
 	b[0] = de_getbyte(pos);
 	if(b[0]==0 && len<=2) return LHA_WN_TRAILER;
 	if(b[0]==0 && len<21) return LHA_WN_TRAILER_AND_JUNK;
 	de_read(&b[1], pos+1, sizeof(b)-1);
-	if(d->swg_fmt) hlev = 0;
+	if(d->basefmt==BASEFMT_SWG) hlev = 0;
 	else hlev = b[20];
 	if(b[0]==0 && b[1]==0) return LHA_WN_TRAILER_AND_JUNK;
 	if(b[0]==0 && hlev!=2) return LHA_WN_TRAILER_AND_JUNK;
@@ -1167,7 +1308,7 @@ static void do_special_swg_fields(deark *c, lctx *d, struct member_data *md, i64
 
 // This single function parses all the different header formats, using lots of
 // "if" statements. It is messy, but it's a no-win situation.
-// The alternative of four separate functions would be have a lot of redundant
+// The alternative of many separate functions would be have a lot of redundant
 // code, and be harder to maintain.
 //
 // Caller allocates and initializes md.
@@ -1200,7 +1341,9 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 	wn = lha_classify_whats_next(c, d, pos1, nbytes_avail);
 	if(wn!=LHA_WN_MEMBER) {
 		if(d->member_count==0) {
-			de_err(c, "Not an LHA file");
+			de_err(c, "Not a%s %s file",
+				((d->basefmt==BASEFMT_LHA || d->basefmt==BASEFMT_SWG)?"n":""),
+				d->basefmt_name);
 		}
 		else if(wn==LHA_WN_TRAILER || wn==LHA_WN_TRAILER_AND_JUNK) {
 			d->trailer_found = 1;
@@ -1208,8 +1351,8 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 			de_dbg(c, "trailer at %"I64_FMT, d->trailer_pos);
 		}
 		else if(wn==LHA_WN_JUNK) {
-			de_warn(c, "%"I64_FMT" bytes of non-LHA data found at end of file (offset %"I64_FMT")",
-				nbytes_avail, pos1);
+			de_warn(c, "%"I64_FMT" bytes of non-%s data found at end of file (offset %"I64_FMT")",
+				nbytes_avail, d->basefmt_name, pos1);
 		}
 		goto done;
 	}
@@ -1222,9 +1365,15 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 	// which happened to always be zero.
 	// In later LHA versions, it is overloaded to identify the header format
 	// version (called "header level" in LHA jargon).
-	if(d->swg_fmt) {
+	if(d->basefmt==BASEFMT_SWG) {
 		md->hlev = 0; // SWG is most similar to header level 0
 	}
+	else if(d->basefmt==BASEFMT_PAKLEO) {
+		md->hlev = 0; // hlev field is present, but we only support one format
+	}
+	else if(d->basefmt==BASEFMT_TPK) {
+		md->hlev = 0;
+	}
 	else {
 		md->hlev = de_getbyte(pos1+20);
 	}
@@ -1237,19 +1386,34 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		d->hlev_of_first_member = md->hlev;
 	}
 
-	if(md->hlev==0) {
-		lev0_header_size = (i64)de_getbyte_p(&pos);
-		de_dbg(c, "header size: (2+)%d", (int)lev0_header_size);
+	if(d->basefmt==BASEFMT_PAKLEO) {
+		pos += 2; // What is this field?
+	}
+	else if(md->hlev==0) {
+		if(d->basefmt==BASEFMT_TPK) {
+			lev0_header_size = (i64)de_getbyte_p(&pos);
+			de_dbg(c, "header size: %d", (int)lev0_header_size);
+			lev0_header_size -= 2; // hack
+		}
+		else {
+			lev0_header_size = (i64)de_getbyte_p(&pos);
+			de_dbg(c, "header size: (2+)%d", (int)lev0_header_size);
+		}
+
+		if(lev0_header_size<0) goto done;
+
 		hdr_checksum_reported = (UI)de_getbyte_p(&pos);
 		has_hdr_checksum = 1;
-		dbuf_buffered_read(c->infile, pos, lev0_header_size, cksum_cbfn, (void*)&md->hdr_checksum_calc);
+		md->hdr_checksum_calc = lha_calc_checksum(c->infile, pos, lev0_header_size,
+			d->crco_cksum);
 	}
 	else if(md->hlev==1) {
 		lev1_base_header_size = (i64)de_getbyte_p(&pos);
 		de_dbg(c, "base header size: %d", (int)lev1_base_header_size);
 		hdr_checksum_reported = (UI)de_getbyte_p(&pos);
 		has_hdr_checksum = 1;
-		dbuf_buffered_read(c->infile, pos, lev1_base_header_size, cksum_cbfn, (void*)&md->hdr_checksum_calc);
+		md->hdr_checksum_calc = lha_calc_checksum(c->infile, pos, lev1_base_header_size,
+			d->crco_cksum);
 	}
 	else if(md->hlev==2) {
 		lev2_total_header_size = de_getu16le_p(&pos);
@@ -1267,7 +1431,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 
 	if(has_hdr_checksum) {
 		de_dbg(c, "header checksum (reported): 0x%02x", hdr_checksum_reported);
-		de_dbg(c, "header checksum (calculated): 0x%02x", md->hdr_checksum_calc);
+		de_dbg(c, "header checksum (calculated): 0x%02x", (UI)md->hdr_checksum_calc);
 		if(md->hdr_checksum_calc != hdr_checksum_reported) {
 			de_err(c, "Wrong header checksum: reported=0x%02x, calculated=0x%02x",
 				hdr_checksum_reported, md->hdr_checksum_calc);
@@ -1276,7 +1440,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 
 	de_read(cmpr_meth_raw, pos, 5);
 	md->cmi = de_malloc(c, sizeof(struct cmpr_meth_info));
-	get_cmpr_meth_info(cmpr_meth_raw, md->cmi);
+	get_cmpr_meth_info(cmpr_meth_raw, d->basefmt, md->cmi);
 	de_dbg(c, "cmpr method: '%s' (%s)", md->cmi->id_printable_sz, md->cmi->descr);
 	pos+=5;
 
@@ -1295,7 +1459,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		// lev1_skip_size is the distance from the third byte of the extended
 		// header section, to the end of the compressed data.
 		lev1_skip_size = de_getu32le_p(&pos);
-		de_dbg(c, "skip size: %u", (unsigned int)lev1_skip_size);
+		de_dbg(c, "skip size: %u", (UI)lev1_skip_size);
 		md->total_size = 2 + lev1_base_header_size + lev1_skip_size;
 	}
 	else {
@@ -1303,7 +1467,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		de_dbg(c, "compressed size: %"I64_FMT, md->compressed_data_len);
 		pos += 4;
 
-		if(md->hlev==0) {
+		if(md->hlev==0 && d->basefmt!=BASEFMT_PAKLEO) {
 			md->total_size = 2 + lev0_header_size + md->compressed_data_len;
 		}
 		else if(md->hlev==2) {
@@ -1312,7 +1476,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 	}
 
 	md->orig_size = de_getu32le(pos);
-	de_dbg(c, "original size: %u", (unsigned int)md->orig_size);
+	de_dbg(c, "original size: %u", (UI)md->orig_size);
 	pos += 4;
 
 	if(md->hlev==0 || md->hlev==1) {
@@ -1342,12 +1506,21 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		pos++; // header level, already handled
 	}
 
-	if(d->swg_fmt) {
+	if(d->basefmt==BASEFMT_SWG) {
 		do_special_swg_fields(c, d, md, pos);
 		pos += 165;
 	}
 
-	if(md->hlev<=1) {
+	if(d->basefmt==BASEFMT_PAKLEO) {
+		md->crc_reported = (u32)de_getu32le_p(&pos);
+		de_dbg(c, "crc32 (reported): 0x%08x", (UI)md->crc_reported);
+	}
+
+	if(d->basefmt==BASEFMT_TPK) {
+		read_filename_tpk(c, d, md, pos,
+			(md->member_pos + 2 + lev0_header_size) - pos);
+	}
+	else if(md->hlev<=1) {
 		fnlen = de_getbyte(pos++);
 		de_dbg(c, "filename len: %d", (int)fnlen);
 		if(md->hlev==0) {
@@ -1359,12 +1532,18 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		pos += fnlen;
 	}
 
-	md->crc16 = (u32)de_getu16le_p(&pos);
-	de_dbg(c, "crc16 (reported): 0x%04x", (unsigned int)md->crc16);
+	if(d->basefmt==BASEFMT_TPK) {
+		md->have_crc_reported = 0;
+	}
+
+	if(d->basefmt!=BASEFMT_PAKLEO && d->basefmt!=BASEFMT_TPK) {
+		md->crc_reported = (u32)de_getu16le_p(&pos);
+		de_dbg(c, "crc16 (reported): 0x%04x", (UI)md->crc_reported);
+	}
 
 	if(md->hlev==1 || md->hlev==2 || md->hlev==3) {
 		md->os_id = de_getbyte_p(&pos);
-		de_dbg(c, "OS id: %d ('%c') (%s)", (int)md->os_id,
+		de_dbg(c, "OS id: %u ('%c') (%s)", (UI)md->os_id,
 			de_byte_to_printable_char(md->os_id), get_os_name(md->os_id));
 	}
 
@@ -1373,11 +1552,18 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		md->total_size = lev3_header_size + md->compressed_data_len;
 	}
 
-	if(md->hlev==0) {
+	if(d->basefmt==BASEFMT_PAKLEO) {
+		md->compressed_data_pos = pos;
+		md->total_size = md->compressed_data_pos + md->compressed_data_len - md->member_pos;
+	}
+	else if(d->basefmt==BASEFMT_TPK) {
+		md->compressed_data_pos = md->member_pos + 2 + lev0_header_size;
+	}
+	else if(md->hlev==0) {
 		i64 ext_headers_size = (2+lev0_header_size) - (pos-pos1);
 		md->compressed_data_pos = pos1 + 2 + lev0_header_size;
 		if(ext_headers_size>0) {
-			de_dbg(c, "extended header area at %d, len=%d", (int)pos, (int)ext_headers_size);
+			de_dbg(c, "extended header area at %"I64_FMT", len=%"I64_FMT, pos, ext_headers_size);
 			de_dbg_indent(c, 1);
 			do_lev0_ext_area(c, d, md, pos, ext_headers_size);
 			de_dbg_indent(c, -1);
@@ -1390,14 +1576,14 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		pos = pos1 + 2 + lev1_base_header_size - 2;
 		// TODO: sanitize pos?
 		first_ext_hdr_size = de_getu16le_p(&pos);
-		de_dbg(c, "first ext hdr size: %d", (int)first_ext_hdr_size);
+		de_dbg(c, "first ext hdr size: %"I64_FMT, first_ext_hdr_size);
 
 		ret = do_read_ext_headers(c, d, md, pos, lev1_skip_size, first_ext_hdr_size,
 			&exthdr_bytes_consumed);
 
 		if(!ret) {
-			de_err(c, "Error parsing extended headers at %d. Cannot extract this file.",
-				(int)pos);
+			de_err(c, "Error parsing extended headers at %"I64_FMT". Cannot extract this file.",
+				pos);
 			retval = 1;
 			goto done;
 		}
@@ -1416,14 +1602,14 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 			// would be better than just guessing.
 			lev2_total_header_size += 2;
 			md->total_size = lev2_total_header_size + md->compressed_data_len;
-			de_dbg(c, "attempting bug workaround: changing total header size to %d",
-				(int)lev2_total_header_size);
+			de_dbg(c, "attempting bug workaround: changing total header size to %"I64_FMT,
+				lev2_total_header_size);
 		}
 
 		md->compressed_data_pos = pos1+lev2_total_header_size;
 
 		first_ext_hdr_size = de_getu16le_p(&pos);
-		de_dbg(c, "first ext hdr size: %d", (int)first_ext_hdr_size);
+		de_dbg(c, "first ext hdr size: %"I64_FMT, first_ext_hdr_size);
 
 		do_read_ext_headers(c, d, md, pos, pos1+lev2_total_header_size-pos,
 			first_ext_hdr_size, &exthdr_bytes_consumed);
@@ -1434,7 +1620,7 @@ static int do_read_member(deark *c, lctx *d, struct member_data *md)
 		md->compressed_data_pos = pos1+lev3_header_size;
 
 		first_ext_hdr_size = de_getu32le_p(&pos);
-		de_dbg(c, "first ext hdr size: %d", (int)first_ext_hdr_size);
+		de_dbg(c, "first ext hdr size: %"I64_FMT, first_ext_hdr_size);
 
 		do_read_ext_headers(c, d, md, pos, pos1+lev3_header_size-pos,
 			first_ext_hdr_size, &exthdr_bytes_consumed);
@@ -1486,7 +1672,7 @@ static void do_lha_footer(deark *c, lctx *d)
 	extra_bytes_len = c->infile->len - extra_bytes_pos;
 	if(extra_bytes_len<=1) goto done;
 
-	if(d->swg_fmt && extra_bytes_len==129) {
+	if(d->basefmt==BASEFMT_SWG && extra_bytes_len==129) {
 		do_swg_footer(c, d, extra_bytes_pos);
 		goto done;
 	}
@@ -1497,30 +1683,63 @@ done:
 	;
 }
 
-static void do_run_lha_internal(deark *c, de_module_params *mparams, int is_swg)
+static lctx *lha_create_lctx(deark *c)
 {
-	lctx *d = NULL;
-	i64 pos;
-	struct member_data *md = NULL;
+	lctx *d;
 
 	d = de_malloc(c, sizeof(lctx));
-	if(is_swg) d->swg_fmt = 1;
+	return d;
+}
 
+static void lha_destroy_lctx(deark *c, lctx *d)
+{
+	if(!d) return;
+	de_crcobj_destroy(d->crco);
+	de_crcobj_destroy(d->crco_cksum);
+	de_free(c, d);
+}
+
+static void do_run_lha_internal(deark *c, lctx *d, de_module_params *mparams)
+{
+	i64 pos;
+	struct member_data *md = NULL;
+	de_encoding guessed_encoding;
+
+	if(!d->basefmt_name) {
+		d->basefmt_name = "LHA";
+	}
 	d->lhark_req = de_get_ext_option_bool(c, "lha:lhark", -1);
 	d->lhark_policy = d->lhark_req;
 
 	// It's not really safe to guess CP437, because Japanese-encoded (CP932?)
 	// filenames are common.
-	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
+	if(d->basefmt==BASEFMT_SWG || d->basefmt==BASEFMT_PAKLEO) {
+		guessed_encoding = DE_ENCODING_CP437;
+	}
+	else if(d->basefmt==BASEFMT_TPK) {
+		guessed_encoding = DE_ENCODING_WINDOWS1252;
+	}
+	else {
+		guessed_encoding = DE_ENCODING_ASCII;
+	}
+	d->input_encoding = de_get_input_encoding(c, NULL, guessed_encoding);
 
 	d->hlev_of_first_member = 0xff;
-	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
+	if(d->basefmt==BASEFMT_PAKLEO) {
+		d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC32_PL);
+	}
+	else {
+		d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
+	}
+	d->crco_cksum = de_crcobj_create(c, DE_CRCOBJ_SUM_BYTES);
 
 	pos = 0;
+	if(d->basefmt==BASEFMT_PAKLEO) pos += 37;
+
 	while(1) {
 		if(pos >= c->infile->len) break;
 
-		md = de_malloc(c, sizeof(struct member_data));
+		md = create_member_data(c);
 		md->encoding = d->input_encoding;
 		md->member_pos = pos;
 		if(!do_read_member(c, d, md)) goto done;
@@ -1535,17 +1754,17 @@ static void do_run_lha_internal(deark *c, de_module_params *mparams, int is_swg)
 
 done:
 	do_lha_footer(c, d);
-
 	destroy_member_data(c, md);
-	if(d) {
-		de_crcobj_destroy(d->crco);
-		de_free(c, d);
-	}
 }
 
 static void de_run_lha(deark *c, de_module_params *mparams)
 {
-	do_run_lha_internal(c, mparams, 0);
+	lctx *d;
+
+	d = lha_create_lctx(c);
+	d->basefmt = BASEFMT_LHA;
+	do_run_lha_internal(c, d, mparams);
+	lha_destroy_lctx(c, d);
 }
 
 static int is_swg_sig(const u8 *b)
@@ -1554,6 +1773,12 @@ static int is_swg_sig(const u8 *b)
 		(b[3]=='0' || b[3]=='1') && b[4]=='-';
 }
 
+static int is_tpk_sig(const u8 *b)
+{
+	return b[0]=='-' && b[1]=='T' && b[2]=='K' &&
+		(b[3]=='0' || b[3]=='1') && b[4]=='-';
+}
+
 static int de_identify_lha(deark *c)
 {
 	int has_ext = 0;
@@ -1583,7 +1808,7 @@ static int de_identify_lha(deark *c)
 	}
 
 	de_zeromem(&cmi, sizeof(struct cmpr_meth_info));
-	get_cmpr_meth_info(&b[2], &cmi);
+	get_cmpr_meth_info(&b[2], BASEFMT_LHA, &cmi);
 	if(!cmi.is_recognized) {
 		return 0;
 	}
@@ -1620,11 +1845,14 @@ void de_module_lha(deark *c, struct deark_module_info *mi)
 
 static void de_run_swg(deark *c, de_module_params *mparams)
 {
-	de_declare_fmt(c, "SWAG packet (LHA-like)");
-	// TODO?: Some diagnostic messages in the LHA module are hardcoded to
-	// say "LHA", which is likely to be confusing to uses who know this as
-	// SWG format.
-	do_run_lha_internal(c, mparams, 1);
+	lctx *d;
+
+	d = lha_create_lctx(c);
+	d->basefmt = BASEFMT_SWG;
+	d->basefmt_name = "SWG";
+	de_declare_fmt(c, "SWAG packet");
+	do_run_lha_internal(c, d, mparams);
+	lha_destroy_lctx(c, d);
 }
 
 static int de_identify_swg(deark *c)
@@ -1645,6 +1873,70 @@ void de_module_swg(deark *c, struct deark_module_info *mi)
 	mi->desc = "SWAG packet";
 	mi->run_fn = de_run_swg;
 	mi->identify_fn = de_identify_swg;
+	mi->flags |= DE_MODFLAG_WARNPARSEONLY;
+}
+
+/////////////////////// TPK (by Thomas Haukap?)
+
+static void de_run_tpk(deark *c, de_module_params *mparams)
+{
+	lctx *d;
+
+	d = lha_create_lctx(c);
+	d->basefmt = BASEFMT_TPK;
+	d->basefmt_name = "TPK";
+	de_declare_fmt(c, "TPK");
+	do_run_lha_internal(c, d, mparams);
+	lha_destroy_lctx(c, d);
+}
+
+static int de_identify_tpk(deark *c)
+{
+	u8 b[5];
+
+	de_read(b, 2, sizeof(b));
+	if(is_tpk_sig(b)) {
+		if(de_input_file_has_ext(c, "tpk")) return 100;
+		return 75;
+	}
+	return 0;
+}
+
+void de_module_tpk(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "tpk";
+	mi->desc = "TPK archive";
+	mi->run_fn = de_run_tpk;
+	mi->identify_fn = de_identify_tpk;
+}
+
+/////////////////////// PAKLEO
+
+static void de_run_pakleo(deark *c, de_module_params *mparams)
+{
+	lctx *d;
+
+	d = lha_create_lctx(c);
+	d->basefmt = BASEFMT_PAKLEO;
+	d->basefmt_name = "PAKLEO";
+	de_declare_fmt(c, "PAKLEO");
+	do_run_lha_internal(c, d, mparams);
+	lha_destroy_lctx(c, d);
+}
+
+static int de_identify_pakleo(deark *c)
+{
+	if(dbuf_memcmp(c->infile, 0, "LEOLZW", 6)) return 0;
+	if(dbuf_memcmp(c->infile, 39, "-l", 2)) return 0;
+	return 100;
+}
+
+void de_module_pakleo(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "pakleo";
+	mi->desc = "PAKLEO archive";
+	mi->run_fn = de_run_pakleo;
+	mi->identify_fn = de_identify_pakleo;
 }
 
 /////////////////////// CAR (MylesHi!)
@@ -1652,12 +1944,13 @@ void de_module_swg(deark *c, struct deark_module_info *mi)
 struct car_member_data {
 	i64 member_pos;
 	i64 total_size;
-	UI hdr_checksum_calc;
+	u32 hdr_checksum_calc;
 };
 
 struct car_ctx {
 	dbuf *hdr_tmp;
 	dbuf *lha_outf;
+	struct de_crcobj *crco_cksum;
 };
 
 static int looks_like_car_member(deark *c, i64 pos)
@@ -1716,9 +2009,8 @@ static int do_car_member(deark *c, struct car_ctx *d, struct car_member_data *md
 	dbuf_writebyte(d->hdr_tmp, 77); // OS ID = 'M' = MS-DOS
 
 	// Recalculate checksum
-	dbuf_buffered_read(d->hdr_tmp, 2, lev1_base_header_size, cksum_cbfn,
-		(void*)&md->hdr_checksum_calc);
-	de_dbg(c, "header checksum (calculated): 0x%02x", md->hdr_checksum_calc);
+	md->hdr_checksum_calc = lha_calc_checksum(d->hdr_tmp, 2, lev1_base_header_size, d->crco_cksum);
+	de_dbg(c, "header checksum (calculated): 0x%02x", (UI)md->hdr_checksum_calc);
 	dbuf_writebyte_at(d->hdr_tmp, 1, (u8)md->hdr_checksum_calc);
 	dbuf_truncate(d->hdr_tmp, 2+lev1_base_header_size);
 
@@ -1748,7 +2040,8 @@ static void de_run_car_lha(deark *c, de_module_params *mparams)
 		goto done;
 	}
 
-	d->lha_outf = dbuf_create_output_file(c, "lha", NULL, 0);
+	d->crco_cksum = de_crcobj_create(c, DE_CRCOBJ_SUM_BYTES);
+	d->lha_outf = dbuf_create_output_file(c, "lzh", NULL, 0);
 	d->hdr_tmp = dbuf_create_membuf(c, 0, 0);
 
 	md = de_malloc(c, sizeof(struct car_member_data));
@@ -1772,6 +2065,7 @@ done:
 	de_free(c, md);
 	if(d) {
 		if(d->lha_outf) {
+			de_crcobj_destroy(d->crco_cksum);
 			dbuf_close(d->lha_outf);
 			if(!ok) {
 				de_err(c, "Conversion to LHA format failed");
@@ -1809,12 +2103,14 @@ struct arx_member_data {
 	i64 unc_data_len;
 	int is_uncompressed;
 	u32 crc_calc;
+	u32 hdr_checksum_calc;
 };
 
 struct arx_ctx {
 	dbuf *hdr_tmp;
 	dbuf *lha_outf;
 	struct de_crcobj *crco;
+	struct de_crcobj *crco_cksum;
 };
 
 static int looks_like_arx_member(deark *c, i64 pos)
@@ -1867,7 +2163,6 @@ static int do_arx_member(deark *c, struct arx_ctx *d, struct arx_member_data *md
 {
 	i64 lev0_header_size;
 	i64 pos1 = md->member_pos;
-	UI hdr_checksum_calc = 0;
 	u8 extra_crc_byte;
 	int retval = 0;
 	int saved_indent_level;
@@ -1930,10 +2225,10 @@ static int do_arx_member(deark *c, struct arx_ctx *d, struct arx_member_data *md
 	}
 
 	// Recalculate header checksum
-	dbuf_buffered_read(d->hdr_tmp, 2, lev0_header_size, cksum_cbfn,
-		(void*)&hdr_checksum_calc);
-	de_dbg(c, "header checksum (calculated): 0x%02x", hdr_checksum_calc);
-	dbuf_writebyte_at(d->hdr_tmp, 1, (u8)hdr_checksum_calc);
+	md->hdr_checksum_calc = lha_calc_checksum(d->hdr_tmp, 2, lev0_header_size,
+		d->crco_cksum);
+	de_dbg(c, "header checksum (calculated): 0x%02x", (UI)md->hdr_checksum_calc);
+	dbuf_writebyte_at(d->hdr_tmp, 1, (u8)md->hdr_checksum_calc);
 	dbuf_truncate(d->hdr_tmp, 2+lev0_header_size);
 
 	// Write everything out
@@ -1963,7 +2258,8 @@ static void de_run_arx(deark *c, de_module_params *mparams)
 	}
 
 	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
-	d->lha_outf = dbuf_create_output_file(c, "lha", NULL, 0);
+	d->crco_cksum = de_crcobj_create(c, DE_CRCOBJ_SUM_BYTES);
+	d->lha_outf = dbuf_create_output_file(c, "lzh", NULL, 0);
 	d->hdr_tmp = dbuf_create_membuf(c, 0, 0);
 
 	md = de_malloc(c, sizeof(struct arx_member_data));
@@ -1994,6 +2290,7 @@ done:
 		}
 		dbuf_close(d->hdr_tmp);
 		de_crcobj_destroy(d->crco);
+		de_crcobj_destroy(d->crco_cksum);
 		de_free(c, d);
 	}
 }
@@ -2069,7 +2366,7 @@ static int do_read_ar001_member(deark *c, lctx *d, struct member_data *md)
 	case 1: de_memcpy(cmpr_meth_raw, (const void*)"-lh4-", 5); break; // ...
 	}
 	md->cmi = de_malloc(c, sizeof(struct cmpr_meth_info));
-	get_cmpr_meth_info(cmpr_meth_raw, md->cmi);
+	get_cmpr_meth_info(cmpr_meth_raw, BASEFMT_LHA, md->cmi);
 	de_dbg(c, "cmpr method: %u (%s)", cmpr_method, md->cmi->descr);
 
 	pos += 1; // file type
@@ -2084,8 +2381,8 @@ static int do_read_ar001_member(deark *c, lctx *d, struct member_data *md)
 	md->orig_size = de_getu32le_p(&pos);
 	de_dbg(c, "original size: %"I64_FMT, md->orig_size);
 
-	md->crc16 = (u32)de_getu16le_p(&pos);
-	de_dbg(c, "crc16 (reported): 0x%04x", (UI)md->crc16);
+	md->crc_reported = (u32)de_getu16le_p(&pos);
+	de_dbg(c, "crc16 (reported): 0x%04x", (UI)md->crc_reported);
 
 	read_filename_hlev0(c, d, md, pos, fnlen);
 	pos += fnlen;
@@ -2133,7 +2430,7 @@ static void de_run_ar001(deark *c, de_module_params *mparams)
 	i64 pos;
 	struct member_data *md = NULL;
 
-	d = de_malloc(c, sizeof(lctx));
+	d = lha_create_lctx(c);
 	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
 
 	d->crco = de_crcobj_create(c, DE_CRCOBJ_CRC16_IBMSDLC);
@@ -2142,7 +2439,7 @@ static void de_run_ar001(deark *c, de_module_params *mparams)
 	while(1) {
 		if(pos >= c->infile->len) break;
 
-		md = de_malloc(c, sizeof(struct member_data));
+		md = create_member_data(c);
 		md->encoding = d->input_encoding;
 		md->member_pos = pos;
 		if(!do_read_ar001_member(c, d, md)) goto done;
@@ -2157,10 +2454,7 @@ static void de_run_ar001(deark *c, de_module_params *mparams)
 
 done:
 	destroy_member_data(c, md);
-	if(d) {
-		de_crcobj_destroy(d->crco);
-		de_free(c, d);
-	}
+	lha_destroy_lctx(c, d);
 }
 
 static int slice_is_printable_ascii(dbuf *f, i64 pos, i64 len)
@@ -2316,9 +2610,13 @@ static void de_run_lharc_sfx_com(deark *c, de_module_params *mparams)
 
 	de_dbg(c, "payload found at: %"I64_FMT, d->payload_offs);
 
+	// Note: Our practice is to use the .lzh extension if we think the file
+	// is for the DOS platform, as it was near universal there.
+	// If we were to support certain other platforms, e.g. Amiga, we might
+	// use .lha instead.
 	dbuf_create_file_from_slice(c->infile, d->payload_offs,
 		c->infile->len-d->payload_offs,
-		(d->sfx_container_is_larc ? "lzs" : "lha"), NULL, 0);
+		(d->sfx_container_is_larc ? "lzs" : "lzh"), NULL, 0);
 
 done:
 	if(d->errflag && d->need_errmsg) {
@@ -2332,7 +2630,7 @@ static int de_identify_lharc_sfx_com(deark *c)
 	int is_larc = 0;
 	int ret;
 
-	if(!de_input_file_has_ext(c, "com")) return 0;
+	if(c->infile->len>65280) return 0;
 	ret = looks_like_lharc_sfx_com(c, &is_larc);
 	if(ret) return 70;
 	return 0;
