@@ -10,58 +10,154 @@
 DE_DECLARE_MODULE(de_module_macpaint);
 
 #define MACPAINT_WIDTH 576
+#define MACPAINT_WIDTH_BYTES (MACPAINT_WIDTH/8)
 #define MACPAINT_HEIGHT 720
-#define MACPAINT_IMAGE_BYTES ((MACPAINT_WIDTH/8)*MACPAINT_HEIGHT)
+#define MACPAINT_IMAGE_BYTES (MACPAINT_WIDTH_BYTES*MACPAINT_HEIGHT)
 
 typedef struct localctx_struct {
 	int has_macbinary_header;
 	u8 is_fmac2com;
 	u8 df_known, rf_known;
+	u8 row_issue_flag;
 	i64 expected_dfpos, expected_rfpos;
 	i64 expected_dflen, expected_rflen;
 	de_ucstring *filename;
 	struct de_timestamp mod_time_from_macbinary;
+	i64 hdr_pos;
+	i64 img_data_pos;
 } lctx;
 
-static void do_read_bitmap(deark *c, lctx *d, i64 pos)
+// We'd like to use fmtutil_decompress_packbits_ex() instead of this custom
+// decompressor, but it would make it difficult to test for row boundary
+// violations.
+// Alternatively, merging this and test_valid_image() is something to consider,
+// but probably not worth it.
+static void decompress_packbits_for_macpaint(deark *c, lctx *d,
+	struct de_dfilter_in_params *dcmpri, struct de_dfilter_out_params *dcmpro,
+	struct de_dfilter_results *dres)
+{
+	i64 pos;
+	u8 b, b2;
+	i64 count;
+	i64 endpos;
+	i64 xpos_b = 0;
+	i64 nbytes_written = 0;
+
+	pos = dcmpri->pos;
+	endpos = dcmpri->pos + dcmpri->len;
+	d->row_issue_flag = 0;
+
+	while(1) {
+		if(dcmpro->len_known && nbytes_written >= dcmpro->expected_len) {
+			goto done; // Decompressed the requested amount of dst data.
+		}
+		if(pos+2 > endpos) { // Min item size is 2 bytes
+			goto done; // Reached the end of source data
+		}
+
+		b = dbuf_getbyte_p(dcmpri->f, &pos);
+
+		if(b<=127) { // An uncompressed run
+			count = 1 + (i64)b;
+			if(pos+count > endpos) {
+				pos--;
+				goto done;
+			}
+
+			xpos_b += count;
+			dbuf_copy(dcmpri->f, pos, count, dcmpro->f);
+			pos += count;
+			nbytes_written += count;
+		}
+		else if(b>=129) { // A compressed run
+			count = 257 - (i64)b;
+			xpos_b += count;
+			b2 = dbuf_getbyte_p(dcmpri->f, &pos);
+			dbuf_write_run(dcmpro->f, b2, count);
+			nbytes_written += count;
+		}
+
+		if(xpos_b==MACPAINT_WIDTH_BYTES) {
+			xpos_b = 0;
+		}
+		else if(xpos_b > MACPAINT_WIDTH_BYTES) {
+			d->row_issue_flag = 1;
+			xpos_b = xpos_b % MACPAINT_WIDTH_BYTES;
+		}
+	}
+
+done:
+	dbuf_flush(dcmpro->f);
+	dres->bytes_consumed = pos - dcmpri->pos;
+	dres->bytes_consumed_valid = 1;
+}
+
+static void do_read_bitmap(deark *c, lctx *d)
 {
 	i64 cmpr_bytes_consumed = 0;
 	dbuf *unc_pixels = NULL;
 	de_finfo *fi = NULL;
+	int saved_indent_level;
+	i64 ipos;
+	struct de_packbits_params *pbparams = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
 
+	de_dbg_indent_save(c, &saved_indent_level);
 	if(!d->is_fmac2com) {
 		i64 ver_num;
 
-		ver_num = de_getu32be(pos);
+		de_dbg(c, "header at %"I64_FMT, d->hdr_pos);
+		de_dbg_indent(c, 1);
+		ver_num = de_getu32be(d->hdr_pos);
 		de_dbg(c, "version number: %u", (unsigned int)ver_num);
 		if(ver_num!=0 && ver_num!=2 && ver_num!=3) {
 			de_warn(c, "Unrecognized version number: %u", (unsigned int)ver_num);
 		}
+
+		// We wait until later to read the brush patterns, only so that the patterns
+		// won't be the first file extracted.
+
+		de_dbg_indent(c, -1);
 	}
 
-	pos += 512;
-
+	ipos = d->img_data_pos;
+	de_dbg(c, "image data at %"I64_FMT, ipos);
+	de_dbg_indent(c, 1);
 	unc_pixels = dbuf_create_membuf(c, MACPAINT_IMAGE_BYTES, 1);
 	dbuf_enable_wbuffer(unc_pixels);
 
-	fmtutil_decompress_packbits(c->infile, pos, c->infile->len - pos,
-		unc_pixels, &cmpr_bytes_consumed);
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = ipos;
+	dcmpri.len = c->infile->len - ipos;
+	dcmpro.f = unc_pixels;
+	dcmpro.len_known = 1;
+	dcmpro.expected_len = MACPAINT_IMAGE_BYTES;
+	pbparams = de_malloc(c, sizeof(struct de_packbits_params));
 
+	decompress_packbits_for_macpaint(c, d, &dcmpri, &dcmpro, &dres);
 	dbuf_flush(unc_pixels);
-	de_dbg(c, "decompressed %d to %d bytes", (int)cmpr_bytes_consumed,
-		(int)unc_pixels->len);
+
+	cmpr_bytes_consumed = dres.bytes_consumed;
+	de_dbg(c, "decompressed %"I64_FMT" to %"I64_FMT" bytes", cmpr_bytes_consumed,
+		unc_pixels->len);
 
 	if(d->df_known) {
-		if(pos+cmpr_bytes_consumed > d->expected_dfpos+d->expected_dflen) {
+		if(ipos+cmpr_bytes_consumed > d->expected_dfpos+d->expected_dflen) {
 			de_warn(c, "Image (ends at %"I64_FMT") goes beyond end of "
 				"MacBinary data fork (ends at %"I64_FMT")",
-				pos+cmpr_bytes_consumed, d->expected_dfpos+d->expected_dflen);
+				ipos+cmpr_bytes_consumed, d->expected_dfpos+d->expected_dflen);
 		}
 	}
 
 	if(unc_pixels->len < MACPAINT_IMAGE_BYTES) {
-		de_warn(c, "Image decompressed to %d bytes, expected %d.",
-			(int)unc_pixels->len, (int)MACPAINT_IMAGE_BYTES);
+		de_warn(c, "Image decompressed to %"I64_FMT" bytes, expected %u.",
+			unc_pixels->len, (UI)MACPAINT_IMAGE_BYTES);
+	}
+	else if(d->row_issue_flag) {
+		de_warn(c, "Rows not compressed independently. Decompression may have failed.");
 	}
 
 	fi = de_finfo_create(c);
@@ -79,81 +175,109 @@ static void do_read_bitmap(deark *c, lctx *d, i64 pos)
 
 	dbuf_close(unc_pixels);
 	de_finfo_destroy(c, fi);
+	de_free(c, pbparams);
+	de_dbg_indent_restore(c, saved_indent_level);
 }
 
+struct validity_info {
+	i64 pos;
+	int retval;
+	char msg[80];
+};
+
 // A function to help determine if the file has a MacBinary header.
 // Each row is RLE-compressed independently, so once we assume one possibility
 // or the other, we can do sanity checks to see if any code crosses a row
 // boundary, or the image is too small to be a MacPaint image.
 // It's inefficient to decompress whole image -- twice -- just to try to
 // figure this out, but hopefully it's pretty reliable.
-// Returns an integer (0, 1, 2) reflecting the likelihood that this is the
+// Returns an integer (0, 1, 2...) reflecting the likelihood that this is the
 // correct position.
-static int valid_file_at(deark *c, lctx *d, i64 pos1)
+// The special 'struct validity_info' is overkill, but it makes it easier to
+// try different things.
+static void test_valid_image(deark *c, lctx *d, struct validity_info *vi)
 {
 	u8 b;
-	i64 x;
-	i64 xpos, ypos;
+	i64 count;
 	i64 pos;
 	i64 imgstart;
+	i64 xpos_b = 0;
+	i64 ypos = 0;
+	int v = 0;
 
-	imgstart = pos1+512;
+	imgstart = vi->pos+512;
+	de_dbg(c, "checking for image at offset %d", (int)imgstart);
+	de_dbg_indent(c, 1);
 
 	// Minimum bytes per row is 2.
 	// For a valid (non-truncated) file, file size must be at least
 	// pos1 + 512 + 2*MACPAINT_HEIGHT. But we want to tolerate truncated
 	// files as well.
 	if(c->infile->len < imgstart + 4) {
-		de_dbg(c, "file too small");
-		return 0;
+		de_strlcpy(vi->msg, "file too small", sizeof(vi->msg));
+		vi->retval = 0;
+		goto done;
 	}
 
-	xpos=0; ypos=0;
-	pos = pos1 + 512;
+	// Look for a boundary between all-0 bytes, and not-all-0 bytes.
+	if(dbuf_is_all_zeroes(c->infile, imgstart-16, 16) &&
+		!dbuf_is_all_zeroes(c->infile, imgstart, 8))
+	{
+		v++;
+	}
+
+	pos = imgstart;
 
 	while(pos < c->infile->len) {
 		if(ypos>=MACPAINT_HEIGHT) {
 			break;
 		}
 
-		b = de_getbyte(pos);
-		pos++;
+		b = de_getbyte_p(&pos);
 
 		if(b<=127) {
-			x = 1+(i64)b;
-			pos+=x;
-			xpos+=8*x;
-			if(xpos==MACPAINT_WIDTH) {
-				xpos=0;
+			count = 1+(i64)b;
+			pos += count;
+			xpos_b += count;
+			if(xpos_b==MACPAINT_WIDTH_BYTES) {
+				xpos_b = 0;
 				ypos++;
 			}
-			else if(xpos>MACPAINT_WIDTH) {
-				de_dbg(c, "image at offset %d: literal too long", (int)imgstart);
-				return 0;
+			else if(xpos_b > MACPAINT_WIDTH_BYTES) {
+				de_strlcpy(vi->msg, "literal too long", sizeof(vi->msg));
+				vi->retval = v+0;
+				goto done;
 			}
 		}
 		else if(b>=129) {
-			x = 257 - (i64)b;
+			count = 257 - (i64)b;
 			pos++;
-			xpos+=8*x;
-			if(xpos==MACPAINT_WIDTH) {
-				xpos=0;
+			xpos_b += count;
+			if(xpos_b == MACPAINT_WIDTH_BYTES) {
+				xpos_b = 0;
 				ypos++;
 			}
-			else if(xpos>MACPAINT_WIDTH) {
-				de_dbg(c, "image at offset %d: run too long", (int)imgstart);
-				return 0;
+			else if(xpos_b > MACPAINT_WIDTH_BYTES) {
+				de_strlcpy(vi->msg, "run too long", sizeof(vi->msg));
+				vi->retval = v+0;
+				goto done;
 			}
 		}
 	}
 
-	if(xpos==0 && ypos==MACPAINT_HEIGHT) {
-		de_dbg(c, "image at offset %d decodes okay", (int)imgstart);
-		return 2;
+	if(xpos_b==0 && ypos==MACPAINT_HEIGHT) {
+		de_strlcpy(vi->msg, "decodes okay", sizeof(vi->msg));
+		vi->retval = v+2;
+		goto done;
 	}
 
-	de_dbg(c, "image at offset %d: premature end of file (x=%d, y=%d)", (int)imgstart, (int)xpos, (int)ypos);
-	return 1;
+	de_snprintf(vi->msg, sizeof(vi->msg), "premature end of file (x=%d, y=%d)",
+		(int)(xpos_b*8), (int)ypos);
+	vi->retval = v+1;
+
+done:
+	de_dbg(c, "image at offset %d: %s", (int)(vi->pos+512), vi->msg);
+	de_dbg_indent(c, -1);
 }
 
 static const char *get_pattern_set_info(u32 patcrc, int *is_blank)
@@ -171,31 +295,42 @@ static const char *get_pattern_set_info(u32 patcrc, int *is_blank)
 // Some MacPaint files contain a collection of brush patterns.
 // Essentially, MacPaint saves workspace settings inside image files.
 // (But these patterns are the only setting.)
-static void do_read_patterns(deark *c, lctx *d, i64 pos)
+static void do_read_patterns(deark *c, lctx *d)
 {
-	i64 cell;
+	i64 pos1;
+	i64 cell_idx;
 	i64 i, j;
-	u8 x;
-	const i64 dispwidth = 19;
-	const i64 dispheight = 17;
-	i64 xpos, ypos;
+	const i64 cell_width = 19;
+	const i64 cell_height = 16;
+	const i64 cells_per_row = 19;
 	int is_blank;
-	de_bitmap *pat = NULL;
+	de_bitmap *gallery = NULL;
+	de_bitmap *rawimg = NULL;
 	u32 patcrc;
 	const char *patsetname;
 	de_finfo *fi = NULL;
 	de_ucstring *tmpname = NULL;
 	struct de_crcobj *crc32o;
+	int saved_indent_level;
 
-	pos += 4;
+	de_dbg_indent_save(c, &saved_indent_level);
+	de_dbg(c, "header (continued)");
+	de_dbg_indent(c, 1);
+	pos1 = d->hdr_pos + 4;
 
+	de_dbg(c, "brush patterns at %"I64_FMT, pos1);
+	de_dbg_indent(c, 1);
 	crc32o = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
-	de_crcobj_addslice(crc32o, c->infile, pos, 38*8);
+	de_crcobj_addslice(crc32o, c->infile, pos1, 38*8);
 	patcrc = de_crcobj_getval(crc32o);
 	de_crcobj_destroy(crc32o);
 	patsetname = get_pattern_set_info(patcrc, &is_blank);
 	de_dbg(c, "brush patterns crc: 0x%08x (%s)", (unsigned int)patcrc, patsetname);
 
+	rawimg = de_bitmap_create(c, 8, 38*8, 1);
+	rawimg->is_internal = 1;
+	de_convert_image_bilevel(c->infile, pos1, 1, rawimg, DE_CVTF_WHITEISZERO);
+
 	if(c->extract_level<2) {
 		goto done;
 	}
@@ -205,23 +340,33 @@ static void do_read_patterns(deark *c, lctx *d, i64 pos)
 		goto done;
 	}
 
-	pat = de_bitmap_create(c, (dispwidth+1)*19+1, (dispheight+1)*2+1, 1);
+	gallery = de_bitmap_create(c, (cell_width+1)*19+1, (cell_height+1)*2+1, 1);
 
-	for(cell=0; cell<38; cell++) {
-		xpos = (dispwidth+1)*(cell%19)+1;
-		ypos = (dispheight+1)*(cell/19)+1;
+	for(cell_idx=0; cell_idx<38; cell_idx++) {
+		i64 cell_x, cell_y; // dst cell pos in cells
+		i64 cell_xpos, cell_ypos; // dst cell pos in pixels
 
-		for(j=0; j<dispheight; j++) {
-			for(i=0; i<dispwidth; i++) {
-				// TODO: Figure out the proper "brush origin" of these patterns.
-				// Some of them may be shifted differently than MacPaint displays them.
-				x = de_get_bits_symbol(c->infile, 1, pos+cell*8+j%8, i%8);
+		cell_x = cell_idx%cells_per_row;
+		cell_y = cell_idx/cells_per_row;
 
-				// 0 = white. Only need to set the white pixels, since de_bitmap
-				// pixels default to black.
-				if(x==0) {
-					de_bitmap_setpixel_gray(pat, xpos+i, ypos+j, 255);
-				}
+		cell_xpos = (cell_width+1)*cell_x+1;
+		cell_ypos = (cell_height+1)*cell_y+1;
+
+		for(j=0; j<cell_height; j++) {
+			i64 yoffs;
+
+			// The cell sizes and "brush origin" are believed to be correct for
+			// at least one version of MacPaint, under some conditions.
+			yoffs = (cell_ypos+j+4)%8;
+
+			for(i=0; i<cell_width; i++) {
+				de_colorsample x;
+				i64 xoffs;
+
+				xoffs = (cell_xpos+i+7)%8;
+
+				x = DE_COLOR_K(de_bitmap_getpixel(rawimg, xoffs, 8*cell_idx + yoffs));
+				de_bitmap_setpixel_gray(gallery, cell_xpos+i, cell_ypos+j, x);
 			}
 		}
 	}
@@ -234,12 +379,14 @@ static void do_read_patterns(deark *c, lctx *d, i64 pos)
 	ucstring_append_sz(tmpname, "pat", DE_ENCODING_LATIN1);
 	fi = de_finfo_create(c);
 	de_finfo_set_name_from_ucstring(c, fi, tmpname, 0);
-	de_bitmap_write_to_file_finfo(pat, fi, DE_CREATEFLAG_IS_AUX);
+	de_bitmap_write_to_file_finfo(gallery, fi, DE_CREATEFLAG_IS_AUX|DE_CREATEFLAG_IS_BWIMG);
 
 done:
-	de_bitmap_destroy(pat);
+	de_bitmap_destroy(gallery);
+	de_bitmap_destroy(rawimg);
 	de_finfo_destroy(c, fi);
 	ucstring_destroy(tmpname);
+	de_dbg_indent_restore(c, saved_indent_level);
 }
 
 // Not many MacPaint-in-MacBinary files have a resource fork, but a few do.
@@ -271,7 +418,7 @@ static void do_macbinary(deark *c, lctx *d)
 	if(b0!=0) goto done;
 	if(b1<1 || b1>63) goto done;
 
-	de_dbg(c, "MacBinary header at %d", 0);
+	de_dbg(c, "MacBinary header");
 	de_dbg_indent(c, 1);
 	mparams = de_malloc(c, sizeof(de_module_params));
 	mparams->in_params.codes = "D"; // = decode only, don't extract
@@ -333,7 +480,8 @@ static u8 is_fmac2com(deark *c)
 static void de_run_macpaint(deark *c, de_module_params *mparams)
 {
 	lctx *d;
-	i64 pos;
+	struct validity_info *vi1 = NULL;
+	struct validity_info *vi2 = NULL;
 
 	d = de_malloc(c, sizeof(lctx));
 	d->has_macbinary_header = de_get_ext_option_bool(c, "macpaint:macbinary", -1);
@@ -349,17 +497,20 @@ static void de_run_macpaint(deark *c, de_module_params *mparams)
 	if(d->has_macbinary_header == -1) {
 		int v512;
 		int v640;
-		de_dbg(c, "trying to determine if file has a MacBinary header");
 
+		de_dbg(c, "trying to determine if file has a MacBinary header");
 		de_dbg_indent(c, 1);
-		de_dbg(c, "checking for image at offset 512");
-		de_dbg_indent(c, 1);
-		v512 = valid_file_at(c, d, 0);
-		de_dbg_indent(c, -1);
-		de_dbg(c, "checking for image at offset 640");
-		de_dbg_indent(c, 1);
-		v640 = valid_file_at(c, d, 128);
-		de_dbg_indent(c, -1);
+
+		vi1 = de_malloc(c, sizeof(struct validity_info));
+		vi1->pos = 0;
+		test_valid_image(c, d, vi1);
+		v512 = vi1->retval;
+
+		vi2 = de_malloc(c, sizeof(struct validity_info));
+		vi2->pos = 128;
+		test_valid_image(c, d, vi2);
+		v640 = vi2->retval;
+
 		de_dbg_indent(c, -1);
 
 		if(v512 > v640) {
@@ -388,22 +539,30 @@ static void de_run_macpaint(deark *c, de_module_params *mparams)
 	else
 		de_declare_fmt(c, "MacPaint without MacBinary header");
 
-	pos = 0;
+	if(d->has_macbinary_header) {
+		d->hdr_pos = 128;
+	}
+	else {
+		d->hdr_pos = 0;
+	}
+	d->img_data_pos = d->hdr_pos + 512;
+
 	if(d->has_macbinary_header) {
 		do_macbinary(c, d);
-		pos += 128;
 	}
 
-	do_read_bitmap(c, d, pos);
+	do_read_bitmap(c, d);
 
 	if(!d->is_fmac2com) {
-		do_read_patterns(c, d, pos);
+		do_read_patterns(c, d);
 	}
 
 	if(d) {
 		ucstring_destroy(d->filename);
 		de_free(c, d);
 	}
+	de_free(c, vi1);
+	de_free(c, vi2);
 }
 
 // Note: This must be coordinated with the macbinary detection routine.
