@@ -27,6 +27,7 @@ struct icns_stream_item {
 
 typedef struct localctx_struct {
 	u8 extract_raw;
+	u8 combine_SICN;
 	i64 data_offs, map_offs;
 	i64 data_size, map_size;
 
@@ -317,26 +318,80 @@ done:
 	de_finfo_destroy(c, fi);
 }
 
-// SICN - Small icons - One or more 16x16 images
-static void do_SICN_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
-	struct rsrcinstanceinfo *rii, i64 pos1, i64 len)
+// Write each small icon in an SICN set to an individual file.
+static void do_SICN_resource_1(deark *c, lctx *d, struct rsrctypeinfo *rti,
+	struct rsrcinstanceinfo *rii, i64 pos1, i64 numicons)
 {
 	de_finfo *fi = NULL;
-	i64 numicons;
+	de_bitmap *img = NULL;
 	i64 i;
 
-	numicons = len/32;
 	fi = de_finfo_create(c);
 	set_resource_filename(c, d, fi, rti, rii, NULL);
+	img = de_bitmap_create(c, 16, 16, 1);
+
+	for(i=0; i<numicons; i++) {
+		de_bitmap_rect(img, 0, 0, img->width, img->height, DE_STOCKCOLOR_BLACK, 0);
+		de_convert_image_bilevel(c->infile, pos1+32*i, 2, img, DE_CVTF_WHITEISZERO);
+		de_bitmap_write_to_file_finfo(img, fi, DE_CREATEFLAG_IS_BWIMG);
+	}
+
+	de_bitmap_destroy(img);
+	de_finfo_destroy(c, fi);
+}
+
+// Write all icons in an SICN set to a single "gallery" file.
+static void do_SICN_resource_gallery(deark *c, lctx *d, struct rsrctypeinfo *rti,
+	struct rsrcinstanceinfo *rii, i64 pos1, i64 numicons)
+{
+#define SICN_PER_ROW 16
+	de_finfo *fi = NULL;
+	i64 ncols, nrows;
+	i64 canvas_w, canvas_h;
+	de_bitmap *cnv = NULL;
+	i64 i;
+
+	fi = de_finfo_create(c);
+	set_resource_filename(c, d, fi, rti, rii, "SICN_gallery");
+
+	ncols = (numicons<SICN_PER_ROW) ? numicons : SICN_PER_ROW;
+	nrows = (numicons+SICN_PER_ROW-1) / SICN_PER_ROW;
+	canvas_w = 1 + ncols*17;
+	canvas_h = 1 + nrows*17;
+
+	cnv = de_bitmap_create(c, canvas_w, canvas_h, 3);
+	de_bitmap_rect(cnv, 0, 0, cnv->width, cnv->height, DE_MAKE_RGB(32,32,144), 0);
 
 	for(i=0; i<numicons; i++) {
 		de_bitmap *img = NULL;
 
 		img = de_bitmap_create(c, 16, 16, 1);
 		de_convert_image_bilevel(c->infile, pos1+32*i, 2, img, DE_CVTF_WHITEISZERO);
-		de_bitmap_write_to_file_finfo(img, fi, 0);
+		de_bitmap_copy_rect(img, cnv, 0, 0, 16, 16,
+			1 + 17*(i%SICN_PER_ROW), 1 + 17*(i/SICN_PER_ROW), 0);
 		de_bitmap_destroy(img);
 	}
+
+	de_bitmap_write_to_file_finfo(cnv, fi, 0);
+
+	de_bitmap_destroy(cnv);
+	de_finfo_destroy(c, fi);
+}
+
+// SICN - Small icons - One or more 16x16 images
+static void do_SICN_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
+	struct rsrcinstanceinfo *rii, i64 pos1, i64 len)
+{
+	i64 numicons;
+
+	numicons = len/32;
+	de_dbg(c, "num icons: %"I64_FMT, numicons);
+	if(numicons>1 && d->combine_SICN) {
+		do_SICN_resource_gallery(c, d, rti, rii, pos1, numicons);
+	}
+	else {
+		do_SICN_resource_1(c, d, rti, rii, pos1, numicons);
+	}
 }
 
 static void do_cicn_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
@@ -357,7 +412,9 @@ static void do_cicn_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
 	i64 pos = dpos;
 	int needmsg = 1;
 	int ok = 0;
+	int saved_indent_level;
 
+	de_dbg_indent_save(c, &saved_indent_level);
 	bi_fgcolor = de_malloc(c, sizeof(struct fmtutil_macbitmap_info));
 	bi_mask = de_malloc(c, sizeof(struct fmtutil_macbitmap_info));
 	bi_bw = de_malloc(c, sizeof(struct fmtutil_macbitmap_info));
@@ -372,6 +429,15 @@ static void do_cicn_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
 	pos += 10;
 	fmtutil_macbitmap_read_pixmap_only_fields(c, c->infile, bi_fgcolor, pos);
 	pos += 36;
+
+	// Some images have 0 in one or both of these fields.
+	if(bi_fgcolor->cmpcount==0) {
+		bi_fgcolor->cmpcount = 1;
+	}
+	if(bi_fgcolor->cmpsize==0) {
+		bi_fgcolor->cmpsize = bi_fgcolor->pixelsize;
+	}
+
 	de_dbg_indent(c, -1);
 
 	de_dbg(c, "[mask bitmap header]");
@@ -402,6 +468,7 @@ static void do_cicn_resource(deark *c, lctx *d, struct rsrctypeinfo *rti,
 	if(bw_exists && !de_good_image_dimensions_noerr(c, bi_bw->npwidth, bi_bw->height)) goto done;
 
 	if(bi_fgcolor->pixeltype!=0) goto done;
+	// TODO: pixsize=8 w/ cmpsize=4 is possibly legal
 	if(bi_fgcolor->pixelsize!=bi_fgcolor->cmpsize) goto done;
 	if(bi_fgcolor->cmpcount!=1) goto done;
 	if(bi_fgcolor->pixelsize!=1 && bi_fgcolor->pixelsize!=2 &&
@@ -472,6 +539,7 @@ done:
 	de_free(c, bi_mask);
 	de_free(c, bi_bw);
 	de_finfo_destroy(c, fi);
+	de_dbg_indent_restore(c, saved_indent_level);
 }
 
 static int looks_like_pict(deark *c, lctx *d, struct rsrcinstanceinfo *rii,
@@ -618,6 +686,15 @@ static void do_resource_name(deark *c, lctx *d, struct rsrcinstanceinfo *rii)
 	de_dbg(c, "name: \"%s\"", ucstring_getpsz_d(rii->name ));
 }
 
+static const char *get_ownertype_name(UI n)
+{
+	static const char *names[8] = { "driver/desk accessory",
+		"window def", "menu def", "widget def", "printer driver",
+		"package", "control panel", "?" };
+
+	return names[n%8];
+}
+
 static void do_resource_record(deark *c, lctx *d, struct rsrctypeinfo *rti,
 	i64 pos1, UI idx)
 {
@@ -630,6 +707,21 @@ static void do_resource_record(deark *c, lctx *d, struct rsrctypeinfo *rti,
 	rii.idx = idx;
 	rii.id = (int)de_geti16be_p(&pos);
 	de_dbg(c, "id: %d", rii.id);
+	if(rii.id>=(-16384) && rii.id<=(-1)) {
+		UI u;
+		UI ownertype, ownerid, subid;
+
+		u = (UI)(rii.id + 65536);
+		ownertype = (u & 0xf800) >> 11;
+		ownerid = (u & 0x07e0) >> 5;
+		subid = (u & 0x001f);
+		de_dbg_indent(c, 1);
+		de_dbg(c, "owner type: %u (%s)", ownertype, get_ownertype_name(ownertype));
+		de_dbg(c, "owner id: %u", ownerid);
+		de_dbg(c, "sub id: %u", subid);
+		de_dbg_indent(c, -1);
+	}
+
 	nameOffset_rel = de_getu16be_p(&pos);
 	if(nameOffset_rel!=0xffff) {
 		rii.has_name = 1;
@@ -772,7 +864,7 @@ static void do_map(deark *c, lctx *d, i64 map_offs, i64 map_size)
 	do_type_list(c, d);
 
 done:
-	de_dbg_indent(c, 1);
+	de_dbg_indent(c, -1);
 }
 
 static void de_run_macrsrc(deark *c, de_module_params *mparams)
@@ -791,6 +883,7 @@ static void de_run_macrsrc(deark *c, de_module_params *mparams)
 	if(de_get_ext_option(c, "macrsrc:extractraw")) {
 		d->extract_raw = 1;
 	}
+	d->combine_SICN = (u8)de_get_ext_option_bool(c, "macrsrc:combinesicn", 0);
 
 	if(c->infile->len<16) {
 		de_err(c, "%sFile too small to be a valid Resource file", d->errmsgprefix);
@@ -835,6 +928,7 @@ static int de_identify_macrsrc(deark *c)
 static void de_help_macrsrc(deark *c)
 {
 	de_msg(c, "-opt macrsrc:extractraw : Extract all resources to files");
+	de_msg(c, "-opt macrsrc:combinesicn : Put 'small icon' sets in a single image");
 }
 
 void de_module_macrsrc(deark *c, struct deark_module_info *mi)
