@@ -78,6 +78,7 @@ done:
 
 static int de_identify_bld(deark *c)
 {
+	if(de_getbyte(2)>=0x80) return 0; // Assume height can't be negative
 	if(de_input_file_has_ext(c, "bld")) return 20;
 	// TODO: We could try to test if the dimensions are sane, but we'd risk
 	// getting it wrong, because we probably don't know what every edition of
@@ -117,6 +118,7 @@ static void de_run_megapaint_pat(deark *c, de_module_params *mparams)
 		i64 imgpos_x, imgpos_y;
 
 		img = de_bitmap_create(c, 32, 32, 1);
+		img->is_internal = 1;
 		de_convert_image_bilevel(c->infile, pos, 4, img, DE_CVTF_WHITEISZERO);
 		pos += 4*32;
 
