@@ -8,25 +8,30 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_copy);
 DE_DECLARE_MODULE(de_module_null);
+DE_DECLARE_MODULE(de_module_join);
 DE_DECLARE_MODULE(de_module_split);
 DE_DECLARE_MODULE(de_module_xor);
 DE_DECLARE_MODULE(de_module_plaintext);
 DE_DECLARE_MODULE(de_module_cp437);
 DE_DECLARE_MODULE(de_module_crc);
+DE_DECLARE_MODULE(de_module_datetime);
 DE_DECLARE_MODULE(de_module_hexdump);
 DE_DECLARE_MODULE(de_module_bytefreq);
 DE_DECLARE_MODULE(de_module_deflate);
 DE_DECLARE_MODULE(de_module_zlib);
-DE_DECLARE_MODULE(de_module_winzle);
 DE_DECLARE_MODULE(de_module_mrw);
-DE_DECLARE_MODULE(de_module_vgafont);
-DE_DECLARE_MODULE(de_module_pcrfont);
 DE_DECLARE_MODULE(de_module_zbr);
 DE_DECLARE_MODULE(de_module_compress);
 DE_DECLARE_MODULE(de_module_hpi);
 DE_DECLARE_MODULE(de_module_dclimplode);
+DE_DECLARE_MODULE(de_module_lgcompress);
 DE_DECLARE_MODULE(de_module_lzss_oku);
 DE_DECLARE_MODULE(de_module_lzhuf);
+DE_DECLARE_MODULE(de_module_compress_lzh);
+DE_DECLARE_MODULE(de_module_lzstac);
+DE_DECLARE_MODULE(de_module_npack);
+DE_DECLARE_MODULE(de_module_lzs221);
+DE_DECLARE_MODULE(de_module_xpk);
 
 // **************************************************************************
 // "copy" module
@@ -65,6 +70,52 @@ void de_module_null(deark *c, struct deark_module_info *mi)
 	mi->flags |= DE_MODFLAG_NOEXTRACT;
 }
 
+// **************************************************************************
+// "join"
+//
+// This is basically a multi-part example module.
+// **************************************************************************
+
+static void de_run_join(deark *c, de_module_params *mparams)
+{
+	int k;
+	int num_parts = 1;
+	dbuf *outf = NULL;
+
+	if(c->mp_data) {
+		num_parts += c->mp_data->count;
+	}
+
+	outf = dbuf_create_output_file(c, "bin", NULL, 0);
+
+	for(k=0; k<num_parts; k++) {
+		dbuf *inf;
+
+		if(k>0 && c->mp_data) {
+			de_dbg(c, "[mp file %d: %s]", k-1, c->mp_data->item[k-1].fn);
+		}
+		inf = de_mp_acquire_dbuf(c, k);
+		if(!inf) {
+			goto done;
+		}
+
+		dbuf_copy(inf, 0, inf->len, outf);
+
+		de_mp_release_dbuf(c, k, &inf);
+	}
+
+done:
+	dbuf_close(outf);
+}
+
+void de_module_join(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "join";
+	mi->desc = "Concatenate files";
+	mi->run_fn = de_run_join;
+	mi->flags |= DE_MODFLAG_MULTIPART;
+}
+
 // **************************************************************************
 // split
 // Split the input file into equal-sized chunks.
@@ -376,50 +427,102 @@ void de_module_cp437(deark *c, struct deark_module_info *mi)
 // Prints various CRCs and checksums. Does not create any files.
 // **************************************************************************
 
+struct crcm_alg_info {
+	UI crctype;
+	u8 flags; // 1 = computed by default
+	u8 nbits; // 0 = special
+	const char *name;
+};
+
+#define CRCM_NUMCRCS 11
+static const struct crcm_alg_info crcm_map[CRCM_NUMCRCS] = {
+	{ DE_CRCOBJ_CRC32_IEEE, 1, 32, "CRC-32-IEEE" },
+	{ DE_CRCOBJ_CRC16_ARC, 1, 16, "CRC-16/ARC"},
+	{ DE_CRCOBJ_CRC16_XMODEM, 1, 16, "CRC-16/XMODEM" },
+	{ DE_CRCOBJ_CRC32_JAMCRC, 0, 32, "CRC-32/JAMCRC" },
+	{ DE_CRCOBJ_CRC32_PL, 0, 32, "CRC-32/PL" },
+	{ DE_CRCOBJ_ADLER32, 0, 32, "Adler-32" },
+	{ DE_CRCOBJ_CRC16_IBMSDLC, 0, 16, "CRC-16/IBM-SDLC" },
+	{ DE_CRCOBJ_CRC16_IBM3740, 0, 16, "CRC-16/IBM-3740" },
+	{ DE_CRCOBJ_SUM_BYTES, 1, 0, "Sum of bytes" },
+	{ DE_CRCOBJ_SUM_U16LE, 0, 0, "Sum of uint16-LE" },
+	{ DE_CRCOBJ_SUM_U16BE, 0, 0, "Sum of uint16-BE" }
+};
+
 struct crcctx_struct {
-	struct de_crcobj *crco_32ieee;
-	struct de_crcobj *crco_16arc;
-	struct de_crcobj *crco_16ccitt;
-	u64 sum_of_bytes;
+	u8 opt_all;
+	struct de_crcobj *crcos[CRCM_NUMCRCS];
 };
 
 static int crc_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
 	i64 buf_len)
 {
 	struct crcctx_struct *crcctx = (struct crcctx_struct*)brctx->userdata;
-	i64 i;
+	size_t n;
 
-	de_crcobj_addbuf(crcctx->crco_32ieee, buf, buf_len);
-	de_crcobj_addbuf(crcctx->crco_16arc, buf, buf_len);
-	de_crcobj_addbuf(crcctx->crco_16ccitt, buf, buf_len);
-	for(i=0; i<buf_len; i++) {
-		crcctx->sum_of_bytes += buf[i];
+	for(n=0; n<CRCM_NUMCRCS; n++) {
+		if(!crcctx->crcos[n]) continue;
+		de_crcobj_addbuf(crcctx->crcos[n], buf, buf_len);
 	}
+
 	return 1;
 }
 
 static void de_run_crc(deark *c, de_module_params *mparams)
 {
 	struct crcctx_struct crcctx;
+	size_t n;
 
 	de_zeromem(&crcctx, sizeof(struct crcctx_struct));
-	crcctx.crco_32ieee = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
-	crcctx.crco_16arc = de_crcobj_create(c, DE_CRCOBJ_CRC16_ARC);
-	crcctx.crco_16ccitt = de_crcobj_create(c, DE_CRCOBJ_CRC16_XMODEM);
+	crcctx.opt_all = (u8)de_get_ext_option_bool(c, "crc:all", 0xff);
+	if(crcctx.opt_all==0xff) {
+		if(c->extract_level>=2) {
+			crcctx.opt_all = 1;
+		}
+		else {
+			crcctx.opt_all = 0;
+		}
+	}
+
+	for(n=0; n<CRCM_NUMCRCS; n++) {
+		if((crcm_map[n].flags & 0x1)==0 && !crcctx.opt_all) continue;
+		crcctx.crcos[n] = de_crcobj_create(c, crcm_map[n].crctype);
+	}
 
 	dbuf_buffered_read(c->infile, 0, c->infile->len, crc_cbfn, (void*)&crcctx);
 
-	de_msg(c, "CRC-32-IEEE: 0x%08x",
-		(unsigned int)de_crcobj_getval(crcctx.crco_32ieee));
-	de_msg(c, "CRC-16-IBM/ARC: 0x%04x",
-		(unsigned int)de_crcobj_getval(crcctx.crco_16arc));
-	de_msg(c, "CRC-16-CCITT: 0x%04x",
-		(unsigned int)de_crcobj_getval(crcctx.crco_16ccitt));
-	de_msg(c, "Sum of bytes: 0x%"U64_FMTx, crcctx.sum_of_bytes);
+	for(n=0; n<CRCM_NUMCRCS; n++) {
+		u32 val = 0;
+		u64 val64 = 0;
 
-	de_crcobj_destroy(crcctx.crco_32ieee);
-	de_crcobj_destroy(crcctx.crco_16arc);
-	de_crcobj_destroy(crcctx.crco_16ccitt);
+		if(!crcctx.crcos[n]) continue;
+
+		if(crcm_map[n].nbits==0) {
+			val64 = de_crcobj_getval64(crcctx.crcos[n]);
+		}
+		else {
+			val = de_crcobj_getval(crcctx.crcos[n]);
+		}
+
+		if(crcm_map[n].nbits==0) {
+			de_msg(c, "%-18s: 0x%"U64_FMTx, crcm_map[n].name, val64);
+		}
+		else if(crcm_map[n].nbits==16) {
+			de_msg(c, "%-18s: 0x%04x", crcm_map[n].name, (UI)val);
+		}
+		else {
+			de_msg(c, "%-18s: 0x%08x", crcm_map[n].name, (UI)val);
+		}
+	}
+
+	for(n=0; n<CRCM_NUMCRCS; n++) {
+		de_crcobj_destroy(crcctx.crcos[n]);
+	}
+}
+
+static void de_help_crc(deark *c)
+{
+	de_msg(c, "-a : Also compute uncommon checksum types");
 }
 
 void de_module_crc(deark *c, struct deark_module_info *mi)
@@ -428,9 +531,64 @@ void de_module_crc(deark *c, struct deark_module_info *mi)
 	mi->id_alias[0] = "crc32";
 	mi->desc = "Calculate various CRCs";
 	mi->run_fn = de_run_crc;
+	mi->help_fn = de_help_crc;
 	mi->flags |= DE_MODFLAG_NOEXTRACT;
 }
 
+// **************************************************************************
+// datetime
+// Interpret the start of the file as a date/time field, in various formats.
+// **************************************************************************
+
+static void datetime_msg(deark *c, struct de_timestamp *ts, const char *name)
+{
+	char timestamp_buf[64];
+
+	de_timestamp_to_string(ts, timestamp_buf, sizeof(timestamp_buf), 0);
+	de_msg(c, "%s: %s", name, timestamp_buf);
+}
+
+static void de_run_datetime(deark *c, de_module_params *mparams)
+{
+	struct de_timestamp timestamp;
+	i64 dtime, ddate;
+	i64 t;
+
+	dtime = de_getu16le(0);
+	ddate = de_getu16le(2);
+	de_dos_datetime_to_timestamp(&timestamp, ddate, dtime);
+	datetime_msg(c, &timestamp, "DOS time,date");
+
+	ddate = de_getu16le(0);
+	dtime = de_getu16le(2);
+	de_dos_datetime_to_timestamp(&timestamp, ddate, dtime);
+	datetime_msg(c, &timestamp, "DOS date,time");
+
+	t = de_geti32le(0);
+	de_unix_time_to_timestamp(t, &timestamp, 0x1);
+	datetime_msg(c, &timestamp, "Unix-LE");
+
+	t = de_geti32be(0);
+	de_unix_time_to_timestamp(t, &timestamp, 0x1);
+	datetime_msg(c, &timestamp, "Unix-BE");
+
+	t = de_getu32be(0);
+	de_mac_time_to_timestamp(t, &timestamp);
+	datetime_msg(c, &timestamp, "Mac/HFS-BE");
+
+	t = de_geti64le(0);
+	de_FILETIME_to_timestamp(t, &timestamp, 0x1);
+	datetime_msg(c, &timestamp, "Windows FILETIME");
+}
+
+void de_module_datetime(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "datetime";
+	mi->desc = "Interpret a date/time field";
+	mi->run_fn = de_run_datetime;
+	mi->flags |= DE_MODFLAG_HIDDEN | DE_MODFLAG_NOEXTRACT;
+}
+
 // **************************************************************************
 // hexdump
 // Prints a hex dump. Does not create any files.
@@ -648,53 +806,6 @@ void de_module_zlib(deark *c, struct deark_module_info *mi)
 	mi->identify_fn = de_identify_zlib;
 }
 
-// **************************************************************************
-// Winzle! puzzle image
-// **************************************************************************
-
-static void de_run_winzle(deark *c, de_module_params *mparams)
-{
-	u8 buf[256];
-	i64 xorsize;
-	i64 i;
-	dbuf *f = NULL;
-
-	xorsize = c->infile->len >= 256 ? 256 : c->infile->len;
-	de_read(buf, 0, xorsize);
-	for(i=0; i<xorsize; i++) {
-		buf[i] ^= 0x0d;
-	}
-
-	f = dbuf_create_output_file(c, "bmp", NULL, 0);
-	dbuf_write(f, buf, xorsize);
-	if(c->infile->len > 256) {
-		dbuf_copy(c->infile, 256, c->infile->len - 256, f);
-	}
-	dbuf_close(f);
-}
-
-static int de_identify_winzle(deark *c)
-{
-	u8 b[18];
-	de_read(b, 0, sizeof(b));
-
-	if(b[0]==0x4f && b[1]==0x40) {
-		if(b[14]==0x25 && b[15]==0x0d && b[16]==0x0d && b[17]==0x0d) {
-			return 95;
-		}
-		return 40;
-	}
-	return 0;
-}
-
-void de_module_winzle(deark *c, struct deark_module_info *mi)
-{
-	mi->id = "winzle";
-	mi->desc = "Winzle! puzzle image";
-	mi->run_fn = de_run_winzle;
-	mi->identify_fn = de_identify_winzle;
-}
-
 // **************************************************************************
 // Minolta RAW (MRW)
 // **************************************************************************
@@ -742,155 +853,6 @@ void de_module_mrw(deark *c, struct deark_module_info *mi)
 	mi->identify_fn = de_identify_mrw;
 }
 
-// **************************************************************************
-// 8xN "VGA" font (intended for development/debugging use)
-// **************************************************************************
-
-static void do_vgafont_internal(deark *c, dbuf *inf)
-{
-	u8 *fontdata = NULL;
-	struct de_bitmap_font *font = NULL;
-	i64 i;
-	i64 height;
-	de_encoding encoding_req;
-	de_encoding encoding_to_use;
-	struct de_encconv_state es;
-
-#define VGAFONT_MINH 3
-#define VGAFONT_MAXH 20
-	height = inf->len / 256;
-	if((inf->len % 256) || height<VGAFONT_MINH || height>VGAFONT_MAXH) {
-		de_err(c, "Bad file size");
-		goto done;
-	}
-
-	fontdata = de_malloc(c, height*256);
-	dbuf_read(inf, fontdata, 0, height*256);
-
-	if(de_get_ext_option(c, "vgafont:c")) {
-		dbuf *ff;
-		ff = dbuf_create_output_file(c, "h", NULL, 0);
-		for(i=0; i<(height*256); i++) {
-			if(i%height==0) dbuf_puts(ff, "\t");
-			dbuf_printf(ff, "%d", (int)fontdata[i]);
-			if(i!=(height*256-1)) dbuf_puts(ff, ",");
-			if(i%height==(height-1)) dbuf_puts(ff, "\n");
-		}
-		dbuf_close(ff);
-		goto done;
-	}
-
-	encoding_req = de_get_input_encoding(c, NULL, DE_ENCODING_UNKNOWN);
-	if(encoding_req!=DE_ENCODING_UNKNOWN)
-		encoding_to_use = encoding_req;
-	else
-		encoding_to_use = DE_ENCODING_CP437;
-
-	font = de_create_bitmap_font(c);
-	font->num_chars = 256;
-	font->has_nonunicode_codepoints = 1;
-	font->has_unicode_codepoints = 1;
-	font->prefer_unicode = (encoding_req!=DE_ENCODING_UNKNOWN);
-	font->nominal_width = 8;
-	font->nominal_height = (int)height;
-	font->char_array = de_mallocarray(c, font->num_chars, sizeof(struct de_bitmap_font_char));
-
-	de_encconv_init(&es, encoding_to_use);
-
-	for(i=0; i<font->num_chars; i++) {
-		font->char_array[i].codepoint_nonunicode = (i32)i;
-		if(font->has_unicode_codepoints) {
-			font->char_array[i].codepoint_unicode = de_char_to_unicode_ex((i32)i, &es);
-		}
-		font->char_array[i].width = font->nominal_width;
-		font->char_array[i].height = font->nominal_height;
-		font->char_array[i].rowspan = 1;
-		font->char_array[i].bitmap = &fontdata[i*font->nominal_height];
-	}
-
-	de_font_bitmap_font_to_image(c, font, NULL, 0);
-
-done:
-	if(font) {
-		de_free(c, font->char_array);
-		de_destroy_bitmap_font(c, font);
-	}
-	de_free(c, fontdata);
-}
-
-static void de_run_vgafont(deark *c, de_module_params *mparams)
-{
-	do_vgafont_internal(c, c->infile);
-}
-
-static void de_help_vgafont(deark *c)
-{
-	de_msg(c, "-opt vgafont:c : Emit C code");
-}
-
-void de_module_vgafont(deark *c, struct deark_module_info *mi)
-{
-	mi->id = "vgafont";
-	mi->desc = "Raw 8xN bitmap font";
-	mi->run_fn = de_run_vgafont;
-	mi->help_fn = de_help_vgafont;
-}
-
-// **************************************************************************
-// PCR font (OPTICKS)
-// **************************************************************************
-
-static void de_run_pcrfont(deark *c, de_module_params *mparams)
-{
-	u8 hdr[11];
-	i64 h;
-	int need_errmsg = 0;
-	dbuf *inf = NULL;
-
-	de_read(hdr, 0, 11);
-	// I assume either hdr[7] or hdr[10] is the high byte of the font data size,
-	// but I don't know which.
-	if(hdr[6]!=0x1 || hdr[7]!=hdr[10] || hdr[8]!=0 || hdr[9]!=0) {
-		need_errmsg = 1;
-		goto done;
-	}
-	h = (i64)hdr[7];
-	de_dbg(c, "height: %d", (int)h);
-	if(h<VGAFONT_MINH || h>VGAFONT_MAXH || c->infile->len < 11+h*256) {
-		need_errmsg = 1;
-		goto done;
-	}
-
-	inf = dbuf_open_input_subfile(c->infile, 11, h*256);
-	do_vgafont_internal(c, inf);
-
-done:
-	dbuf_close(inf);
-	if(need_errmsg) {
-		de_err(c, "Unsupported type of PCR font");
-	}
-}
-
-static int de_identify_pcrfont(deark *c)
-{
-	u8 h;
-
-	if(dbuf_memcmp(c->infile, 0, "KPG", 3)) return 0;
-	if(de_getbyte(5)!=0x20) return 0;
-	h = de_getbyte(7);
-	if(h<6 || h>16) return 0;
-	if(de_getbyte(10)!=h) return 0;
-	return 80;
-}
-
-void de_module_pcrfont(deark *c, struct deark_module_info *mi)
-{
-	mi->id = "pcrfont";
-	mi->desc = "PCR font";
-	mi->run_fn = de_run_pcrfont;
-	mi->identify_fn = de_identify_pcrfont;
-}
-
 // **************************************************************************
 // ZBR (Zoner Zebra Metafile)
 // **************************************************************************
@@ -948,6 +910,7 @@ static void de_run_compress(deark *c, de_module_params *mparams)
 	dbuf *f = NULL;
 
 	f = dbuf_create_output_file(c, "bin", NULL, 0);
+	dbuf_enable_wbuffer(f);
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
@@ -961,6 +924,7 @@ static void de_run_compress(deark *c, de_module_params *mparams)
 	delzwp.flags |= DE_LZWFLAG_HAS3BYTEHEADER;
 
 	fmtutil_decompress_lzw(c, &dcmpri, &dcmpro, &dres, &delzwp);
+	dbuf_flush(f);
 	if(dres.errcode!=0) {
 		de_err(c, "%s", de_dfilter_get_errmsg(c, &dres));
 	}
@@ -1030,7 +994,8 @@ void de_module_hpi(deark *c, struct deark_module_info *mi)
 // PKWARE DCL Implode compressed file
 // **************************************************************************
 
-static void de_run_dclimplode(deark *c, de_module_params *mparams)
+static void dclimplode_main(deark *c, i64 cmpr_pos, i64 cmpr_len,
+	u8 orig_len_known, i64 orig_len)
 {
 	dbuf *outf = NULL;
 	struct de_dfilter_in_params dcmpri;
@@ -1041,9 +1006,13 @@ static void de_run_dclimplode(deark *c, de_module_params *mparams)
 	dbuf_enable_wbuffer(outf);
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = c->infile;
-	dcmpri.pos = 0;
-	dcmpri.len = c->infile->len;
+	dcmpri.pos = cmpr_pos;
+	dcmpri.len = cmpr_len;
 	dcmpro.f = outf;
+	if(orig_len_known) {
+		dcmpro.len_known = 1;
+		dcmpro.expected_len = orig_len;
+	}
 
 	fmtutil_dclimplode_codectype1(c, &dcmpri, &dcmpro, &dres, NULL);
 	dbuf_flush(outf);
@@ -1054,13 +1023,18 @@ static void de_run_dclimplode(deark *c, de_module_params *mparams)
 	dbuf_close(outf);
 }
 
+static void de_run_dclimplode(deark *c, de_module_params *mparams)
+{
+	dclimplode_main(c, 0, c->infile->len, 0, 0);
+}
+
 static int de_identify_dclimplode(deark *c)
 {
 	u8 b0, b1;
 	int i;
 	u32 x;
 
-	if(c->infile->len<5) return 0;
+	if(c->infile->len<4) return 0;
 	b0 = de_getbyte(0);
 	if(b0>1) return 0;
 	b1 = de_getbyte(1);
@@ -1071,7 +1045,7 @@ static int de_identify_dclimplode(deark *c)
 	// no extraneous bytes after that.
 	x = (u32)de_getu32le(c->infile->len-4);
 	for(i=0; i<8; i++) {
-		if((x & 0xfffffc00U)==0x01fe0000U) {
+		if((x & 0xfffffe00U)==0x01fe0200U) {
 			if(b0==0 && b1==6) return 40;
 			return 10;
 		}
@@ -1089,6 +1063,41 @@ void de_module_dclimplode(deark *c, struct deark_module_info *mi)
 	mi->identify_fn = de_identify_dclimplode;
 }
 
+// **************************************************************************
+// Logitech Compress / LGEXPAND (v2)
+// **************************************************************************
+
+static void de_run_lgcompress(deark *c, de_module_params *mparams)
+{
+	u8 mfnc;
+	i64 orig_len;
+
+	mfnc = de_getbyte(2);
+	if(mfnc>=0x33 && mfnc<=126) {
+		de_dbg(c, "missing filename char: '%c'", (int)mfnc);
+	}
+	orig_len = de_getu32le(4);
+	de_dbg(c, "orig len: %"I64_FMT, orig_len);
+
+	dclimplode_main(c, 8, c->infile->len-8, 1, orig_len);
+}
+
+static int de_identify_lgcompress(deark *c)
+{
+	if(de_getu16be(0) != 0xdafa) return 0;
+	if(de_getu16be(8) != 0x0006) return 0;
+	// TODO?: We could do more checks, especially at EOF.
+	return 80;
+}
+
+void de_module_lgcompress(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lgcompress";
+	mi->desc = "Logitech Compress";
+	mi->run_fn = de_run_lgcompress;
+	mi->identify_fn = de_identify_lgcompress;
+}
+
 // **************************************************************************
 // LZSS (Haruhiko Okumura) compressed file
 // **************************************************************************
@@ -1170,3 +1179,220 @@ void de_module_lzhuf(deark *c, struct deark_module_info *mi)
 	mi->desc = "LZHUF compressed file";
 	mi->run_fn = de_run_lzhuf;
 }
+
+// **************************************************************************
+// SCO compress LZH
+// **************************************************************************
+
+static void de_run_compress_lzh(deark *c, de_module_params *mparams)
+{
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+	struct de_lh5x_params lzhparams;
+
+#define SCOLZH_HDRSIZE 2
+	if(c->infile->len<SCOLZH_HDRSIZE) goto done;
+	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = SCOLZH_HDRSIZE;
+	dcmpri.len = c->infile->len-SCOLZH_HDRSIZE;
+	dcmpro.f = outf;
+
+	de_zeromem(&lzhparams, sizeof(struct de_lh5x_params));
+	lzhparams.fmt = DE_LH5X_FMT_LH5;
+	lzhparams.zero_codes_block_behavior = DE_LH5X_ZCB_STOP;
+
+	fmtutil_decompress_lh5x(c, &dcmpri, &dcmpro, &dres, &lzhparams);
+	dbuf_flush(outf);
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
+	}
+done:
+	dbuf_close(outf);
+}
+
+static int de_identify_compress_lzh(deark *c)
+{
+	if((UI)de_getu16be(0) != 0x1fa0) return 0;
+	if(de_input_file_has_ext(c, "z")) return 90;
+	return 10;
+}
+
+void de_module_compress_lzh(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "compress_lzh";
+	mi->desc = "SCO compress LZH";
+	mi->run_fn = de_run_compress_lzh;
+	mi->identify_fn = de_identify_compress_lzh;
+}
+
+// **************************************************************************
+// Raw LZS (Stac) compressed data
+// **************************************************************************
+
+static void do_lzstac_internal(deark *c, i64 cmpr_pos, i64 cmpr_len, UI flags)
+{
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+	struct de_lzstac_params lzstacparams;
+
+	outf = dbuf_create_output_file(c, "unc", NULL, 0);
+	dbuf_enable_wbuffer(outf);
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = cmpr_pos;
+	dcmpri.len = cmpr_len;
+	dcmpro.f = outf;
+
+	de_zeromem(&lzstacparams, sizeof(struct de_lzstac_params));
+	lzstacparams.flags = flags;
+	fmtutil_lzstac_codectype1(c, &dcmpri, &dcmpro, &dres, (void*)&lzstacparams);
+
+	dbuf_flush(outf);
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", de_dfilter_get_errmsg(c, &dres));
+		goto done;
+	}
+	if(dres.bytes_consumed_valid) {
+		de_dbg(c, "decompressed %"I64_FMT" bytes to %"I64_FMT, dres.bytes_consumed,
+			outf->len);
+		if(dres.bytes_consumed<cmpr_len) {
+			de_warn(c, "%"I64_FMT" extra bytes at end of file. File might not "
+				"have decompressed correctly.", cmpr_len - dres.bytes_consumed);
+		}
+	}
+
+done:
+	dbuf_close(outf);
+}
+
+static void de_run_lzstac(deark *c, de_module_params *mparams)
+{
+	do_lzstac_internal(c, 0, c->infile->len, 0);
+}
+
+void de_module_lzstac(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lzstac";
+	mi->desc = "Raw LZS (Stac) compressed data";
+	mi->run_fn = de_run_lzstac;
+}
+
+// **************************************************************************
+// NPack
+// An installer format by Symantec & Stac
+// **************************************************************************
+
+static void de_run_npack(deark *c, de_module_params *mparams)
+{
+	// NPack decompresses at most 1 LZS block, hence the 0x2 flag.
+	do_lzstac_internal(c, 5, c->infile->len-5, 0x2);
+}
+
+static int de_identify_npack(deark *c)
+{
+	if(dbuf_memcmp(c->infile, 0, (const void*)"MSTSM", 5)) {
+		return 0;
+	}
+	return 85;
+}
+
+void de_module_npack(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "npack";
+	mi->desc = "NPack compressed file";
+	mi->run_fn = de_run_npack;
+	mi->identify_fn = de_identify_npack;
+}
+
+// **************************************************************************
+// LZS221
+// e.g. LZSDEMO v3.1 by Stac
+// **************************************************************************
+
+static void de_run_lzs221(deark *c, de_module_params *mparams)
+{
+	do_lzstac_internal(c, 4, c->infile->len-4, 0);
+}
+
+static int de_identify_lzs221(deark *c)
+{
+	if(dbuf_memcmp(c->infile, 0, (const void*)"sTaC", 4)) {
+		return 0;
+	}
+	return 85;
+}
+
+void de_module_lzs221(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "lzs221";
+	mi->desc = "LZS221 compressed file";
+	mi->run_fn = de_run_lzs221;
+	mi->identify_fn = de_identify_lzs221;
+}
+
+// **************************************************************************
+// XPK compressed file format (XPKF)
+// **************************************************************************
+
+#define CODE_XPKF 0x58504b46U
+
+static void xpkf_internal(deark *c)
+{
+	dbuf *tmpoutf = NULL;
+	dbuf *outf = NULL;
+	struct de_dfilter_in_params dcmpri;
+	struct de_dfilter_out_params dcmpro;
+	struct de_dfilter_results dres;
+
+	tmpoutf = dbuf_create_membuf(c, 0, 0);
+
+	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
+	dcmpri.f = c->infile;
+	dcmpri.pos = 0;
+	dcmpri.len = c->infile->len;
+	dcmpro.f = tmpoutf;
+
+	fmtutil_xpk_codectype1(c, &dcmpri, &dcmpro, &dres, NULL);
+	dbuf_flush(tmpoutf);
+
+	if(dres.errcode==0 || tmpoutf->len>0) {
+		outf = dbuf_create_output_file(c, "unc", NULL, 0);
+		dbuf_copy(tmpoutf, 0, tmpoutf->len, outf);
+	}
+
+	if(dres.errcode) {
+		de_err(c, "Decompression failed: %s", dres.errmsg);
+		goto done;
+	}
+
+done:
+	dbuf_close(outf);
+	dbuf_close(tmpoutf);
+}
+
+static void de_run_xpk(deark *c, de_module_params *mparams)
+{
+	xpkf_internal(c);
+}
+
+static int de_identify_xpk(deark *c)
+{
+	if((u32)de_getu32be(0)!=CODE_XPKF) return 0;
+	return 85;
+}
+
+void de_module_xpk(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "xpk";
+	mi->desc = "XPK compressed file";
+	mi->run_fn = de_run_xpk;
+	mi->identify_fn = de_identify_xpk;
+	mi->flags |= DE_MODFLAG_HIDDEN;
+}
