@@ -74,7 +74,8 @@ static int do_ape_binary_item(deark *c, struct ape_tag_header_footer *ah,
 	i64 nbytes_to_scan;
 	i64 img_pos, img_len;
 	de_finfo *fi = NULL;
-	char *ext = NULL;
+	struct fmtutil_fmtid_ctx *idctx = NULL;
+	const char *ext = NULL;
 	int retval = 0;
 
 	if(de_strncasecmp(key->sz, "cover art", 9)) {
@@ -92,18 +93,20 @@ static int do_ape_binary_item(deark *c, struct ape_tag_header_footer *ah,
 	if(len < 16) goto done;
 
 	fi = de_finfo_create(c);
+
 	if(c->filenames_from_file) {
 		de_finfo_set_name_from_ucstring(c, fi, name->str, 0);
 	}
 	else {
-		u8 sig[2];
-
 		de_finfo_set_name_from_sz(c, fi, "cover_art", 0, DE_ENCODING_LATIN1);
 
-		de_read(sig, img_pos, 2);
-		if(sig[0]==0x89 && sig[1]==0x50) ext="png";
-		else if(sig[0]==0xff && sig[1]==0xd8) ext="jpg";
-		else ext="bin";
+		idctx = de_malloc(c, sizeof(struct fmtutil_fmtid_ctx));
+		idctx->inf = c->infile;
+		idctx->inf_pos = img_pos;
+		idctx->inf_len = img_len;
+		idctx->mode = FMTUTIL_FMTIDMODE_ALL_IMG; // expecting PNG or JPEG
+		fmtutil_fmtid(c, idctx);
+		ext = idctx->ext_sz;
 	}
 	dbuf_create_file_from_slice(c->infile, img_pos, img_len, ext,
 		fi, DE_CREATEFLAG_IS_AUX);
@@ -112,6 +115,7 @@ static int do_ape_binary_item(deark *c, struct ape_tag_header_footer *ah,
 done:
 	de_finfo_destroy(c, fi);
 	de_destroy_stringreaderdata(c, name);
+	de_free(c, idctx);
 	return retval;
 }
 
