@@ -3,29 +3,38 @@
 // See the file COPYING for terms of use.
 
 // MS-DOS installation compression (compress.exe, expand.exe, MSLZ, etc.)
+// and some related formats
 
 #include <deark-private.h>
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_mscompress);
+DE_DECLARE_MODULE(de_module_is_ibt);
+DE_DECLARE_MODULE(de_module_mrnz);
 
 #define FMT_SZDD 1
 #define FMT_KWAJ 2
+#define FMT_SZ   3
 
 #define CMPR_NONE    0
 #define CMPR_XOR     1
-#define CMPR_SZDD    2
+#define CMPR_LZSS18  2 // Used by KWAJ:2 and SZ
 #define CMPR_LZHUFF  3
 #define CMPR_MSZIP   4
+#define CMPR_LZSS16  65536 // Used by SZDD
 
-typedef struct localctx_struct {
+typedef struct localctx_mscompress {
 	int fmt;
 	int input_encoding;
+	u8 errflag;
 	UI cmpr_meth;
 	i64 cmpr_data_pos;
 	i64 cmpr_data_len;
 	u8 uncmpr_len_known;
 	i64 uncmpr_len;
 	de_ucstring *filename;
+	de_finfo *fi_override; // Do not free; this is a copy.
+	dbuf *inf; // Input file to use (this is a copy)
+	dbuf *combined_inf;
 } lctx;
 
 static int cmpr_meth_is_supported(lctx *d, UI n)
@@ -33,7 +42,8 @@ static int cmpr_meth_is_supported(lctx *d, UI n)
 	switch(n) {
 	case CMPR_NONE:
 	case CMPR_XOR:
-	case CMPR_SZDD:
+	case CMPR_LZSS16:
+	case CMPR_LZSS18:
 	case CMPR_LZHUFF:
 	case CMPR_MSZIP:
 		return 1;
@@ -48,7 +58,7 @@ static const char *get_cmpr_meth_name(UI n)
 	switch(n) {
 	case CMPR_NONE: name="uncompressed"; break;
 	case CMPR_XOR: name="XOR"; break;
-	case CMPR_SZDD: name="SZDD"; break;
+	case CMPR_LZSS16: case CMPR_LZSS18: name="LZSS"; break;
 	case CMPR_LZHUFF: name="LZ+Huffman"; break;
 	case CMPR_MSZIP: name="MSZIP"; break;
 	}
@@ -67,20 +77,20 @@ static int do_header_SZDD(deark *c, lctx *d, i64 pos1)
 	de_dbg_indent(c, 1);
 
 	d->cmpr_data_pos = 14;
-	d->cmpr_data_len = c->infile->len - d->cmpr_data_pos;
+	d->cmpr_data_len = d->inf->len - d->cmpr_data_pos;
 
 	pos += 8; // signature
 
-	cmpr_mode = de_getbyte(pos++);
+	cmpr_mode = dbuf_getbyte_p(d->inf, &pos);
 	de_dbg(c, "compression mode: 0x%02x ('%c')", (unsigned int)cmpr_mode,
 		de_byte_to_printable_char(cmpr_mode));
 	if(cmpr_mode != 0x41) {
 		de_err(c, "Unsupported compression mode");
 		goto done;
 	}
-	d->cmpr_meth = CMPR_SZDD;
+	d->cmpr_meth = CMPR_LZSS16;
 
-	fnchar = de_getbyte(pos++);
+	fnchar = dbuf_getbyte_p(d->inf, &pos);
 	if(fnchar>=32 && fnchar<=126) {
 		de_snprintf(tmps, sizeof(tmps), " ('%c')", fnchar);
 	}
@@ -92,10 +102,38 @@ static int do_header_SZDD(deark *c, lctx *d, i64 pos1)
 	}
 	de_dbg(c, "missing filename char: 0x%02x%s", (unsigned int)fnchar, tmps);
 
-	d->uncmpr_len = de_getu32le(pos);
+	d->uncmpr_len = dbuf_getu32le_p(d->inf, &pos);
 	d->uncmpr_len_known = 1;
-	de_dbg(c, "uncompressed len: %"I64_FMT"", d->uncmpr_len);
-	//pos += 4;
+	de_dbg(c, "uncompressed len: %"I64_FMT, d->uncmpr_len);
+
+	retval = 1;
+done:
+	de_dbg_indent(c, -1);
+	return retval;
+}
+
+static int do_header_SZ(deark *c, lctx *d, i64 pos1)
+{
+	u8 fragment_id;
+	i64 pos = pos1;
+	int retval = 0;
+
+	de_dbg(c, "header at %d", (int)pos);
+	de_dbg_indent(c, 1);
+
+	pos += 7; // signature
+	fragment_id = dbuf_getbyte_p(d->inf, &pos);
+	if(fragment_id!=0xd1) {
+		de_err(c, "First file is not volume 1");
+		goto done;
+	}
+
+	d->cmpr_meth = CMPR_LZSS18;
+	d->uncmpr_len = dbuf_getu32le_p(d->inf, &pos);
+	d->uncmpr_len_known = 1;
+	de_dbg(c, "uncompressed len: %"I64_FMT, d->uncmpr_len);
+	d->cmpr_data_pos = pos;
+	d->cmpr_data_len = d->inf->len - d->cmpr_data_pos;
 
 	retval = 1;
 done:
@@ -117,43 +155,43 @@ static int do_header_KWAJ(deark *c, lctx *d, i64 pos1)
 
 	pos += 8; // signature
 
-	d->cmpr_meth = (UI)de_getu16le_p(&pos);
+	d->cmpr_meth = (UI)dbuf_getu16le_p(d->inf, &pos);
 	de_dbg(c, "compression method: %u (%s)", d->cmpr_meth, get_cmpr_meth_name(d->cmpr_meth));
 
-	d->cmpr_data_pos = de_getu16le_p(&pos);
+	d->cmpr_data_pos = dbuf_getu16le_p(d->inf, &pos);
 	de_dbg(c, "compressed data offset: %"I64_FMT, d->cmpr_data_pos);
-	d->cmpr_data_len = c->infile->len - d->cmpr_data_pos;
+	d->cmpr_data_len = d->inf->len - d->cmpr_data_pos;
 
-	flags = (UI)de_getu16le_p(&pos);
+	flags = (UI)dbuf_getu16le_p(d->inf, &pos);
 	de_dbg(c, "header extension flags: 0x%04x", flags);
 
 	if(flags & 0x0001) { // bit 0
-		d->uncmpr_len = de_getu32le_p(&pos);
+		d->uncmpr_len = dbuf_getu32le_p(d->inf, &pos);
 		d->uncmpr_len_known = 1;
-		de_dbg(c, "uncompressed len: %"I64_FMT"", d->uncmpr_len);
+		de_dbg(c, "uncompressed len: %"I64_FMT, d->uncmpr_len);
 	}
 	if(flags & 0x0002) { // bit 1
 		pos += 2;
 	}
 	if(flags & 0x0004) { // bit 2
-		n = de_getu16le_p(&pos);
+		n = dbuf_getu16le_p(d->inf, &pos);
 		pos += n;
 	}
 	if(flags & 0x0008) { // bit 3, base part of filename
 		foundpos = 0;
-		ret = dbuf_search_byte(c->infile, 0x00, pos, 9, &foundpos);
+		ret = dbuf_search_byte(d->inf, 0x00, pos, 9, &foundpos);
 		if(!ret) goto header_extensions_done;
 		d->filename = ucstring_create(c);
-		dbuf_read_to_ucstring(c->infile, pos, foundpos-pos, d->filename, 0, d->input_encoding);
+		dbuf_read_to_ucstring(d->inf, pos, foundpos-pos, d->filename, 0, d->input_encoding);
 		pos = foundpos+1;
 	}
 	if(flags & 0x0010) { // bit 4, filename extension
 		foundpos = 0;
-		ret = dbuf_search_byte(c->infile, 0x00, pos, 4, &foundpos);
+		ret = dbuf_search_byte(d->inf, 0x00, pos, 4, &foundpos);
 		if(!ret) goto header_extensions_done;
 		if(d->filename && (foundpos-pos > 0)) {
 			ucstring_append_char(d->filename, '.');
-			dbuf_read_to_ucstring(c->infile, pos, foundpos-pos, d->filename, 0, d->input_encoding);
+			dbuf_read_to_ucstring(d->inf, pos, foundpos-pos, d->filename, 0, d->input_encoding);
 		}
 		pos = foundpos+1;
 	}
@@ -625,7 +663,7 @@ static void do_decompress(deark *c, lctx *d, dbuf *outf)
 	struct de_dfilter_results dres;
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
-	dcmpri.f = c->infile;
+	dcmpri.f = d->inf;
 	dcmpri.pos = d->cmpr_data_pos;
 	dcmpri.len = d->cmpr_data_len;
 
@@ -640,7 +678,10 @@ static void do_decompress(deark *c, lctx *d, dbuf *outf)
 	case CMPR_XOR:
 		do_decompress_XOR(c, &dcmpri, &dcmpro, &dres);
 		break;
-	case CMPR_SZDD:
+	case CMPR_LZSS18:
+		fmtutil_decompress_lzss1(c, &dcmpri, &dcmpro, &dres, 0x0);
+		break;
+	case CMPR_LZSS16:
 		fmtutil_decompress_lzss1(c, &dcmpri, &dcmpro, &dres, 0x1);
 		break;
 	case CMPR_LZHUFF:
@@ -693,7 +734,7 @@ static void do_extract_file(deark *c, lctx *d)
 	else {
 		de_finfo_set_name_from_sz(c, fi, "bin", 0, DE_ENCODING_LATIN1);
 	}
-	outf = dbuf_create_output_file(c, NULL, fi, 0);
+	outf = dbuf_create_output_file(c, NULL, (d->fi_override ? d->fi_override : fi), 0);
 	dbuf_enable_wbuffer(outf);
 	do_decompress(c, d, outf);
 	de_dbg_indent(c, -1);
@@ -710,34 +751,91 @@ static int detect_fmt_internal(deark *c)
 	de_read(buf, 0, sizeof(buf));
 	if(!de_memcmp(buf, "\x53\x5a\x44\x44\x88\xf0\x27\x33", 8))
 		return FMT_SZDD;
-
 	if(!de_memcmp(buf, "\x4b\x57\x41\x4a\x88\xf0\x27\xd1", 8))
 		return FMT_KWAJ;
+	if(!de_memcmp(buf, "SZ \x88\xf0\x27\x33", 7))
+		return FMT_SZ;
 	return 0;
 }
 
+static void mscompress_handle_multipart(deark *c, lctx *d)
+{
+	int k;
+
+	if(!c->mp_data) goto done;
+
+	if(d->fmt!=FMT_SZ) {
+		de_err(c, "Multi-part archives not supported for this format");
+		d->errflag = 1;
+		goto done;
+	}
+
+	d->combined_inf = dbuf_create_membuf(c, c->infile->len, 0);
+	d->inf = d->combined_inf;
+	dbuf_copy(c->infile, 0, c->infile->len, d->inf);
+	for(k=0; k<c->mp_data->count; k++) {
+		u8 v;
+		dbuf *inf_part;
+
+		inf_part = de_mp_acquire_dbuf(c, k+1);
+		if(!inf_part) {
+			d->errflag = 1;
+			goto done;
+		}
+
+		v = dbuf_getbyte(inf_part, 7);
+		if((int)v != k+2) {
+			de_err(c, "(part %d) Bad file or wrong order", k+2);
+			d->errflag = 1;
+			goto done;
+		}
+
+		dbuf_copy(inf_part, 8, inf_part->len-8, d->inf);
+		de_mp_release_dbuf(c, k+1, &inf_part);
+	}
+
+done:
+	;
+}
+
 static void de_run_mscompress(deark *c, de_module_params *mparams)
 {
 	lctx *d = NULL;
+	const char *varname;
 
 	d = de_malloc(c, sizeof(lctx));
 	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
 
 	d->fmt = detect_fmt_internal(c);
 	if(d->fmt==FMT_SZDD) {
-		de_declare_fmt(c, "MS Installation Compression, SZDD variant");
+		varname = "SZDD";
 	}
 	else if(d->fmt==FMT_KWAJ) {
-		de_declare_fmt(c, "MS Installation Compression, KWAJ variant");
+		varname = "KWAJ";
+	}
+	else if(d->fmt==FMT_SZ) {
+		varname = "SZ";
 	}
 	else {
 		de_err(c, "Unidentified format");
 		goto done;
 	}
+	de_declare_fmtf(c, "MS Installation Compression, %s variant", varname);
+
+	if(mparams && mparams->in_params.fi) {
+		d->fi_override = mparams->in_params.fi;
+	}
+
+	d->inf = c->infile;
+	mscompress_handle_multipart(c, d);
+	if(d->errflag) goto done;
 
 	if(d->fmt==FMT_KWAJ) {
 		if(!do_header_KWAJ(c, d, 0)) goto done;
 	}
+	else if(d->fmt==FMT_SZ) {
+		if(!do_header_SZ(c, d, 0)) goto done;
+	}
 	else {
 		if(!do_header_SZDD(c, d, 0)) goto done;
 	}
@@ -747,6 +845,7 @@ static void de_run_mscompress(deark *c, de_module_params *mparams)
 done:
 	if(d) {
 		ucstring_destroy(d->filename);
+		dbuf_close(d->combined_inf);
 		de_free(c, d);
 	}
 }
@@ -765,4 +864,224 @@ void de_module_mscompress(deark *c, struct deark_module_info *mi)
 	mi->desc = "MS-DOS Installation Compression";
 	mi->run_fn = de_run_mscompress;
 	mi->identify_fn = de_identify_mscompress;
+	mi->flags |= DE_MODFLAG_MULTIPART;
+}
+
+// **************************************************************************
+// InstallShield setup.ibt
+// A container for SZDD-compressed files.
+// **************************************************************************
+
+struct ibt_member_ctx {
+	i64 cmpr_pos;
+	i64 cmpr_len;
+	de_ucstring *tmpstr;
+	de_ucstring *cmpr_fn;
+	de_ucstring *orig_fn;
+};
+
+struct ibt_ctx {
+	de_encoding input_encoding;
+	u8 need_errmsg;
+	i64 last_member_size;
+};
+
+// maxlen: max length including NUL byte
+static int read_sz_p(dbuf *f, i64 maxlen, de_ucstring *s, de_encoding enc, i64 *ppos)
+{
+	i64 startpos = *ppos;
+	i64 foundpos;
+	i64 len;
+	int ret;
+
+	ret = dbuf_search_byte(f, 0x00, startpos, maxlen, &foundpos);
+	if(!ret) return 0;
+	len = foundpos - startpos; // length w/o NUL
+	dbuf_read_to_ucstring(f, startpos, len, s, 0, enc);
+	*ppos = foundpos+1;
+	return 1;
+}
+
+static void ibt_decompress_and_extract(deark *c, struct ibt_ctx *d,
+	struct ibt_member_ctx *md)
+{
+	de_module_params *mparams = NULL;
+	de_finfo *fi = NULL;
+
+	de_dbg(c, "SZDD data at %"I64_FMT", size=%"I64_FMT, md->cmpr_pos, md->cmpr_len);
+	de_dbg_indent(c, 1);
+
+	fi = de_finfo_create(c);
+	de_finfo_set_name_from_ucstring(c, fi, md->orig_fn, 0);
+	fi->original_filename_flag = 1;
+
+	mparams = de_malloc(c, sizeof(de_module_params));
+	mparams->in_params.fi = fi;
+
+	de_run_module_by_id_on_slice(c, "mscompress", mparams, c->infile,
+		md->cmpr_pos, md->cmpr_len);
+
+	de_free(c, mparams);
+	de_finfo_destroy(c, fi);
+	de_dbg_indent(c, -1);
+}
+
+// Sets d->last_member_size. On fatal error, sets it to 0.
+static void do_ibt_member(deark *c, struct ibt_ctx *d, i64 pos1)
+{
+	struct ibt_member_ctx *md = NULL;
+	i64 pos = pos1;
+
+	d->last_member_size = 0;
+
+	md = de_malloc(c, sizeof(struct ibt_member_ctx));
+	md->tmpstr = ucstring_create(c);
+	md->cmpr_fn = ucstring_create(c);
+	md->orig_fn = ucstring_create(c);
+
+	if(!read_sz_p(c->infile, 80, md->cmpr_fn, d->input_encoding, &pos)) {
+		d->need_errmsg = 1;
+		goto done;
+	}
+	de_dbg(c, "cmpr name: \"%s\"", ucstring_getpsz_d(md->cmpr_fn));
+
+	if(!read_sz_p(c->infile, 80, md->orig_fn, d->input_encoding, &pos)) {
+		d->need_errmsg = 1;
+		goto done;
+	}
+	de_dbg(c, "orig name: \"%s\"", ucstring_getpsz_d(md->orig_fn));
+
+	if(!read_sz_p(c->infile, 80, md->tmpstr, DE_ENCODING_ASCII, &pos)) {
+		d->need_errmsg = 1;
+		goto done;
+	}
+	de_dbg(c, "file version: \"%s\"", ucstring_getpsz_d(md->tmpstr));
+
+	ucstring_empty(md->tmpstr);
+	if(!read_sz_p(c->infile, 80, md->tmpstr, DE_ENCODING_ASCII, &pos)) {
+		d->need_errmsg = 1;
+		goto done;
+	}
+	md->cmpr_pos = pos;
+	md->cmpr_len = de_atoi64(ucstring_getpsz(md->tmpstr));
+	de_dbg(c, "cmpr len: %"I64_FMT, md->cmpr_len);
+	if(md->cmpr_len<8 || md->cmpr_pos+md->cmpr_len>c->infile->len) {
+		d->need_errmsg = 1;
+		goto done;
+	}
+
+	d->last_member_size = md->cmpr_pos + md->cmpr_len - pos1;
+
+	// TODO? An option to extract without decompressing.
+	// But we'd lose the original filename, and it's overkill for such an
+	// unimportant format.
+
+	ibt_decompress_and_extract(c, d, md);
+
+done:
+	if(md) {
+		ucstring_destroy(md->tmpstr);
+		ucstring_destroy(md->cmpr_fn);
+		ucstring_destroy(md->orig_fn);
+		de_free(c, md);
+	}
+}
+
+static void de_run_is_ibt(deark *c, de_module_params *mparams)
+{
+	struct ibt_ctx *d = NULL;
+	i64 pos = 0;
+
+	d = de_malloc(c, sizeof(struct ibt_ctx));
+	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_WINDOWS1252);
+
+	while(1) {
+		if(pos+16 > c->infile->len) break;
+		de_dbg(c, "member at %"I64_FMT, pos);
+		de_dbg_indent(c, 1);
+		do_ibt_member(c, d, pos);
+		de_dbg_indent(c, -1);
+		if(d->last_member_size<=0) goto done;
+		pos += d->last_member_size;
+	}
+
+done:
+	if(d) {
+		if(d->need_errmsg) {
+			de_err(c, "Bad or unsupported IBT file");
+		}
+		de_free(c, d);
+	}
+}
+
+static int de_identify_is_ibt(deark *c)
+{
+	// TODO? Better identification
+	if(!dbuf_memcmp(c->infile, 0, (const void*)"setup.dl_\0setup.dll\0", 20)) {
+		return 90;
+	}
+	return 0;
+}
+
+void de_module_is_ibt(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "is_ibt";
+	mi->desc = "InstallShield IBT archive";
+	mi->run_fn = de_run_is_ibt;
+	mi->identify_fn = de_identify_is_ibt;
+}
+
+// **************************************************************************
+// MRNZ
+// Obfuscated file used by a PC DOS installer
+// **************************************************************************
+
+static void de_run_mrnz(deark *c, de_module_params *mparams)
+{
+	dbuf *outf = NULL;
+	i64 ipos;
+	i64 i;
+	i64 ocount;
+	u8 errflag = 0;
+
+	ipos = 8;
+	// Guessing that this is the number of obfuscated bytes
+	ocount = de_getu32le_p(&ipos);
+	if(ocount>0xff) {
+		errflag = 1;
+		goto done;
+	}
+
+	outf = dbuf_create_output_file(c, "bin", NULL, 0);
+	for(i=0; i<ocount; i++) {
+		u8 b;
+
+		if(ipos >= c->infile->len) goto done;
+		b = de_getbyte_p(&ipos);
+		dbuf_writebyte(outf, b^0xae);
+	}
+
+	dbuf_copy(c->infile, ipos, c->infile->len-ipos, outf);
+
+done:
+	dbuf_close(outf);
+	if(errflag) {
+		de_err(c, "Failed to decode MRNZ file");
+	}
+}
+
+static int de_identify_mrnz(deark *c)
+{
+	if(!dbuf_memcmp(c->infile, 0, (const void*)"MRNZ\x88\xf0\x27\x33", 8)) {
+		return 100;
+	}
+	return 0;
+}
+
+void de_module_mrnz(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "mrnz";
+	mi->desc = "MRNZ obfuscated file";
+	mi->run_fn = de_run_mrnz;
+	mi->identify_fn = de_identify_mrnz;
 }
