@@ -35,6 +35,8 @@ struct page_ctx {
 	int has_custom_pal;
 	unsigned int cmpr_type_field;
 	enum palm_cmpr_type cmpr_type;
+	i64 endpos;
+	i64 expected_num_uncmpr_image_bytes;
 	u32 custom_pal[256];
 };
 
@@ -83,6 +85,7 @@ static void do_decompress_scanline_compression(deark *c, lctx *d, struct page_ct
 	i64 j;
 	i64 blocknum;
 	i64 blocksperrow;
+	i64 nbytes_written = 0;
 	u8 bf;
 	u8 dstb;
 	unsigned int k;
@@ -92,6 +95,8 @@ static void do_decompress_scanline_compression(deark *c, lctx *d, struct page_ct
 	for(j=0; j<pg->h; j++) {
 		i64 bytes_written_this_row = 0;
 
+		if(nbytes_written >= dcmpro->expected_len) goto done;
+
 		for(blocknum=0; blocknum<blocksperrow; blocknum++) {
 			// For each byte-per-row, we expect a lead byte, which is a
 			// bitfield that tells us which of the next 8 bytes are stored
@@ -111,9 +116,13 @@ static void do_decompress_scanline_compression(deark *c, lctx *d, struct page_ct
 				dbuf_writebyte(dcmpro->f, dstb);
 
 				bytes_written_this_row++;
+				nbytes_written++;
 			}
 		}
 	}
+done:
+	dres->bytes_consumed = srcpos - dcmpri->pos;
+	dres->bytes_consumed_valid = 1;
 }
 
 // Note that this is distinct from ImageViewer RLE compression.
@@ -121,15 +130,21 @@ static void do_decompress_rle_compression(deark *c, struct de_dfilter_in_params
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres)
 {
 	i64 srcpos = dcmpri->pos;
+	i64 nbytes_written = 0;
 
 	while(srcpos <= (dcmpri->pos + dcmpri->len - 2)) {
 		i64 count;
 		u8 val;
 
+		if(nbytes_written > dcmpro->expected_len) goto done;
 		count = (i64)dbuf_getbyte_p(dcmpri->f, &srcpos);
 		val = dbuf_getbyte_p(dcmpri->f, &srcpos);
 		dbuf_write_run(dcmpro->f, val, count);
+		nbytes_written += count;
 	}
+done:
+	dres->bytes_consumed = srcpos - dcmpri->pos;
+	dres->bytes_consumed_valid = 1;
 }
 
 static void make_stdpal256(deark *c, lctx *d, u32 *stdpal)
@@ -155,18 +170,17 @@ static void make_stdpal256(deark *c, lctx *d, u32 *stdpal)
 	}
 }
 
-static void do_generate_unc_image(deark *c, lctx *d, struct page_ctx *pg,
+static void do_generate_img_from_unc_pixels(deark *c, lctx *d, struct page_ctx *pg,
 	dbuf *unc_pixels)
 {
 	i64 i, j;
 	i64 pdwidth;
-	u8 b;
-	u8 b_adj;
 	u32 clr;
 	int has_color;
 	de_bitmap *img = NULL;
-	u32 stdpal[256];
+	u32 pal[256];
 
+	de_zeromem(pal, sizeof(pal));
 	has_color = (pg->bitsperpixel>4 || pg->has_custom_pal);
 
 	if(pg->bitsperpixel==1 && !has_color) {
@@ -175,17 +189,15 @@ static void do_generate_unc_image(deark *c, lctx *d, struct page_ctx *pg,
 		goto done;
 	}
 
-	make_stdpal256(c, d, stdpal);
-
 	pdwidth = (pg->rowbytes*8) / pg->bitsperpixel;
 	if(pdwidth<pg->w) pdwidth = pg->w;
 
 	img = de_bitmap_create2(c, pg->w, pdwidth, pg->h,
 		(has_color?3:1) + (pg->has_trns?1:0));
 
-	for(j=0; j<pg->h; j++) {
-		for(i=0; i<pdwidth; i++) {
-			if(pg->bitsperpixel==16) {
+	if(pg->bitsperpixel==16) {
+		for(j=0; j<pg->h; j++) {
+			for(i=0; i<pdwidth; i++) {
 				u32 clr1;
 				clr1 = (u32)dbuf_getu16be(unc_pixels, pg->rowbytes*j + 2*i);
 				clr = de_rgb565_to_888(clr1);
@@ -194,30 +206,30 @@ static void do_generate_unc_image(deark *c, lctx *d, struct page_ctx *pg,
 					de_bitmap_setsample(img, i, j, 3, 0);
 				}
 			}
-			else {
-				b = de_get_bits_symbol(unc_pixels, pg->bitsperpixel, pg->rowbytes*j, i);
-				if(has_color) {
-					if(pg->has_custom_pal)
-						clr = pg->custom_pal[(unsigned int)b];
-					else
-						clr = stdpal[(unsigned int)b];
-				}
-				else {
-					// TODO: What are the correct colors (esp. for 4bpp)?
-					b_adj = 255 - de_sample_nbit_to_8bit(pg->bitsperpixel, (unsigned int)b);
-					clr = DE_MAKE_GRAY(b_adj);
-				}
-
-				de_bitmap_setpixel_rgb(img, i, j, clr);
-
-				if(pg->has_trns && (u32)b==pg->trns_value) {
-					de_bitmap_setsample(img, i, j, 3, 0);
-				}
-			}
 		}
 	}
+	else {
+		if(!has_color) {
+			if(pg->bitsperpixel>8) goto done;
+			// TODO: What are the correct colors (esp. for 4bpp)?
+			de_make_grayscale_palette(pal, 1ULL<<pg->bitsperpixel, 0x1);
+		}
+		else if(pg->has_custom_pal) {
+			de_memcpy(pal, pg->custom_pal, sizeof(pg->custom_pal));
+		}
+		else {
+			make_stdpal256(c, d, pal);
+		}
 
-	de_bitmap_write_to_file(img, NULL, 0);
+		if(pg->has_trns && pg->trns_value<256) {
+			pal[pg->trns_value] = DE_SET_ALPHA(pal[pg->trns_value], 0);
+		}
+
+		de_convert_image_paletted(unc_pixels, 0,pg->bitsperpixel, pg->rowbytes,
+			pal, img, 0);
+	}
+
+	de_bitmap_write_to_file(img, NULL, DE_CREATEFLAG_OPT_IMAGE);
 
 done:
 	de_bitmap_destroy(img);
@@ -230,27 +242,33 @@ static int de_decompress_image(deark *c, lctx *d, struct page_ctx *pg,
 	struct de_dfilter_out_params dcmpro;
 	struct de_dfilter_results dres;
 	int retval = 0;
+	i64 n;
 
 	de_dfilter_init_objects(c, &dcmpri, &dcmpro, &dres);
 	dcmpri.f = inf;
 	dcmpri.pos = pos;
 	dcmpri.len = len;
 	dcmpro.f = unc_pixels;
+	dcmpro.expected_len = pg->expected_num_uncmpr_image_bytes;
+	dcmpro.len_known = 1;
 
 	if(pg->cmpr_type==PCMPR_SCANLINE) {
 		do_decompress_scanline_compression(c, d, pg, &dcmpri, &dcmpro, &dres);
 	}
 	else if(pg->cmpr_type==PCMPR_RLE) {
+		dbuf_enable_wbuffer(unc_pixels);
 		do_decompress_rle_compression(c, &dcmpri, &dcmpro, &dres);
 	}
 	else if(pg->cmpr_type==PCMPR_PACKBITS8) {
+		dbuf_enable_wbuffer(unc_pixels);
 		fmtutil_decompress_packbits_ex(c, &dcmpri, &dcmpro, &dres, NULL);
 	}
 	else if(pg->cmpr_type==PCMPR_PACKBITS16) {
 			struct de_packbits_params pbparams;
 
 			de_zeromem(&pbparams, sizeof(struct de_packbits_params));
-			pbparams.is_packbits16 = 1;
+			pbparams.nbytes_per_unit = 2;
+			dbuf_enable_wbuffer(unc_pixels);
 			fmtutil_decompress_packbits_ex(c, &dcmpri, &dcmpro, &dres, &pbparams);
 	}
 	else {
@@ -258,13 +276,21 @@ static int de_decompress_image(deark *c, lctx *d, struct page_ctx *pg,
 		goto done;
 	}
 
+	dbuf_flush(unc_pixels);
+
 	if(dres.errcode) {
 		de_err(c, "%s", de_dfilter_get_errmsg(c, &dres));
 		goto done;
 	}
 
-	// TODO: The byte counts in this message are not very accurate.
-	de_dbg(c, "decompressed %"I64_FMT" bytes to %"I64_FMT" bytes", len,
+	if(dres.bytes_consumed_valid) {
+		n = dres.bytes_consumed;
+	}
+	else {
+		n = len;
+	}
+
+	de_dbg(c, "decompressed %"I64_FMT" bytes to %"I64_FMT" bytes", n,
 		unc_pixels->len);
 	retval = 1;
 
@@ -272,18 +298,19 @@ done:
 	return retval;
 }
 
-// A wrapper that decompresses the image if necessary, then calls do_generate_unc_image().
+// A wrapper that decompresses the image if necessary, then calls
+// do_generate_img_from_unc_pixels().
 static void do_generate_image(deark *c, lctx *d, struct page_ctx *pg,
 	dbuf *inf, i64 pos, i64 len)
 {
 	dbuf *unc_pixels = NULL;
-	i64 expected_num_uncmpr_image_bytes;
 
-	expected_num_uncmpr_image_bytes = pg->rowbytes*pg->h;
+	pg->expected_num_uncmpr_image_bytes = pg->rowbytes*pg->h;
 
 	if(pg->cmpr_type==PCMPR_NONE) {
-		if(expected_num_uncmpr_image_bytes > len) {
-			de_warn(c, "Not enough data for image");
+		if(pg->expected_num_uncmpr_image_bytes > len) {
+			de_err(c, "Not enough data for image");
+			goto done;
 		}
 		unc_pixels = dbuf_open_input_subfile(inf, pos, len);
 	}
@@ -299,24 +326,21 @@ static void do_generate_image(deark *c, lctx *d, struct page_ctx *pg,
 			hdr_len = 2;
 			cmpr_len = dbuf_getu16x(inf, pos, d->is_le);
 		}
-		de_dbg(c, "cmpr len: %d", (int)cmpr_len);
-		if(cmpr_len < len) {
-			// Reduce the number of available bytes, based on the cmpr_len field.
-			len = cmpr_len;
-		}
+		de_dbg(c, "cmpr len: %d", (int)cmpr_len); // Note - We don't trust this field
+
 		// Account for the size of the cmpr_len field.
 		pos += hdr_len;
 		len -= hdr_len;
 		if(len<0) goto done;
 
-		unc_pixels = dbuf_create_membuf(c, expected_num_uncmpr_image_bytes, 1);
+		unc_pixels = dbuf_create_membuf(c, pg->expected_num_uncmpr_image_bytes, 1);
 
 		if(!de_decompress_image(c, d, pg, inf, pos, len, unc_pixels)) {
 			goto done;
 		}
 	}
 
-	do_generate_unc_image(c, d, pg, unc_pixels);
+	do_generate_img_from_unc_pixels(c, d, pg, unc_pixels);
 
 done:
 	dbuf_close(unc_pixels);
@@ -452,9 +476,9 @@ static void do_palm_BitmapType_internal(deark *c, lctx *d, i64 pos1, i64 len,
 	de_dbg_indent_save(c, &saved_indent_level);
 	pg = de_malloc(c, sizeof(struct page_ctx));
 
-	de_dbg(c, "BitmapType at %d, len"DE_CHAR_LEQ"%d", (int)pos1, (int)len);
+	de_dbg(c, "BitmapType at %"I64_FMT, pos1);
 	de_dbg_indent(c, 1);
-	de_dbg(c, "bitmap header at %d", (int)pos1);
+	de_dbg(c, "bitmap header at %"I64_FMT, pos1);
 	de_dbg_indent(c, 1);
 
 	// Look ahead to get the version
@@ -584,7 +608,7 @@ static void do_palm_BitmapType_internal(deark *c, lctx *d, i64 pos1, i64 len,
 		densitycode = dbuf_getu16x(c->infile, pos1+14, d->is_le);
 		de_dbg(c, "density: %d", (int)densitycode);
 		// The density is an indication of the target screen density.
-		// It's tempting to interpet it as pixels per inch, and copy it to the
+		// It's tempting to interpret it as pixels per inch, and copy it to the
 		// output image -- though the documentation says it "should not be
 		// interpreted as representing pixels per inch".
 	}
@@ -693,9 +717,16 @@ static void do_palm_BitmapType_internal(deark *c, lctx *d, i64 pos1, i64 len,
 		goto done;
 	}
 
+	if(nextbitmapoffs_in_bytes>0) {
+		pg->endpos = pos1 + nextbitmapoffs_in_bytes;
+	}
+	else {
+		pg->endpos = c->infile->len;
+	}
+
 	de_dbg(c, "image data at %d", (int)pos);
 	de_dbg_indent(c, 1);
-	do_generate_image(c, d, pg, c->infile, pos, pos1+len-pos);
+	do_generate_image(c, d, pg, c->infile, pos, pg->endpos-pos);
 	de_dbg_indent(c, -1);
 
 done:
