@@ -550,8 +550,15 @@ static int do_read_table_entry(deark *c, lctx *d, struct table_entry *te, i64 po
 	te->offset = de_getu32le_p(&pos);
 	de_dbg(c, "offset: %"I64_FMT", size: %"I64_FMT, te->offset, te->size);
 	if(te->offset+te->size > c->infile->len) {
-		de_warn(c, "Table entry goes beyond end of file (type=%s, at %"I64_FMT
-			", size=%"I64_FMT")", te->type_name, te->offset, te->size);
+		if(te->type==0x100) {
+			// A truncated BDF accelerators table is too common to warn about,
+			// and doesn't concern us.
+			de_dbg(c, "[table goes beyond end of file]");
+		}
+		else {
+			de_warn(c, "Table entry goes beyond end of file (type=%s, at %"I64_FMT
+				", size=%"I64_FMT")", te->type_name, te->offset, te->size);
+		}
 	}
 	if(te->offset > c->infile->len) {
 		goto done;
@@ -621,8 +628,8 @@ static void do_make_font_image(deark *c, lctx *d)
 
 		ch->width = ci->width_raw;
 		ch->height = ci->height_raw;
-		if(ch->width<1) ch->width=1;
-		if(ch->height<1) ch->height=1;
+		if(ch->width<0) ch->width=0;
+		if(ch->height<0) ch->height=0;
 		ch->v_offset = max_ascent - ci->ascent;
 		ch->extraspace_l = ci->extraspace_l;
 		ch->extraspace_r = ci->extraspace_r;
