@@ -2,13 +2,14 @@
 // Copyright (C) 2016 Jason Summers
 // See the file COPYING for terms of use.
 
-// PCX (PC Paintbrush) and DCX (multi-image PCX)
+// PCX (PC Paintbrush) and related formats
 
 #include <deark-config.h>
 #include <deark-private.h>
 DE_DECLARE_MODULE(de_module_pcx);
 DE_DECLARE_MODULE(de_module_mswordscr);
 DE_DECLARE_MODULE(de_module_dcx);
+DE_DECLARE_MODULE(de_module_pcx2com);
 
 #define PCX_HDRSIZE 128
 
@@ -19,20 +20,24 @@ enum resmode_type {
 	RESMODE_SCREENDIMENSIONS
 };
 
-typedef struct localctx_struct {
+typedef struct localctx_PCX {
 	u8 version;
 	u8 encoding;
 	enum resmode_type resmode;
 	i64 bits;
 	i64 bits_per_pixel;
 	i64 margin_L, margin_T, margin_R, margin_B;
+	i64 hscreensize, vscreensize;
 	i64 planes;
 	i64 rowspan_raw;
 	i64 rowspan;
 	i64 ncolors;
-	u8 palette_info;
+	UI palette_info;
 	u8 reserved1;
-	i64 width, height;
+	i64 reported_width;
+	i64 padded_width;
+	i64 width_to_use;
+	i64 height;
 	u8 is_mswordscr;
 	u8 is_pcxsfx;
 	int has_vga_pal;
@@ -44,7 +49,7 @@ typedef struct localctx_struct {
 
 	dbuf *unc_pixels;
 	de_finfo *fi;
-	u32 pal[256];
+	de_color pal[256];
 } lctx;
 
 static void simplify_dens(i64 *pxdens, i64 *pydens, i64 factor)
@@ -86,6 +91,8 @@ static void do_decode_resolution(deark *c, lctx *d, i64 hres, i64 vres)
 
 	if(hres==0 || vres==0) return;
 
+	// TODO: Account for d->hscreensize, d->vscreensize.
+
 	if(resmode==RESMODE_AUTO) {
 		if((hres==320 && vres==200) ||
 			(hres==640 && vres==480) ||
@@ -94,12 +101,12 @@ static void do_decode_resolution(deark *c, lctx *d, i64 hres, i64 vres)
 			(hres==800 && vres==600) ||
 			(hres==1024 && vres==768))
 		{
-			if(d->width<=hres && d->height<=hres) {
+			if(d->reported_width<=hres && d->height<=hres) {
 				// Looks like screen dimensions, and image fits on the screen
 				resmode = RESMODE_SCREENDIMENSIONS;
 			}
 		}
-		else if(hres==d->width && vres==d->height) {
+		else if(hres==d->reported_width && vres==d->height) {
 			;
 		}
 		else {
@@ -109,7 +116,7 @@ static void do_decode_resolution(deark *c, lctx *d, i64 hres, i64 vres)
 		}
 	}
 
-	if(resmode==RESMODE_DPI) { // dpi
+	if(resmode==RESMODE_DPI) {
 		d->fi->density.code = DE_DENSITY_DPI;
 		d->fi->density.xdens = (double)hres;
 		d->fi->density.ydens = (double)vres;
@@ -119,24 +126,36 @@ static void do_decode_resolution(deark *c, lctx *d, i64 hres, i64 vres)
 	}
 }
 
+static int sane_screensize(i64 h, i64 v)
+{
+	if(h<320 || v<200) return 0;
+	if(h>4096 || v>4096) return 0;
+	if((h%8 != 0) || (v%2 != 0)) return 0;
+	if(v*5 < h) return 0;
+	if(h*3 < v) return 0;
+	return 1;
+}
+
 static int do_read_header(deark *c, lctx *d)
 {
 	u8 initialbyte;
 	int retval = 0;
 	i64 hres, vres;
+	i64 pos = 0;
 	const char *imgtypename = "";
 
-	de_dbg(c, "header at %d", 0);
+	de_dbg(c, "header at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
-	initialbyte = de_getbyte(0);
-	d->version = de_getbyte(1);
+	initialbyte = de_getbyte_p(&pos);
+	d->version = de_getbyte_p(&pos);
 	if(!d->is_mswordscr) {
 		if(initialbyte==0xeb && d->version==0x0e) {
 			d->is_pcxsfx = 1;
 			d->version = 5;
 		}
 	}
+	de_dbg(c, "format version: %u", (UI)d->version);
 
 	if(d->is_mswordscr) {
 		de_declare_fmt(c, "Word for DOS screen capture");
@@ -148,39 +167,76 @@ static int do_read_header(deark *c, lctx *d)
 		de_declare_fmt(c, "PCX");
 	}
 
-	d->encoding = de_getbyte(2);
-	d->bits = (i64)de_getbyte(3); // Bits per pixel per plane
-	d->margin_L = de_getu16le(4);
-	d->margin_T = de_getu16le(6);
-	d->margin_R = de_getu16le(8);
-	d->margin_B = de_getu16le(10);
+	d->encoding = de_getbyte_p(&pos);
+	de_dbg(c, "encoding: %u", (UI)d->encoding);
 
-	hres = de_getu16le(12);
-	vres = de_getu16le(14);
+	d->bits = (i64)de_getbyte_p(&pos); // Bits per pixel per plane
+	de_dbg(c, "bits: %d", (int)d->bits);
+	if(d->bits<1) d->bits = 1;
+
+	d->margin_L = de_getu16le_p(&pos);
+	d->margin_T = de_getu16le_p(&pos);
+	d->margin_R = de_getu16le_p(&pos);
+	d->margin_B = de_getu16le_p(&pos);
+	de_dbg(c, "margins: %d, %d, %d, %d", (int)d->margin_L, (int)d->margin_T,
+		(int)d->margin_R, (int)d->margin_B);
+	d->reported_width = d->margin_R - d->margin_L +1;
+	d->height = d->margin_B - d->margin_T +1;
+	de_dbg_dimensions(c, d->reported_width, d->height);
+
+	hres = de_getu16le_p(&pos);
+	vres = de_getu16le_p(&pos);
+	de_dbg(c, "resolution: %d"DE_CHAR_TIMES"%d", (int)hres, (int)vres);
 
 	// The palette (offset 16-63) will be read later.
 
+	pos = 64;
 	// For older versions of PCX, this field might be useful to help identify
 	// the intended video mode. Documentation is lacking, though.
-	d->reserved1 = de_getbyte(64);
+	d->reserved1 = de_getbyte_p(&pos);
+	de_dbg(c, "vmode: 0x%02x", (UI)d->reserved1);
 
-	d->planes = (i64)de_getbyte(65);
-	d->rowspan_raw = de_getu16le(66);
-	d->palette_info = de_getbyte(68);
+	d->planes = (i64)de_getbyte_p(&pos);
+	de_dbg(c, "planes: %d", (int)d->planes);
+	d->rowspan_raw = de_getu16le_p(&pos);
+	de_dbg(c, "bytes/plane/row: %d", (int)d->rowspan_raw);
 
-	de_dbg(c, "format version: %d, encoding: %d, planes: %d, bits: %d", (int)d->version,
-		(int)d->encoding, (int)d->planes, (int)d->bits);
-	de_dbg(c, "bytes/plane/row: %d, palette info: %d, vmode: 0x%02x", (int)d->rowspan_raw,
-		(int)d->palette_info, (unsigned int)d->reserved1);
-	de_dbg(c, "margins: %d, %d, %d, %d", (int)d->margin_L, (int)d->margin_T,
-		(int)d->margin_R, (int)d->margin_B);
+	// TODO: Is this field (@68) 1 byte or 2?
+	d->palette_info = (UI)de_getbyte_p(&pos);
+	pos++;
+	de_dbg(c, "palette info: %u", (UI)d->palette_info);
 
-	de_dbg(c, "resolution: %d"DE_CHAR_TIMES"%d", (int)hres, (int)vres);
+	if(d->version>=5) {
+		d->hscreensize = de_getu16le_p(&pos);
+		d->vscreensize = de_getu16le_p(&pos);
+		if(!sane_screensize(d->hscreensize, d->vscreensize)) {
+			d->hscreensize = 0;
+			d->vscreensize = 0;
+		}
+	}
+	if(d->hscreensize) {
+		de_dbg(c, "screen size: %d" DE_CHAR_TIMES "%d", (int)d->hscreensize,
+			(int)d->vscreensize);
+	}
 
-	d->width = d->margin_R - d->margin_L +1;
-	d->height = d->margin_B - d->margin_T +1;
-	de_dbg_dimensions(c, d->width, d->height);
-	if(!de_good_image_dimensions(c, d->width, d->height)) goto done;
+	//-----
+
+	d->padded_width = (d->rowspan_raw*8) / d->bits;
+	d->width_to_use = d->reported_width;
+	if(c->padpix) {
+		if(d->padded_width>d->reported_width) {
+			d->width_to_use = d->padded_width;
+		}
+	}
+	else {
+		if(d->width_to_use<1 && d->padded_width>0) {
+			de_warn(c, "Invalid width %"I64_FMT"; using %"I64_FMT" instead",
+				d->width_to_use, d->padded_width);
+			d->width_to_use = d->padded_width;
+		}
+	}
+
+	if(!de_good_image_dimensions(c, d->width_to_use, d->height)) goto done;
 
 	d->rowspan = d->rowspan_raw * d->planes;
 	de_dbg(c, "calculated bytes/row: %d", (int)d->rowspan);
@@ -204,6 +260,10 @@ static int do_read_header(deark *c, lctx *d)
 		imgtypename = "4-color";
 		d->ncolors = 4;
 	}
+	else if(d->planes==1 && d->bits==4) {
+		imgtypename = "16-color nonplanar";
+		d->ncolors = 16;
+	}
 	else if(d->planes==3 && d->bits==1) {
 		imgtypename = "8-color";
 		d->ncolors = 8;
@@ -244,12 +304,6 @@ static int do_read_header(deark *c, lctx *d)
 
 	de_dbg(c, "image type: %s", imgtypename);
 
-	// Sanity check
-	if(d->rowspan > d->width * 4 + 100) {
-		de_err(c, "Bad bytes/line (%d)", (int)d->rowspan_raw);
-		goto done;
-	}
-
 	do_decode_resolution(c, d, hres, vres);
 
 	retval = 1;
@@ -271,7 +325,7 @@ static int do_read_vga_palette(deark *c, lctx *d)
 		return 0;
 	}
 
-	de_dbg(c, "VGA palette at %d", (int)pos);
+	de_dbg(c, "VGA palette at %"I64_FMT, pos);
 	d->has_vga_pal = 1;
 	pos++;
 	de_dbg_indent(c, 1);
@@ -331,18 +385,11 @@ done:
 	return retval;
 }
 
-// 16-color palettes to use, if there is no palette in the file.
-// (8-color version-3 PCXs apparently use only the first 8 colors of the
-// palette.)
-static const u32 ega16pal[2][16] = {
-	// This palette seems to be correct for at least some files.
-	{0x000000,0x000080,0x008000,0x008080,0x800000,0x800080,0x808000,0x808080,
-	 0xc0c0c0,0x0000ff,0x00ff00,0x00ffff,0xff0000,0xff00ff,0xffff00,0xffffff},
-
-	// This is the "default EGA palette" used by several PCX viewers.
-	// I don't know its origin.
-	{0x000000,0xbf0000,0x00bf00,0xbfbf00,0x0000bf,0xbf00bf,0x00bfbf,0xc0c0c0,
-	 0x808080,0xff0000,0x00ff00,0xffff00,0x0000ff,0xff00ff,0x00ffff,0xffffff}
+static const de_color ega16pal_1[16] = {
+	0xff000000U,0xffbf0000U,0xff00bf00U,0xffbfbf00U,
+	0xff0000bfU,0xffbf00bfU,0xff00bfbfU,0xffc0c0c0U,
+	0xff808080U,0xffff0000U,0xff00ff00U,0xffffff00U,
+	0xff0000ffU,0xffff00ffU,0xff00ffffU,0xffffffffU
 };
 
 static void do_palette_stuff(deark *c, lctx *d)
@@ -356,7 +403,7 @@ static void do_palette_stuff(deark *c, lctx *d)
 	if(d->ncolors==256) {
 		// For 256-color images, start with a default grayscale palette.
 		for(k=0; k<256; k++) {
-			d->pal[k] = DE_MAKE_GRAY((unsigned int)k);
+			d->pal[k] = DE_MAKE_GRAY((UI)k);
 		}
 	}
 
@@ -374,13 +421,23 @@ static void do_palette_stuff(deark *c, lctx *d)
 	}
 
 	if(d->version==3 && d->ncolors>=8 && d->ncolors<=16) {
+		// Come up with a 16-color palette, if there is no palette in the file.
+		// (8-color version-3 PCXs apparently use only the first 8 colors of the
+		// palette.)
+
 		if(!d->default_pal_set) {
 			de_info(c, "Note: This paletted PCX file does not contain a palette. "
 				"If it is not decoded correctly, try \"-opt pcx:pal=1\".");
 		}
 		de_dbg(c, "using a default EGA palette");
-		for(k=0; k<16; k++) {
-			d->pal[k] = ega16pal[d->default_pal_num][k];
+		if(d->default_pal_num==1) {
+			// This is the "default EGA palette" used by several PCX viewers.
+			// I don't know its origin.
+			de_memcpy(d->pal, ega16pal_1, sizeof(ega16pal_1));
+		}
+		else {
+			// This palette seems to be correct for at least some files.
+			de_copy_std_palette(DE_PALID_WIN16, 2, 0, d->pal, 16, 0);
 		}
 		return;
 	}
@@ -396,8 +453,8 @@ static void do_palette_stuff(deark *c, lctx *d)
 
 	if(d->ncolors==4) {
 		u8 p0, p3;
-		unsigned int bgcolor;
-		unsigned int fgpal;
+		UI bgcolor;
+		UI fgpal;
 		int pal_subid;
 
 		de_warn(c, "4-color PCX images might not be supported correctly");
@@ -427,7 +484,7 @@ static void do_palette_stuff(deark *c, lctx *d)
 		default: // 0, 2
 			pal_subid = 2; break; // C=0 P=? I=0
 		}
-		de_copy_std_palette(DE_PALID_CGA, pal_subid, 1, 3, &d->pal[1], 3, 0);
+		de_copy_std_palette(DE_PALID_CGA, pal_subid, 1, &d->pal[1], 3, 0);
 		return;
 	}
 
@@ -451,7 +508,7 @@ static int do_uncompress(deark *c, lctx *d)
 	i64 endpos;
 
 	pos = PCX_HDRSIZE;
-	de_dbg(c, "compressed bitmap at %d", (int)pos);
+	de_dbg(c, "compressed bitmap at %"I64_FMT, pos);
 
 	expected_bytes = d->rowspan * d->height;
 	d->unc_pixels = dbuf_create_membuf(c, expected_bytes, 0);
@@ -497,54 +554,41 @@ static void do_bitmap_1bpp(deark *c, lctx *d)
 	// The paletted algorithm would work here (if we construct a palette),
 	// but this special case is easy and efficient.
 	de_convert_and_write_image_bilevel2(d->unc_pixels, 0,
-		d->width, d->height, d->rowspan_raw, 0, d->fi, 0);
+		d->width_to_use, d->height, d->rowspan_raw, 0, d->fi, 0);
 }
 
 static void do_bitmap_paletted(deark *c, lctx *d)
 {
 	de_bitmap *img = NULL;
-	i64 pdwidth;
-	i64 i, j;
-	i64 plane;
-	u8 b;
-	unsigned int palent;
 
-	// bits_per_plane(_per_row) / bits_per_pixel_per_plane
-	pdwidth = (d->rowspan_raw*8) / d->bits;
+	img = de_bitmap_create(c, d->width_to_use, d->height, 3);
 
-	img = de_bitmap_create2(c, d->width, pdwidth, d->height, 3);
-
-	for(j=0; j<d->height; j++) {
-		for(i=0; i<pdwidth; i++) {
-			palent = 0;
-			for(plane=0; plane<d->planes; plane++) {
-				b = de_get_bits_symbol(d->unc_pixels, d->bits,
-					j*d->rowspan + plane*d->rowspan_raw, i);
-				palent |= b<<(plane*d->bits);
-			}
-			if(palent>255) palent=0; // Should be impossible.
-			de_bitmap_setpixel_rgb(img, i, j, d->pal[palent]);
-		}
+	// Impossible to get here unless one of the following conditions is true.
+	if(d->planes==1) {
+		de_convert_image_paletted(d->unc_pixels, 0, d->bits, d->rowspan,
+			d->pal, img, 0);
+	}
+	else if(d->bits==1) {
+		de_convert_image_paletted_planar(d->unc_pixels, 0, d->planes, d->rowspan,
+			d->rowspan_raw, d->pal, img, 0x2);
 	}
 
-	de_bitmap_write_to_file_finfo(img, d->fi, 0);
+	de_bitmap_write_to_file_finfo(img, d->fi, DE_CREATEFLAG_OPT_IMAGE);
 	de_bitmap_destroy(img);
 }
 
 static void do_bitmap_24bpp(deark *c, lctx *d)
 {
 	de_bitmap *img = NULL;
-	i64 pdwidth;
 	i64 i, j;
 	i64 plane;
 	u8 s[4];
 
 	de_memset(s, 0xff, sizeof(s));
-	pdwidth = (d->rowspan_raw*8) / d->bits;
-	img = de_bitmap_create2(c, d->width, pdwidth, d->height, d->has_transparency?4:3);
+	img = de_bitmap_create(c, d->width_to_use, d->height, d->has_transparency?4:3);
 
 	for(j=0; j<d->height; j++) {
-		for(i=0; i<pdwidth; i++) {
+		for(i=0; i<d->width_to_use; i++) {
 			for(plane=0; plane<d->planes; plane++) {
 				s[plane] = dbuf_getbyte(d->unc_pixels, j*d->rowspan + plane*d->rowspan_raw +i);
 			}
@@ -746,15 +790,15 @@ static void de_run_dcx(deark *c, de_module_params *mparams)
 
 	for(page=0; page<num_pages; page++) {
 		if(page == num_pages-1) {
-			// Last page. Asssume it goes to the end of file.
+			// Last page. Assume it goes to the end of file.
 			page_size = c->infile->len - page_offset[page];
 		}
 		else {
 			page_size = page_offset[page+1] - page_offset[page];
 		}
 		if(page_size<0) page_size=0;
-		de_dbg(c, "page %d at %d, size=%d", (int)page, (int)page_offset[page],
-			(int)page_size);
+		de_dbg(c, "page %d at %u, size=%"I64_FMT, (int)page, (UI)page_offset[page],
+			page_size);
 
 		dbuf_create_file_from_slice(c->infile, page_offset[page], page_size, "pcx", NULL, 0);
 	}
@@ -774,3 +818,63 @@ void de_module_dcx(deark *c, struct deark_module_info *mi)
 	mi->run_fn = de_run_dcx;
 	mi->identify_fn = de_identify_dcx;
 }
+
+// **************************************************************************
+// PCX2COM
+// DOS utility by "Dr.Destiny".
+// graph/pcx2com.zip in the SAC archive.
+// **************************************************************************
+
+static void de_run_pcx2com(deark *c, de_module_params *mparams)
+{
+	i64 pos;
+	dbuf *outf = NULL;
+
+	outf = dbuf_create_output_file(c, "pcx", NULL, 0);
+
+	// header
+	dbuf_enable_wbuffer(outf);
+	dbuf_write(outf, (const u8*)"\x0a\x05\x01\x08\x00\x00\x00\x00\x3f\x01\xc7", 11);
+	dbuf_truncate(outf, 64);
+	dbuf_write(outf, (const u8*)"\x00\x01\x40\x01\x01\x00\x40\x01\xc8", 9);
+	dbuf_truncate(outf, 128);
+
+	// image data, and 0x0c palette marker
+	dbuf_copy(c->infile, 920, c->infile->len-920, outf);
+
+	// VGA palette
+	pos = 152;
+	while(pos < 152+768) {
+		u8 x;
+
+		x = de_getbyte_p(&pos);
+		dbuf_writebyte(outf, de_scale_63_to_255(x));
+	}
+
+	dbuf_close(outf);
+}
+
+static int de_identify_pcx2com(deark *c)
+{
+	if(c->infile->len<922 || c->infile->len>65280) return 0;
+
+	if((UI)de_getu32be(0)!=0xb81300cdU) return 0;
+	if(de_getbyte(c->infile->len-1) != 0x0c) return 0;
+
+	// The is the substring "Self PCX", xor 0x80.
+	if(dbuf_memcmp(c->infile, 104,
+		(const u8*)"\xd3\xe5\xec\xe6\xa0\xd0\xc3\xd8", 8))
+	{
+		return 0;
+	}
+
+	return 100;
+}
+
+void de_module_pcx2com(deark *c, struct deark_module_info *mi)
+{
+	mi->id = "pcx2com";
+	mi->desc = "PCX2COM self-displaying image";
+	mi->run_fn = de_run_pcx2com;
+	mi->identify_fn = de_identify_pcx2com;
+}
