@@ -22,16 +22,28 @@ struct detection_info {
 	int has_fileheader;
 };
 
-typedef struct localctx_struct {
+typedef struct localctx_PICT {
 	struct detection_info dti;
+	de_encoding input_encoding;
 	int version; // 1 or 2: The version mode that the parser is currently using
 	int is_extended_v2;
 	int decode_qt;
 	dbuf *iccprofile_file;
+	struct pict_rect framerect;
 } lctx;
 
-typedef int (*item_decoder_fn)(deark *c, lctx *d, i64 opcode, i64 data_pos,
-	i64 *bytes_used);
+struct item_handler_data {
+	deark *c;
+	lctx *d;
+	dbuf *inf; // always = c->infile
+	UI opcode;
+	i64 opcode_pos;
+	i64 data_pos;
+	i64 data_len; // Handler sets this, unless it's the default for this opcode
+	u8 fatal_errflag;
+};
+
+typedef void (*item_handler_fn)(struct item_handler_data *h);
 
 struct opcode_info {
 	u16 opcode;
@@ -40,9 +52,9 @@ struct opcode_info {
 #define SZCODE_REGION  2
 #define SZCODE_POLYGON 3
 	u16 size_code;
-	u32 size; // Data size, not including opcode. Logic depends on size_code.
+	u32 size; // If SZCODE_EXACT: Data size, not including opcode.
 	const char *name;
-	item_decoder_fn fn;
+	item_handler_fn fn;
 };
 
 static double pict_read_fixed(dbuf *f, i64 pos)
@@ -82,130 +94,121 @@ static void pict_read_rect(dbuf *f, i64 pos,
 	}
 }
 
-static int handler_RGBColor(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_RGBColor(struct item_handler_data *h)
 {
-	unsigned int clr16[3];
+	UI clr16[3];
 	u8 clr8[3];
-	u32 clr;
+	de_color clr;
 	char csamp[16];
-	i64 pos = data_pos;
+	i64 pos = h->data_pos;
 	i64 k;
 
 	for(k=0; k<3; k++) {
-		clr16[k] = (unsigned int)de_getu16be_p(&pos);
+		clr16[k] = (UI)dbuf_getu16be_p(h->inf, &pos);
 		clr8[k] = (u8)(clr16[k]>>8);
 	}
 	clr = DE_MAKE_RGB(clr8[0], clr8[1], clr8[2]);
-	de_get_colorsample_code(c, clr, csamp, sizeof(csamp));
-	de_dbg(c, "color: (0x%04x,0x%04x,0x%04x)%s", clr16[0], clr16[1], clr16[2], csamp);
-	return 1;
+	de_get_colorsample_code(h->c, clr, csamp, sizeof(csamp));
+	de_dbg(h->c, "color: (0x%04x,0x%04x,0x%04x)%s", clr16[0], clr16[1], clr16[2], csamp);
 }
 
 // Version
-static int handler_11(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_11(struct item_handler_data *h)
 {
-	i64 ver;
+	u8 ver;
 
-	*bytes_used = 1;
-	ver = de_getbyte(data_pos);
-	de_dbg(c, "version: %d", (int)ver);
+	ver = dbuf_getbyte(h->inf, h->data_pos);
+	de_dbg(h->c, "version: %u", (UI)ver);
 
 	if(ver==2) {
-		d->version = 2;
+		h->d->version = 2;
 	}
 	else if(ver!=1) {
-		de_err(c, "Unsupported PICT version: %d", (int)ver);
-		return 0;
+		de_err(h->c, "Unsupported PICT version: %u", (UI)ver);
+		h->fatal_errflag = 1;
 	}
-	return 1;
 }
 
 // LongText
-static int handler_28(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_28(struct item_handler_data *h)
 {
 	i64 tlen;
 	de_ucstring *s = NULL;
 	struct pict_point pt;
 
-	pict_read_point(c->infile, data_pos, &pt, "txLoc");
-	tlen = (i64)de_getbyte(data_pos+4);
-	s = ucstring_create(c);
-	dbuf_read_to_ucstring(c->infile, data_pos+5, tlen, s, 0, DE_ENCODING_MACROMAN);
-	de_dbg(c, "text: \"%s\"", ucstring_getpsz(s));
-	*bytes_used = 5+tlen;
+	pict_read_point(h->inf, h->data_pos, &pt, "txLoc");
+	tlen = (i64)dbuf_getbyte(h->inf, h->data_pos+4);
+	s = ucstring_create(h->c);
+	dbuf_read_to_ucstring(h->inf, h->data_pos+5, tlen, s, 0, h->d->input_encoding);
+	de_dbg(h->c, "text: \"%s\"", ucstring_getpsz(s));
+	h->data_len = 5+tlen;
 	ucstring_destroy(s);
-	return 1;
 }
 
 // DVText
-static int handler_DxText(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_DxText(struct item_handler_data *h)
 {
 	i64 tlen;
 	i64 dx;
 	de_ucstring *s = NULL;
 
-	dx = (i64)de_getbyte(data_pos);
-	de_dbg(c, "%s: %d", opcode==0x2a?"dv":"dh", (int)dx);
+	dx = (i64)dbuf_getbyte(h->inf, h->data_pos);
+	de_dbg(h->c, "%s: %d", h->opcode==0x2a?"dv":"dh", (int)dx);
 
-	tlen = (i64)de_getbyte(data_pos+1);
-	*bytes_used = 2+tlen;
+	tlen = (i64)dbuf_getbyte(h->inf, h->data_pos+1);
+	h->data_len = 2+tlen;
 
-	s = ucstring_create(c);
-	dbuf_read_to_ucstring(c->infile, data_pos+2, tlen, s, 0, DE_ENCODING_MACROMAN);
-	de_dbg(c, "text: \"%s\"", ucstring_getpsz(s));
+	s = ucstring_create(h->c);
+	dbuf_read_to_ucstring(h->inf, h->data_pos+2, tlen, s, 0, h->d->input_encoding);
+	de_dbg(h->c, "text: \"%s\"", ucstring_getpsz(s));
 
 	ucstring_destroy(s);
-	return 1;
 }
 
 // DHDVText
-static int handler_2b(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_2b(struct item_handler_data *h)
 {
 	i64 tlen;
 	i64 dh, dv;
 	de_ucstring *s = NULL;
 
-	dh = (i64)de_getbyte(data_pos);
-	dv = (i64)de_getbyte(data_pos+1);
-	de_dbg(c, "dh,dv: (%d,%d)", (int)dh, (int)dv);
+	dh = (i64)dbuf_getbyte(h->inf, h->data_pos);
+	dv = (i64)dbuf_getbyte(h->inf, h->data_pos+1);
+	de_dbg(h->c, "dh,dv: (%d,%d)", (int)dh, (int)dv);
 
-	tlen = (i64)de_getbyte(data_pos+2);
-	de_dbg(c, "text size: %d", (int)tlen);
-	*bytes_used = 3+tlen;
+	tlen = (i64)dbuf_getbyte(h->inf, h->data_pos+2);
+	de_dbg(h->c, "text size: %d", (int)tlen);
+	h->data_len = 3+tlen;
 
-	s = ucstring_create(c);
-	dbuf_read_to_ucstring(c->infile, data_pos+3, tlen, s, 0, DE_ENCODING_MACROMAN);
-	de_dbg(c, "text: \"%s\"", ucstring_getpsz(s));
-
-	return 1;
+	s = ucstring_create(h->c);
+	dbuf_read_to_ucstring(h->inf, h->data_pos+3, tlen, s, 0, h->d->input_encoding);
+	de_dbg(h->c, "text: \"%s\"", ucstring_getpsz(s));
 }
 
 // fontName
-static int handler_2c(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_2c(struct item_handler_data *h)
 {
 	i64 n;
 	i64 tlen;
 	i64 id;
 	de_ucstring *s = NULL;
 
-	n = de_getu16be(data_pos);
-	*bytes_used = 2+n;
-	id = de_getu16be(data_pos+2);
-	de_dbg(c, "old font id: %d", (int)id);
-	tlen = (i64)de_getbyte(data_pos+4);
-	s = ucstring_create(c);
-	dbuf_read_to_ucstring(c->infile, data_pos+5, tlen, s, 0, DE_ENCODING_MACROMAN);
-	de_dbg(c, "font name: \"%s\"", ucstring_getpsz(s));
+	n = dbuf_getu16be(h->inf, h->data_pos);
+	h->data_len = 2+n;
+	id = dbuf_getu16be(h->inf, h->data_pos+2);
+	de_dbg(h->c, "old font id: %d", (int)id);
+	tlen = (i64)dbuf_getbyte(h->inf, h->data_pos+4);
+	s = ucstring_create(h->c);
+	dbuf_read_to_ucstring(h->inf, h->data_pos+5, tlen, s, 0, h->d->input_encoding);
+	de_dbg(h->c, "font name: \"%s\"", ucstring_getpsz(s));
 	ucstring_destroy(s);
-	return 1;
 }
 
-static int handler_Rectangle(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_Rectangle(struct item_handler_data *h)
 {
 	struct pict_rect rect;
 
-	pict_read_rect(c->infile, data_pos, &rect, "rect");
-	return 1;
+	pict_read_rect(h->inf, h->data_pos, &rect, "rect");
 }
 
 // final few bitmap header fields (18 bytes)
@@ -214,7 +217,7 @@ static void read_src_dst_mode(deark *c, lctx *d, struct fmtutil_macbitmap_info *
 	struct pict_rect tmprect;
 	i64 n;
 
-	de_dbg(c, "src/dst/mode part of bitmap header, at %d", (int)pos);
+	de_dbg(c, "src/dst/mode part of bitmap header, at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	pict_read_rect(c->infile, pos, &tmprect, "srcRect");
@@ -222,9 +225,8 @@ static void read_src_dst_mode(deark *c, lctx *d, struct fmtutil_macbitmap_info *
 	pict_read_rect(c->infile, pos, &tmprect, "dstRect");
 	pos += 8;
 
-	n = de_getu16be(pos);
+	n = de_getu16be_p(&pos);
 	de_dbg(c, "transfer mode: %d", (int)n);
-	//pos += 2;
 	de_dbg_indent(c, -1);
 }
 
@@ -241,11 +243,11 @@ static int get_pixdata_size(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi
 	int retval = 0;
 
 	pos = pos1;
-	de_dbg(c, "PixData at %d", (int)pos);
+	de_dbg(c, "PixData at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	if(bi->height<0 || bi->height>65535) {
-		de_err(c, "Invalid bitmap height (%d)", (int)bi->height);
+		de_err(c, "Invalid bitmap height (%"I64_FMT")", (i64)bi->height);
 		goto done;
 	}
 
@@ -259,12 +261,10 @@ static int get_pixdata_size(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi
 	if(bi->packing_type>=3 || (bi->packing_type==0 && bi->rowbytes>=8)) {
 		for(j=0; j<bi->height; j++) {
 			if(bi->rowbytes > 250) {
-				bytecount = de_getu16be(pos);
-				pos+=2;
+				bytecount = de_getu16be_p(&pos);
 			}
 			else {
-				bytecount = (i64)de_getbyte(pos);
-				pos+=1;
+				bytecount = (i64)de_getbyte_p(&pos);
 			}
 			pos += bytecount;
 		}
@@ -278,7 +278,7 @@ static int get_pixdata_size(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi
 	}
 
 	*pixdata_size = pos - pos1;
-	de_dbg(c, "PixData size: %d", (int)*pixdata_size);
+	de_dbg(c, "PixData size: %"I64_FMT, (i64)*pixdata_size);
 	retval = 1;
 
 done:
@@ -306,16 +306,17 @@ static void decode_bitmap_rgb16(deark *c, lctx *d, struct fmtutil_macbitmap_info
 	dbuf *unc_pixels, de_bitmap *img, i64 pos)
 {
 	i64 i, j;
-	u8 c0, c1; //, cg, cb;
-	u32 clr;
+	u8 c0, c1;
+	u32 clr555;
+	de_color clr888;
 
 	for(j=0; j<bi->height; j++) {
 		for(i=0; i<bi->pdwidth; i++) {
 			c0 = dbuf_getbyte(unc_pixels, j*bi->rowspan + i*2);
 			c1 = dbuf_getbyte(unc_pixels, j*bi->rowspan + i*2+1);
-			clr = ((u32)c0 << 8)|c1;
-			clr = de_rgb555_to_888(clr);
-			de_bitmap_setpixel_rgb(img, i, j, clr);
+			clr555 = ((u32)c0 << 8)|c1;
+			clr888 = de_rgb555_to_888(clr555);
+			de_bitmap_setpixel_rgb(img, i, j, clr888);
 		}
 	}
 }
@@ -323,17 +324,8 @@ static void decode_bitmap_rgb16(deark *c, lctx *d, struct fmtutil_macbitmap_info
 static void decode_bitmap_paletted(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi,
 	dbuf *unc_pixels, de_bitmap *img, i64 pos)
 {
-	i64 i, j;
-	u8 b;
-	u32 clr;
-
-	for(j=0; j<bi->height; j++) {
-		for(i=0; i<bi->pdwidth; i++) {
-			b = de_get_bits_symbol(unc_pixels, bi->pixelsize, j*bi->rowspan, i);
-			clr = bi->pal[(unsigned int)b];
-			de_bitmap_setpixel_rgb(img, i, j, clr);
-		}
-	}
+	de_convert_image_paletted(unc_pixels, 0, bi->pixelsize, bi->rowspan,
+		bi->pal, img, 0);
 }
 
 static int decode_bitmap(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi, i64 pos)
@@ -368,12 +360,10 @@ static int decode_bitmap(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi, i
 			bytecount = bi->rowbytes;
 		}
 		else if(bi->rowbytes > 250) {
-			bytecount = de_getu16be(pos);
-			pos+=2;
+			bytecount = de_getu16be_p(&pos);
 		}
 		else {
-			bytecount = (i64)de_getbyte(pos);
-			pos+=1;
+			bytecount = (i64)de_getbyte_p(&pos);
 		}
 
 		if(bi->packing_type==1 || bi->rowbytes<8) {
@@ -383,7 +373,7 @@ static int decode_bitmap(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi, i
 			struct de_packbits_params pbparams;
 
 			de_zeromem(&pbparams, sizeof(struct de_packbits_params));
-			pbparams.is_packbits16 = 1;
+			pbparams.nbytes_per_unit = 2;
 			dcmpri.pos = pos;
 			dcmpri.len = bytecount;
 			fmtutil_decompress_packbits_ex(c, &dcmpri, &dcmpro, &dres, &pbparams);
@@ -432,7 +422,7 @@ static int decode_bitmap(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi, i
 		}
 	}
 
-	de_bitmap_write_to_file_finfo(img, fi, 0);
+	de_bitmap_write_to_file_finfo(img, fi, DE_CREATEFLAG_OPT_IMAGE);
 
 	de_bitmap_destroy(img);
 	de_finfo_destroy(c, fi);
@@ -481,7 +471,9 @@ static int decode_pixdata(deark *c, lctx *d, struct fmtutil_macbitmap_info *bi,
 		(bi->packing_type==0 || bi->packing_type==1) &&
 		(bi->pixelsize==1 || bi->pixelsize==2 || bi->pixelsize==4 || bi->pixelsize==8) &&
 		bi->cmpcount==1 && bi->cmpsize==bi->pixelsize) ||
-		(!bi->uses_pal && bi->packing_type==3 && bi->pixelsize==16 && bi->cmpcount==3 && bi->cmpsize==5) ||
+		(!bi->uses_pal && (bi->packing_type==1 || bi->packing_type==3) && bi->pixelsize==16
+			&& bi->cmpcount==3 && bi->cmpsize==5) ||
+		(!bi->uses_pal && bi->packing_type==1 && bi->pixelsize==16 && bi->cmpcount==3 && bi->cmpsize==5) ||
 		(!bi->uses_pal && bi->packing_type==4 && bi->pixelsize==32 && bi->cmpcount==3 && bi->cmpsize==8) ||
 		(!bi->uses_pal && bi->packing_type==4 && bi->pixelsize==32 && bi->cmpcount==4 && bi->cmpsize==8))
 	{
@@ -504,36 +496,37 @@ done:
 }
 
 // For opcodes 0x90, 0x91, 0x98, 0x99, 0x9a, 0x9b
-static int handler_98_9a(deark *c, lctx *d, i64 opcode, i64 pos1, i64 *bytes_used)
+static void handler_98_9a(struct item_handler_data *h)
 {
 	struct fmtutil_macbitmap_info *bi = NULL;
 	i64 pixdata_size = 0;
 	i64 colortable_size = 0;
 	int retval = 0;
 	i64 pos;
+	deark *c = h->c;
 
 	bi = de_malloc(c, sizeof(struct fmtutil_macbitmap_info));
-	pos = pos1;
+	pos = h->data_pos;
 
-	if(opcode==0x9a || opcode==0x9b) {
-		fmtutil_macbitmap_read_baseaddr(c, c->infile, bi, pos);
+	if(h->opcode==0x9a || h->opcode==0x9b) {
+		fmtutil_macbitmap_read_baseaddr(c, h->inf, bi, pos);
 		pos += 4;
 	}
 
-	fmtutil_macbitmap_read_rowbytes_and_bounds(c, c->infile, bi, pos);
+	fmtutil_macbitmap_read_rowbytes_and_bounds(c, h->inf, bi, pos);
 	pos += 10;
 
 	if(bi->pixmap_flag) {
-		fmtutil_macbitmap_read_pixmap_only_fields(c, c->infile, bi, pos);
+		fmtutil_macbitmap_read_pixmap_only_fields(c, h->inf, bi, pos);
 		pos += 36;
 	}
 
-	if((opcode==0x90 || opcode==0x91 || opcode==0x98 || opcode==0x99) && bi->pixmap_flag) {
+	if((h->opcode==0x90 || h->opcode==0x91 || h->opcode==0x98 || h->opcode==0x99) && bi->pixmap_flag) {
 		// Prepare to read the palette
 		bi->uses_pal = 1;
 		bi->has_colortable = 1;
 	}
-	else if((opcode==0x90 || opcode==0x91 || opcode==0x98 || opcode==0x99) && !bi->pixmap_flag) {
+	else if((h->opcode==0x90 || h->opcode==0x91 || h->opcode==0x98 || h->opcode==0x99) && !bi->pixmap_flag) {
 		// Settings implied by the lack of a PixMap header
 		bi->pixelsize = 1;
 		bi->cmpcount = 1;
@@ -543,20 +536,20 @@ static int handler_98_9a(deark *c, lctx *d, i64 opcode, i64 pos1, i64 *bytes_use
 		bi->pal[0] = DE_STOCKCOLOR_WHITE;
 		bi->pal[1] = DE_STOCKCOLOR_BLACK;
 	}
-	else if((opcode==0x9a || opcode==0x9b) && !bi->pixmap_flag) {
+	else if((h->opcode==0x9a || h->opcode==0x9b) && !bi->pixmap_flag) {
 		de_err(c, "DirectBitsRect image without PixMap flag is not supported");
 		goto done;
 	}
 
 	if(bi->has_colortable) {
-		if(!fmtutil_macbitmap_read_colortable(c, c->infile, bi, pos, &colortable_size)) goto done;
+		if(!fmtutil_macbitmap_read_colortable(c, h->inf, bi, pos, &colortable_size)) goto done;
 		pos += colortable_size;
 	}
 
-	read_src_dst_mode(c, d, bi, pos);
+	read_src_dst_mode(c, h->d, bi, pos);
 	pos += 18;
 
-	if(opcode==0x91 || opcode==0x99 || opcode==0x9b) {
+	if(h->opcode==0x91 || h->opcode==0x99 || h->opcode==0x9b) {
 		i64 rgnsize;
 
 		de_dbg(c, "region at %"I64_FMT, pos);
@@ -572,32 +565,35 @@ static int handler_98_9a(deark *c, lctx *d, i64 opcode, i64 pos1, i64 *bytes_use
 		}
 	}
 
-	if(!get_pixdata_size(c, d, bi, pos, &pixdata_size)) {
+	if(!get_pixdata_size(c, h->d, bi, pos, &pixdata_size)) {
 		goto done;
 	}
-	decode_pixdata(c, d, bi, pos);
+	decode_pixdata(c, h->d, bi, pos);
 	pos += pixdata_size;
 
-	*bytes_used = pos - pos1;
+	h->data_len = pos - h->data_pos;
 
 	retval = 1;
 
 done:
+	if(!retval) {
+		h->fatal_errflag = 1;
+	}
 	de_free(c, bi);
-	return retval;
 }
 
-static int handler_pixpat(deark *c, lctx *d, i64 opcode, i64 pos1, i64 *bytes_used)
+static void handler_pixpat(struct item_handler_data *h)
 {
-	unsigned int pattype;
-	i64 pos = pos1;
+	UI pattype;
+	i64 pos = h->data_pos;
 	int needmsg = 1;
 	int retval = 0;
 	i64 colortable_size = 0;
 	i64 pixdata_size = 0;
 	struct fmtutil_macbitmap_info *bi = NULL;
+	deark *c = h->c;
 
-	pattype = (unsigned int)de_getu16be_p(&pos);
+	pattype = (UI)dbuf_getu16be_p(h->inf, &pos);
 	de_dbg(c, "PatType: %u", pattype);
 	pos += 8; // Pat1Data
 
@@ -609,16 +605,16 @@ static int handler_pixpat(deark *c, lctx *d, i64 opcode, i64 pos1, i64 *bytes_us
 
 	bi = de_malloc(c, sizeof(struct fmtutil_macbitmap_info));
 
-	fmtutil_macbitmap_read_rowbytes_and_bounds(c, c->infile, bi, pos);
+	fmtutil_macbitmap_read_rowbytes_and_bounds(c, h->inf, bi, pos);
 	pos += 10;
-	fmtutil_macbitmap_read_pixmap_only_fields(c, c->infile, bi, pos);
+	fmtutil_macbitmap_read_pixmap_only_fields(c, h->inf, bi, pos);
 	pos += 36;
 
 	bi->uses_pal = 1;
-	if(!fmtutil_macbitmap_read_colortable(c, c->infile, bi, pos, &colortable_size)) goto done;
+	if(!fmtutil_macbitmap_read_colortable(c, h->inf, bi, pos, &colortable_size)) goto done;
 	pos += colortable_size;
 
-	if(!get_pixdata_size(c, d, bi, pos, &pixdata_size)) {
+	if(!get_pixdata_size(c, h->d, bi, pos, &pixdata_size)) {
 		goto done;
 	}
 	// Note: We could extract the "PixMap" pattern easily enough here, by calling
@@ -632,10 +628,12 @@ done:
 		de_err(c, "Failed to parse PixPat data");
 	}
 	if(retval) {
-		*bytes_used = pos - pos1;
+		h->data_len = pos - h->data_pos;
+	}
+	if(!retval) {
+		h->fatal_errflag = 1;
 	}
 	de_free(c, bi);
-	return retval;
 }
 
 static void do_iccprofile_item(deark *c, lctx *d, i64 pos, i64 len)
@@ -646,8 +644,8 @@ static void do_iccprofile_item(deark *c, lctx *d, i64 pos, i64 len)
 	if(len<4) return;
 	selector = de_getu32be(pos);
 	data_len = len-4;
-	de_dbg(c, "ICC profile segment, selector=%d, data len=%d", (int)selector,
-		(int)data_len);
+	de_dbg(c, "ICC profile segment, selector=%d, data len=%"I64_FMT, (int)selector,
+		data_len);
 
 	if(selector!=1) {
 		// If this is not a Continuation segment, close any current file.
@@ -672,65 +670,62 @@ static void do_iccprofile_item(deark *c, lctx *d, i64 pos, i64 len)
 }
 
 // ShortComment
-static int handler_a0(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_a0(struct item_handler_data *h)
 {
 	i64 kind;
-	kind = de_getu16be(data_pos);
-	de_dbg(c, "comment kind: %d", (int)kind);
-	return 1;
+
+	kind = dbuf_getu16be(h->inf, h->data_pos);
+	de_dbg(h->c, "comment kind: %d", (int)kind);
 }
 
 // LongComment
-static int handler_a1(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_a1(struct item_handler_data *h)
 {
 	i64 kind;
 	i64 len;
+	deark *c = h->c;
 
-	kind = de_getu16be(data_pos);
-	len = de_getu16be(data_pos+2);
+	kind = dbuf_getu16be(h->inf, h->data_pos);
+	len = dbuf_getu16be(h->inf, h->data_pos+2);
 	de_dbg(c, "comment kind: %d, size: %d", (int)kind, (int)len);
-	*bytes_used = 4+len;
+	h->data_len = 4+len;
 
 	if(kind==100 && len>=4) {
 		struct de_fourcc sig4cc;
 
-		dbuf_read_fourcc(c->infile, data_pos+4, &sig4cc, 4, 0x0);
+		dbuf_read_fourcc(h->inf, h->data_pos+4, &sig4cc, 4, 0x0);
 		de_dbg(c, "application comment, signature=0x%08x '%s'",
-			(unsigned int)sig4cc.id, sig4cc.id_dbgstr);
-		de_dbg_hexdump(c, c->infile, data_pos+8, len-4, 256, NULL, 0x1);
+			(UI)sig4cc.id, sig4cc.id_dbgstr);
+		de_dbg_hexdump(c, h->inf, h->data_pos+8, len-4, 256, NULL, 0x1);
 	}
 	else if(kind==224) {
-		do_iccprofile_item(c, d, data_pos+4, len);
+		do_iccprofile_item(c, h->d, h->data_pos+4, len);
 	}
 	else {
-		de_dbg_hexdump(c, c->infile, data_pos+4, len, 256, NULL, 0x1);
+		de_dbg_hexdump(c, h->inf, h->data_pos+4, len, 256, NULL, 0x1);
 	}
-
-	return 1;
 }
 
 // HeaderOp
-static int handler_0c00(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_0c00(struct item_handler_data *h)
 {
 	i64 hdrver;
 	double hres, vres;
 	struct pict_rect srcrect;
 
-	hdrver = de_getu16be(data_pos);
-	d->is_extended_v2 = (hdrver==0xfffe);
+	hdrver = dbuf_getu16be(h->inf, h->data_pos);
+	h->d->is_extended_v2 = (hdrver==0xfffe);
 
-	de_dbg(c, "extended v2: %s", d->is_extended_v2?"yes":"no");
-	if(d->is_extended_v2) {
-		hres = pict_read_fixed(c->infile, data_pos+4);
-		vres = pict_read_fixed(c->infile, data_pos+8);
-		de_dbg(c, "dpi: %.2f"DE_CHAR_TIMES"%.2f", hres, vres);
-		pict_read_rect(c->infile, data_pos+12, &srcrect, "srcRect");
+	de_dbg(h->c, "extended v2: %s", h->d->is_extended_v2?"yes":"no");
+	if(h->d->is_extended_v2) {
+		hres = pict_read_fixed(h->inf, h->data_pos+4);
+		vres = pict_read_fixed(h->inf, h->data_pos+8);
+		de_dbg(h->c, "dpi: %.2f"DE_CHAR_TIMES"%.2f", hres, vres);
+		pict_read_rect(h->inf, h->data_pos+12, &srcrect, "srcRect");
 	}
-
-	return 1;
 }
 
-// Returns 1 if image decoding was successul,
+// Returns 1 if image decoding was successful,
 // 0 if (e.g.) unsupported compression type.
 static int do_decode_qt(deark *c, lctx *d, i64 pos, i64 len)
 {
@@ -801,26 +796,28 @@ done:
 }
 
 // CompressedQuickTime (0x8200) & UncompressedQuickTime (0x8201)
-static int handler_QuickTime(deark *c, lctx *d, i64 opcode, i64 data_pos, i64 *bytes_used)
+static void handler_QuickTime(struct item_handler_data *h)
 {
 	i64 payload_pos;
 	i64 payload_len;
 	i64 endpos;
 	i64 idsc_pos;
 
-	payload_len = de_getu32be(data_pos);
-	payload_pos = data_pos+4;
-	de_dbg(c, "payload: pos=%"I64_FMT", len=%"I64_FMT, payload_pos, payload_len);
+	payload_len = dbuf_getu32be(h->inf, h->data_pos);
+	payload_pos = h->data_pos+4;
+	de_dbg(h->c, "payload: pos=%"I64_FMT", len=%"I64_FMT, payload_pos, payload_len);
 	endpos = payload_pos+payload_len;
-	if(endpos > c->infile->len) return 0;
-	*bytes_used = 4+payload_len;
+	if(endpos > h->inf->len) {
+		h->fatal_errflag = 1;
+		return;
+	}
+	h->data_len = 4+payload_len;
 
 	// Following the size field seems to be 68 or 50 bytes of data,
 	// followed by QuickTime "idsc" data, followed by image data.
-	idsc_pos = payload_pos + ((opcode==0x8201) ? 50 : 68);
+	idsc_pos = payload_pos + ((h->opcode==0x8201) ? 50 : 68);
 
-	do_handle_qtif_idsc(c, d, idsc_pos, endpos-idsc_pos);
-	return 1;
+	do_handle_qtif_idsc(h->c, h->d, idsc_pos, endpos-idsc_pos);
 }
 
 static const struct opcode_info opcode_info_arr[] = {
@@ -931,92 +928,89 @@ static const struct opcode_info opcode_info_arr[] = {
 	{ 0x8201, SZCODE_SPECIAL, 0,  "UncompressedQuickTime", handler_QuickTime }
 };
 
-static const struct opcode_info *find_opcode_info(i64 opcode)
+static const struct opcode_info *find_opcode_info(UI opcode)
 {
 	size_t i;
 
 	for(i=0; i<DE_ARRAYCOUNT(opcode_info_arr); i++) {
-		if(opcode_info_arr[i].opcode == opcode) {
+		if((UI)opcode_info_arr[i].opcode == opcode) {
 			return &opcode_info_arr[i];
 		}
 	}
 	return NULL;
 }
 
-static int do_handle_item(deark *c, lctx *d, i64 opcode_pos, i64 opcode,
-						   i64 data_pos, i64 *data_bytes_used)
+// Caller sets h->opcode_pos and h->data_pos (among other fields).
+static void do_handle_item(struct item_handler_data *h)
 {
 	const char *opcode_name;
 	const struct opcode_info *opi;
 	i64 n;
 	struct pict_rect tmprect;
-	int ret = 0;
+	deark *c = h->c;
 
-	*data_bytes_used = 0;
-
-	opi = find_opcode_info(opcode);
+	opi = find_opcode_info(h->opcode);
 	if(opi && opi->name) opcode_name = opi->name;
 	else opcode_name = "?";
 
-	if(d->version==2)
-		de_dbg(c, "opcode 0x%04x (%s) at %d", (unsigned int)opcode, opcode_name, (int)opcode_pos);
+	if(h->d->version==2)
+		de_dbg(c, "opcode 0x%04x (%s) at %"I64_FMT, h->opcode, opcode_name, h->opcode_pos);
 	else
-		de_dbg(c, "opcode 0x%02x (%s) at %d", (unsigned int)opcode, opcode_name, (int)opcode_pos);
+		de_dbg(c, "opcode 0x%02x (%s) at %"I64_FMT, h->opcode, opcode_name, h->opcode_pos);
+
+	if(opi) {
+		h->data_len = (i64)opi->size; // Default to the size in the table.
+	}
 
 	if(opi && opi->fn) {
 		de_dbg_indent(c, 1);
-		*data_bytes_used = opi->size; // Default to the size in the table.
-		ret = opi->fn(c, d, opcode, data_pos, data_bytes_used);
+		opi->fn(h);
 		de_dbg_indent(c, -1);
 	}
 	else if(opi && opi->size_code==SZCODE_EXACT) {
-		*data_bytes_used = opi->size;
-		ret = 1;
+		;
 	}
 	else if(opi && opi->size_code==SZCODE_REGION) {
-		n = de_getu16be(data_pos);
+		n = dbuf_getu16be(h->inf, h->data_pos);
 		de_dbg_indent(c, 1);
 		de_dbg(c, "region size: %d", (int)n);
 		if(n>=10) {
-			pict_read_rect(c->infile, data_pos+2, &tmprect, "rect");
+			pict_read_rect(h->inf, h->data_pos+2, &tmprect, "rect");
 		}
 		de_dbg_indent(c, -1);
-		*data_bytes_used = n;
-		ret = 1;
+		h->data_len = n;
 	}
 	else if(opi && opi->size_code==SZCODE_POLYGON) {
-		n = de_getu16be(data_pos);
+		n = dbuf_getu16be(h->inf, h->data_pos);
 		de_dbg_indent(c, 1);
 		de_dbg(c, "polygon size: %d", (int)n);
 		de_dbg_indent(c, -1);
-		*data_bytes_used = n;
-		ret = 1;
+		h->data_len = n;
 	}
-	else if(opcode>=0x2c && opcode<=0x2f) {
+	else if(h->opcode>=0x2c && h->opcode<=0x2f) {
 		// Starts with 2-byte size, size does not include the "size" field.
-		n = de_getu16be(data_pos);
-		*data_bytes_used = 2+n;
-		ret = 1;
+		n = dbuf_getu16be(h->inf, h->data_pos);
+		h->data_len = 2+n;
 	}
-	else if(opcode>=0x8100 && opcode<=0xffff) {
+	else if(h->opcode>=0x8100 && h->opcode<=0xffff) {
 		// Starts with 4-byte size, size does not include the "size" field.
-		n = de_getu32be(data_pos);
-		*data_bytes_used = 4+n;
-		ret = 1;
+		n = dbuf_getu32be(h->inf, h->data_pos);
+		h->data_len = 4+n;
 	}
 	else {
-		de_err(c, "Unsupported opcode: 0x%04x", (unsigned int)opcode);
+		de_err(c, "Unsupported opcode: 0x%04x", h->opcode);
+		h->fatal_errflag = 1;
 	}
-
-	return ret;
 }
 
 static void do_read_items(deark *c, lctx *d, i64 pos)
 {
-	i64 opcode;
-	i64 opcode_pos;
-	i64 bytes_used;
-	int ret;
+	struct item_handler_data *h = NULL;
+
+	h = de_malloc(c, sizeof(struct item_handler_data));
+	h->c = c;
+	h->d = d;
+	h->inf = c->infile;
 
 	while(1) {
 		if(pos%2 && d->version==2) {
@@ -1025,25 +1019,28 @@ static void do_read_items(deark *c, lctx *d, i64 pos)
 
 		if(pos >= c->infile->len) break;
 
-		opcode_pos = pos;
+		h->opcode_pos = pos;
 
 		if(d->version==2) {
-			opcode = de_getu16be(pos);
-			pos+=2;
+			h->opcode = (UI)de_getu16be_p(&pos);
 		}
 		else {
-			opcode = (i64)de_getbyte(pos);
-			pos+=1;
+			h->opcode = (UI)de_getbyte_p(&pos);
 		}
 
-		ret = do_handle_item(c, d, opcode_pos, opcode, pos, &bytes_used);
-		if(!ret) goto done;
-		if(opcode==0x00ff) goto done; // End of image
+		h->data_pos = pos;
 
-		pos += bytes_used;
+		// Must initialize all remaining relevant fields.
+		h->data_len = 0;
+
+		do_handle_item(h);
+		if(h->fatal_errflag) goto done;
+		if(h->opcode==0x00ff) goto done; // End of image
+
+		pos += h->data_len;
 	}
 done:
-	;
+	de_free(c, h);
 }
 
 // mode: 0=called from de_identify..., 1=called from de_run...
@@ -1118,10 +1115,10 @@ static void de_run_pict(deark *c, de_module_params *mparams)
 	lctx *d = NULL;
 	i64 pos = 0;
 	i64 picsize;
-	struct pict_rect framerect;
 
 	d = de_malloc(c, sizeof(lctx));
 
+	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_MACROMAN);
 	d->decode_qt = de_get_ext_option_bool(c, "pict:decodeqt", 0);
 
 	do_detect_version(c, &d->dti, 1);
@@ -1136,10 +1133,9 @@ static void de_run_pict(deark *c, de_module_params *mparams)
 		pos += 512;
 	}
 
-	picsize = de_getu16be(pos);
-	de_dbg(c, "picSize: %d", (int)picsize);
-	pos+=2;
-	pict_read_rect(c->infile, pos, &framerect, "picFrame");
+	picsize = de_getu16be_p(&pos);
+	de_dbg(c, "picSize: %"I64_FMT, picsize);
+	pict_read_rect(c->infile, pos, &d->framerect, "picFrame");
 	pos+=8;
 
 	do_read_items(c, d, pos);
