@@ -7,6 +7,8 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_tga);
 
+#define CODE_IGCH 0x49474348U
+
 struct tgaimginfo {
 	i64 width, height;
 	i64 img_size_in_bytes;
@@ -14,11 +16,17 @@ struct tgaimginfo {
 	int respect_alpha_channel;
 };
 
-typedef struct localctx_struct {
+typedef struct localctx_structTGA {
 	i64 id_field_len;
+	de_encoding input_encoding;
 #define FMT_TGA 0
 #define FMT_VST 1
 	int file_format;
+#define TRANS_AUTO     100
+#define TRANS_REMOVE   101
+#define TRANS_RESPECT  102
+#define TRANS_ASSOC    103
+	u8 trans_setting;
 	u8 color_map_type;
 	u8 img_type;
 	struct tgaimginfo main_image;
@@ -30,6 +38,7 @@ typedef struct localctx_struct {
 	u8 image_descriptor;
 	i64 num_attribute_bits;
 	u8 attributes_type;
+	u8 alpha_is_premultiplied;
 	u8 top_down, right_to_left;
 	u8 interleave_mode;
 	i64 v2_footer_pos;
@@ -63,9 +72,22 @@ typedef struct localctx_struct {
 static int should_use_alpha_channel(deark *c, lctx *d, struct tgaimginfo *imginfo,
 	int has_alpha_0, int has_alpha_partial, int has_alpha_255)
 {
+	const char *prefix = "";
+
 	if(imginfo->is_thumb) {
-		// Do the same thing with the thumbnail as we did with the main image.
-		return d->main_image.respect_alpha_channel;
+		prefix = "[thumbnail] ";
+	}
+
+	if(d->trans_setting==TRANS_REMOVE) return 0;
+
+	if(d->has_ext_area_attribs_type) {
+		// attributes_type==0 technically also means there is no transparency,
+		// but this cannot be trusted.
+		if(d->attributes_type!=0 && d->attributes_type!=3 &&
+			d->attributes_type!=4)
+		{
+			return 0;
+		}
 	}
 
 	if(d->pixel_depth!=32 || d->color_type!=TGA_CLRTYPE_TRUECOLOR) {
@@ -73,47 +95,39 @@ static int should_use_alpha_channel(deark *c, lctx *d, struct tgaimginfo *imginf
 	}
 
 	if(d->num_attribute_bits!=0 && d->num_attribute_bits!=8) {
-		de_warn(c, "%d-bit attribute channel not supported. Transparency disabled.",
-			(int)d->num_attribute_bits);
+		if(!imginfo->is_thumb) { // Hack to suppress duplicate warning
+			de_warn(c, "%d-bit attribute channel not supported. Transparency disabled.",
+				(int)d->num_attribute_bits);
+		}
 		return 0;
 	}
 
+	if(d->trans_setting==TRANS_RESPECT || d->trans_setting==TRANS_ASSOC) {
+		return 1;
+	}
+
 	if(d->has_ext_area_attribs_type) {
-		if(d->attributes_type==0) {
-			// attributes_type==0 technically also means there is no transparency,
-			// but this cannot be trusted.
-			;
-		}
-		if(d->attributes_type==1 || d->attributes_type==2) {
-			// Indicates that attribute data can be ignored.
-			return 0;
-		}
-		else if(d->attributes_type==3 && d->num_attribute_bits==8) {
+		if(d->attributes_type==3 && d->num_attribute_bits==8) {
 			// Alpha channel seems to be labeled correctly.
 			// Trust it.
 			return 1;
 		}
-		else if(d->attributes_type==4) {
-			// Sigh. The spec shows that Field 24 (Attributes Type) == 4 is for
-			// pre-multiplied alpha. Then the discussion section says that
-			// Field *23* ("Attributes Type") == *3* is for premultiplied alpha.
-			// I have to guess that the "23" and "3" are clerical errors, but that
-			// doesn't do me much good unless all TGA developers made the same guess.
-			de_warn(c, "Pre-multiplied alpha is not supported. Disabling transparency.");
-			return 0;
+		else if(d->attributes_type==4 && d->num_attribute_bits==8) {
+			// Premultiplied alpha
+			return 1;
 		}
 	}
 
 	if(has_alpha_partial || (has_alpha_0 && has_alpha_255)) {
 		if(d->num_attribute_bits==0) {
-			de_warn(c, "Detected likely alpha channel. Enabling transparency, even though "
-				"the image is labeled as non-transparent.");
+			de_warn(c, "%sDetected likely alpha channel. Enabling transparency, even though "
+				"the image is labeled as non-transparent.", prefix);
 		}
 		return 1;
 	}
 	else if(has_alpha_0) { // All 0x00
 		if(d->num_attribute_bits!=0) {
-			de_warn(c, "Non-visible image detected. Disabling transparency.");
+			de_warn(c, "%sNon-visible image detected. Disabling transparency.", prefix);
 		}
 		else {
 			de_dbg(c, "potential alpha channel ignored: all 0 bits");
@@ -126,8 +140,25 @@ static int should_use_alpha_channel(deark *c, lctx *d, struct tgaimginfo *imginf
 	}
 }
 
+static void bitmap_unpremultiply_alpha(de_bitmap *img)
+{
+	i64 i, j;
+
+	if(img->bytes_per_pixel!=2 && img->bytes_per_pixel!=4) return;
+
+	for(j=0; j<img->height; j++) {
+		for(i=0; i<img->width; i++) {
+			de_color clr;
+
+			clr = de_bitmap_getpixel(img, i, j);
+			clr = de_unpremultiply_alpha_clr(clr);
+			de_bitmap_setpixel_rgba(img, i, j, clr);
+		}
+	}
+}
+
 static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf *unc_pixels,
-	const char *token, UI createflags)
+	const char *token, UI createflags1)
 {
 	de_bitmap *img = NULL;
 	de_finfo *fi = NULL;
@@ -145,6 +176,7 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 	int has_alpha_0 = 0;
 	int has_alpha_partial = 0;
 	int has_alpha_255 = 0;
+	UI createflags = createflags1;
 
 	fi = de_finfo_create(c);
 
@@ -191,10 +223,7 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 	for(j=0; j<imginfo->height; j++) {
 		i64 j_adj;
 
-		if(d->top_down)
-			j_adj = cur_rownum;
-		else
-			j_adj = imginfo->height-1-cur_rownum;
+		j_adj = cur_rownum;
 
 		// Update the row number for next time
 		cur_rownum += interleave_stride;
@@ -210,23 +239,16 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 		}
 
 		for(i=0; i<pdwidth; i++) {
-			i64 i_adj;
-
-			if(d->right_to_left)
-				i_adj = imginfo->width-1-i;
-			else
-				i_adj = i;
-
 			if(d->color_type==TGA_CLRTYPE_TRUECOLOR && (d->pixel_depth==15 || d->pixel_depth==16)) {
 				clr = (u32)dbuf_getu16le(unc_pixels, j*rowspan + i*d->bytes_per_pixel);
 				clr = de_rgb555_to_888(clr);
-				de_bitmap_setpixel_rgb(img, i_adj, j_adj, clr);
+				de_bitmap_setpixel_rgb(img, i, j_adj, clr);
 			}
 			else if(d->color_type==TGA_CLRTYPE_TRUECOLOR) {
 				clr = dbuf_getRGB(unc_pixels, j*rowspan + i*d->bytes_per_pixel, getrgbflags);
 				if(d->pixel_depth==32) {
 					a = dbuf_getbyte(unc_pixels, j*rowspan + i*d->bytes_per_pixel+3);
-					de_bitmap_setpixel_rgba(img, i_adj, j_adj, DE_SET_ALPHA(clr, a));
+					de_bitmap_setpixel_rgba(img, i, j_adj, DE_SET_ALPHA(clr, a));
 
 					// Collect metrics that we may need, to decide whether to keep the
 					// might-be-alpha channel.
@@ -241,16 +263,16 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 					}
 				}
 				else {
-					de_bitmap_setpixel_rgb(img, i_adj, j_adj, clr);
+					de_bitmap_setpixel_rgb(img, i, j_adj, clr);
 				}
 			}
 			else if(d->color_type==TGA_CLRTYPE_GRAYSCALE) {
 				b = dbuf_getbyte(unc_pixels, j*rowspan + i*d->bytes_per_pixel);
-				de_bitmap_setpixel_gray(img, i_adj, j_adj, b);
+				de_bitmap_setpixel_gray(img, i, j_adj, b);
 			}
 			else if(d->color_type==TGA_CLRTYPE_PALETTE) {
 				b = dbuf_getbyte(unc_pixels, j*rowspan + i*d->bytes_per_pixel);
-				de_bitmap_setpixel_rgb(img, i_adj, j_adj, d->pal[(UI)b]);
+				de_bitmap_setpixel_rgb(img, i, j_adj, d->pal[(UI)b]);
 			}
 		}
 	}
@@ -258,6 +280,9 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 	// TODO: 16-bit images could theoretically have a transparency bit, but I don't
 	// know how to detect that.
 	if(d->pixel_depth==32) {
+		// TODO: The transparency logic is a bit messy. It probably ought to be
+		// redesigned from scratch.
+
 		imginfo->respect_alpha_channel = should_use_alpha_channel(c, d, imginfo,
 			has_alpha_0, has_alpha_partial, has_alpha_255);
 
@@ -265,11 +290,23 @@ static void do_decode_image(deark *c, lctx *d, struct tgaimginfo *imginfo, dbuf
 			de_dbg(c, "using alpha channel: %s", imginfo->respect_alpha_channel?"yes":"no");
 		}
 
-		if(!imginfo->respect_alpha_channel) {
+		if(!imginfo->respect_alpha_channel || (!has_alpha_0 && !has_alpha_partial)) {
 			de_bitmap_remove_alpha(img);
 		}
+
+		if(imginfo->respect_alpha_channel && d->alpha_is_premultiplied) {
+			bitmap_unpremultiply_alpha(img);
+		}
 	}
 
+	if(d->right_to_left) {
+		de_bitmap_mirror(img);
+	}
+	if(!d->top_down) {
+		createflags |= DE_CREATEFLAG_FLIP_IMAGE;
+	}
+
+	createflags |= DE_CREATEFLAG_OPT_IMAGE;
 	de_bitmap_write_to_file_finfo(img, fi, createflags);
 
 	de_bitmap_destroy(img);
@@ -347,7 +384,7 @@ static void do_decode_thumbnail(deark *c, lctx *d)
 	dbuf *unc_pixels = NULL;
 	i64 hdrsize = 2;
 
-	de_dbg(c, "thumbnail image at %d", (int)d->thumbnail_offset);
+	de_dbg(c, "thumbnail image at %"I64_FMT, d->thumbnail_offset);
 	de_dbg_indent(c, 1);
 
 	// The thumbnail image is supposed to use the same format as the main image,
@@ -435,12 +472,12 @@ static void do_read_extension_area(deark *c, lctx *d, i64 pos1)
 	s = ucstring_create(c);
 
 	ext_area_size = de_getu16le_p(&pos);
-	de_dbg(c, "extension area size: %d", (int)ext_area_size);
+	de_dbg(c, "extension area size: %"I64_FMT, ext_area_size);
 	endpos = pos1+ext_area_size;
 
 	if(pos+41>endpos) goto done;
 	ucstring_empty(s);
-	dbuf_read_to_ucstring(c->infile, pos, 41, s, DE_CONVFLAG_STOP_AT_NUL, DE_ENCODING_ASCII);
+	dbuf_read_to_ucstring(c->infile, pos, 41, s, DE_CONVFLAG_STOP_AT_NUL, d->input_encoding);
 	ucstring_strip_trailing_spaces(s);
 	de_dbg(c, "author: \"%s\"", ucstring_getpsz_d(s));
 	pos += 41;
@@ -448,7 +485,7 @@ static void do_read_extension_area(deark *c, lctx *d, i64 pos1)
 	for(k=0; k<4; k++) {
 		if(pos+81>endpos) goto done;
 		ucstring_empty(s);
-		dbuf_read_to_ucstring(c->infile, pos, 81, s, DE_CONVFLAG_STOP_AT_NUL, DE_ENCODING_ASCII);
+		dbuf_read_to_ucstring(c->infile, pos, 81, s, DE_CONVFLAG_STOP_AT_NUL, d->input_encoding);
 		ucstring_strip_trailing_spaces(s);
 		de_dbg(c, "comment line %d: \"%s\"", (int)k, ucstring_getpsz_d(s));
 		pos += 81;
@@ -477,7 +514,7 @@ static void do_read_extension_area(deark *c, lctx *d, i64 pos1)
 
 	if(pos+41>endpos) goto done;
 	ucstring_empty(s);
-	dbuf_read_to_ucstring(c->infile, pos, 41, s, DE_CONVFLAG_STOP_AT_NUL, DE_ENCODING_ASCII);
+	dbuf_read_to_ucstring(c->infile, pos, 41, s, DE_CONVFLAG_STOP_AT_NUL, d->input_encoding);
 	ucstring_strip_trailing_spaces(s);
 	de_dbg(c, "software id: \"%s\"", ucstring_getpsz_d(s));
 	pos += 41;
@@ -522,6 +559,9 @@ static void do_read_extension_area(deark *c, lctx *d, i64 pos1)
 	d->attributes_type = de_getbyte_p(&pos);
 	d->has_ext_area_attribs_type = 1;
 	de_dbg(c, "attributes type: %u", (UI)d->attributes_type);
+	if(d->attributes_type==4) {
+		d->alpha_is_premultiplied = 1;
+	}
 	if(d->attributes_type==0 && d->num_attribute_bits!=0) {
 		de_warn(c, "Incompatible \"number of attribute bits\" (%d) and \"attributes type\" "
 			"(%u) fields. Transparency may not be handled correctly.",
@@ -553,8 +593,8 @@ static void do_read_developer_area(deark *c, lctx *d, i64 pos)
 		tag_id = de_getu16le(pos + 2 + 10*i);
 		tag_data_pos = de_getu32le(pos + 2 + 10*i + 2);
 		tag_data_size = de_getu32le(pos + 2 + 10*i + 6);
-		de_dbg(c, "tag #%d: id=%d, pos=%d, size=%d", (int)i, (int)tag_id,
-			(int)tag_data_pos, (int)tag_data_size);
+		de_dbg(c, "tag #%d: id=%d, pos=%"I64_FMT", size=%"I64_FMT, (int)i, (int)tag_id,
+			tag_data_pos, tag_data_size);
 
 		if(tag_id==20) {
 			// Tag 20 seems to contain Photoshop resources, though this is unconfirmed.
@@ -621,12 +661,17 @@ static int do_read_tga_headers(deark *c, lctx *d)
 	de_dbg_indent(c, 1);
 
 	d->id_field_len = (i64)de_getbyte(0);
+	de_dbg(c, "image ID len: %"I64_FMT, d->id_field_len);
 	d->color_map_type = de_getbyte(1);
 	de_dbg(c, "color map type: %d", (int)d->color_map_type);
 	d->img_type = de_getbyte(2);
 	de_dbg(c, "image type: %d", (int)d->img_type);
 
 	switch(d->img_type) {
+	case 0:
+		d->color_type = TGA_CLRTYPE_UNKNOWN;
+		d->clrtype_name = "no image";
+		break;
 	case 1: case 9:
 	case 32: case 33:
 		d->color_type = TGA_CLRTYPE_PALETTE;
@@ -660,16 +705,20 @@ static int do_read_tga_headers(deark *c, lctx *d)
 	}
 
 	de_dbg_indent(c, 1);
-	de_dbg(c, "color type: %d (%s)", (int)d->color_type, d->clrtype_name);
-	de_dbg(c, "compression: %d (%s)", (int)d->cmpr_type, d->cmpr_name);
+	de_dbg(c, "color type: %s", d->clrtype_name);
+	de_dbg(c, "compression: %s", d->cmpr_name);
 	de_dbg_indent(c, -1);
 
 	if(d->color_map_type != 0) {
 		d->cmap_start = de_getu16le(3);
 		d->cmap_length = de_getu16le(5);
 		d->cmap_depth = (i64)de_getbyte(7);
-		de_dbg(c, "color map start: %d, len: %d, depth: %d", (int)d->cmap_start,
-			(int)d->cmap_length, (int)d->cmap_depth);
+		de_dbg(c, "color map spec. at 3");
+		de_dbg_indent(c, 1);
+		de_dbg(c, "starting idx: %u", (UI)d->cmap_start);
+		de_dbg(c, "num entries: %u", (UI)d->cmap_length);
+		de_dbg(c, "bits/entry: %u", (UI)d->cmap_depth);
+		de_dbg_indent(c, -1);
 	}
 
 	d->main_image.width = de_getu16le(12);
@@ -811,7 +860,7 @@ static void detect_file_format(deark *c, lctx *d)
 
 	img_type = de_getbyte(2);
 	if(img_type==0) {
-		has_igch = !dbuf_memcmp(c->infile, 20, "IGCH", 4);
+		has_igch = (de_getu32be(20)==CODE_IGCH);
 		if(has_igch) {
 			d->file_format = FMT_VST;
 			return;
@@ -821,6 +870,18 @@ static void detect_file_format(deark *c, lctx *d)
 	d->file_format = FMT_TGA;
 }
 
+static void read_image_id(deark *c, lctx *d, i64 pos)
+{
+	de_ucstring *s = NULL;
+
+	if(d->id_field_len<1) return;
+	if(d->file_format!=FMT_TGA) return;
+	s = ucstring_create(c);
+	dbuf_read_to_ucstring(c->infile, pos, d->id_field_len, s, 0, d->input_encoding);
+	de_dbg(c, "image ID: \"%s\"", ucstring_getpsz_d(s));
+	ucstring_destroy(s);
+}
+
 static void de_run_tga(deark *c, de_module_params *mparams)
 {
 	lctx *d = NULL;
@@ -828,9 +889,28 @@ static void de_run_tga(deark *c, de_module_params *mparams)
 	dbuf *unc_pixels = NULL;
 	int saved_indent_level;
 	i64 rowspan_tmp;
+	const char *opt_trans_str;
 
 	de_dbg_indent_save(c, &saved_indent_level);
 	d = de_malloc(c, sizeof(lctx));
+	d->input_encoding = de_get_input_encoding(c, NULL, DE_ENCODING_ASCII);
+	d->trans_setting = TRANS_AUTO;
+
+	opt_trans_str = de_get_ext_option(c, "tga:trans");
+	if(opt_trans_str) {
+		if(!de_strcmp(opt_trans_str, "auto")) {
+			d->trans_setting = TRANS_AUTO;
+		}
+		else if(!de_strcmp(opt_trans_str, "0")) {
+			d->trans_setting = TRANS_REMOVE;
+		}
+		else if(!de_strcmp(opt_trans_str, "1")) {
+			d->trans_setting = TRANS_RESPECT;
+		}
+		else if(!de_strcmp(opt_trans_str, "2")) {
+			d->trans_setting = TRANS_ASSOC;
+		}
+	}
 
 	detect_file_format(c, d);
 
@@ -851,17 +931,23 @@ static void de_run_tga(deark *c, de_module_params *mparams)
 	else {
 		if(!do_read_tga_headers(c, d)) goto done;
 	}
+
+	if(d->trans_setting==TRANS_ASSOC) {
+		d->alpha_is_premultiplied = 1;
+	}
+
 	pos += 18;
 
-	if(d->id_field_len>0) {
-		de_dbg(c, "image ID at %d (len=%d)", (int)pos, (int)d->id_field_len);
-		pos += d->id_field_len;
-	}
+	de_dbg(c, "image/color data at %"I64_FMT, pos);
+	de_dbg_indent(c, 1);
+
+	read_image_id(c, d, pos);
+	pos += d->id_field_len;
 
 	if(d->color_map_type!=0) {
 		d->bytes_per_pal_entry = (d->cmap_depth+7)/8;
 		d->pal_size_in_bytes = d->cmap_length * d->bytes_per_pal_entry;
-		de_dbg(c, "color map at %d (%d colors, %d bytes)", (int)pos,
+		de_dbg(c, "color map at %"I64_FMT" (%d colors, %d bytes)", pos,
 			(int)d->cmap_length, (int)d->pal_size_in_bytes);
 
 		de_dbg_indent(c, 1);
@@ -871,7 +957,7 @@ static void de_run_tga(deark *c, de_module_params *mparams)
 		pos += d->pal_size_in_bytes;
 	}
 
-	de_dbg(c, "bitmap at %d", (int)pos);
+	de_dbg(c, "bitmap at %"I64_FMT, pos);
 	de_dbg_indent(c, 1);
 
 	d->bytes_per_pixel = ((d->pixel_depth+7)/8);
@@ -920,13 +1006,13 @@ static void de_run_tga(deark *c, de_module_params *mparams)
 		unc_pixels = dbuf_open_input_subfile(c->infile, pos, d->main_image.img_size_in_bytes);
 	}
 	else {
-		de_err(c, "Unsupported compression type (%d, %s)", (int)d->cmpr_type, d->cmpr_name);
+		de_err(c, "Unsupported image or compression type");
 		goto done;
 	}
 
 	do_decode_image(c, d, &d->main_image, unc_pixels, NULL, 0);
 
-	de_dbg_indent(c, -1);
+	de_dbg_indent(c, -2);
 
 	if(d->thumbnail_offset!=0) {
 		do_decode_thumbnail(c, d);
@@ -945,27 +1031,39 @@ static int de_identify_tga(deark *c)
 	int has_tga_ext;
 	i64 v2_footer_pos = 0;
 
-	if(look_for_v2_signature(c, 1, &v2_footer_pos)) {
-		return 100;
-	}
-
 	// TGA v1 format has no signature, but there are only a few common types of
 	// it. We'll at least try to identify anything that we support.
 	de_read(b, 0, 18);
 
 	if(b[1]>1) return 0; // Color map type should be 0 or 1.
 
-	// bits/pixel:
-	if(b[16]!=1 && b[16]!=8 && b[16]!=15 && b[16]!=16 && b[16]!=24 && b[16]!=32) return 0;
+	if(b[12]==0 && b[13]==0) return 0; // Width can't be 0.
+	if(b[14]==0 && b[15]==0) return 0; // Height can't be 0.
 
-	if(b[2]!=0 && b[2]!=1 && b[2]!=2 && b[2]!=3 &&
-		b[2]!=9 && b[2]!=10 && b[2]!=11 && b[2]!=32 && b[2]!=33)
+	if(look_for_v2_signature(c, 1, &v2_footer_pos)) {
+		return 100;
+	}
+
+	// 0 is valid here, but it means "no image", and we need all the
+	// can't-be-0 bytes we can find...
+	if(b[2]==0) {
+		// ...except that we have to allow 0 in .vst format.
+		// TODO: .vst should maybe be a separate module.
+		if((UI)de_getu32be(20)!=CODE_IGCH) {
+			return 0;
+		}
+	}
+	else if(b[2]==1 || b[2]==2 || b[2]==3 ||
+		b[2]==9 || b[2]==10 || b[2]==11 || b[2]==32 || b[2]==33)
 	{
+		;
+	}
+	else {
 		return 0; // Unknown image type
 	}
 
-	if(b[12]==0 && b[13]==0) return 0; // Width can't be 0.
-	if(b[14]==0 && b[15]==0) return 0; // Height can't be 0.
+	// bits/pixel:
+	if(b[16]!=1 && b[16]!=8 && b[16]!=15 && b[16]!=16 && b[16]!=24 && b[16]!=32) return 0;
 
 	// Bits per palette entry. Supposed to be 0 if there is no palette, but
 	// in practice it may be 24 instead.
@@ -990,10 +1088,17 @@ static int de_identify_tga(deark *c)
 	return 8;
 }
 
+static void de_help_tga(deark *c)
+{
+	de_msg(c, "-opt tga:trans=<0|1|2|auto> : Transparency: 0=remove, 1=respect, "
+		"2=premultiplied");
+}
+
 void de_module_tga(deark *c, struct deark_module_info *mi)
 {
 	mi->id = "tga";
 	mi->desc = "Truevision TGA, a.k.a. TARGA";
 	mi->run_fn = de_run_tga;
 	mi->identify_fn = de_identify_tga;
+	mi->help_fn = de_help_tga;
 }
