@@ -9,7 +9,7 @@
 #include <deark-fmtutil.h>
 DE_DECLARE_MODULE(de_module_wmf);
 
-typedef struct localctx_struct {
+typedef struct localctx_WMF {
 	int has_aldus_header;
 	int input_encoding;
 	i64 wmf_file_type;
@@ -22,7 +22,6 @@ typedef struct localctx_struct {
 struct escape_info {
 	u16 escfn;
 	const char *name;
-	void *reserved1;
 };
 
 struct decoder_params {
@@ -157,66 +156,66 @@ done:
 }
 
 static const struct escape_info escape_info_arr[] = {
-	{ 0x0001, "NEWFRAME", NULL },
-	{ 0x0002, "ABORTDOC", NULL },
-	{ 0x0003, "NEXTBAND", NULL },
-	{ 0x0004, "SETCOLORTABLE", NULL },
-	{ 0x0005, "GETCOLORTABLE", NULL },
-	{ 0x0006, "FLUSHOUT", NULL },
-	{ 0x0007, "DRAFTMODE", NULL },
-	{ 0x0008, "QUERYESCSUPPORT", NULL },
-	{ 0x0009, "SETABORTPROC", NULL },
-	{ 0x000a, "STARTDOC", NULL },
-	{ 0x000b, "ENDDOC", NULL },
-	{ 0x000c, "GETPHYSPAGESIZE", NULL },
-	{ 0x000d, "GETPRINTINGOFFSET", NULL },
-	{ 0x000e, "GETSCALINGFACTOR", NULL },
-	{ 0x000f, "MFCOMMENT", NULL }, // a.k.a. META_ESCAPE_ENHANCED_METAFILE
-	{ 0x0010, "SETPENWIDTH", NULL },
-	{ 0x0011, "SETCOPYCOUNT", NULL },
-	{ 0x0012, "SETPAPERSOURCE", NULL },
-	{ 0x0013, "PASSTHROUGH", NULL },
-	{ 0x0014, "GETTECHNOLOGY", NULL },
-	{ 0x0015, "SETLINECAP", NULL },
-	{ 0x0016, "SETLINEJOIN", NULL },
-	{ 0x0017, "SETMITERLIMIT", NULL },
-	{ 0x0018, "BANDINFO", NULL },
-	{ 0x0019, "DRAWPATTERNRECT", NULL },
-	{ 0x001a, "GETVECTORPENSIZE", NULL },
-	{ 0x001b, "GETVECTORBRUSHSIZE", NULL },
-	{ 0x001c, "ENABLEDUPLEX", NULL },
-	{ 0x001d, "GETSETPAPERBINS", NULL },
-	{ 0x001e, "GETSETPRINTORIENT", NULL },
-	{ 0x001f, "ENUMPAPERBINS", NULL },
-	{ 0x0020, "SETDIBSCALING", NULL },
-	{ 0x0021, "EPSPRINTING", NULL },
-	{ 0x0022, "ENUMPAPERMETRICS", NULL },
-	{ 0x0023, "GETSETPAPERMETRICS", NULL },
-	{ 0x0025, "POSTSCRIPT_DATA", NULL },
-	{ 0x0026, "POSTSCRIPT_IGNORE", NULL },
-	{ 0x002a, "GETDEVICEUNITS", NULL },
-	{ 0x0100, "GETEXTENDEDTEXTMETRICS", NULL },
-	{ 0x0102, "GETPAIRKERNTABLE", NULL },
-	{ 0x0200, "EXTTEXTOUT", NULL },
-	{ 0x0201, "GETFACENAME", NULL },
-	{ 0x0202, "DOWNLOADFACE", NULL },
-	{ 0x0801, "METAFILE_DRIVER", NULL },
-	{ 0x0c01, "QUERYDIBSUPPORT", NULL },
-	{ 0x1000, "BEGIN_PATH", NULL },
-	{ 0x1001, "CLIP_TO_PATH", NULL },
-	{ 0x1002, "END_PATH", NULL },
-	{ 0x100e, "OPEN_CHANNEL", NULL },
-	{ 0x100f, "DOWNLOADHEADER", NULL },
-	{ 0x1010, "CLOSE_CHANNEL", NULL },
-	{ 0x1013, "POSTSCRIPT_PASSTHROUGH", NULL },
-	{ 0x1014, "ENCAPSULATED_POSTSCRIPT", NULL },
-	{ 0x1015, "POSTSCRIPT_IDENTIFY", NULL },
-	{ 0x1016, "POSTSCRIPT_INJECTION", NULL },
-	{ 0x1017, "CHECKJPEGFORMAT", NULL },
-	{ 0x1018, "CHECKPNGFORMAT", NULL },
-	{ 0x1019, "GET_PS_FEATURESETTING", NULL },
-	{ 0x101a, "MXDC_ESCAPE", NULL },
-	{ 0x11d8, "SPCLPASSTHROUGH2", NULL }
+	{ 0x0001, "NEWFRAME" },
+	{ 0x0002, "ABORTDOC" },
+	{ 0x0003, "NEXTBAND" },
+	{ 0x0004, "SETCOLORTABLE" },
+	{ 0x0005, "GETCOLORTABLE" },
+	{ 0x0006, "FLUSHOUT" },
+	{ 0x0007, "DRAFTMODE" },
+	{ 0x0008, "QUERYESCSUPPORT" },
+	{ 0x0009, "SETABORTPROC" },
+	{ 0x000a, "STARTDOC" },
+	{ 0x000b, "ENDDOC" },
+	{ 0x000c, "GETPHYSPAGESIZE" },
+	{ 0x000d, "GETPRINTINGOFFSET" },
+	{ 0x000e, "GETSCALINGFACTOR" },
+	{ 0x000f, "MFCOMMENT" }, // a.k.a. META_ESCAPE_ENHANCED_METAFILE
+	{ 0x0010, "SETPENWIDTH" },
+	{ 0x0011, "SETCOPYCOUNT" },
+	{ 0x0012, "SETPAPERSOURCE" },
+	{ 0x0013, "PASSTHROUGH" },
+	{ 0x0014, "GETTECHNOLOGY" },
+	{ 0x0015, "SETLINECAP" },
+	{ 0x0016, "SETLINEJOIN" },
+	{ 0x0017, "SETMITERLIMIT" },
+	{ 0x0018, "BANDINFO" },
+	{ 0x0019, "DRAWPATTERNRECT" },
+	{ 0x001a, "GETVECTORPENSIZE" },
+	{ 0x001b, "GETVECTORBRUSHSIZE" },
+	{ 0x001c, "ENABLEDUPLEX" },
+	{ 0x001d, "GETSETPAPERBINS" },
+	{ 0x001e, "GETSETPRINTORIENT" },
+	{ 0x001f, "ENUMPAPERBINS" },
+	{ 0x0020, "SETDIBSCALING" },
+	{ 0x0021, "EPSPRINTING" },
+	{ 0x0022, "ENUMPAPERMETRICS" },
+	{ 0x0023, "GETSETPAPERMETRICS" },
+	{ 0x0025, "POSTSCRIPT_DATA" },
+	{ 0x0026, "POSTSCRIPT_IGNORE" },
+	{ 0x002a, "GETDEVICEUNITS" },
+	{ 0x0100, "GETEXTENDEDTEXTMETRICS" },
+	{ 0x0102, "GETPAIRKERNTABLE" },
+	{ 0x0200, "EXTTEXTOUT" },
+	{ 0x0201, "GETFACENAME" },
+	{ 0x0202, "DOWNLOADFACE" },
+	{ 0x0801, "METAFILE_DRIVER" },
+	{ 0x0c01, "QUERYDIBSUPPORT" },
+	{ 0x1000, "BEGIN_PATH" },
+	{ 0x1001, "CLIP_TO_PATH" },
+	{ 0x1002, "END_PATH" },
+	{ 0x100e, "OPEN_CHANNEL" },
+	{ 0x100f, "DOWNLOADHEADER" },
+	{ 0x1010, "CLOSE_CHANNEL" },
+	{ 0x1013, "POSTSCRIPT_PASSTHROUGH" },
+	{ 0x1014, "ENCAPSULATED_POSTSCRIPT" },
+	{ 0x1015, "POSTSCRIPT_IDENTIFY" },
+	{ 0x1016, "POSTSCRIPT_INJECTION" },
+	{ 0x1017, "CHECKJPEGFORMAT" },
+	{ 0x1018, "CHECKPNGFORMAT" },
+	{ 0x1019, "GET_PS_FEATURESETTING" },
+	{ 0x101a, "MXDC_ESCAPE" },
+	{ 0x11d8, "SPCLPASSTHROUGH2" }
 };
 
 static void do_ESCAPE_MFCOMMENT_EMF(deark *c, lctx *d, struct decoder_params *dp,
@@ -842,23 +841,26 @@ done:
 
 static int de_identify_wmf(deark *c)
 {
-	u8 buf[4];
+	UI w0, w1, ver;
+	i64 sz;
+	int has_ext;
 
-	de_read(buf, 0, 4);
-
-	if(!de_memcmp(buf, "\xd7\xcd\xc6\x9a", 4))
+	w0 = (UI)de_getu16le(0);
+	w1 = (UI)de_getu16le(2);
+	if(w0==0xcdd7 && w1==0x9ac6) {
 		return 100;
-
-	if(de_input_file_has_ext(c, "wmf")) {
-		i64 ftype, hsize;
-		ftype = de_getu16le_direct(&buf[0]);
-		hsize = de_getu16le_direct(&buf[2]);
-		if(hsize==9 && (ftype==1 || ftype==2)) {
-			return 80;
-		}
 	}
+	if(w1!=0x09) return 0;
+	if(w0!=0x01 && w0!=0x02) return 0;
+	ver = (UI)de_getu16le(4);
+	if(ver!=0x0100 && ver!=0x0300) return 0;
 
-	return 0;
+	has_ext = de_input_file_has_ext(c, "wmf");
+	if(has_ext) return 95;
+	sz = de_getu32le(6) * 2;
+	if(sz > c->infile->len || sz < 20) return 0;
+	if(sz >= c->infile->len-8) return 70;
+	return 14;
 }
 
 void de_module_wmf(deark *c, struct deark_module_info *mi)
