@@ -12,6 +12,7 @@ export DEARK_OBJDIR="mingwobj"
 TOOLSPREFIX="x86_64-w64-mingw32-"
 export CC=${TOOLSPREFIX}gcc.exe
 export AR=${TOOLSPREFIX}ar.exe
+export DEARK_RC=src/_deark_a.rc
 export DEARK_WINDRES=${TOOLSPREFIX}windres.exe
 export LDFLAGS="-Wall -municode"
 
@@ -21,7 +22,11 @@ then
 else
  mkdir -p $DEARK_OBJDIR/src
  mkdir -p $DEARK_OBJDIR/modules
+ # (I don't know how to get windres to accept UTF-16 input.)
+ iconv -f utf16 -t cp1252 < src/deark.rc > ${DEARK_RC}
+ touch --reference=src/deark.rc ${DEARK_RC}
  make -j4 dep
  make -j4
+ rm ${DEARK_RC}
 fi
 
