@@ -1,6 +1,6 @@
 #!/bin/bash
 
-VER=1.6.2
+VER=1.7.1
 
 if [ ! -f formats.txt ]
 then
@@ -24,7 +24,7 @@ D=".build-tmp/deark-$VER"
 mkdir $D
 
 mkdir $D/src
-cp -p src/*.c src/*.h src/*.rc src/*.manifest $D/src/
+cp -p src/*.c src/*.h src/*.rc src/*.ico src/*.manifest $D/src/
 
 mkdir $D/modules
 cp -p modules/*.c $D/modules/
@@ -36,8 +36,8 @@ mkdir $D/scripts
 cp -p scripts/*.sh scripts/*.pl $D/scripts/
 
 mkdir $D/proj
-mkdir $D/proj/vs2019
-cp -p proj/vs2019/*.sln proj/vs2019/*.vcxproj proj/vs2019/*.vcxproj.filters $D/proj/vs2019/
+mkdir $D/proj/vs2022
+cp -p proj/vs2022/*.sln proj/vs2022/*.vcxproj proj/vs2022/*.vcxproj.filters $D/proj/vs2022/
 
 mkdir $D/obj
 mkdir $D/obj/src
