@@ -28,6 +28,38 @@ int de_good_image_dimensions(deark *c, i64 w, i64 h)
 	return 1;
 }
 
+#define MAX_INTERNAL_IMAGE_DIMENSION 10000
+
+// flags: 0x1: Emit an error message for a bad non-internal image.
+int de_bitmap_good_dimensions(de_bitmap *img, UI flags)
+{
+	if(img->is_internal) {
+		// Images for internal use, that aren't going to be written to a file,
+		// should not respect the "-maxdim" setting from the user.
+		//
+		// Note - The is_internal flag should only be used for images with
+		// a fixed size, that does not depend on width/height fields in the
+		// input file.
+		if(img->width<1 || img->width>MAX_INTERNAL_IMAGE_DIMENSION ||
+			img->height<1 || img->height>MAX_INTERNAL_IMAGE_DIMENSION)
+		{
+			de_internal_err_nonfatal(img->c, "Bad temp bitmap size");
+			return 0;
+		}
+	}
+	else if(flags & 0x1) {
+		if(!de_good_image_dimensions(img->c, img->width, img->height)) {
+			return 0;
+		}
+	}
+	else {
+		if(!de_good_image_dimensions_noerr(img->c, img->width, img->height)) {
+			return 0;
+		}
+	}
+	return 1;
+}
+
 #define DE_MAX_IMAGES_PER_FILE 10000
 
 // This is meant as a sanity check for fields that indicate how many images
@@ -82,7 +114,7 @@ static void de_bitmap_alloc_pixels(de_bitmap *img)
 		de_bitmap_free_pixels(img);
 	}
 
-	if(!de_good_image_dimensions(img->c, img->width, img->height)) {
+	if(!de_bitmap_good_dimensions(img, 0x1)) {
 		// This function is not allowed to fail. If something goes wrong, create
 		// a dummy image, and set invalid_image_flag.
 		img->invalid_image_flag = 1;
@@ -95,25 +127,44 @@ static void de_bitmap_alloc_pixels(de_bitmap *img)
 	img->bitmap = de_malloc(img->c, img->bitmap_size);
 }
 
-struct image_scan_results {
-	int has_color;
-	int has_trns;
-	int has_visible_pixels;
+struct image_scan_opt_data {
+	u8 has_color;
+	u8 has_trns;
+	u8 has_visible_pixels;
+	// bilevel means every pixel is black or white, and opaque.
+	u8 is_nonbilevel;
+	u8 has_only_invis_black_pixels;
+	u8 has_only_invis_white_pixels;
+	de_bitmap *optimg;
 };
 
 // Scan the image's pixels, and report whether any are transparent, etc.
-static void scan_image(de_bitmap *img, struct image_scan_results *isres)
+// Caller initializes optctx.
+static void scan_image(de_bitmap *img, struct image_scan_opt_data *optctx)
 {
 	i64 i, j;
 	de_color clr;
 	de_colorsample a, r, g, b;
 
-	de_zeromem(isres, sizeof(struct image_scan_results));
-	if(img->bytes_per_pixel==1) {
-		// No reason to scan opaque grayscale images.
-		isres->has_visible_pixels = 1;
+	if(img->bytes_per_pixel==1) { // Special case
+		optctx->has_visible_pixels = 1;
+
+		for(j=0; j<img->height; j++) {
+			for(i=0; i<img->width; i++) {
+				clr = de_bitmap_getpixel(img, i, j);
+				r = DE_COLOR_K(clr);
+				if(r!=0 && r!=255) {
+					optctx->is_nonbilevel = 1;
+					return;
+				}
+			}
+		}
 		return;
 	}
+
+	optctx->has_only_invis_black_pixels = 1;
+	optctx->has_only_invis_white_pixels = 1;
+
 	for(j=0; j<img->height; j++) {
 		for(i=0; i<img->width; i++) {
 			clr = de_bitmap_getpixel(img, i, j);
@@ -123,26 +174,60 @@ static void scan_image(de_bitmap *img, struct image_scan_results *isres)
 			r = DE_COLOR_R(clr);
 			g = DE_COLOR_G(clr);
 			b = DE_COLOR_B(clr);
-			if(!isres->has_visible_pixels && a!=0) {
-				isres->has_visible_pixels = 1;
+
+			if(!optctx->has_visible_pixels && a==0) {
+				if(optctx->has_only_invis_black_pixels) {
+					if(r || g || b) {
+						optctx->has_only_invis_black_pixels = 0;
+					}
+				}
+				if(optctx->has_only_invis_white_pixels) {
+					if((r!=0xff) || (g!=0xff) || (b!=0xff)) {
+						optctx->has_only_invis_white_pixels = 0;
+					}
+				}
 			}
-			if(!isres->has_trns && a<255) {
-				isres->has_trns = 1;
+
+			if(!optctx->has_visible_pixels && a!=0) {
+				optctx->has_visible_pixels = 1;
+				optctx->has_only_invis_black_pixels = 0;
+				optctx->has_only_invis_white_pixels = 0;
 			}
-			if(!isres->has_color && img->bytes_per_pixel>=3 &&
-				((g!=r || b!=r) && a!=0) )
+			if(!optctx->has_trns && a<255) {
+				optctx->has_trns = 1;
+				optctx->is_nonbilevel = 1;
+			}
+			if(!optctx->has_color && img->bytes_per_pixel>=3 &&
+				(g!=r || b!=r))
 			{
-				isres->has_color = 1;
+				// This doesn't test for "&& a!=0". It *could*, but our (undocumented)
+				// behavior is to try not to change the underlying color of invisible
+				// pixels.
+				// If we *were* going to do that, it would be more logical, and better
+				// for compression, to change the underlying color to black, instead
+				// of ultimately writing what amounts to garbage (probably just the
+				// red channel would survive).
+				optctx->has_color = 1;
+				optctx->is_nonbilevel = 1;
+			}
+
+			if(!optctx->is_nonbilevel && (r!=0 && r!=255)) {
+				// Only need to test one of the color samples. The has_color and
+				// has_trns tests will take care of the rest.
+				optctx->is_nonbilevel = 1;
 			}
 		}
 
 		// After each row, test whether we've learned everything we can learn
 		// about this image.
-		if((isres->has_trns || img->bytes_per_pixel==1 || img->bytes_per_pixel==3) &&
-			(isres->has_visible_pixels) &&
-			(isres->has_color || img->bytes_per_pixel<=2))
-		{
-			return;
+		if(img->bytes_per_pixel==2) {
+			if(optctx->has_trns && optctx->has_visible_pixels) return;
+		}
+		else if(img->bytes_per_pixel==3) {
+			if(optctx->has_color) return;
+		}
+		else { // bypp==4
+			if(optctx->has_color && optctx->has_trns && optctx->has_visible_pixels) return;
 		}
 	}
 }
@@ -174,68 +259,145 @@ static de_bitmap *de_bitmap_clone(de_bitmap *img1)
 	return img2;
 }
 
-// Returns NULL if there's no need to optimize the image
-static de_bitmap *get_optimized_image(de_bitmap *img1)
+// Caller initializes optctx.
+// Returns:
+//   optctx->optimg: NULL if no optimized image was produced.
+//     Caller must free if non-NULL.
+static void get_optimized_image(de_bitmap *img1, struct image_scan_opt_data *optctx)
 {
-	struct image_scan_results isres;
 	int opt_bytes_per_pixel;
-	de_bitmap *optimg;
 
-	scan_image(img1, &isres);
-	opt_bytes_per_pixel = isres.has_color ? 3 : 1;
-	if(isres.has_trns) opt_bytes_per_pixel++;
+	scan_image(img1, optctx);
 
-	if(opt_bytes_per_pixel>=img1->bytes_per_pixel) {
-		return NULL;
+	if(!optctx->is_nonbilevel) {
+		// We don't optimize this type of image here. It will be handled
+		// by the PNG encoder.
+		return;
 	}
 
-	optimg = de_bitmap_clone_noalloc(img1);
-	optimg->bytes_per_pixel = opt_bytes_per_pixel;
-	de_bitmap_copy_rect(img1, optimg, 0, 0, img1->width, img1->height, 0, 0, 0);
-	return optimg;
+	opt_bytes_per_pixel = optctx->has_color ? 3 : 1;
+	if(optctx->has_trns) opt_bytes_per_pixel++;
+
+	if(opt_bytes_per_pixel>=img1->bytes_per_pixel) {
+		return;
+	}
+
+	optctx->optimg = de_bitmap_clone_noalloc(img1);
+	optctx->optimg->bytes_per_pixel = opt_bytes_per_pixel;
+	de_bitmap_copy_rect(img1, optctx->optimg, 0, 0, img1->width, img1->height, 0, 0, 0);
+}
+
+static int valid_imglo(de_bitmap *img, de_bitmap *imglo)
+{
+	if(imglo->invalid_image_flag) return 0;
+	if(img->bytes_per_pixel != imglo->bytes_per_pixel) return 0;
+	if(img->width != imglo->width) return 0;
+	if(img->height != imglo->height) return 0;
+	return 1;
+}
+
+static int bitmap16_low_bits_important(de_bitmap *imghi, de_bitmap *imglo)
+{
+	i64 i, j;
+
+	for(j=0; j<imghi->height; j++) {
+		for(i=0; i<imghi->width; i++) {
+			de_color c1, c2;
+
+			c1 = de_bitmap_getpixel(imghi, i, j);
+			c2 = de_bitmap_getpixel(imglo, i, j);
+			if(c1 != c2) return 1;
+		}
+	}
+	return 0;
 }
 
 // When calling this function, the "name" data associated with fi, if set, should
 // be set to something like a filename, but *without* a final ".png" extension.
 // Image-specific createflags:
 //  - DE_CREATEFLAG_OPT_IMAGE
+//  - DE_CREATEFLAG_IS_BWIMG - Declares that image is bi-level B&W.
 //  - DE_CREATEFLAG_FLIP_IMAGE
 //     Write the rows in reverse order ("bottom-up"). This affects only the pixels,
 //     not the finfo metadata (e.g. hotspot). It's equivalent to flipping the image
 //     immediately before writing it, then flipping it back immediately after.
-void de_bitmap_write_to_file_finfo(de_bitmap *img, de_finfo *fi,
-	unsigned int createflags)
+// imglo: If non-NULL, contains the low 8 bits of each sample, and a 16 bits/sample
+//     output image will potentially be written. (This is obviously a hack, but
+//     it's not worth doing anything more for such a rarely used feature.)
+void de_bitmap16_write_to_file_finfo(de_bitmap *img, de_bitmap *imglo,
+	de_finfo *fi, unsigned int createflags)
 {
 	deark *c;
-	dbuf *f;
-	de_bitmap *optimg = NULL;
+	struct image_scan_opt_data optctx;
+	struct de_write_image_params wp;
 
 	if(!img) return;
 	c = img->c;
 	if(img->invalid_image_flag) return;
+	de_zeromem(&optctx, sizeof(struct image_scan_opt_data));
 
 	if(!img->bitmap) de_bitmap_alloc_pixels(img);
+	if(imglo) {
+		if(!imglo->bitmap) de_bitmap_alloc_pixels(imglo);
+		if(!valid_imglo(img, imglo)) return;
+	}
 
-	if(createflags & DE_CREATEFLAG_OPT_IMAGE) {
-		// This should probably be the default, but our optimization routine
-		// isn't very efficient, and wouldn't change anything in most cases.
-		optimg = get_optimized_image(img);
-		if(optimg) {
-			de_dbg3(c, "reducing image depth (%d->%d)", img->bytes_per_pixel,
-				optimg->bytes_per_pixel);
+	de_zeromem(&wp, sizeof(struct de_write_image_params));
+	wp.createflags = createflags;
+
+	if(imglo && (createflags & DE_CREATEFLAG_OPT_IMAGE)) {
+		// If the high and low bytes are the same in every sample, we don't need
+		// the low byte.
+		if(!bitmap16_low_bits_important(img, imglo)) {
+			imglo = NULL;
 		}
 	}
 
-	f = dbuf_create_output_file(c, "png", fi, createflags);
-	if(optimg) {
-		de_write_png(c, optimg, f, createflags);
+	if(imglo) {
+		; // The routines below don't support 16-bit images
+	}
+	else if(createflags & DE_CREATEFLAG_IS_BWIMG) {
+		// The BWIMG flag/optimization has to be handled in a different way than the
+		// other optimizations, because our de_bitmap object does not support a
+		// 1 bit/pixel image type.
+		wp.flags2 |= 0x1;
+	}
+	else if(createflags & DE_CREATEFLAG_OPT_IMAGE) {
+		// This should probably be the default, but our optimization routine
+		// isn't very efficient, and wouldn't change anything in most cases.
+		get_optimized_image(img, &optctx);
+		if(optctx.optimg) {
+			de_dbg3(c, "reducing image depth (%d->%d)", img->bytes_per_pixel,
+				optctx.optimg->bytes_per_pixel);
+		}
+		if(!optctx.is_nonbilevel) {
+			de_dbg3(c, "reducing to bilevel (from %d samples)", img->bytes_per_pixel);
+			wp.flags2 |= 0x1;
+		}
+	}
+
+	if(fi && fi->linear_colorpace) {
+		wp.flags2 |= 0x2;
+	}
+
+	wp.f = dbuf_create_output_file(c, "png", fi, createflags);
+	if(optctx.optimg) {
+		wp.img = optctx.optimg;
 	}
 	else {
-		de_write_png(c, img, f, createflags);
+		wp.img = img;
+		wp.imglo = imglo;
 	}
-	dbuf_close(f);
+	de_write_png(c, &wp);
+	dbuf_close(wp.f);
 
-	if(optimg) de_bitmap_destroy(optimg);
+	if(optctx.optimg) de_bitmap_destroy(optctx.optimg);
+}
+
+void de_bitmap_write_to_file_finfo(de_bitmap *img, de_finfo *fi,
+	unsigned int createflags)
+{
+	de_bitmap16_write_to_file_finfo(img, NULL, fi, createflags);
 }
 
 // "token" - A (UTF-8) filename component, like "output.000.<token>.png".
@@ -257,6 +419,7 @@ void de_bitmap_write_to_file(de_bitmap *img, const char *token,
 }
 
 // samplenum 0=Red, 1=Green, 2=Blue, 3=Alpha
+// If img is grayscale or grayscale+alpha, samplenum 0=Gray.
 void de_bitmap_setsample(de_bitmap *img, i64 x, i64 y,
 	i64 samplenum, de_colorsample v)
 {
@@ -497,6 +660,7 @@ u8 de_get_bits_symbol_lsb(dbuf *f, i64 bps, i64 rowstart, i64 index)
 
 // Read a symbol (up to 8 bits) that starts at an arbitrary bit position.
 // It may span (two) bytes.
+// TODO: Delete this function, if it's not being used.
 u8 de_get_bits_symbol2(dbuf *f, int nbits, i64 bytepos, i64 bitpos)
 {
 	u8 b0, b1;
@@ -606,7 +770,7 @@ void de_convert_and_write_image_bilevel(dbuf *f, i64 fpos,
 
 	img = de_bitmap_create(c, width, height, 1);
 	de_convert_image_bilevel(f, fpos, rowspan, img, cvtflags);
-	de_bitmap_write_to_file_finfo(img, fi, createflags);
+	de_bitmap_write_to_file_finfo(img, fi, createflags | DE_CREATEFLAG_IS_BWIMG);
 	de_bitmap_destroy(img);
 }
 
@@ -623,7 +787,7 @@ void de_convert_and_write_image_bilevel2(dbuf *f, i64 fpos,
 	if(!de_good_image_dimensions(c, width, height)) return;
 	img = de_bitmap_create2(c, width, rowspan*8, height, 1);
 	de_convert_image_bilevel(f, fpos, rowspan, img, cvtflags);
-	de_bitmap_write_to_file_finfo(img, fi, createflags);
+	de_bitmap_write_to_file_finfo(img, fi, createflags | DE_CREATEFLAG_IS_BWIMG);
 	de_bitmap_destroy(img);
 }
 
@@ -673,7 +837,7 @@ void de_read_simple_palette(deark *c, dbuf *f, i64 fpos,
 		de_dbg_indent(c, 1);
 	}
 
-	if(ncolors_to_save < ncolors_to_read) {
+	if(ncolors_to_save > ncolors_to_read) {
 		ncolors_to_save = ncolors_to_read;
 	}
 
@@ -761,22 +925,121 @@ done:
 	}
 }
 
+// flags:
+//  0x01 = lsb bit order
 void de_convert_image_paletted(dbuf *f, i64 fpos,
 	i64 bpp, i64 rowspan, const de_color *pal,
 	de_bitmap *img, unsigned int flags)
 {
 	i64 i, j;
 	unsigned int palent;
+	u8 mask;
 
 	if(bpp!=1 && bpp!=2 && bpp!=4 && bpp!=8) return;
-	if(!de_good_image_dimensions_noerr(f->c, img->width, img->height)) return;
+	if(!de_bitmap_good_dimensions(img, 0)) return;
+
+	mask = (1U<<(UI)bpp)-1;
 
 	for(j=0; j<img->height; j++) {
-		for(i=0; i<img->width; i++) {
-			palent = (unsigned int)de_get_bits_symbol(f, bpp, fpos+j*rowspan, i);
-			de_bitmap_setpixel_rgba(img, i, j, pal[palent]);
+		i64 pos = fpos + j*rowspan;
+
+		i = 0;
+
+		while(1) {
+			UI nbits_avail;
+			u8 b = 0;
+
+			b = dbuf_getbyte_p(f, &pos);
+			nbits_avail = 8;
+
+			while(nbits_avail >= (UI)bpp) {
+				nbits_avail -= (UI)bpp;
+				if(flags & 0x1) {
+					palent = b & mask;
+					b >>= (UI)bpp;
+				}
+				else {
+					palent = (b >> nbits_avail) & mask;
+				}
+				de_bitmap_setpixel_rgba(img, i++, j, pal[palent]);
+				if(i>=img->width) goto nextrow;
+			}
+		}
+	nextrow:
+		;
+	}
+}
+
+// Decode some planar paletted images.
+// Rows and planes must be byte-aligned.
+// All image data must be in the same dbuf.
+// row_stride = Dist. in bytes from first byte of row 0 plane 0 to first byte of row 1 plane 0
+// plane_stride = Dist. in bytes from first byte of row 0 plane 0 to first byte of row 0 plane 1
+// Note: row_stride may be smaller, or larger, than plane_stride.
+// Note: nplanes = bits/pixel
+// flags:
+//  0x01 = lsb bit order
+//  0x02 = lsb plane order
+void de_convert_image_paletted_planar(dbuf *f, i64 fpos, i64 nplanes,
+	i64 row_stride, i64 plane_stride, const de_color *pal, de_bitmap *img, UI flags)
+{
+	i64 ypos;
+	u8 bit_order_is_lsb = 0;
+	u8 plane_order_is_lsb = 0;
+	u8 pbit[8]; // [0] is for bits from the least-significant plane, etc.
+	i64 units_per_row; // num bytes per row per plane that we will process
+
+	if(nplanes<1 || nplanes>8) goto done;
+	de_zeromem(pbit, sizeof(pbit));
+
+	if(flags & 0x01) bit_order_is_lsb = 1;
+	if(flags & 0x02) plane_order_is_lsb = 1;
+	units_per_row = (img->width + 7)/8;
+
+	for(ypos=0; ypos<img->height; ypos++) {
+		i64 n;
+
+		// Read 8 bits from each plane, then rearrange them to make 8
+		// output pixels.
+		for(n=0; n<units_per_row; n++) {
+			UI k;
+			UI pn;
+			u8 b;
+			i64 xpos;
+
+			for(pn=0; pn<(UI)nplanes; pn++) {
+				b = dbuf_getbyte(f, fpos + ypos*row_stride + pn*plane_stride + n);
+				if(plane_order_is_lsb) {
+					pbit[pn] = b;
+				}
+				else {
+					pbit[(UI)nplanes-1-pn] = b;
+				}
+			}
+
+			for(k=0; k<8; k++) {
+				UI palent;
+
+				palent = 0;
+				for(pn=0; pn<(UI)nplanes; pn++) {
+					if((pbit[pn] & (1U<<k))!=0) {
+						palent |= 1U<<pn;
+					}
+				}
+
+				if(bit_order_is_lsb) {
+					xpos = n*8 + (i64)k;
+				}
+				else {
+					xpos = n*8 + (i64)(7-k);
+				}
+				de_bitmap_setpixel_rgba(img, xpos, ypos, pal[palent]);
+			}
 		}
 	}
+
+done:
+	;
 }
 
 void de_convert_image_rgb(dbuf *f, i64 fpos,
@@ -1004,22 +1267,33 @@ void de_bitmap_remove_alpha(de_bitmap *img)
 // If the image is 100% opaque, remove the alpha channel.
 // Otherwise do nothing.
 // flags:
-//  0x1: Make 100% invisible images 100% opaque
+//  0x1: Make 100% invisible images 100% opaque (always)
 //  0x2: Warn if an invisible image was made opaque
+//  0x4: Make 100% invisible images 100% opaque, unless the image seems
+//       sufficiently "boring".
 void de_bitmap_optimize_alpha(de_bitmap *img, unsigned int flags)
 {
-	struct image_scan_results isres;
+	struct image_scan_opt_data optctx;
 
 	if(img->bytes_per_pixel!=2 && img->bytes_per_pixel!=4) return;
 
-	scan_image(img, &isres);
+	de_zeromem(&optctx, sizeof(struct image_scan_opt_data));
+	scan_image(img, &optctx);
 
-	if(isres.has_trns && !isres.has_visible_pixels && (flags&0x1)) {
+	if((flags&0x4) && !optctx.has_visible_pixels &&
+		(optctx.has_only_invis_black_pixels || optctx.has_only_invis_white_pixels))
+	{
+		// No visible pixels, but the image is boring enough that it may be by
+		// design, so don't remove the alpha channel.
+		return;
+	}
+
+	if(optctx.has_trns && !optctx.has_visible_pixels && (flags&(0x1|0x4))) {
 		if(flags&0x2) {
 			de_warn(img->c, "Invisible image detected. Ignoring transparency.");
 		}
 	}
-	else if(isres.has_trns) {
+	else if(optctx.has_trns) {
 		return;
 	}
 
@@ -1041,3 +1315,31 @@ void de_make_grayscale_palette(de_color *pal, i64 num_entries, unsigned int flag
 		pal[k] = DE_MAKE_GRAY(b);
 	}
 }
+
+de_colorsample de_unpremultiply_alpha_samp(de_colorsample cval, de_colorsample a)
+{
+	if(a==0xff) {
+		return cval;
+	}
+	if(a==0 || cval==0) {
+		return 0;
+	}
+	if(cval>=a) {
+		return 0xff;
+	}
+	return (de_colorsample)(0.5 + (double)cval / ((double)a/255.0));
+}
+
+de_color de_unpremultiply_alpha_clr(de_color clr)
+{
+	de_colorsample r, g, b, a;
+
+	r = DE_COLOR_R(clr);
+	g = DE_COLOR_G(clr);
+	b = DE_COLOR_B(clr);
+	a = DE_COLOR_A(clr);
+	r = de_unpremultiply_alpha_samp(r, a);
+	g = de_unpremultiply_alpha_samp(g, a);
+	b = de_unpremultiply_alpha_samp(b, a);
+	return DE_MAKE_RGBA(r, g, b, a);
+}
