@@ -40,6 +40,20 @@
 
 #ifdef DE_UNIX
 
+#if defined(__amigaos4__) || defined(AMIGA)
+#ifndef DE_BUILDFLAG_AMIGA
+#define DE_BUILDFLAG_AMIGA 1
+#endif
+#endif
+
+#ifndef _POSIX_C_SOURCE
+#define _POSIX_C_SOURCE 200809L
+#endif
+#ifndef _FILE_OFFSET_BITS
+#define _FILE_OFFSET_BITS 64
+#define _TIME_BITS 64
+#endif
+
 #ifndef DE_USE_FSEEKO
 #define DE_USE_FSEEKO 1
 #endif
