@@ -14,6 +14,8 @@
 #define DE_MAX_MEMBUF_SIZE 2000000000
 #define DE_RCACHE_SIZE 262144
 #define DE_WBUFFER_SIZE 512
+// Support at least this many virtual bytes before or after the actual file.
+#define DE_ALLOWED_VIRTUAL_BYTES 16384
 
 // Fill the cache that remembers the first part of the file.
 // TODO: We should probably use memory-mapped files instead when possible,
@@ -82,6 +84,35 @@ static void populate_rcache_from_pipe(dbuf *f)
 	f->len = f->rcache_bytes_used;
 }
 
+// Use if we read a 'offset' field representing an absolute file position.
+// Returns 0 if we changed *plen.
+int dbuf_constrain_offset(dbuf *f, i64 *ppos)
+{
+	return de_constrain_int(ppos, 0, f->len);
+}
+
+// Use if we read a 'length' field associated with a segment with a known offset.
+// pos should be a valid position for this dbuf:
+// 0 to f->len inclusive. If not, sets *plen to 0, and returns 0.
+// Returns 0 if we changed *plen.
+int dbuf_constrain_length(dbuf *f, i64 pos, i64 *plen)
+{
+	i64 maxlen;
+
+	if(*plen < 0 || pos < 0 || pos > f->len ) {
+		*plen = 0;
+		return 0;
+	}
+
+	maxlen = f->len - pos;
+	if(*plen > maxlen) {
+		*plen = maxlen;
+		return 0;
+	}
+
+	return 1;
+}
+
 // Read len bytes, starting at file position pos, into buf.
 // Unread bytes will be set to 0.
 void dbuf_read(dbuf *f, u8 *buf, i64 pos, i64 len)
@@ -92,8 +123,14 @@ void dbuf_read(dbuf *f, u8 *buf, i64 pos, i64 len)
 
 	c = f->c;
 
+	if(len <= 0) goto done_read;
+	if(len > DE_MAX_MALLOC) {
+		de_fatalerror(c);
+		return;
+	}
+
 	if(pos < 0) {
-		if((-pos) >= len) {
+		if(pos <= (-len)) {
 			// All requested bytes are before the beginning of the file
 			de_zeromem(buf, (size_t)len);
 			return;
@@ -191,26 +228,19 @@ i64 dbuf_standard_read(dbuf *f, u8 *buf, i64 n, i64 *fpos)
 
 u8 dbuf_getbyte(dbuf *f, i64 pos)
 {
+	u8 b;
+
 	if(pos<0 || pos>=f->len) return 0x00;
 
 	if(pos<f->rcache_bytes_used) {
 		return f->rcache[pos];
 	}
 	if(f->btype==DBUF_TYPE_MEMBUF) {
-		// Note that it is necessary to handle read+write dbuf types specially,
-		// so that the "rcache2" feature isn't used.
 		return f->membuf_buf[pos];
 	}
 
-	// TODO: I don't like that rcache2 exists, but without it some large images
-	// are decoded too slowly (especially on Windows), and I haven't figured out
-	// a solution I like better.
-	if(pos==f->rcache2_pos) {
-		return f->rcache2;
-	}
-	f->rcache2_pos = pos;
-	dbuf_read(f, &f->rcache2, pos, 1);
-	return f->rcache2;
+	dbuf_read(f, &b, pos, 1);
+	return b;
 }
 
 i64 de_geti8_direct(const u8 *m)
@@ -1001,6 +1031,7 @@ static void finfo_shallow_copy(deark *c, de_finfo *src, de_finfo *dst)
 	UI k;
 
 	dst->is_directory = src->is_directory;
+	dst->is_volume_label = src->is_volume_label;
 	dst->has_riscos_data = src->has_riscos_data;
 	dst->riscos_appended_type = src->riscos_appended_type;
 	dst->riscos_attribs = src->riscos_attribs;
@@ -1023,7 +1054,6 @@ static dbuf *create_dbuf_lowlevel(deark *c)
 
 	f = de_malloc(c, sizeof(dbuf));
 	f->c = c;
-	f->rcache2_pos = -1; // Any offset outside the bounds of the file will do.
 	f->file_id = -1;
 	return f;
 }
@@ -1144,6 +1174,14 @@ dbuf *dbuf_create_output_file(deark *c, const char *ext1, de_finfo *fi,
 	f->max_len_hard = c->max_output_file_size;
 	f->is_managed = 1;
 
+	if(fi && fi->is_volume_label) {
+		if(c->output_style!=DE_OUTPUTSTYLE_ARCHIVE || c->archive_fmt!=DE_ARCHIVEFMT_ZIP) {
+			de_dbg(c, "skipping volume label");
+			f->btype = DBUF_TYPE_NULL;
+			goto done;
+		}
+	}
+
 	if(fi && fi->is_directory) {
 		is_directory = 1;
 	}
@@ -2344,6 +2382,8 @@ static int buffered_read_zero_len(struct de_bufferedreadctx *brctx,
 //   - If the source dbuf is a MEMBUF, and the requested bytes are all in range,
 //     then all requested bytes will be provided in the first call to the callback
 //     function.
+// As is normal for Deark, a slice may extend slightly before or after the file,
+// with nonexistent bytes getting the value 0.
 // Return value: 1 normally, 0 if the callback function ever returned 0.
 int dbuf_buffered_read(dbuf *f, i64 pos1, i64 len,
 	de_buffered_read_cbfn cbfn, void *userdata)
@@ -2353,6 +2393,14 @@ int dbuf_buffered_read(dbuf *f, i64 pos1, i64 len,
 	brctx.c = f->c;
 	brctx.userdata = userdata;
 
+	if((pos1 < -DE_ALLOWED_VIRTUAL_BYTES) ||
+		(pos1 > f->len+DE_ALLOWED_VIRTUAL_BYTES) ||
+		(len > f->len+DE_ALLOWED_VIRTUAL_BYTES) ||
+		(pos1+len > f->len+DE_ALLOWED_VIRTUAL_BYTES))
+	{
+		len = 0;
+	}
+
 	if(len<=0) { // Get this special case out of the way.
 		return buffered_read_zero_len(&brctx, cbfn);
 	}
@@ -2503,18 +2551,6 @@ static int slice_has_BOM(dbuf *inf, i64 pos, i64 len, de_encoding enc)
 	return 0;
 }
 
-static void constrain_slice_to_file(dbuf *f, i64 pos, i64 *plen)
-{
-	if(*plen < 0 || pos < 0 || pos > f->len) {
-		*plen = 0;
-		return;
-	}
-
-	if(*plen > f->len - pos) {
-		*plen = f->len - pos;
-	}
-}
-
 // Write a slice with a known encoding, to an output file, generally as UTF-8.
 //
 // This is a messy function intended to be used when extracting a text segment
@@ -2539,7 +2575,7 @@ void dbuf_copy_slice_convert_to_utf8(dbuf *inf, i64 pos, i64 len,
 	struct textconvctx_struct tcctx;
 
 	de_zeromem(&tcctx, sizeof(struct textconvctx_struct));
-	constrain_slice_to_file(inf, pos, &len);
+	dbuf_constrain_length(inf, pos, &len);
 
 	if((flags & 0x4) && (enc==DE_ENCODING_UNKNOWN || enc==DE_ENCODING_ASCII)) {
 		dbuf_copy(inf, pos, len, outf);
@@ -2575,6 +2611,19 @@ done:
 	ucstring_destroy(tcctx.tmpstr);
 }
 
+// Unused bits in n are required to be 0.
+void de_bitbuf_lowlevel_add_bits(struct de_bitbuf_lowlevel *bbll, u64 n, UI nbits)
+{
+	if(bbll->nbits_in_bitbuf+nbits>64) return;
+	if(bbll->is_lsb==0) {
+		bbll->bit_buf = (bbll->bit_buf<<nbits) | n;
+	}
+	else {
+		bbll->bit_buf |= (u64)n << bbll->nbits_in_bitbuf;
+	}
+	bbll->nbits_in_bitbuf += nbits;
+}
+
 void de_bitbuf_lowlevel_add_byte(struct de_bitbuf_lowlevel *bbll, u8 n)
 {
 	if(bbll->nbits_in_bitbuf>56) return;
