@@ -2,12 +2,15 @@
 // Copyright (C) 2016 Jason Summers
 // See the file COPYING for terms of use.
 
+#define DEARK_FMTUTIL_H_INC
+
 struct de_lz77buffer;
 
 struct de_dfilter_in_params {
 	dbuf *f;
 	i64 pos;
 	i64 len;
+	UI id; // Mainly for debugging
 };
 
 struct de_dfilter_out_params {
@@ -24,6 +27,14 @@ struct de_dfilter_results {
 	char errmsg[80];
 };
 
+// de_module_in_params::obj1
+// Used if in_params::flags & 0x02
+struct fmtutil_bmp_mparams_indata {
+	de_bitmap *img;
+	de_finfo *fi;
+	UI createflags;
+};
+
 struct de_bmpinfo {
 #define DE_BMPINFO_FMT_BMP 0
 #define DE_BMPINFO_FMT_PNG 1
@@ -111,7 +122,7 @@ void fmtutil_deflate_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 	void *codec_private_params);
 
 struct de_packbits_params {
-	u8 is_packbits16;
+	UI nbytes_per_unit; // 0=default (1)
 };
 void fmtutil_decompress_packbits_ex(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
@@ -138,8 +149,27 @@ void fmtutil_hlp_lz77_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 void fmtutil_huff_squeeze_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	void *codec_private_params);
+void fmtutil_huff_packit_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
 int fmtutil_decompress_exepack_reloc_tbl(deark *c, i64 pos1, i64 endpos, dbuf *outf);
 
+void fmtutil_xpk_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+int fmtutil_xpk_ismethodsupported(u32 method);
+
+void fmtutil_xpkMASH_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+void fmtutil_ic1_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+
+void fmtutil_decompress_stos_pictbank(deark *c, dbuf *inf,
+	i64 picdatapos, i64 rledatapos, i64 pointspos,
+	dbuf *unc_pixels, i64 unc_image_size);
+
 struct de_dfilter_ctx;
 typedef void (*dfilter_codec_type)(struct de_dfilter_ctx *dfctx, void *codec_private_params);
 typedef void (*dfilter_codec_addbuf_type)(struct de_dfilter_ctx *dfctx,
@@ -162,14 +192,19 @@ struct de_dfilter_ctx {
 };
 
 enum de_lzwfmt_enum {
-	DE_LZWFMT_GENERIC = 0,
+	DE_LZWFMT_UNKNOWN = 0,
 	DE_LZWFMT_UNIXCOMPRESS,
 	DE_LZWFMT_GIF,
 	DE_LZWFMT_ZIPSHRINK,
 	DE_LZWFMT_ZOOLZD,
-	DE_LZWFMT_TIFF,
+	DE_LZWFMT_TIFFOLD,
+	DE_LZWFMT_TIFFNEW,
 	DE_LZWFMT_ARC5,
-	DE_LZWFMT_DWC
+	DE_LZWFMT_DWC,
+	DE_LZWFMT_SHRINKIT1,
+	DE_LZWFMT_SHRINKIT2,
+	DE_LZWFMT_PAKLEO,
+	DE_LZWFMT_ASC2COM
 };
 
 struct de_lzw_params {
@@ -180,16 +215,21 @@ struct de_lzw_params {
 	UI flags;
 	unsigned int gif_root_code_size;
 	unsigned int max_code_size; // 0 = no info
-	u8 tifflzw_oldversion; // 1 = old version
 	u8 arc5_has_stop_code;
 };
 void fmtutil_decompress_lzw(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	struct de_lzw_params *lzwp);
 
+void fmtutil_ibmlzw_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+
 void dfilter_lzw_codec(struct de_dfilter_ctx *dfctx, void *codec_private_params);
 void dfilter_rle90_codec(struct de_dfilter_ctx *dfctx, void *codec_private_params);
 void dfilter_packbits_codec(struct de_dfilter_ctx *dfctx, void *codec_private_params);
+void dfilter_deflate_codec_miniz(struct de_dfilter_ctx *dfctx,
+	void *codec_private_params);
 
 struct de_lh5x_params {
 #define DE_LH5X_FMT_LH5     5
@@ -207,6 +247,9 @@ struct de_lh5x_params {
 	u8 warn_about_zero_codes_block;
 
 	u8 history_fill_val; // Set to 0x20 (space) if not sure.
+
+	// Returned to caller:
+	i64 max_offset_used;
 };
 void fmtutil_decompress_lh5x(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
@@ -244,6 +287,8 @@ void de_dfilter_finish(struct de_dfilter_ctx *dfctx);
 void de_dfilter_destroy(struct de_dfilter_ctx *dfctx);
 void de_dfilter_transfer_error(deark *c, struct de_dfilter_results *src,
 	struct de_dfilter_results *dst);
+void de_dfilter_transfer_error2(deark *c, struct de_dfilter_results *src,
+	struct de_dfilter_results *dst, const char *dst_modname);
 
 void de_dfilter_decompress_oneshot(deark *c,
 	dfilter_codec_type codec_init_fn, void *codec_private_params,
@@ -289,6 +334,13 @@ void fmtutil_distilled_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
 	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
 	void *codec_private_params);
 
+struct de_lzstac_params {
+	UI flags;
+};
+void fmtutil_lzstac_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params);
+
 void fmtutil_get_lzhuf_d_code_and_len(UI n, UI *pd_code, UI *pd_len);
 
 struct de_lh1_params {
@@ -512,7 +564,7 @@ const char *fmtutil_tiff_orientation_name(i64 n);
 const char *fmtutil_get_windows_charset_name(u8 cs);
 const char *fmtutil_get_windows_cb_data_type_name(unsigned int ty);
 
-int fmtutil_find_zip_eocd(deark *c, dbuf *f, i64 *foundpos);
+int fmtutil_find_zip_eocd(deark *c, dbuf *f, UI flags, i64 *foundpos);
 
 struct de_id3info {
 	int has_id3v1, has_id3v2;
@@ -552,8 +604,7 @@ struct de_advfile {
 	unsigned int createflags;
 	u8 enable_wbuffer;
 	u8 original_filename_flag;
-	u8 no_applesingle;
-	u8 no_appledouble;
+	u8 originally_appledouble;
 	u8 has_typecode;
 	u8 has_creatorcode;
 	u8 has_finderflags;
@@ -651,6 +702,7 @@ i64 fmtutil_huffman_get_num_codes(struct fmtutil_huffman_codebook *bk);
 void fmtutil_huffman_dump(deark *c, struct fmtutil_huffman_decoder *ht);
 int fmtutil_huffman_record_a_code_length(deark *c, struct fmtutil_huffman_code_builder *builder,
 	fmtutil_huffman_valtype val, UI len);
+i64 fmtutil_huffman_hcb_get_num_codes(struct fmtutil_huffman_code_builder *builder);
 #define FMTUTIL_MCTFLAG_LEFT_ALIGN_LEAVES     0x0 // default
 #define FMTUTIL_MCTFLAG_LEFT_ALIGN_BRANCHES   0x1
 #define FMTUTIL_MCTFLAG_LAST_CODE_FIRST       0x2 // Pretend codes were added in the reverse order
@@ -685,7 +737,17 @@ struct de_lz77buffer {
 	i64 start_of_dos_code;
 	i64 entry_point;
 	i64 end_of_dos_code;
+	i64 overlay_len;
+	i64 ext_hdr_pos;
+	u8 is_extended;
+	u8 is_ne;
+	u8 is_pe;
+	u8 is_dll;
+	u8 have_epcrcs;
+	u8 have_testbytes; // Are ep/ovl fields populated?
 	u64 entrypoint_crcs;
+	u8 ep64b[64]; // Some bytes at entry point
+	u8 ovl64b[64]; // Some bytes in overlay
 };
 void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei);
 
@@ -697,6 +759,14 @@ void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei);
 #define DE_SPECIALEXEFMT_EXECOMP   99 // Misc. unsupported executable compression
 #define DE_SPECIALEXEFMT_SFX       100 // Generic extractable self-extracting archive
 #define DE_SPECIALEXEFMT_ZIPSFX    101
+#define DE_SPECIALEXEFMT_ARJSFX    102
+#define DE_SPECIALEXEFMT_PAK16SFX  103
+#define DE_SPECIALEXEFMT_ISHIELDNE 110
+#define DE_SPECIALEXEFMT_GWS_EXEPIC 201 // Graphic Workshop
+#define DE_SPECIALEXEFMT_READMAKE   202
+#define DE_SPECIALEXEFMT_TEXE       203
+#define DE_SPECIALEXEFMT_READAMATIC 204
+#define DE_SPECIALEXEFMT_TEXTLIFE   205
 
  struct fmtutil_specialexe_detection_data {
 	u8 restrict_to_fmt; // DE_SPECIALEXEFMT_*; 0 = any
@@ -705,9 +775,14 @@ void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei);
 	u8 payload_valid;
 	u8 zip_eocd_looked_for;
 	u8 zip_eocd_found;
+	UI flags_in; // 0x1 = set when called by main exe module
+	UI flags_out; // format-specific
 	i64 payload_pos;
 	i64 payload_len;
+	i64 special_pos_1; // format-specific
 	i64 zip_eocd_pos;
+	i64 regCS_2; // For some patched files, the original CS/IP
+	i64 regIP_2;
 	const char *payload_file_ext;
 	const char *modname; // Non-NULL if we think we can decompress
 	char detected_fmt_name[40];
@@ -716,3 +791,50 @@ void fmtutil_detect_execomp(deark *c, struct fmtutil_exe_info *ei,
 	struct fmtutil_specialexe_detection_data *edd);
 void fmtutil_detect_exesfx(deark *c, struct fmtutil_exe_info *ei,
 	struct fmtutil_specialexe_detection_data *edd);
+void fmtutil_detect_specialexe(deark *c, struct fmtutil_exe_info *ei,
+	struct fmtutil_specialexe_detection_data *edd);
+int fmtutil_scan_for_arj_data(dbuf *f, i64 startpos, i64 max_skip,
+	UI flags, i64 *pfoundpos);
+void fmtutil_get_std_jpeg_qtable(UI tbl_id, u8 tbl[64]);
+void fmtutil_write_std_jpeg_dht(dbuf *outf, UI tbl_id);
+UI fmtutil_detect_pklite_by_exe_ep(deark *c, const u8 *mem, i64 mem_len, UI flags);
+
+struct fmtutil_char_simplectx {
+	dbuf *inf;
+	i64 inf_pos;
+	i64 inf_len;
+	de_encoding input_encoding;
+	u8 use_default_pal;
+	u8 nonblink;
+	i64 width_in_chars, height_in_chars;
+	i64 fg_stride, attr_offset;
+};
+void fmtutil_char_simple_run(deark *c, struct fmtutil_char_simplectx *csctx,
+	struct de_char_context *charctx);
+
+struct fmtutil_fmtid_ctx {
+	// default mode 0 = Detect all fmts that are reasonably portable.
+#define FMTUTIL_FMTIDMODE_ALL_IMG     1
+#define FMTUTIL_FMTIDMODE_ISH_SFX     10
+	u8 mode;
+	u8 have_bof64bytes;
+	const char *default_ext;
+	dbuf *inf; // Can be NULL if have_bof64bytes is set, but may reduce quality.
+	i64 inf_pos;
+	i64 inf_len;
+	u8 bof64bytes[64];
+
+#define FMTUTIL_FMTID_OTHER   1 // Used when we only need the extension.
+#define FMTUTIL_FMTID_JPEG    10
+#define FMTUTIL_FMTID_BMP     11
+#define FMTUTIL_FMTID_PNG     12
+#define FMTUTIL_FMTID_GIF     13
+#define FMTUTIL_FMTID_TIFF    14
+#define FMTUTIL_FMTID_CDR     17
+#define FMTUTIL_FMTID_WAVE    40
+#define FMTUTIL_FMTID_ZIP     60
+	UI fmtid; // 0 = unknown
+	char ext_sz[8];
+};
+
+void fmtutil_fmtid(deark *c, struct fmtutil_fmtid_ctx *idctx);
