@@ -36,7 +36,9 @@ DE_MODULE(de_module_eps)
 DE_MODULE(de_module_wmf)
 DE_MODULE(de_module_emf)
 DE_MODULE(de_module_midi)
+DE_MODULE(de_module_rgfx)
 DE_MODULE(de_module_corel_ccx)
+DE_MODULE(de_module_mmm)
 DE_MODULE(de_module_sunras)
 DE_MODULE(de_module_exe)
 DE_MODULE(de_module_riff)
@@ -55,16 +57,21 @@ DE_MODULE(de_module_mpegaudio)
 DE_MODULE(de_module_monkeys_audio)
 DE_MODULE(de_module_rm)
 DE_MODULE(de_module_mscompress)
+DE_MODULE(de_module_is_ibt)
+DE_MODULE(de_module_mrnz)
 DE_MODULE(de_module_wpg)
 DE_MODULE(de_module_car_lha)
 DE_MODULE(de_module_lha)
 DE_MODULE(de_module_swg)
+DE_MODULE(de_module_tpk)
+DE_MODULE(de_module_pakleo)
 DE_MODULE(de_module_arx)
 DE_MODULE(de_module_zoo)
 DE_MODULE(de_module_zoo_filter)
 DE_MODULE(de_module_zoo_z)
 DE_MODULE(de_module_binhex)
 DE_MODULE(de_module_anim)
+DE_MODULE(de_module_deep)
 DE_MODULE(de_module_jbf)
 DE_MODULE(de_module_dcx)
 DE_MODULE(de_module_bpg)
@@ -83,6 +90,7 @@ DE_MODULE(de_module_mrw)
 DE_MODULE(de_module_zip)
 DE_MODULE(de_module_tga)
 DE_MODULE(de_module_dwc)
+DE_MODULE(de_module_rar)
 DE_MODULE(de_module_tscomp)
 DE_MODULE(de_module_pict)
 DE_MODULE(de_module_ico)
@@ -90,15 +98,23 @@ DE_MODULE(de_module_clp)
 DE_MODULE(de_module_fli)
 DE_MODULE(de_module_arj)
 DE_MODULE(de_module_is_z)
+DE_MODULE(de_module_is_instarch)
+DE_MODULE(de_module_is_sfx)
 DE_MODULE(de_module_amigaicon)
 DE_MODULE(de_module_amiga_adf)
 DE_MODULE(de_module_amiga_dms)
+DE_MODULE(de_module_apple2_dsk)
 DE_MODULE(de_module_woz)
+DE_MODULE(de_module_moof)
 DE_MODULE(de_module_sauce)
 DE_MODULE(de_module_xbin)
 DE_MODULE(de_module_ansiart)
 DE_MODULE(de_module_bintext)
 DE_MODULE(de_module_icedraw)
+DE_MODULE(de_module_thedraw_com)
+DE_MODULE(de_module_aciddraw_com)
+DE_MODULE(de_module_ldog_com)
+DE_MODULE(de_module_grabber)
 DE_MODULE(de_module_sis)
 DE_MODULE(de_module_hsiraw)
 DE_MODULE(de_module_gemraster)
@@ -107,8 +123,10 @@ DE_MODULE(de_module_pgx)
 DE_MODULE(de_module_fnt)
 DE_MODULE(de_module_uuencode)
 DE_MODULE(de_module_ascii85)
+DE_MODULE(de_module_binscii)
 DE_MODULE(de_module_hpicn)
 DE_MODULE(de_module_hpi)
+DE_MODULE(de_module_hta)
 DE_MODULE(de_module_flif)
 DE_MODULE(de_module_psf)
 DE_MODULE(de_module_nol)
@@ -118,6 +136,10 @@ DE_MODULE(de_module_nlm)
 DE_MODULE(de_module_nsl)
 DE_MODULE(de_module_abk)
 DE_MODULE(de_module_amos_source)
+DE_MODULE(de_module_stos_pp1)
+DE_MODULE(de_module_stos_pp2)
+DE_MODULE(de_module_stos_pp3)
+DE_MODULE(de_module_stos_daj)
 DE_MODULE(de_module_mbk)
 DE_MODULE(de_module_pff2)
 DE_MODULE(de_module_tivariable)
@@ -133,9 +155,13 @@ DE_MODULE(de_module_zlib)
 DE_MODULE(de_module_deflate)
 DE_MODULE(de_module_tim)
 DE_MODULE(de_module_drhalocut)
+DE_MODULE(de_module_drhalopic)
+DE_MODULE(de_module_xwd)
 DE_MODULE(de_module_makichan)
 DE_MODULE(de_module_cserve_rle)
 DE_MODULE(de_module_lotus_mscr)
+DE_MODULE(de_module_fastgraph_spr)
+DE_MODULE(de_module_fastgraph_ppr)
 DE_MODULE(de_module_crg)
 DE_MODULE(de_module_zbr)
 DE_MODULE(de_module_cdr_wl)
@@ -154,8 +180,27 @@ DE_MODULE(de_module_crlzh)
 DE_MODULE(de_module_zsq)
 DE_MODULE(de_module_cpshrink)
 DE_MODULE(de_module_edi_pack)
+DE_MODULE(de_module_qip)
+DE_MODULE(de_module_pcxlib)
+DE_MODULE(de_module_gxlib)
+DE_MODULE(de_module_mdcd)
+DE_MODULE(de_module_cazip)
+DE_MODULE(de_module_cmz)
+DE_MODULE(de_module_pcshrink)
+DE_MODULE(de_module_arcv)
+DE_MODULE(de_module_red)
+DE_MODULE(de_module_lif_kdc)
+DE_MODULE(de_module_ain)
+DE_MODULE(de_module_hit)
+DE_MODULE(de_module_binary_ii)
+DE_MODULE(de_module_tome)
+DE_MODULE(de_module_os2pack)
+DE_MODULE(de_module_os2pack2)
+DE_MODULE(de_module_nufx)
 DE_MODULE(de_module_lharc_sfx_com)
 DE_MODULE(de_module_pklite)
+DE_MODULE(de_module_diet)
+DE_MODULE(de_module_bsave_cmpr)
 DE_MODULE(de_module_bsave)
 DE_MODULE(de_module_xpuzzle)
 DE_MODULE(de_module_autocad_slb)
@@ -173,12 +218,17 @@ DE_MODULE(de_module_msa)
 DE_MODULE(de_module_pasti)
 DE_MODULE(de_module_vbm)
 DE_MODULE(de_module_winzle)
+DE_MODULE(de_module_fontmania)
 DE_MODULE(de_module_pcrfont)
+DE_MODULE(de_module_fontedit)
+DE_MODULE(de_module_evafont)
+DE_MODULE(de_module_cpi)
 DE_MODULE(de_module_bob)
 DE_MODULE(de_module_hr)
 DE_MODULE(de_module_applevol)
 DE_MODULE(de_module_basic_c64)
 DE_MODULE(de_module_mswordscr)
+DE_MODULE(de_module_pcx2com)
 DE_MODULE(de_module_ps_gradient)
 DE_MODULE(de_module_ps_styles)
 DE_MODULE(de_module_ps_action)
@@ -202,21 +252,26 @@ DE_MODULE(de_module_neochrome_ani)
 DE_MODULE(de_module_iccprofile)
 DE_MODULE(de_module_iptc)
 DE_MODULE(de_module_ripicon)
-DE_MODULE(de_module_gws_thn)
 DE_MODULE(de_module_insetpix)
 DE_MODULE(de_module_macbinary)
+DE_MODULE(de_module_packit)
 DE_MODULE(de_module_macpaint)
 DE_MODULE(de_module_printshop)
 DE_MODULE(de_module_newprintshop)
 DE_MODULE(de_module_printmaster)
+DE_MODULE(de_module_printshop_gs)
 DE_MODULE(de_module_pp_gph)
 DE_MODULE(de_module_rosprite)
 DE_MODULE(de_module_rodraw)
 DE_MODULE(de_module_palmdb)
 DE_MODULE(de_module_macrsrc)
+DE_MODULE(de_module_pif)
+DE_MODULE(de_module_desqview_dvp)
 DE_MODULE(de_module_fat)
 DE_MODULE(de_module_loaddskf)
 DE_MODULE(de_module_ea_data)
+DE_MODULE(de_module_txt2com)
+DE_MODULE(de_module_show_gmr)
 DE_MODULE(de_module_wad)
 DE_MODULE(de_module_crush)
 DE_MODULE(de_module_eggpaint)
@@ -230,18 +285,47 @@ DE_MODULE(de_module_xface)
 DE_MODULE(de_module_mdesk_icn)
 DE_MODULE(de_module_animator_pic)
 DE_MODULE(de_module_young_picasso)
+DE_MODULE(de_module_iconmgr_ica)
+DE_MODULE(de_module_thumbsplus)
+DE_MODULE(de_module_fmtowns_icn)
+DE_MODULE(de_module_fmtowns_hel)
+DE_MODULE(de_module_pixfolio)
+DE_MODULE(de_module_apple2icons)
+DE_MODULE(de_module_div_map)
+DE_MODULE(de_module_div_fpg)
+DE_MODULE(de_module_imggal_alch)
+DE_MODULE(de_module_gws_thn)
+DE_MODULE(de_module_asc2com)
+DE_MODULE(de_module_doc2com)
+DE_MODULE(de_module_doc2com_dkn)
+DE_MODULE(de_module_gtxt)
+DE_MODULE(de_module_readmake)
+DE_MODULE(de_module_texe)
+DE_MODULE(de_module_readamatic)
+DE_MODULE(de_module_ascom)
+DE_MODULE(de_module_textlife)
+DE_MODULE(de_module_sgiimage)
 DE_MODULE(de_module_dgi)
 DE_MODULE(de_module_cdi_imag)
 DE_MODULE(de_module_storyboard)
 DE_MODULE(de_module_adex)
 DE_MODULE(de_module_pkm)
 DE_MODULE(de_module_dclimplode)
+DE_MODULE(de_module_lgcompress)
+DE_MODULE(de_module_compress_lzh)
+DE_MODULE(de_module_lzstac)
+DE_MODULE(de_module_npack)
+DE_MODULE(de_module_lzs221)
+DE_MODULE(de_module_xpk)
+DE_MODULE(de_module_tc_trs80)
 DE_MODULE(de_module_ar001)
+DE_MODULE(de_module_aldus_inst)
 DE_MODULE(de_module_vort)
 DE_MODULE(de_module_ybm)
 DE_MODULE(de_module_qdv)
 DE_MODULE(de_module_jovianvi)
 DE_MODULE(de_module_vitec)
+DE_MODULE(de_module_colorix)
 DE_MODULE(de_module_palmbitmap)
 DE_MODULE(de_module_fpaint_pi4)
 DE_MODULE(de_module_fpaint_pi9)
@@ -250,9 +334,14 @@ DE_MODULE(de_module_falcon_xga)
 DE_MODULE(de_module_coke)
 DE_MODULE(de_module_videomaster)
 DE_MODULE(de_module_alphabmp)
-DE_MODULE(de_module_kqp)
+DE_MODULE(de_module_syberia_syj)
+DE_MODULE(de_module_esm_pix)
+DE_MODULE(de_module_pegasus_pic2)
+DE_MODULE(de_module_picjpeg)
 DE_MODULE(de_module_dib)
+DE_MODULE(de_module_jigsaw_wk)
 DE_MODULE(de_module_win1ico)
+DE_MODULE(de_module_os2bootlogo)
 DE_MODULE(de_module_artworx_adf)
 DE_MODULE(de_module_lumena_cel)
 DE_MODULE(de_module_pm_xv)
@@ -260,10 +349,16 @@ DE_MODULE(de_module_bld)
 DE_MODULE(de_module_megapaint_pat)
 DE_MODULE(de_module_megapaint_lib)
 DE_MODULE(de_module_dlmaker)
+DE_MODULE(de_module_dosbackup33)
+DE_MODULE(de_module_dosbackup20)
+DE_MODULE(de_module_comicchat)
 DE_MODULE(de_module_nrg)
 DE_MODULE(de_module_apm)
 DE_MODULE(de_module_cd_raw)
 DE_MODULE(de_module_hfs)
+DE_MODULE(de_module_reko)
+DE_MODULE(de_module_wizsolitaire)
+DE_MODULE(de_module_gws_exepic)
 DE_MODULE(de_module_compfacei)
 DE_MODULE(de_module_hs2)
 DE_MODULE(de_module_apetag)
@@ -285,12 +380,14 @@ DE_MODULE(de_module_lzexe)
 DE_MODULE(de_module_ddb)
 DE_MODULE(de_module_vgafont)
 DE_MODULE(de_module_crc)
+DE_MODULE(de_module_datetime)
 DE_MODULE(de_module_hexdump)
 DE_MODULE(de_module_bytefreq)
 DE_MODULE(de_module_plaintext)
 DE_MODULE(de_module_cp437)
 DE_MODULE(de_module_copy)
 DE_MODULE(de_module_xor)
+DE_MODULE(de_module_join)
 DE_MODULE(de_module_split)
 DE_MODULE(de_module_null)
 DE_MODULE_LAST(de_module_unsupported)
