@@ -18,6 +18,7 @@
 #define CODE_IDAT 0x49444154U
 #define CODE_IEND 0x49454e44U
 #define CODE_IHDR 0x49484452U
+#define CODE_gAMA 0x67414d41U
 #define CODE_htSP 0x68745350U
 #define CODE_pHYs 0x70485973U
 #define CODE_tEXt 0x74455874U
@@ -26,20 +27,27 @@
 struct deark_png_encode_info {
 	deark *c;
 	dbuf *outf;
+	de_bitmap *img;
+	de_bitmap *imglo;
 	int width, height;
-	int rowspan;
-	int num_chans;
+	int src_rowspan;
+	int num_chans; // both src and dst
+	int samples_per_row; // both src and dst
 	int flip;
 	unsigned int level;
 	int has_phys;
 	u32 xdens;
 	u32 ydens;
 	u8 phys_units;
+	u8 linear_colorspace;
 	struct de_timestamp internal_mod_time;
 	u8 include_text_chunk_software;
 	u8 has_hotspot;
+	u8 encode_as_bwimg;
 	int hotspot_x, hotspot_y;
 	struct de_crcobj *crco;
+	size_t dst_rowspan;
+	u8 *tmprow;
 };
 
 static void write_png_chunk_from_mem(struct deark_png_encode_info *pei,
@@ -87,12 +95,25 @@ static void write_png_chunk_IHDR(struct deark_png_encode_info *pei,
 
 	dbuf_writeu32be(cdbuf, (i64)pei->width);
 	dbuf_writeu32be(cdbuf, (i64)pei->height);
-	dbuf_writebyte(cdbuf, 8); // bit depth
-	dbuf_writebyte(cdbuf, color_type_code[pei->num_chans]);
+	if(pei->encode_as_bwimg) {
+		dbuf_writebyte(cdbuf, 1); // bit depth
+		dbuf_writebyte(cdbuf, 0x00);
+	}
+	else {
+		dbuf_writebyte(cdbuf, (pei->imglo ? 16 : 8)); // bit depth
+		dbuf_writebyte(cdbuf, color_type_code[pei->num_chans]);
+	}
 	dbuf_truncate(cdbuf, 13); // rest of chunk is zeroes
 	write_png_chunk_from_cdbuf(pei, cdbuf, CODE_IHDR);
 }
 
+static void write_png_chunk_gAMA_linear(struct deark_png_encode_info *pei,
+	dbuf *cdbuf)
+{
+	dbuf_writeu32be(cdbuf, 100000);
+	write_png_chunk_from_cdbuf(pei, cdbuf, CODE_gAMA);
+}
+
 static void write_png_chunk_pHYs(struct deark_png_encode_info *pei,
 	dbuf *cdbuf)
 {
@@ -189,11 +210,62 @@ static void my_IDAT_write_cb(dbuf *f, void *userdata,
 	}
 }
 
-static int write_png_chunk_IDATs(struct deark_png_encode_info *pei, dbuf *cdbuf,
-	const u8 *src_pixels)
+static void compress_png_row(struct deark_png_encode_info *pei, struct fmtutil_tdefl_ctx *tdctx,
+	int y)
+{
+	static const char nulbyte = '\0';
+
+	// Filter byte
+	fmtutil_tdefl_compress_buffer(tdctx, &nulbyte, 1, FMTUTIL_TDEFL_NO_FLUSH);
+
+	if(pei->encode_as_bwimg) {
+		int x;
+
+		if(pei->tmprow) {
+			de_zeromem(pei->tmprow, pei->dst_rowspan);
+		}
+		else {
+			pei->tmprow = de_malloc(pei->c, pei->dst_rowspan);
+		}
+
+		for(x=0; x<pei->width; x++) {
+			u8 k;
+
+			// We just use the red sample, like DE_COLOR_K().
+			k = pei->img->bitmap[y*pei->src_rowspan + x*pei->img->bytes_per_pixel];
+			if(k>=0x80) {
+				pei->tmprow[x/8] |= 1U<<(7-x%8);
+			}
+		}
+
+		fmtutil_tdefl_compress_buffer(tdctx, pei->tmprow, pei->dst_rowspan, FMTUTIL_TDEFL_NO_FLUSH);
+	}
+	else if(pei->imglo) {
+		int x;
+
+		if(pei->tmprow) {
+			de_zeromem(pei->tmprow, pei->dst_rowspan);
+		}
+		else {
+			pei->tmprow = de_malloc(pei->c, pei->dst_rowspan);
+		}
+
+		for(x=0; x<pei->samples_per_row; x++) {
+			pei->tmprow[x*2] = pei->img->bitmap[y*pei->src_rowspan + x];
+			pei->tmprow[x*2+1] = pei->imglo->bitmap[y*pei->src_rowspan + x];
+		}
+
+		fmtutil_tdefl_compress_buffer(tdctx, pei->tmprow, pei->dst_rowspan, FMTUTIL_TDEFL_NO_FLUSH);
+	}
+	else {
+		fmtutil_tdefl_compress_buffer(tdctx, &pei->img->bitmap[y*pei->src_rowspan],
+			pei->dst_rowspan, FMTUTIL_TDEFL_NO_FLUSH);
+	}
+}
+
+static int write_png_chunk_IDATs(struct deark_png_encode_info *pei, dbuf *cdbuf)
 {
 	int y;
-	static const char nulbyte = '\0';
 	int retval = 0;
 	deark *c = pei->c;
 	struct fmtutil_tdefl_ctx *tdctx = NULL;
@@ -213,11 +285,18 @@ static int write_png_chunk_IDATs(struct deark_png_encode_info *pei, dbuf *cdbuf,
 	tdctx = fmtutil_tdefl_create(c, outf_IDAT,
 		my_s_tdefl_num_probes[MY_MZ_MIN(10, pei->level)] | MY_TDEFL_WRITE_ZLIB_HEADER);
 
-	for (y = 0; y < pei->height; ++y) {
-		fmtutil_tdefl_compress_buffer(tdctx, &nulbyte, 1, FMTUTIL_TDEFL_NO_FLUSH);
-		fmtutil_tdefl_compress_buffer(tdctx, &src_pixels[(pei->flip ? (pei->height - 1 - y) : y) * pei->rowspan],
-			(size_t)pei->width * (size_t)pei->num_chans, FMTUTIL_TDEFL_NO_FLUSH);
+	if(pei->encode_as_bwimg) {
+		pei->dst_rowspan = ((size_t)pei->width+7)/8;
 	}
+	else {
+		pei->dst_rowspan = (size_t)pei->width * (size_t)pei->num_chans;
+		if(pei->imglo) pei->dst_rowspan *= 2;
+	}
+
+	for (y = 0; y<pei->height; y++) {
+		compress_png_row(pei, tdctx, (pei->flip ? (pei->height - 1 - y) : y));
+	}
+
 	if (fmtutil_tdefl_compress_buffer(tdctx, NULL, 0, FMTUTIL_TDEFL_FINISH) !=
 		FMTUTIL_TDEFL_STATUS_DONE)
 	{
@@ -249,6 +328,11 @@ static int do_generate_png(struct deark_png_encode_info *pei, const u8 *src_pixe
 
 	write_png_chunk_IHDR(pei, cdbuf);
 
+	if(pei->linear_colorspace) {
+		dbuf_truncate(cdbuf, 0);
+		write_png_chunk_gAMA_linear(pei, cdbuf);
+	}
+
 	if(pei->has_phys) {
 		dbuf_truncate(cdbuf, 0);
 		write_png_chunk_pHYs(pei, cdbuf);
@@ -270,7 +354,7 @@ static int do_generate_png(struct deark_png_encode_info *pei, const u8 *src_pixe
 	}
 
 	dbuf_truncate(cdbuf, 0);
-	if(!write_png_chunk_IDATs(pei, cdbuf, src_pixels)) goto done;
+	if(!write_png_chunk_IDATs(pei, cdbuf)) goto done;
 
 	dbuf_truncate(cdbuf, 0);
 	write_png_chunk_from_cdbuf(pei, cdbuf, CODE_IEND);
@@ -281,7 +365,10 @@ done:
 	return retval;
 }
 
-int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags)
+// wp->flags2:
+//   0x1 = image can be encoded as bi-level, black&white, opaque
+//   0x2 = linear colorspace
+int de_write_png(deark *c, struct de_write_image_params *wp)
 {
 	const char *opt_level;
 	int retval = 0;
@@ -290,29 +377,38 @@ int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags)
 	pei = de_malloc(c, sizeof(struct deark_png_encode_info));
 	pei->c = c;
 
-	if(img->invalid_image_flag) {
+	if(wp->img->invalid_image_flag) {
 		goto done;
 	}
-	if(!de_good_image_dimensions(c, img->width, img->height)) {
+	if(!de_good_image_dimensions(c, wp->img->width, wp->img->height)) {
 		goto done;
 	}
 
 	// Optimization to speed up list mode
-	if(f->btype==DBUF_TYPE_NULL && !c->enable_oinfo) {
+	if(wp->f->btype==DBUF_TYPE_NULL && !c->enable_oinfo) {
 		goto done;
 	}
 
-	if(f->fi_copy && f->fi_copy->density.code>0 && c->write_density) {
+	pei->img = wp->img;
+	pei->imglo = wp->imglo;
+	if(wp->flags2 & 0x1) {
+		pei->encode_as_bwimg = 1;
+	}
+	if(wp->flags2 & 0x2) {
+		pei->linear_colorspace = 1;
+	}
+
+	if(wp->f->fi_copy && wp->f->fi_copy->density.code>0 && c->write_density) {
 		pei->has_phys = 1;
-		if(f->fi_copy->density.code==1) { // unspecified units
+		if(wp->f->fi_copy->density.code==1) { // unspecified units
 			pei->phys_units = 0;
-			pei->xdens = (u32)(f->fi_copy->density.xdens+0.5);
-			pei->ydens = (u32)(f->fi_copy->density.ydens+0.5);
+			pei->xdens = (u32)(wp->f->fi_copy->density.xdens+0.5);
+			pei->ydens = (u32)(wp->f->fi_copy->density.ydens+0.5);
 		}
-		else if(f->fi_copy->density.code==2) { // dpi
+		else if(wp->f->fi_copy->density.code==2) { // dpi
 			pei->phys_units = 1; // pixels/meter
-			pei->xdens = (u32)(0.5+f->fi_copy->density.xdens/0.0254);
-			pei->ydens = (u32)(0.5+f->fi_copy->density.ydens/0.0254);
+			pei->xdens = (u32)(0.5+wp->f->fi_copy->density.xdens/0.0254);
+			pei->ydens = (u32)(0.5+wp->f->fi_copy->density.ydens/0.0254);
 		}
 	}
 
@@ -330,17 +426,18 @@ int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags)
 		}
 	}
 
-	pei->outf = f;
-	if(!c->padpix && img->unpadded_width>0 && img->unpadded_width<img->width) {
-		pei->width = (int)img->unpadded_width;
+	pei->outf = wp->f;
+	if(!c->padpix && wp->img->unpadded_width>0 && wp->img->unpadded_width<wp->img->width) {
+		pei->width = (int)wp->img->unpadded_width;
 	}
 	else {
-		pei->width = (int)img->width;
+		pei->width = (int)wp->img->width;
 	}
-	pei->rowspan = (int)(img->width * img->bytes_per_pixel);
-	pei->height = (int)img->height;
-	pei->flip = (createflags & DE_CREATEFLAG_FLIP_IMAGE)?1:0;
-	pei->num_chans = img->bytes_per_pixel;
+	pei->src_rowspan = (int)(wp->img->width * wp->img->bytes_per_pixel);
+	pei->height = (int)wp->img->height;
+	pei->flip = (wp->createflags & DE_CREATEFLAG_FLIP_IMAGE)?1:0;
+	pei->num_chans = wp->img->bytes_per_pixel;
+	pei->samples_per_row = pei->width * pei->num_chans;
 	pei->include_text_chunk_software = 0;
 
 	if(!c->pngcprlevel_valid) {
@@ -363,21 +460,21 @@ int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags)
 	}
 	pei->level = c->pngcmprlevel;
 
-	if(f->fi_copy && f->fi_copy->internal_mod_time.is_valid) {
-		pei->internal_mod_time = f->fi_copy->internal_mod_time;
+	if(wp->f->fi_copy && wp->f->fi_copy->internal_mod_time.is_valid) {
+		pei->internal_mod_time = wp->f->fi_copy->internal_mod_time;
 	}
 
-	if(f->fi_copy && f->fi_copy->has_hotspot) {
+	if(wp->f->fi_copy && wp->f->fi_copy->has_hotspot) {
 		pei->has_hotspot = 1;
-		pei->hotspot_x = f->fi_copy->hotspot_x;
-		pei->hotspot_y = f->fi_copy->hotspot_y;
+		pei->hotspot_x = wp->f->fi_copy->hotspot_x;
+		pei->hotspot_y = wp->f->fi_copy->hotspot_y;
 		// Leave a hint as to where our custom Hotspot chunk came from.
 		pei->include_text_chunk_software = 1;
 	}
 
 	pei->crco = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
 
-	if(!do_generate_png(pei, img->bitmap)) {
+	if(!do_generate_png(pei, wp->img->bitmap)) {
 		de_err(c, "PNG write failed");
 		goto done;
 	}
@@ -387,6 +484,7 @@ int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags)
 done:
 	if(pei) {
 		de_crcobj_destroy(pei->crco);
+		de_free(c, pei->tmprow);
 		de_free(c, pei);
 	}
 	return retval;
