@@ -13,6 +13,7 @@
 #include "deark.h"
 #endif
 
+#define DE_MAX_MALLOC           500000000
 #define DE_MAX_SANE_OBJECT_SIZE 100000000
 
 typedef i32 de_rune; // A Unicode codepoint
@@ -34,12 +35,17 @@ enum de_encoding_enum {
 	DE_ENCODING_WINDOWS1254,
 	DE_ENCODING_WINDOWS874,
 	DE_ENCODING_CP437,
+	DE_ENCODING_CP850,
+	DE_ENCODING_CP866,
+	DE_ENCODING_CP932,
 	DE_ENCODING_MACROMAN,
 	DE_ENCODING_ATARIST,
 	DE_ENCODING_PALM,
 	DE_ENCODING_RISCOS,
 	DE_ENCODING_PETSCII,
-	DE_ENCODING_DEC_SPECIAL_GRAPHICS
+	DE_ENCODING_APPLEII,
+	DE_ENCODING_DEC_SPECIAL_GRAPHICS,
+	DE_ENCODING_ROT13
 };
 #define DE_ENCODING_CP437_G DE_ENCODING_CP437
 
@@ -95,6 +101,9 @@ struct deark_module_info {
 #define DE_MODFLAG_NOEXTRACT    0x04 // Do not warn if no files are extracted
 #define DE_MODFLAG_SECURITYWARNING 0x08
 #define DE_MODFLAG_SHAREDDETECTION 0x10 // Module modifies deark::detection_data
+#define DE_MODFLAG_WARNPARSEONLY   0x20 // Do not list; print warning if autodetected
+#define DE_MODFLAG_MULTIPART       0x40 // Supports multiple input files
+#define DE_MODFLAG_INTERNALONLY    0x80
 #define DE_MODFLAG_DISABLEDETECT 0x100 // Ignore results of autodetection
 	u32 flags;
 	u32 unique_id; // or 0. Rarely used.
@@ -106,7 +115,8 @@ typedef void (*de_module_getinfo_fn)(deark *c, struct deark_module_info *mi);
 struct de_encconv_state;
 typedef de_rune (*de_encconv_fn)(struct de_encconv_state *es, i32 a);
 struct de_encconv_state {
-	de_ext_encoding ee;
+	de_encoding enc;
+	int enc_subtype;
 	de_encconv_fn fn;
 	const void *fn_pvt_data;
 	u8 buf[8];
@@ -130,6 +140,7 @@ struct de_timestamp {
 	// for 0 (UNKNOWN).
 #define DE_TSPREC_UNKNOWN 0 // default, usually treated as 1sec
 #define DE_TSPREC_1DAY 10
+#define DE_TSPREC_1MIN 15
 #define DE_TSPREC_2SEC 20
 #define DE_TSPREC_1SEC 30
 #define DE_TSPREC_HIGH 40 // = better than 1 second
@@ -196,10 +207,6 @@ struct dbuf_struct {
 	i64 rcache_bytes_used;
 	u8 *rcache; // first 'cache_bytes_used' bytes of the file
 
-	// cache2 is a simple 1-byte cache, mainly to speed up de_get_bits_symbol().
-	i64 rcache2_pos;
-	u8 rcache2;
-
 	// Things copied from the de_finfo object at file creation
 	de_finfo *fi_copy;
 };
@@ -220,10 +227,12 @@ struct de_finfo_struct {
 	de_ucstring *file_name_internal; // Modules should avoid using this field directly.
 	u8 original_filename_flag; // Indicates if .file_name_internal is a real file name
 	u8 is_directory; // Does the "file" represent a subdirectory?
+	u8 is_volume_label;
 	u8 is_root_dir; // Is this definitely the unnamed root (".") dir?
 	u8 detect_root_dot_dir; // Directories named "." are special.
 	u8 orig_name_was_dot; // Internal use
 	u8 has_hotspot;
+	u8 linear_colorpace;
 	u8 has_riscos_data; // attribs, load_addr, exec_addr
 	u8 riscos_appended_type; // Internal use
 
@@ -256,8 +265,7 @@ struct deark_bitmap_struct {
 	int bytes_per_pixel;
 	u8 *bitmap;
 	i64 bitmap_size; // bytes allocated for bitmap
-	int orig_colortype; // Optional; can be used by modules
-	int orig_bitdepth; // Optional; can be used by modules
+	u8 is_internal;
 };
 typedef struct deark_bitmap_struct de_bitmap;
 
@@ -293,9 +301,11 @@ struct de_module_in_params {
 	u32 flags;
 	de_encoding input_encoding;
 	u32 uint1;
+	u32 uint2;
 	i64 offset_in_parent;
 	dbuf *parent_dbuf;
 	de_finfo *fi;
+	de_ucstring *str1;
 	void *obj1;
 };
 
@@ -332,6 +342,21 @@ enum de_moddisp_enum {
 	DE_MODDISP_INTERNAL     // Another module is using this module
 };
 
+struct de_mp_item {
+	const char *fn;
+	dbuf *f;
+};
+
+struct de_mp_data {
+	int count;
+	int alloc;
+	struct de_mp_item *item; // array[alloc]
+};
+
+#define DE_FONTFMT_AUTO 1
+#define DE_FONTFMT_FONT 2
+#define DE_FONTFMT_IMAGE 3
+
 struct deark_struct {
 	int debug_level;
 	void *userdata;
@@ -357,6 +382,8 @@ struct deark_struct {
 
 	// Always valid during identify(); can be NULL during run().
 	struct de_detection_data_struct *detection_data;
+
+	struct de_mp_data *mp_data;
 	////////////////////////////////////////////////////
 
 	int file_count; // The number of extractable files encountered so far.
@@ -374,8 +401,10 @@ struct deark_struct {
 	const char *modcodes_req;
 	i64 slice_start_req; // Used if we're only to look at part of the file.
 	i64 slice_size_req;
-	int slice_size_req_valid;
-	int suppress_detection_by_filename;
+	u8 slice_start_req_special;
+	u8 slice_size_req_valid;
+	u8 suppress_detection_by_filename;
+	u8 mp_opt_used;
 
 	int output_style; // DE_OUTPUTSTYLE_*
 	int archive_fmt; // If output_style==DE_OUTPUTSTYLE_ARCHIVE
@@ -404,7 +433,9 @@ struct deark_struct {
 	u8 filenames_from_file;
 	u8 macformat_known;
 	u8 macformat;
+	u8 macmetaflag;
 	u8 append_riscos_type;
+	u8 font_fmt_req; // DE_FONTFMT_*
 	u8 padpix;
 	int overwrite_mode;
 	u8 preserve_file_times;
@@ -455,6 +486,9 @@ struct deark_struct {
 #define DE_MAX_EXT_OPTIONS 16
 	int num_ext_options;
 	struct deark_ext_option ext_option[DE_MAX_EXT_OPTIONS];
+
+#define DE_NUM_PERSISTENT_MEM_ITEMS 6
+	void *persistent_item[DE_NUM_PERSISTENT_MEM_ITEMS];
 };
 
 void de_fatalerror(deark *c);
@@ -535,11 +569,16 @@ de_encoding de_get_input_encoding(deark *c, de_module_params *mparams,
 void de_dbg_indent(deark *c, int n);
 void de_dbg_indent_save(deark *c, int *saved_indent_level);
 void de_dbg_indent_restore(deark *c, int saved_indent_level);
+char *de_render_hexbytes_from_mem(const u8 *inbytes, i64 ilen,
+	char *outbuf, size_t outbuf_len);
+char *de_render_hexbytes_from_dbuf(dbuf *inf, i64 pos, i64 ilen,
+	char *outbuf, size_t outbuf_len);
 void de_dbg_hexdump(deark *c, dbuf *f, i64 pos1, i64 nbytes_avail,
 	i64 max_nbytes_to_dump, const char *prefix, unsigned int flags);
 void de_hexdump2(deark *c, dbuf *f, i64 pos1, i64 nbytes_avail,
 	i64 max_nbytes_to_dump, unsigned int flags);
 void de_dbg_dimensions(deark *c, i64 w, i64 h);
+void de_stdwarn_execomp(deark *c);
 void de_dbg_pal_entry(deark *c, i64 idx, de_color clr);
 void de_dbg_pal_entry2(deark *c, i64 idx, de_color clr,
 	const char *txt_before, const char *txt_in, const char *txt_after);
@@ -573,10 +612,20 @@ int de_zip_create_file(deark *c);
 void de_zip_add_file_to_archive(deark *c, dbuf *f);
 void de_zip_close_file(deark *c);
 
-int de_write_png(deark *c, de_bitmap *img, dbuf *f, UI createflags);
+struct de_write_image_params { // internal use
+	de_bitmap *img;
+	de_bitmap *imglo;
+	dbuf *f;
+	UI createflags;
+	UI flags2;
+};
+int de_write_png(deark *c, struct de_write_image_params *wp);
 
 ///////////////////////////////////////////
 
+int dbuf_constrain_length(dbuf *f, i64 pos, i64 *plen);
+int dbuf_constrain_offset(dbuf *f, i64 *ppos);
+
 i64 de_geti8_direct(const u8 *m);
 i64 de_getu16be_direct(const u8 *m);
 i64 de_getu16le_direct(const u8 *m);
@@ -678,6 +727,7 @@ void dbuf_read_to_ucstring_n(dbuf *f, i64 pos, i64 len, i64 max_len,
 #define DE_CREATEFLAG_IS_AUX   0x1
 #define DE_CREATEFLAG_OPT_IMAGE 0x2
 #define DE_CREATEFLAG_FLIP_IMAGE 0x4
+#define DE_CREATEFLAG_IS_BWIMG   0x8
 #define DE_CREATEFLAG_NO_WBUFFER 0x200
 dbuf *dbuf_create_output_file(deark *c, const char *ext, de_finfo *fi, unsigned int createflags);
 
@@ -781,6 +831,12 @@ int dbuf_get_utf16_NULterm_len(dbuf *f, i64 pos1, i64 bytes_avail,
 int dbuf_find_line(dbuf *f, i64 pos1, i64 *pcontent_len, i64 *ptotal_len);
 int de_memmatch(const u8 *mem, const u8 *pattern, size_t pattern_len,
 	u8 wildcard, UI flags);
+int de_memsearch_match(const u8 *mem, i64 mem_len,
+	const u8 *pattern, i64 pattern_len, u8 wildcard, i64 *pfoundpos);
+
+int de_decompress_zlib_mem2mem(deark *c, const u8 *src, i64 src_len,
+	u8 *dst, i64 dst_len);
+int de_cp932_lookup(deark *c, u16 n, UI flags, de_rune *pr1, de_rune *pr2);
 
 struct de_fourcc {
   u8 bytes[4];
@@ -816,6 +872,7 @@ struct de_bitbuf_lowlevel {
 	UI nbits_in_bitbuf;
 	u64 bit_buf;
 };
+void de_bitbuf_lowlevel_add_bits(struct de_bitbuf_lowlevel *bbll, u64 n, UI nbits);
 void de_bitbuf_lowlevel_add_byte(struct de_bitbuf_lowlevel *bbll, u8 n);
 u64 de_bitbuf_lowlevel_get_bits(struct de_bitbuf_lowlevel *bbll, UI nbits);
 void de_bitbuf_lowlevel_empty(struct de_bitbuf_lowlevel *bbll);
@@ -837,6 +894,8 @@ char *de_bitreader_describe_curpos(struct de_bitreader *bitrd, char *buf, size_t
 
 void de_bitmap_write_to_file(de_bitmap *img, const char *token, unsigned int createflags);
 void de_bitmap_write_to_file_finfo(de_bitmap *img, de_finfo *fi, unsigned int createflags);
+void de_bitmap16_write_to_file_finfo(de_bitmap *img, de_bitmap *imglo,
+	de_finfo *fi, unsigned int createflags);
 
 void de_bitmap_setsample(de_bitmap *img, i64 x, i64 y,
 	i64 samplenum, de_colorsample v);
@@ -914,12 +973,20 @@ void de_read_simple_palette(deark *c, dbuf *f, i64 fpos,
 void de_convert_image_paletted(dbuf *f, i64 fpos,
 	i64 bpp, i64 rowspan, const de_color *pal,
 	de_bitmap *img, unsigned int flags);
+void de_convert_image_paletted_planar(dbuf *f, i64 fpos, i64 nplanes,
+	i64 row_stride, i64 plane_stride, const de_color *pal,
+	de_bitmap *img, UI flags);
 
 void de_convert_image_rgb(dbuf *f, i64 fpos,
 	i64 rowspan, i64 pixelspan, de_bitmap *img, unsigned int flags);
 
 i64 de_min_int(i64 n1, i64 n2);
 i64 de_max_int(i64 n1, i64 n2);
+int de_int_in_range(i64 n, i64 lv, i64 hv);
+int de_constrain_int(i64 *pn, i64 lv, i64 hv);
+int de_sanitize_count(i64 *pn);
+#define de_sanitize_length de_sanitize_count
+#define de_sanitize_offset de_sanitize_count
 i64 de_pad_to_2(i64 x);
 i64 de_pad_to_4(i64 x);
 i64 de_pad_to_n(i64 x, i64 n);
@@ -932,12 +999,17 @@ i64 de_log2_rounded_up(i64 n);
 
 char *de_print_base2_fixed(char *buf, size_t buf_len, u64 n, UI bitcount);
 
+dbuf *de_mp_acquire_dbuf(deark *c, int xidx);
+void de_mp_release_dbuf(deark *c, int xidx, dbuf **p_oldf);
+
 // Test if the image dimensions are valid and supported.
 int de_good_image_dimensions_noerr(deark *c, i64 w, i64 h);
 
 // Test if the image dimensions are valid and supported. Report an error if not.
 int de_good_image_dimensions(deark *c, i64 w, i64 h);
 
+int de_bitmap_good_dimensions(de_bitmap *img, UI flags);
+
 // Test if the number of images is sane. Report an error if not.
 int de_good_image_count(deark *c, i64 n);
 
@@ -962,6 +1034,8 @@ void de_bitmap_remove_alpha(de_bitmap *img);
 void de_bitmap_optimize_alpha(de_bitmap *img, unsigned int flags);
 
 void de_make_grayscale_palette(de_color *pal, i64 num_entries, unsigned int flags);
+de_colorsample de_unpremultiply_alpha_samp(de_colorsample cval, de_colorsample a);
+de_color de_unpremultiply_alpha_clr(de_color clr);
 
 ///////////////////////////////////////////
 
@@ -972,11 +1046,13 @@ u8 de_decode_hex_digit(u8 x, int *errorflag);
 #define DE_PALID_PC16   20
 #define DE_PALID_EGA64  30
 #define DE_PALID_VGA256 40
+#define DE_PALID_WIN16  50
 
 de_color de_get_std_palette_entry(int pal_id, int pal_subid, int index);
 #define DE_COPYPALFLAG_INITPAL 0x10
-void de_copy_std_palette(int pal_id, int pal_subid, i64 starting_idx, i64 count,
+void de_copy_std_palette(int pal_id, int pal_subid, i64 starting_idx,
 	de_color *dstpal, size_t dstpal_capacity, UI flags);
+void de_copy_palette_from_rgb24(const u8 *src, de_color *dst, size_t ncolors);
 
 const u8 *de_get_8x8ascii_font_ptr(void);
 const u8 *de_get_vga_cp437_font_ptr(void);
@@ -987,6 +1063,7 @@ u8 de_sample_nbit_to_8bit(i64 n, unsigned int x);
 u8 de_scale_63_to_255(u8 x);
 u8 de_scale_1000_to_255(i64 x);
 u8 de_scale_n_to_255(i64 n, i64 x);
+void de_scale_n_to_16bit(int n, int x_in, u8 *x_out_hi, u8 *x_out_lo);
 de_color de_rgb565_to_888(u32 x);
 de_color de_bgr555_to_888(u32 x);
 de_color de_rgb555_to_888(u32 x);
@@ -1032,6 +1109,7 @@ void ucstring_truncate(de_ucstring *s, i64 newlen);
 void ucstring_truncate_at_NUL(de_ucstring *s);
 void ucstring_strip_trailing_NUL(de_ucstring *s);
 void ucstring_strip_trailing_spaces(de_ucstring *s);
+void ucstring_delete_prefix(de_ucstring *s, i64 nchars);
 void ucstring_append_char(de_ucstring *s, de_rune ch);
 void ucstring_append_ucstring(de_ucstring *s1, const de_ucstring *s2);
 void ucstring_vprintf(de_ucstring *s, de_ext_encoding ee, const char *fmt, va_list ap);
@@ -1105,18 +1183,30 @@ int de_inthashtable_remove_item(deark *c, struct de_inthashtable *ht, i64 key, v
 int de_inthashtable_remove_any_item(deark *c, struct de_inthashtable *ht, i64 *pkey, void **pvalue);
 
 #define DE_CRCOBJ_CRC32_IEEE   0x10
-#define DE_CRCOBJ_ADLER32      0x11
+#define DE_CRCOBJ_CRC32_JAMCRC 0x11
+#define DE_CRCOBJ_CRC32_PL     0x12
+#define DE_CRCOBJ_ADLER32      0x15
 #define DE_CRCOBJ_CRC16_XMODEM 0x20
 #define DE_CRCOBJ_CRC16_ARC    0x21
 #define DE_CRCOBJ_CRC16_IBMSDLC 0x22
+#define DE_CRCOBJ_CRC16_IBM3740 0x23
+#define DE_CRCOBJ_SUM_BYTES    0x30
+#define DE_CRCOBJ_SUM_U16LE    0x31
+#define DE_CRCOBJ_SUM_U16BE    0x32
+#define DE_CRCOBJ_SUM_U32LE    0x33
+#define DE_CRCOBJ_SUM_U32BE    0x34
 
 struct de_crcobj *de_crcobj_create(deark *c, UI type_and_flags);
 void de_crcobj_destroy(struct de_crcobj *crco);
+void de_crcobj_setval(struct de_crcobj *crco, u32 v);
 void de_crcobj_reset(struct de_crcobj *crco);
 u32 de_crcobj_getval(struct de_crcobj *crco);
+u64 de_crcobj_getval64(struct de_crcobj *crco);
 void de_crcobj_addbuf(struct de_crcobj *crco, const u8 *buf, i64 buf_len);
+void de_crcobj_addrun(struct de_crcobj *crco, u8 v, i64 len);
 void de_crcobj_addzeroes(struct de_crcobj *crco, i64 len);
 void de_crcobj_addslice(struct de_crcobj *crco, dbuf *f, i64 pos, i64 len);
+u32 de_calccrc_oneshot(dbuf *f, i64 pos, i64 len, UI type_and_flags);
 
 ///////////////////////////////////////////
 
@@ -1149,6 +1239,7 @@ struct de_bitmap_font {
 	// to prefer when displaying the font.
 	u8 prefer_unicode;
 
+	u8 force_fontfile_output;
 	i64 num_chars;
 	struct de_bitmap_font_char *char_array;
 };
@@ -1169,8 +1260,11 @@ void de_font_paint_character_cp(deark *c, de_bitmap *img,
 	struct de_bitmap_font *font, i32 codepoint,
 	i64 xpos, i64 ypos, de_color fgcol, de_color bgcol, unsigned int flags);
 
-void de_font_bitmap_font_to_image(deark *c, struct de_bitmap_font *font, de_finfo *fi, unsigned int createflags);
+void de_font_bitmap_font_write(deark *c, struct de_bitmap_font *font, de_finfo *fi,
+	UI createflags);
+#define de_font_bitmap_font_to_image de_font_bitmap_font_write
 int de_font_is_standard_vga_font(deark *c, u32 crc);
+void de_font_decide_output_fmt(deark *c);
 
 ///////////////////////////////////////////
 
@@ -1200,6 +1294,7 @@ struct de_char_screen {
 struct de_char_context {
 	u8 prefer_image_output;
 	u8 prefer_9col_mode;
+	u8 screen_image_flag; // Hint: non-scrolling format
 	u8 no_density;
 	u8 suppress_custom_font_warning;
 	u8 outfmt_known;
@@ -1239,6 +1334,7 @@ void de_FILETIME_to_timestamp(i64 ft, struct de_timestamp *ts, unsigned int flag
 void de_dos_datetime_to_timestamp(struct de_timestamp *ts,
    i64 ddate, i64 dtime);
 void de_describe_dos_attribs(deark *c, UI attr, de_ucstring *s, UI flags);
+void de_prodos_datetime_to_timestamp(struct de_timestamp *ts, i64 ddate, i64 dtime);
 void de_riscos_loadexec_to_timestamp(u32 load_addr,
 	u32 exec_addr, struct de_timestamp *ts);
 void de_timestamp_set_subsec(struct de_timestamp *ts, double frac);
