@@ -69,6 +69,24 @@ void ucstring_strip_trailing_spaces(de_ucstring *s)
 	}
 }
 
+void ucstring_delete_prefix(de_ucstring *s, i64 nchars)
+{
+	i64 i;
+	i64 new_len;
+
+	if(nchars<1) return;
+	if(nchars >= s->len) {
+		ucstring_empty(s);
+		return;
+	}
+
+	new_len = s->len - nchars;
+	for(i=0; i<new_len; i++) {
+		s->str[i] = s->str[nchars+i];
+	}
+	ucstring_truncate(s, new_len);
+}
+
 // Append s2 to s1
 void ucstring_append_ucstring(de_ucstring *s1, const de_ucstring *s2)
 {
@@ -341,13 +359,71 @@ static void append_bytes_utf16(de_ucstring *s, const u8 *cbuf, i64 buflen,
 
 #undef  UTF16_NBYTES_SAVED
 
+// ---- CP932, Shift-JIS, ...
+
+#define CP932_NBYTES_SAVED    (es->buf[6])
+
+static void append_bytes_cp932(deark *c, de_ucstring *s,
+	const u8 *cbuf, i64 buflen,
+	UI conv_flags, struct de_encconv_state *es)
+{
+	i64 pos = 0;
+
+	for(pos=0; pos<buflen; pos++) {
+		u8 n = cbuf[pos];
+		de_rune ch1, ch2;
+
+		if(CP932_NBYTES_SAVED>0) {
+			u16 tmpch;
+			int nrunes;
+
+			tmpch = (u16)es->buf[0]<<8;
+			tmpch |= n;
+			nrunes = de_cp932_lookup(c, tmpch, 0, &ch1, &ch2);
+			if(nrunes==0 || ch1==0) {
+				ch1 = 0xfffd;
+			}
+			ucstring_append_char(s, ch1);
+			if(nrunes==2) {
+				ucstring_append_char(s, ch2);
+			}
+			CP932_NBYTES_SAVED = 0;
+		}
+		else if((n>=0x81 && n<=0x9f) ||
+			(n>=0xe0 && n<=0xfc))
+		{
+			es->buf[0] = n;
+			CP932_NBYTES_SAVED = 1;
+		}
+		else {
+			if(n<=0x7f || (n>=0xa1 && n<=0xdf)) {
+				(void)de_cp932_lookup(c, (u16)n, 0, &ch1, &ch2);
+			}
+			else {
+				ch1 = 0xfffd;
+			}
+			ucstring_append_char(s, ch1);
+		}
+	}
+
+	// Deal with unprocessed bytes at the end of the string.
+	if(CP932_NBYTES_SAVED!=0 && !(conv_flags & DE_CONVFLAG_PARTIAL_DATA)) {
+		handle_invalid_bytes(s, es->buf, (UI)CP932_NBYTES_SAVED);
+		CP932_NBYTES_SAVED = 0;
+	}
+}
+
+#undef CP932_NBYTES_SAVED
+
+// ----
+
 // conv_flags:
 //  DE_CONVFLAG_PARTIAL_DATA: There might be more data after this; if 'buf' ends
 //   in a way it shouldn't, it's not necessarily an error.
 void ucstring_append_bytes_ex(de_ucstring *s, const u8 *buf, i64 buflen,
 	unsigned int conv_flags, struct de_encconv_state *es)
 {
-	de_encoding encoding = DE_EXTENC_GET_BASE(es->ee);
+	de_encoding encoding = es->enc;
 
 	// Adjust buflen if necessary.
 	if(conv_flags & DE_CONVFLAG_STOP_AT_NUL) {
@@ -364,6 +440,9 @@ void ucstring_append_bytes_ex(de_ucstring *s, const u8 *buf, i64 buflen,
 	else if(encoding==DE_ENCODING_UTF16LE || encoding==DE_ENCODING_UTF16BE) {
 		append_bytes_utf16(s, buf, buflen, conv_flags, es, (encoding==DE_ENCODING_UTF16LE));
 	}
+	else if(encoding==DE_ENCODING_CP932) {
+		append_bytes_cp932(s->c, s, buf, buflen, conv_flags, es);
+	}
 	else {
 		i64 pos;
 
