@@ -17,6 +17,7 @@
 #define DE_DEFAULT_MAX_IMAGE_DIMENSION 10000
 #define DE_DEFAULT_MAX_OUTPUT_FILES 1000 // Limit for direct output (not ZIP)
 #define DE_MAX_OUTPUT_FILES_HARD_LIMIT 250000
+#define DE_MAX_MP_FILES 2047
 
 // Returns the best module to use, by looking at the file contents, etc.
 static struct deark_module_info *detect_module_for_file(deark *c, int *errflag)
@@ -114,8 +115,11 @@ void de_print_module_list(deark *c)
 		i = sort_data[k].module_index;
 		if(!c->module_info[i].id) continue;
 		if(c->extract_level<2) {
-			if(c->module_info[i].flags & DE_MODFLAG_HIDDEN) continue;
-			if(c->module_info[i].flags & DE_MODFLAG_NONWORKING) continue;
+			if(c->module_info[i].flags & (DE_MODFLAG_HIDDEN |
+				DE_MODFLAG_NONWORKING | DE_MODFLAG_WARNPARSEONLY))
+			{
+				continue;
+			}
 		}
 		desc = c->module_info[i].desc ? c->module_info[i].desc : "-";
 		de_printf(c, DE_MSGTYPE_MESSAGE, "%-14s %s\n", c->module_info[i].id, desc);
@@ -127,6 +131,8 @@ void de_print_module_list(deark *c)
 static void do_modhelp_internal(deark *c, struct deark_module_info *module_to_use)
 {
 	int k;
+	u8 printed_something = 0;
+	u8 suppress_nohelpmsg = 0;
 
 	if(!module_to_use) goto done;
 	de_msg(c, "Module: %s", module_to_use->id);
@@ -145,10 +151,25 @@ static void do_modhelp_internal(deark *c, struct deark_module_info *module_to_us
 	}
 	if(module_to_use->desc2) {
 		de_msg(c, "Other notes: %s", module_to_use->desc2);
+		printed_something = 1;
+	}
+
+	if(module_to_use->flags&DE_MODFLAG_MULTIPART) {
+		de_msg(c, "This module supports multiple input files; "
+			"use the \"-mp\" option.");
+		printed_something = 1;
+	}
+
+	if(module_to_use->flags&DE_MODFLAG_INTERNALONLY) {
+		de_msg(c, "This module is intended for internal use only.");
+		suppress_nohelpmsg = 1;
 	}
 
 	if(!module_to_use->help_fn) {
-		de_msg(c, "No help available for module \"%s\"", module_to_use->id);
+		if(!suppress_nohelpmsg) {
+			de_msg(c, "No%s help available for module \"%s\".",
+				(printed_something?" other":""), module_to_use->id);
+		}
 		goto done;
 	}
 
@@ -206,6 +227,46 @@ static void open_extrlist(deark *c)
 		DE_OVERWRITEMODE_STANDARD, flags);
 }
 
+// Modifies c->slice_start_req
+static int do_special_overlay_startpos(deark *c)
+{
+	i64 lfb, nblocks;
+	i64 overlay_pos, overlay_len;
+	int retval = 0;
+	u8 sig[2];
+
+	// Maybe we should use fmtutil_collect_exe_info() here, but it's not
+	// exactly what we want, and might add a dependency on fmtutil.
+	dbuf_read(c->infile, sig, 0, 2);
+	if((sig[0]=='M' && sig[1]=='Z') || (sig[0]=='Z' && sig[1]=='M')) {
+		;
+	}
+	else {
+		goto done;
+	}
+	lfb = dbuf_getu16le(c->infile, 2);
+	nblocks = dbuf_getu16le(c->infile, 4);
+	nblocks &= 0x7ff;
+	if(lfb==0) {
+		overlay_pos = 512*nblocks;
+	}
+	else if(lfb<512 && nblocks>0) {
+		overlay_pos = 512*(nblocks-1) + lfb;
+	}
+	else {
+		goto done;
+	}
+	overlay_len = c->infile->len - overlay_pos;
+	if(overlay_len<0) goto done;
+	de_dbg2(c, "overlay at %"I64_FMT", len=%"I64_FMT, overlay_pos, overlay_len);
+	c->slice_start_req = overlay_pos;
+	c->suppress_detection_by_filename = 1;
+	retval = 1;
+
+done:
+	return retval;
+}
+
 // Returns 0 on "serious" error; e.g. input file not found.
 int de_run(deark *c)
 {
@@ -221,6 +282,8 @@ int de_run(deark *c)
 	de_module_params *mparams = NULL;
 	de_ucstring *friendly_infn = NULL;
 
+	if(c->serious_error_flag) goto done;
+
 	if(c->modhelp_req && c->input_format_req) {
 		do_modhelp(c);
 		goto done;
@@ -236,14 +299,14 @@ int de_run(deark *c)
 	if(c->input_style==DE_INPUTSTYLE_STDIN) {
 		ucstring_append_sz(friendly_infn, "[stdin]", DE_ENCODING_LATIN1);
 	}
-	else {
-		if(!c->input_filename) {
-			de_internal_err_nonfatal(c, "Input file not set");
-			c->serious_error_flag = 1;
-			goto done;
-		}
+	else if(c->input_filename) {
 		ucstring_append_sz(friendly_infn, c->input_filename, DE_ENCODING_UTF8);
 	}
+	else {
+		de_internal_err_nonfatal(c, "Input file not set");
+		c->serious_error_flag = 1;
+		goto done;
+	}
 
 	de_register_modules(c);
 
@@ -256,17 +319,21 @@ int de_run(deark *c)
 		}
 	}
 
-	if(c->slice_size_req_valid) {
-		de_dbg(c, "Input file: %s[%d,%d]", ucstring_getpsz_d(friendly_infn),
-			(int)c->slice_start_req, (int)c->slice_size_req);
+	if(c->slice_start_req_special!=0) {
+		ucstring_append_sz(friendly_infn, "[special]", DE_ENCODING_LATIN1);
+	}
+	else if(c->slice_size_req_valid) {
+		ucstring_printf(friendly_infn, DE_ENCODING_LATIN1, "[%"I64_FMT",%"I64_FMT"]",
+			c->slice_start_req, c->slice_size_req);
 	}
 	else if(c->slice_start_req) {
-		de_dbg(c, "Input file: %s[%d]", ucstring_getpsz_d(friendly_infn),
-			(int)c->slice_start_req);
+		ucstring_printf(friendly_infn, DE_ENCODING_LATIN1, "[%"I64_FMT"]", c->slice_start_req);
 	}
-	else {
-		de_dbg(c, "Input file: %s", ucstring_getpsz_d(friendly_infn));
+	if(c->mp_data && c->mp_data->count>0) {
+		ucstring_printf(friendly_infn, DE_ENCODING_LATIN1, " (+%d more file%s)",
+			c->mp_data->count, (c->mp_data->count==1 ? "" : "s"));
 	}
+	de_dbg(c, "Input file: %s", ucstring_getpsz_d(friendly_infn));
 
 	if(c->input_style==DE_INPUTSTYLE_STDIN) {
 		orig_ifile = dbuf_open_input_stdin(c);
@@ -287,9 +354,23 @@ int de_run(deark *c)
 
 	c->infile = orig_ifile;
 
+	if(c->slice_start_req_special==1) {
+		if(!do_special_overlay_startpos(c)) {
+			de_err(c, "File not compatible with \"-start overlay\"");
+			goto done;
+		}
+	}
+
 	// If we are only supposed to look at a segment of the original file,
 	// do that by creating a child dbuf, using dbuf_open_input_subfile().
-	if(c->slice_start_req>0 || c->slice_size_req_valid) {
+	if(c->slice_start_req!=0 || c->slice_size_req_valid) {
+		if(c->slice_start_req<0 || c->slice_size_req<0 ||
+			(c->slice_start_req+c->slice_size_req > c->infile->len))
+		{
+			de_err(c, "File not compatible with given -start/-size options");
+			goto done;
+		}
+
 		if(c->slice_size_req_valid)
 			subfile_size = c->slice_size_req;
 		else
@@ -323,6 +404,15 @@ int de_run(deark *c)
 
 	de_info(c, "Module: %s", module_to_use->id);
 
+	if(c->mp_data && c->mp_data->count &&
+		!(module_to_use->flags&DE_MODFLAG_MULTIPART))
+	{
+		de_err(c, "The %s module does not support multiple input files",
+			module_to_use->id);
+		c->serious_error_flag = 1;
+		goto done;
+	}
+
 	if(module_was_autodetected && (module_to_use->flags&DE_MODFLAG_SECURITYWARNING)) {
 		de_err(c, "The %s module has not been audited for security. There is a "
 			"greater than average chance that it is unsafe to use with untrusted "
@@ -332,6 +422,10 @@ int de_run(deark *c)
 		goto done;
 	}
 
+	if(module_was_autodetected && (module_to_use->flags&DE_MODFLAG_WARNPARSEONLY)) {
+		de_warn(c, "The %s module can parse files, but does not generally support "
+			"extracting data.", module_to_use->id);
+	}
 	if(module_to_use->flags&DE_MODFLAG_NONWORKING) {
 		de_warn(c, "The %s module is considered to be incomplete, and may "
 			"not work properly. Caveat emptor.",
@@ -466,6 +560,7 @@ done:
 	ucstring_destroy(friendly_infn);
 	if(subfile) dbuf_close(subfile);
 	if(orig_ifile) dbuf_close(orig_ifile);
+	c->infile = NULL;
 	de_free(c, mparams);
 	return c->serious_error_flag ? 0 : 1;
 }
@@ -496,7 +591,7 @@ deark *de_create_internal(void)
 
 void de_destroy(deark *c)
 {
-	i64 i;
+	int i;
 
 	if(!c) return;
 	if(c->zip_data) { de_zip_close_file(c); }
@@ -511,6 +606,19 @@ void de_destroy(deark *c)
 	if(c->output_archive_filename) { de_free(c, c->output_archive_filename); }
 	if(c->extrlist_filename) { de_free(c, c->extrlist_filename); }
 	if(c->detection_data) { de_free(c, c->detection_data); }
+	if(c->mp_data) {
+		for(i=0; i<c->mp_data->count; i++) {
+			dbuf_close(c->mp_data->item[i].f);
+			// (The .fn field is owned by the caller (deark-cmd.c)).
+		}
+		de_free(c, c->mp_data->item);
+		de_free(c, c->mp_data);
+	}
+	for(i=0; i<DE_NUM_PERSISTENT_MEM_ITEMS; i++) {
+		if(c->persistent_item[i]) {
+			de_free(c, c->persistent_item[i]);
+		}
+	}
 	de_free(c, c->module_info);
 	de_free(NULL,c);
 }
@@ -540,7 +648,7 @@ void de_set_fatalerror_callback(deark *c, de_fatalerrorfn_type fn)
 	c->fatalerrorfn = fn;
 }
 
-static int is_pathsep(i32 ch)
+static int is_pathsep(de_rune ch)
 {
 	if(ch=='/') return 1;
 #ifdef DE_WINDOWS
@@ -568,7 +676,7 @@ static const char *get_basename_ptr(const char *fn)
 	return basenameptr;
 }
 
-static i32 ucstring_char_at(de_ucstring *s, i64 pos)
+static de_rune ucstring_char_at(de_ucstring *s, i64 pos)
 {
 	if(!s) return 0;
 	if(pos>=0 && pos<s->len) return s->str[pos];
@@ -588,6 +696,45 @@ static void backslashes_to_slashes(de_ucstring *s)
 }
 #endif
 
+#ifdef DE_WINDOWS
+static int is_alpha_char(de_rune x)
+{
+	return (x>='A' && x<='Z') || (x>='a' && x<='z');
+}
+#endif
+
+static void append_pathsep_if_needed(de_ucstring *s)
+{
+	de_rune lastchar;
+
+	if(s->len<1) return;
+
+	lastchar = ucstring_char_at(s, s->len-1);
+
+#ifdef DE_WINDOWS
+	if(s->len==2 && lastchar==':' &&
+		is_alpha_char(ucstring_char_at(s, 0)))
+	{
+		// This is arguable, but we won't append a backslash to Windows
+		// paths like "D:", so output files will go to that drive's
+		// own "current directory".
+		return;
+	}
+#endif
+
+#if DE_BUILDFLAG_AMIGA
+	if(lastchar==':') {
+		// Assuming this is a device name, so putting a "/" after it is not
+		// allowed.
+		return;
+	}
+#endif
+
+	if(!is_pathsep(lastchar)) {
+		ucstring_append_char(s, DE_PATHSEP);
+	}
+}
+
 // Construct a basename for output files, or a filename for archives.
 // flags:
 //  0x1 = use base filename only
@@ -608,9 +755,7 @@ static char *make_output_filename(deark *c, const char *dirname, const char *fn,
 
 	if(dirname && dirname[0]) {
 		ucstring_append_sz(tmps, dirname, DE_ENCODING_UTF8);
-		if(!is_pathsep(ucstring_char_at(tmps, tmps->len-1))) {
-			ucstring_append_char(tmps, DE_PATHSEP);
-		}
+		append_pathsep_if_needed(tmps);
 	}
 
 	fnpartpos = tmps->len;
@@ -779,6 +924,12 @@ void de_set_std_option_int(deark *c, enum de_stdoptions_enum o, int x)
 	case DE_STDOPT_INFOMESSAGES:
 		c->show_infomessages = x;
 		break;
+	case DE_STDOPT_MP_OPT_USED:
+		// We track this mainly so that modules can use it to construct better
+		// error messages, and not tell the user to use "-mp" when they
+		// already did.
+		c->mp_opt_used = 1;
+		break;
 	case DE_STDOPT_WRITE_BOM:
 		c->write_bom = (u8)x;
 		break;
@@ -807,9 +958,61 @@ void de_set_input_style(deark *c, int x)
 	c->input_style = x;
 }
 
-void de_set_input_filename(deark *c, const char *fn)
+// Notes on the multipart input file feature:
+// The module tests c->mp_data != NULL to see if there are additional
+// input files after the first.
+// The extra files are not opened automatically. The module must do that.
+// The module should use dbuf_open_input_file() to open files as needed.
+// dbuf_open_input_file() may fail, report an error, and return NULL, in
+// which case the module should probably stop immediately.
+// The module can use mp_data->item[].f, but is not required to. If the
+// module leaves it set, it will be closed automatically.
+// (Use de_mp_acquire/release_dbuf() to make this easier.)
+// The first file is still c->infile, the same as for any other module.
+// (It's unfortunate that the first file is such a special case, but that's
+// the way it is for now.)
+// Commmand-line options -start and -size apply only to the first file.
+static void de_add_input_filename_mp(deark *c, const char *fn)
 {
-	c->input_filename = fn;
+	int prev_count, new_count;
+	int new_idx;
+
+	if(!c->mp_data) {
+		c->mp_data = de_malloc(c, sizeof(struct de_mp_data));
+	}
+	prev_count = c->mp_data->count;
+	if(prev_count >= DE_MAX_MP_FILES) {
+		de_err(c, "Too many input files");
+		c->serious_error_flag = 1;
+		return;
+	}
+
+	new_count = prev_count+1;
+	if(new_count > c->mp_data->alloc) {
+		int new_alloc;
+
+		new_alloc = new_count+15;
+		c->mp_data->item = de_reallocarray(c, c->mp_data->item,
+			(i64)c->mp_data->count, sizeof(struct de_mp_item), (i64)new_alloc);
+
+		c->mp_data->alloc = new_alloc;
+	}
+
+	new_idx = prev_count;
+	c->mp_data->item[new_idx].fn = fn;
+	c->mp_data->count = new_count;
+}
+
+// flags&0x1: Allow multiple input files. If this is the first time
+//   this function has been called, this flag doesn't matter.
+void de_set_input_filename(deark *c, const char *fn, UI flags)
+{
+	if(c->input_filename && (flags&0x1)) {
+		de_add_input_filename_mp(c, fn);
+	}
+	else {
+		c->input_filename = fn;
+	}
 }
 
 int de_set_input_encoding(deark *c, const char *encname, int reserved)
@@ -836,6 +1039,11 @@ void de_set_input_file_slice_start(deark *c, i64 n)
 	c->slice_start_req = n;
 }
 
+void de_set_input_file_special_slice_start(deark *c, u8 x)
+{
+	c->slice_start_req_special = x;
+}
+
 void de_set_input_file_slice_size(deark *c, i64 n)
 {
 	c->slice_size_req = n;
