@@ -226,15 +226,17 @@ enum packbits_state_enum {
 	PACKBITS_STATE_READING_UNIT_TO_REPEAT
 };
 
+#define DE_PACKBITS_MAX_NBYTES_PER_UNIT 8
 struct packbitsctx {
 	size_t nbytes_per_unit;
 	size_t nbytes_in_unitbuf;
-	u8 unitbuf[2];
+	u8 unitbuf[DE_PACKBITS_MAX_NBYTES_PER_UNIT];
 	i64 total_nbytes_processed;
 	i64 nbytes_written;
 	enum packbits_state_enum state;
 	i64 nliteral_bytes_remaining;
 	i64 repeat_count;
+	const char *modname;
 };
 
 static void my_packbits_codec_addbuf(struct de_dfilter_ctx *dfctx,
@@ -245,6 +247,8 @@ static void my_packbits_codec_addbuf(struct de_dfilter_ctx *dfctx,
 	struct packbitsctx *rctx = (struct packbitsctx*)dfctx->codec_private;
 
 	if(!rctx) return;
+	if(dfctx->dres->errcode) return;
+	if(dfctx->finished_flag) return;
 
 	for(i=0; i<buf_len; i++) {
 		if(dfctx->dcmpro->len_known &&
@@ -348,15 +352,23 @@ void dfilter_packbits_codec(struct de_dfilter_ctx *dfctx, void *codec_private_pa
 	struct de_packbits_params *pbparams = (struct de_packbits_params*)codec_private_params;
 
 	rctx = de_malloc(dfctx->c, sizeof(struct packbitsctx));
-	rctx->nbytes_per_unit = 1;
-	if(pbparams && pbparams->is_packbits16) {
-		rctx->nbytes_per_unit = 2;
-	}
+	rctx->modname = "packbits";
+
 	dfctx->codec_private = (void*)rctx;
 	dfctx->codec_addbuf_fn = my_packbits_codec_addbuf;
 	dfctx->codec_finish_fn = my_packbits_codec_finish;
 	dfctx->codec_command_fn = my_packbits_codec_command;
 	dfctx->codec_destroy_fn = my_packbits_codec_destroy;
+
+	rctx->nbytes_per_unit = 1;
+	if(pbparams) {
+		if(pbparams->nbytes_per_unit>DE_PACKBITS_MAX_NBYTES_PER_UNIT) {
+			de_dfilter_set_generic_error(dfctx->c, dfctx->dres, rctx->modname);
+		}
+		else if(pbparams->nbytes_per_unit>1) {
+			rctx->nbytes_per_unit = pbparams->nbytes_per_unit;
+		}
+	}
 }
 
 void fmtutil_decompress_packbits_ex(deark *c, struct de_dfilter_in_params *dcmpri,
@@ -367,7 +379,7 @@ void fmtutil_decompress_packbits_ex(deark *c, struct de_dfilter_in_params *dcmpr
 		dcmpri, dcmpro, dres);
 }
 
-// Returns 0 on failure (currently impossible).
+// Returns 0 on failure.
 int fmtutil_decompress_packbits(dbuf *f, i64 pos1, i64 len,
 	dbuf *unc_pixels, i64 *cmpr_bytes_consumed)
 {
@@ -744,6 +756,15 @@ void de_dfilter_transfer_error(deark *c, struct de_dfilter_results *src,
 	}
 }
 
+void de_dfilter_transfer_error2(deark *c, struct de_dfilter_results *src,
+	struct de_dfilter_results *dst, const char *dst_modname)
+{
+	if(src->errcode && !dst->errcode) {
+		dst->errcode = src->errcode;
+		de_snprintf(dst->errmsg, sizeof(dst->errmsg), "[%s] %s", dst_modname, src->errmsg);
+	}
+}
+
 // Decompress an arbitrary two-layer compressed format.
 // tlp->codec1* is the first one that will be used during decompression (i.e. the second
 // method used when during *compression*).
@@ -1074,6 +1095,133 @@ done:
 	}
 }
 
+///////////////////////////////////
+// "PackIt/StuffIt"-style Huffman decoder
+
+struct sit_huffctx {
+	deark *c;
+	const char *modname;
+	struct de_dfilter_in_params *dcmpri;
+	struct de_dfilter_out_params *dcmpro;
+	struct de_dfilter_results *dres;
+	struct fmtutil_huffman_decoder *ht;
+	int errflag;
+	struct de_bitreader bitrd;
+};
+
+// A recursive function to read the tree definition.
+static void sit_huff_read_tree(struct sit_huffctx *hctx, u64 curr_code, UI curr_code_nbits)
+{
+	u8 x;
+
+	if(curr_code_nbits>FMTUTIL_HUFFMAN_MAX_CODE_LENGTH) {
+		hctx->errflag = 1;
+	}
+	if(hctx->bitrd.eof_flag || hctx->errflag) return;
+
+	x = (u8)de_bitreader_getbits(&hctx->bitrd, 1);
+	if(hctx->bitrd.eof_flag) return;
+
+	if(x==0) {
+		sit_huff_read_tree(hctx, curr_code<<1, curr_code_nbits+1);
+		if(hctx->bitrd.eof_flag || hctx->errflag) return;
+		sit_huff_read_tree(hctx, (curr_code<<1) | 1, curr_code_nbits+1);
+	}
+	else {
+		int ret;
+		fmtutil_huffman_valtype val;
+
+		val = (fmtutil_huffman_valtype)de_bitreader_getbits(&hctx->bitrd, 8);
+		if(hctx->c->debug_level>=2) {
+			char b2buf[72];
+
+			de_dbg(hctx->c, "code: \"%s\" = %d",
+				de_print_base2_fixed(b2buf, sizeof(b2buf), curr_code, curr_code_nbits),
+				(int)val);
+		}
+		ret = fmtutil_huffman_add_code(hctx->c, hctx->ht->bk, curr_code, curr_code_nbits, val);
+		if(!ret) {
+			hctx->errflag = 1;
+		}
+	}
+}
+
+// While its code is no longer used by Deark, I credit:
+//   Unsit Version 1 (January 15, 1988), for StuffIt 1.31: unsit.c
+//   by Allan G. Weber
+// for helping me understand the StuffIt type 3 (Huffman) compression format.
+void fmtutil_huff_packit_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres, void *rsvd)
+{
+	struct sit_huffctx *hctx = NULL;
+	i64 nbytes_written = 0;
+	char pos_descr[32];
+
+	hctx = de_malloc(c, sizeof(struct sit_huffctx));
+	hctx->c = c;
+	hctx->modname = "huffman";
+	hctx->dcmpri = dcmpri;
+	hctx->dcmpro = dcmpro;
+	hctx->dres = dres;
+	hctx->ht = fmtutil_huffman_create_decoder(c, 256, 512);
+
+	hctx->bitrd.f = dcmpri->f;
+	hctx->bitrd.curpos = dcmpri->pos;
+	hctx->bitrd.endpos = dcmpri->pos + dcmpri->len;
+
+	// Read the tree definition
+	de_dbg2(c, "interpreted huffman codebook:");
+	de_dbg_indent(c, 1);
+	sit_huff_read_tree(hctx, 0, 0);
+	de_dbg_indent(c, -1);
+	if(hctx->errflag) goto done;
+	if(c->debug_level>=4) {
+		fmtutil_huffman_dump(c, hctx->ht);
+	}
+	if(fmtutil_huffman_get_max_bits(hctx->ht->bk)<1) {
+		goto done;
+	}
+
+	// Read the data section
+	de_bitreader_describe_curpos(&hctx->bitrd, pos_descr, sizeof(pos_descr));
+	de_dbg(c, "cmpr data codes at %s", pos_descr);
+	while(1) {
+		int ret;
+		fmtutil_huffman_valtype val = 0;
+
+		if(dcmpro->len_known) {
+			if(nbytes_written >= dcmpro->expected_len) break;
+		}
+
+		if(hctx->bitrd.eof_flag || hctx->errflag) break;
+
+		ret = fmtutil_huffman_read_next_value(hctx->ht->bk, &hctx->bitrd, &val, NULL);
+		if(!ret) {
+			if(hctx->bitrd.eof_flag) break;
+			hctx->errflag = 1;
+			break;
+		}
+
+		dbuf_writebyte(dcmpro->f, (u8)val);
+		nbytes_written++;
+	}
+
+done:
+	de_bitreader_skip_to_byte_boundary(&hctx->bitrd);
+	dres->bytes_consumed = hctx->bitrd.curpos - hctx->dcmpri->pos;
+	dres->bytes_consumed_valid = 1;
+
+	if(hctx->errflag) {
+		de_dfilter_set_generic_error(c, dres, hctx->modname);
+	}
+	if(hctx) {
+		fmtutil_huffman_destroy_decoder(c, hctx->ht);
+		de_free(c, hctx);
+	}
+}
+
+///////////////////////////////////
+
 // Caller supplies a dbuf to write the decompressed table to.
 // On failure, returns 0. Does not report an error.
 int fmtutil_decompress_exepack_reloc_tbl(deark *c, i64 pos1, i64 endpos, dbuf *outf)
@@ -1121,3 +1269,396 @@ done:
 	de_dbg_indent_restore(c, saved_indent_level);
 	return retval;
 }
+
+///////////////////////////////////
+// XPK (Amiga compression system)
+// Only a very small selection of compression methods are expected to be
+// supported by Deark.
+
+#define CODE_MASH 0x4d415348U
+#define CODE_NONE 0x4e4f4e45U
+#define CODE_XPKF 0x58504b46U
+
+#define XPKSTREAMF_LONGHEADERS  0x01
+#define XPKSTREAMF_PASSWORD     0x02
+#define XPKSTREAMF_EXTHEADER    0x04
+
+#define XPKCHUNK_RAW            0x00
+#define XPKCHUNK_PACKED         0x01
+#define XPKCHUNK_END            0x0f
+
+struct xpkc_chunk {
+	u8 chk_type; // XPKCHUNK_*
+	u8 hchk; // reported chunk hdr checksum
+	UI cchk; // reported chunk data checksum
+	i64 clen; // compressed len
+	i64 ulen; // uncompressed len
+	i64 dpos;
+};
+
+struct xpkc_ctx {
+	i64 pos;
+	i64 cmpr_len;
+	i64 orig_len;
+	struct de_dfilter_in_params *dcmpri;
+	struct de_dfilter_out_params *dcmpro;
+	struct de_dfilter_results *dres;
+	i64 inf_endpos;
+	u8 errflag;
+	u8 flags;
+	u8 hchk;
+	u8 subvrs;
+	u8 masvrs;
+	const char *modname;
+	struct de_fourcc method4cc;
+
+	de_codectype1_type codec;
+	struct xpkc_chunk curchunk;
+};
+
+static int xpk_xor1_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
+	i64 buf_len)
+{
+	u8 *pn = (u8*)brctx->userdata;
+	i64 i;
+
+	for(i=0; i<buf_len; i++) {
+		*pn ^= buf[i];
+	}
+	return 1;
+}
+
+static int xpk_xor2_cbfn(struct de_bufferedreadctx *brctx, const u8 *buf,
+	i64 buf_len)
+{
+	UI *pn = (UI*)brctx->userdata;
+	UI align;
+	i64 i;
+
+	align = brctx->offset & 0x1;
+
+	for(i=0; i<buf_len; i++) {
+		if(align) {
+			*pn ^= buf[i];
+		}
+		else {
+			*pn ^= ((UI)buf[i])<<8;
+		}
+		align = !align;
+	}
+	return 1;
+}
+
+static u8 xpk_checksumh(dbuf *f, i64 pos, i64 len, u8 hck_reported)
+{
+	u8 n = hck_reported;
+
+	dbuf_buffered_read(f, pos, len, xpk_xor1_cbfn, (void*)&n);
+	return n;
+}
+
+static UI xpk_checksumc(dbuf *f, i64 pos, i64 len)
+{
+	UI n = 0;
+
+	len = de_pad_to_4(len);
+	dbuf_buffered_read(f, pos, len, xpk_xor2_cbfn, (void*)&n);
+	return n;
+}
+
+static void xpk_on_checksum_error(deark *c, struct xpkc_ctx *xc)
+{
+	de_dfilter_set_errorf(c, xc->dres, xc->modname, "Checksum error");
+	xc->errflag = 1;
+}
+
+static void do_xpk_chunk(deark *c, struct xpkc_ctx *xc)
+{
+	struct xpkc_chunk *chk;
+	u8 hck;
+	UI ck;
+	i64 pos1;
+	i64 hdrlen;
+	int saved_indent_level;
+	struct de_dfilter_in_params d2i;
+	struct de_dfilter_out_params d2o;
+	struct de_dfilter_results d2r;
+
+	de_dfilter_init_objects(c, &d2i, &d2o, &d2r);
+	de_dbg_indent_save(c, &saved_indent_level);
+	de_dbg(c, "XPK chunk at %"I64_FMT, xc->pos);
+	de_dbg_indent(c, 1);
+	chk = &xc->curchunk;
+	de_zeromem(chk, sizeof(struct xpkc_chunk));
+
+	pos1 = xc->pos;
+	chk->chk_type = dbuf_getbyte_p(xc->dcmpri->f, &xc->pos);
+	de_dbg(c, "type: 0x%02x", (UI)chk->chk_type);
+	chk->hchk = dbuf_getbyte_p(xc->dcmpri->f, &xc->pos);
+	de_dbg(c, "chunk hdr checksum (reported): 0x%02x", (UI)chk->hchk);
+	chk->cchk = (UI)dbuf_getu16be_p(xc->dcmpri->f, &xc->pos);
+	de_dbg(c, "chunk data checksum (reported): 0x%04x", (UI)chk->cchk);
+
+	if(xc->flags & XPKSTREAMF_LONGHEADERS) {
+		hdrlen = 12;
+		chk->clen = dbuf_getu32be_p(xc->dcmpri->f, &xc->pos);
+		chk->ulen = dbuf_getu32be_p(xc->dcmpri->f, &xc->pos);
+	}
+	else {
+		hdrlen = 8;
+		chk->clen = dbuf_getu16be_p(xc->dcmpri->f, &xc->pos);
+		chk->ulen = dbuf_getu16be_p(xc->dcmpri->f, &xc->pos);
+	}
+	de_dbg(c, "clen: %"I64_FMT, chk->clen);
+	de_dbg(c, "ulen: %"I64_FMT, chk->ulen);
+
+	hck = xpk_checksumh(xc->dcmpri->f, pos1, hdrlen, chk->hchk);
+	de_dbg(c, "chunk hdr checksum (calculated): 0x%02x", hck);
+	if(hck != chk->hchk) {
+		xpk_on_checksum_error(c, xc);
+		goto done;
+	}
+
+	if(chk->chk_type==XPKCHUNK_END) {
+		goto done; // EOF marker
+	}
+	if(chk->chk_type!=XPKCHUNK_RAW && chk->chk_type!=XPKCHUNK_PACKED) {
+		xc->errflag = 1;
+		goto done;
+	}
+
+	chk->dpos = xc->pos;
+	de_dbg(c, "chunk dpos: %"I64_FMT, chk->dpos);
+
+	if(chk->dpos + chk->clen > xc->inf_endpos) {
+		xc->errflag = 1;
+		goto done;
+	}
+
+	ck = xpk_checksumc(xc->dcmpri->f, chk->dpos, chk->clen);
+	de_dbg(c, "chunk data checksum (calculated): 0x%04x", ck);
+	if(ck != chk->cchk) {
+		xpk_on_checksum_error(c, xc);
+		goto done;
+	}
+
+	if(chk->chk_type==XPKCHUNK_RAW || xc->method4cc.id==CODE_NONE) {
+		de_dbg(c, "[uncompressed chunk]");
+		dbuf_copy(xc->dcmpri->f, chk->dpos, chk->clen, xc->dcmpro->f);
+	}
+	else if(xc->codec) {
+		i64 old_olen, new_olen;
+
+		d2i.f = xc->dcmpri->f;
+		d2i.pos = chk->dpos;
+		d2i.len = chk->clen;
+		d2o.f = xc->dcmpro->f;
+		d2o.expected_len = chk->ulen;
+		d2o.len_known = 1;
+
+		old_olen = xc->dcmpro->f->len;
+
+		de_dbg(c, "[decompressing chunk]");
+		de_dbg_indent(c, 1);
+		xc->codec(c, &d2i, &d2o, &d2r, NULL);
+		de_dbg_indent(c, -1);
+		dbuf_flush(xc->dcmpro->f);
+
+		new_olen = xc->dcmpro->f->len;
+		if(d2r.errcode) {
+			de_dfilter_transfer_error2(c, &d2r, xc->dres, xc->modname);
+			xc->errflag = 1;
+			goto done;
+		}
+		if(new_olen != old_olen+chk->ulen) {
+			xc->errflag = 1;
+			goto done;
+		}
+	}
+	else {
+		xc->errflag = 1;
+		goto done;
+	}
+
+	xc->pos += chk->clen;
+
+done:
+	de_dbg_indent_restore(c, saved_indent_level);
+}
+
+int fmtutil_xpk_ismethodsupported(u32 method)
+{
+	switch(method) {
+	case CODE_NONE:
+	case CODE_MASH:
+		return 1;
+	}
+	return 0;
+}
+
+void fmtutil_xpk_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params)
+{
+	struct xpkc_ctx *xc = NULL;
+	u8 hchk_calc;
+
+	xc = de_malloc(c, sizeof(struct xpkc_ctx));
+	xc->modname = "xpk";
+	xc->dcmpri = dcmpri;
+	xc->dcmpro = dcmpro;
+	xc->dres = dres;
+	xc->inf_endpos = dcmpri->pos + dcmpri->len;
+
+	if(dcmpri->len<12) {
+		xc->errflag = 1;
+		goto done;
+	}
+	xc->pos = dcmpri->pos;
+
+	if((UI)dbuf_getu32be_p(dcmpri->f, &xc->pos) != CODE_XPKF) {
+		xc->errflag = 1;
+		goto done;
+	}
+
+	xc->cmpr_len = dbuf_getu32be_p(dcmpri->f, &xc->pos);
+	de_dbg(c, "XPK cmpr len: %"I64_FMT, xc->cmpr_len);
+
+	dbuf_read_fourcc(dcmpri->f, xc->pos, &xc->method4cc, 4, 0);
+	de_dbg(c, "XPK cmpr type: '%s'", xc->method4cc.id_dbgstr);
+	xc->pos += 4;
+
+	// TODO: There's too much dbg info, except maybe when used with XPKF format.
+
+	xc->orig_len = dbuf_getu32be_p(dcmpri->f, &xc->pos);
+	de_dbg(c, "orig len: %"I64_FMT, xc->orig_len);
+
+	xc->pos += 16;
+	xc->flags = dbuf_getbyte_p(dcmpri->f, &xc->pos);
+	de_dbg(c, "flags: 0x%02x", (UI)xc->flags);
+	xc->hchk = dbuf_getbyte_p(dcmpri->f, &xc->pos);
+	de_dbg(c, "hchk: 0x%02x", (UI)xc->hchk);
+
+	hchk_calc = xpk_checksumh(dcmpri->f, dcmpri->pos, 36, xc->hchk);
+	de_dbg(c, "hchk (calculated): 0x%02x", (UI)hchk_calc);
+
+	xc->subvrs = dbuf_getbyte_p(dcmpri->f, &xc->pos);
+	xc->masvrs = dbuf_getbyte_p(dcmpri->f, &xc->pos);
+	de_dbg(c, "vers: %u.%u", (UI)xc->masvrs, (UI)xc->subvrs);
+
+	if(hchk_calc != xc->hchk) {
+		xpk_on_checksum_error(c, xc);
+		goto done;
+	}
+
+	if(xc->flags & XPKSTREAMF_EXTHEADER) {
+		i64 ehlen;
+
+		ehlen = dbuf_getu16be_p(dcmpri->f, &xc->pos);
+		xc->pos += ehlen;
+	}
+
+	if(!fmtutil_xpk_ismethodsupported(xc->method4cc.id)) {
+		de_dfilter_set_errorf(c, dres, xc->modname, "Unsupported XPK compession method '%s'",
+			xc->method4cc.id_sanitized_sz);
+		goto done;
+	}
+
+	if(xc->flags & XPKSTREAMF_PASSWORD) {
+		xc->errflag = 1;
+		goto done;
+	}
+
+	switch(xc->method4cc.id) {
+	case CODE_MASH:
+		xc->codec = fmtutil_xpkMASH_codectype1;
+		break;
+	}
+
+	while(1) {
+		if(xc->errflag) goto done;
+		xc->pos = xc->dcmpri->pos + de_pad_to_4(xc->pos - xc->dcmpri->pos);
+		if(xc->pos+8 > xc->inf_endpos) goto done;
+		do_xpk_chunk(c, xc);
+		if(xc->curchunk.chk_type==XPKCHUNK_END) goto done;
+	}
+
+done:
+	if(xc) {
+		if(xc->errflag) {
+			de_dfilter_set_generic_error(c, dres, xc->modname);
+		}
+	}
+	de_free(c, xc);
+}
+
+void fmtutil_decompress_stos_pictbank(deark *c, dbuf *inf,
+	i64 picdatapos1, i64 rledatapos1, i64 pointspos1,
+	dbuf *unc_pixels, i64 unc_image_size)
+{
+	i64 picdatapos;
+	i64 rledatapos;
+	u8 picbyte;
+	u8 rlebyte;
+	u8 rlebit;
+	i64 cmpr_pic_bytes, cmpr_rle_bytes, points_bytes;
+	struct de_bitbuf_lowlevel *bbll_r = NULL;
+	struct de_bitreader *bitrd_p = NULL;
+	i64 t;
+
+	de_dbg(c, "decompressing picture");
+	de_dbg_indent(c, 1);
+
+	bbll_r = de_malloc(c, sizeof(struct de_bitbuf_lowlevel));
+	bitrd_p = de_malloc(c, sizeof(struct de_bitreader));
+	bitrd_p->f = inf;
+	bitrd_p->curpos = pointspos1;
+	bitrd_p->endpos = inf->len;
+
+	picdatapos = picdatapos1;
+	rledatapos = rledatapos1;
+
+	// Note that the first picbyte and/or rlebyte can potentially be overwritten
+	// before being used. That's just the way it is, apparently.
+	picbyte = dbuf_getbyte_p(inf, &picdatapos);
+	rlebyte = dbuf_getbyte_p(inf, &rledatapos);
+
+	for(t=0; t<unc_image_size; t++) {
+		// We'll need an rle bit.
+		// If we've run out of them, read a "points" bit to decide whether
+		// to repeat the previous 8 rle bits (stored in rlebyte), or to read
+		// a new set of 8 rle bits.
+		if(bbll_r->nbits_in_bitbuf==0) {
+			u8 pointsbit;
+
+			pointsbit = (u8)de_bitreader_getbits(bitrd_p, 1);
+			if(pointsbit) {
+				rlebyte = dbuf_getbyte_p(inf, &rledatapos);
+			}
+
+			de_bitbuf_lowlevel_add_byte(bbll_r, rlebyte);
+		}
+
+		rlebit = (u8)de_bitbuf_lowlevel_get_bits(bbll_r, 1);
+		if(rlebit) {
+			picbyte = dbuf_getbyte_p(inf, &picdatapos);
+		}
+
+		dbuf_writebyte(unc_pixels, picbyte);
+	}
+
+	dbuf_flush(unc_pixels);
+	cmpr_pic_bytes = picdatapos - picdatapos1;
+	cmpr_rle_bytes = rledatapos - rledatapos1;
+	de_bitreader_skip_to_byte_boundary(bitrd_p);
+	points_bytes = bitrd_p->curpos - pointspos1;
+	de_dbg(c, "compressed pic bytes: %"I64_FMT, cmpr_pic_bytes);
+	de_dbg(c, "compressed rle bytes: %"I64_FMT, cmpr_rle_bytes);
+	de_dbg(c, "points bytes: %"I64_FMT, points_bytes);
+	de_dbg(c, "decompressed %"I64_FMT" to %"I64_FMT" bytes",
+		cmpr_pic_bytes + cmpr_rle_bytes + points_bytes, unc_pixels->len);
+
+	de_free(c, bbll_r);
+	de_free(c, bitrd_p);
+	de_dbg_indent(c, -1);
+}
