@@ -47,6 +47,7 @@ struct lzh_ctx {
 	UI implode_min_match_len;
 	UI dist_code_extra_bits;
 	struct de_crcobj *crco;
+	i64 max_offset_used;
 };
 
 static void lzh_destroy_trees(struct lzh_ctx *cctx);
@@ -77,6 +78,37 @@ static u64 lzh_getbits(struct lzh_ctx *cctx, UI nbits)
 	return de_bitreader_getbits(&cctx->bitrd, nbits);
 }
 
+static void lzh_make_tree_from_bytearrays(struct lzh_ctx *cctx,
+	struct lzh_tree_wrapper *tree,
+	const u8 *codes, const u8 *lengths, UI ncodes,
+	const char *dbgtitle)
+{
+	deark *c = cctx->c;
+	UI n;
+	char b2buf[72];
+
+	tree->ht = fmtutil_huffman_create_decoder(c, ncodes, ncodes);
+	if(dbgtitle) {
+		de_dbg3(c, "[%s]", dbgtitle);
+	}
+	de_dbg_indent(c, 1);
+	for(n=0; n<ncodes; n++) {
+		UI nbits;
+		u64 code;
+
+		nbits = (UI)lengths[n];
+		code = (u64)codes[n];
+
+		if(dbgtitle && c->debug_level>=3) {
+			de_dbg3(c, "code: \"%s\" = %d",
+				de_print_base2_fixed(b2buf, sizeof(b2buf), code, nbits), (int)n);
+		}
+		fmtutil_huffman_add_code(c, tree->ht->bk, code, nbits,
+			(fmtutil_huffman_valtype)n);
+	}
+	de_dbg_indent(c, -1);
+}
+
 static UI lh5x_read_a_code_length(struct lzh_ctx *cctx)
 {
 	UI n;
@@ -499,6 +531,9 @@ static void lh5x_do_lzh_block(struct lzh_ctx *cctx, int blk_idx)
 			}
 			de_lz77buffer_copy_from_hist(cctx->ringbuf,
 				(UI)(cctx->ringbuf->curpos-offset-1), length);
+			if((i64)offset+1 > cctx->max_offset_used) {
+				cctx->max_offset_used = (i64)offset+1;
+			}
 		}
 
 		ncodes_remaining_this_block--;
@@ -605,7 +640,7 @@ static void decompress_lh5x_internal(struct lzh_ctx *cctx, struct de_lh5x_params
 	}
 
 done:
-	;
+	lzhp->max_offset_used = cctx->max_offset_used;
 }
 
 void fmtutil_decompress_lh5x(deark *c, struct de_dfilter_in_params *dcmpri,
@@ -784,6 +819,7 @@ static int deflate_block_type2_read_trees(deark *c, struct lzh_ctx *cctx)
 	UI cll[19]; // sorted code length lengths
 	UI prev_code = 0;
 	UI num_rle_codes_left = 0;
+	u8 stop_code_defined = 0;
 	int saved_indent_level;
 
 	de_dbg_indent_save(c, &saved_indent_level);
@@ -821,6 +857,14 @@ static int deflate_block_type2_read_trees(deark *c, struct lzh_ctx *cctx)
 				(fmtutil_huffman_valtype)i, cll[i]);
 		}
 	}
+	if(fmtutil_huffman_hcb_get_num_codes(cctx->meta_tree.ht->builder) < 1) {
+		// There must be at least 1, if only because it will ultimately be
+		// needed to define the mandatory "stop" code.
+		// (Actually, I think there must be at least two, for complicated reasons.)
+		de_dfilter_set_errorf(cctx->c, cctx->dres, cctx->modname,
+			"Bad \"code length codes\" table");
+		goto done;
+	}
 
 	if(!fmtutil_huffman_make_canonical_code(c, cctx->meta_tree.ht->bk,
 		cctx->meta_tree.ht->builder, 0, "derived codelengths codebook"))
@@ -854,6 +898,7 @@ static int deflate_block_type2_read_trees(deark *c, struct lzh_ctx *cctx)
 		}
 		else {
 			x = read_next_code_using_tree(cctx, &cctx->meta_tree);
+			if(cctx->err_flag) goto done;
 
 			if(x<=15) {
 				prev_code = x;
@@ -884,9 +929,17 @@ static int deflate_block_type2_read_trees(deark *c, struct lzh_ctx *cctx)
 			}
 		}
 
-		if(i<num_literal_codes) {
+		if(x==0) {
+			ret = 1;
+		}
+		else if(i<num_literal_codes) {
 			ret = fmtutil_huffman_record_a_code_length(c, cctx->literals_tree.ht->builder,
 				(fmtutil_huffman_valtype)i, x);
+			if(ret) {
+				if(i==256) {
+					stop_code_defined = 1;
+				}
+			}
 		}
 		else {
 			ret = fmtutil_huffman_record_a_code_length(c, cctx->offsets_tree.ht->builder,
@@ -895,6 +948,13 @@ static int deflate_block_type2_read_trees(deark *c, struct lzh_ctx *cctx)
 		if(!ret) goto done;
 
 	}
+
+	if(!stop_code_defined) {
+		de_dfilter_set_errorf(cctx->c, cctx->dres, cctx->modname,
+			"No \"end of block\" code definition found");
+		goto done;
+	}
+
 	de_dbg_indent(c, -1);
 
 	if(!fmtutil_huffman_make_canonical_code(c, cctx->literals_tree.ht->bk,
@@ -1560,6 +1620,13 @@ static void dclimplode_internal(struct lzh_ctx *cctx)
 	deark *c = cctx->c;
 	u8 b;
 
+	if(cctx->dcmpri->len==0) {
+		// This special case is possibly a standard part of DCL Implode format:
+		// A zero-length input file, with no header or stop code, decompresses to
+		// a zero-length output file.
+		goto done;
+	}
+
 	b = (u8)lzh_getbits(cctx, 8);
 	if(b==1) {
 		cctx->implode_3_trees = 1;
@@ -1987,3 +2054,725 @@ done:
 		destroy_lzh_ctx(cctx);
 	}
 }
+
+//-------------------------- LZS (Stac) --------------------------
+
+static void lzstac_decompress_block(struct lzh_ctx *cctx)
+{
+	deark *c = cctx->c;
+
+	while(1) {
+		UI n;
+
+		if(cctx->bitrd.eof_flag || cctx->err_flag) {
+			if(!cctx->err_flag) {
+				de_dfilter_set_errorf(c, cctx->dres, cctx->modname,
+					"Decoding error (end-of-data code not found)");
+				cctx->err_flag = 1;
+			}
+			goto done;
+		}
+
+		n = (UI)lzh_getbits(cctx, 1);
+		if(n==0) { // a literal
+			u8 b;
+
+			b = (u8)lzh_getbits(cctx, 8);
+			if(cctx->c->debug_level>=4) {
+				de_dbg(c, "lit %u", (UI)b);
+			}
+			de_lz77buffer_add_literal_byte(cctx->ringbuf, b);
+		}
+		else { // an offset+distance, or a stop code
+
+			UI matchlen_code, matchlen;
+			UI offset_code, offset;
+
+			// Read the offset
+
+			n = (UI)lzh_getbits(cctx, 1);
+			if(n==0) { // an 11-bit offset...
+				offset_code = (UI)lzh_getbits(cctx, 11);
+				if(offset_code==0) { // Expecting at least 128
+					cctx->err_flag = 1;
+					goto done;
+				}
+				offset = offset_code - 1;
+			}
+			else { // 7-bit offset, or stop code
+				offset_code = (UI)lzh_getbits(cctx, 7);
+				if(offset_code==0) {
+					if(cctx->c->debug_level>=4) {
+						de_dbg(c, "stop code");
+					}
+					goto done;
+				}
+				offset = offset_code - 1;
+
+			}
+
+			// Read the match-length
+
+			matchlen_code = (UI)read_next_code_using_tree(cctx, &cctx->matchlengths_tree);
+
+			if(matchlen_code < 6) {
+				matchlen = matchlen_code + 2;
+			}
+			else {
+				matchlen = 8; // This will be increased as we read more bits
+
+				while(1) {
+					matchlen_code = (UI)lzh_getbits(cctx, 4);
+					matchlen += matchlen_code;
+
+					// Arbitrary limit. The matchlen can definitely be *much* larger
+					// than the history buffer.
+					if(matchlen > 0x80000000) {
+						cctx->err_flag = 1;
+						goto done;
+					}
+
+					if(matchlen_code < 15) {
+						break;
+					}
+				}
+			}
+
+			if(cctx->c->debug_level>=4) {
+				de_dbg(c, "match d=%u l=%u", offset+1, matchlen);
+			}
+			de_lz77buffer_copy_from_hist(cctx->ringbuf,
+				(UI)(cctx->ringbuf->curpos-1-offset), matchlen);
+		}
+	}
+
+done:
+	de_bitreader_skip_to_byte_boundary(&cctx->bitrd);
+}
+
+// codec_private_params = struct de_lzstac_params, or NULL.
+//  params.flags:
+//    0x1 = disallow zero blocks
+//    0x2 = disallow more than 1 block
+void fmtutil_lzstac_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params)
+{
+	struct lzh_ctx *cctx = NULL;
+	int block_count = 0;
+	UI flags = 0;
+	static const u8 lzstac_lencodes[7] = {
+		// The first 6 codes are for lengths 2-7, and the last is special.
+		0x00,0x01,0x02,0x0c,0x0d,0x0e,0x0f
+	};
+	static const u8 lzstac_lenlengths[7] = {
+		0x02,0x02,0x02,0x04,0x04,0x04,0x04
+	};
+
+	if(codec_private_params) {
+		struct de_lzstac_params *params;
+
+		params = (struct de_lzstac_params*)codec_private_params;
+		flags = params->flags;
+	}
+
+	cctx = de_malloc(c, sizeof(struct lzh_ctx));
+	cctx->modname = "LZStac";
+	cctx->c = c;
+	cctx->dcmpri = dcmpri;
+	cctx->dcmpro = dcmpro;
+	cctx->dres = dres;
+
+	cctx->bitrd.bbll.is_lsb = 0;
+	cctx->bitrd.f = dcmpri->f;
+	cctx->bitrd.curpos = dcmpri->pos;
+	cctx->bitrd.endpos = dcmpri->pos + dcmpri->len;
+
+	lzh_make_tree_from_bytearrays(cctx, &cctx->matchlengths_tree,
+		lzstac_lencodes, lzstac_lenlengths, 7, NULL);
+
+	cctx->ringbuf = de_lz77buffer_create(cctx->c, 2048);
+	cctx->ringbuf->userdata = (void*)cctx;
+	cctx->ringbuf->writebyte_cb = lzh_lz77buf_writebytecb_flagerrors;
+
+	while(1) {
+		if(block_count==0 && (dcmpri->len<=0) && !(flags & 0x1)) {
+			// No input data (0 blocks), and it is allowed. Stop.
+			break;
+		}
+
+		if(block_count>0 && (flags & 0x2)) {
+			// More than 1 block not allowed, so stop.
+			break;
+		}
+
+		de_dbg3(c, "LZS block at %"I64_FMT, cctx->bitrd.curpos);
+		de_dbg_indent(c, 1);
+		lzstac_decompress_block(cctx);
+		de_dbg_indent(c, -1);
+
+		block_count++;
+		if(cctx->err_flag) break;
+		if(cctx->bitrd.curpos >= cctx->bitrd.endpos) break;
+	}
+
+	if(cctx->err_flag) {
+		de_dfilter_set_errorf(c, dres, cctx->modname, "Decoding error");
+		goto done;
+	}
+
+	cctx->dres->bytes_consumed = cctx->bitrd.curpos - cctx->dcmpri->pos;
+	if(cctx->dres->bytes_consumed<0) {
+		cctx->dres->bytes_consumed = 0;
+	}
+	cctx->dres->bytes_consumed_valid = 1;
+
+done:
+	destroy_lzh_ctx(cctx);
+}
+
+//-------------------------- XPK:MASH --------------------------
+
+static void mash_fill_bitbuf(struct lzh_ctx *cctx)
+{
+	u8 b;
+
+	if(cctx->err_flag) return;
+	if(cctx->bitrd.curpos >= cctx->bitrd.endpos) {
+		cctx->err_flag = 1;
+		return;
+	}
+
+	b = dbuf_getbyte_p(cctx->dcmpri->f, &cctx->bitrd.curpos);
+	de_bitbuf_lowlevel_add_byte(&cctx->bitrd.bbll, b);
+}
+
+static u8 mash_getbit(struct lzh_ctx *cctx)
+{
+	u8 v;
+
+	if(cctx->err_flag) return 0;
+
+	if(cctx->bitrd.bbll.nbits_in_bitbuf==0) {
+		mash_fill_bitbuf(cctx);
+	}
+
+	v = (u8)de_bitbuf_lowlevel_get_bits(&cctx->bitrd.bbll, 1);
+
+	return v;
+}
+
+static UI mash_getbits(struct lzh_ctx *cctx, UI nbits)
+{
+	UI i;
+	UI n = 0;
+
+	for(i=0; i<nbits; i++) {
+		n <<= 1;
+		n |= (UI)mash_getbit(cctx);
+	}
+
+	return n;
+}
+
+static UI mash_read_offset(struct lzh_ctx *cctx)
+{
+	UI ocode;
+	UI offset;
+	static const u8 numextrabits[8] = {5, 7, 9, 10, 11, 12, 13, 14};
+
+	ocode = mash_getbits(cctx, 3);
+	offset = mash_getbits(cctx, numextrabits[ocode]);
+	if(ocode==1) {
+		offset += 32;
+	}
+	else if(ocode>=2) {
+		offset += (1<<numextrabits[ocode]) - 352;
+	}
+	return offset;
+}
+
+static UI mash_read_run_of_1_bits(struct lzh_ctx *cctx, UI max1bits)
+{
+	UI count_of_1s = 0;
+
+	while(1) {
+		u8 b;
+
+		if(count_of_1s>=max1bits) break;
+		b = mash_getbit(cctx);
+		if(b==0) break;
+		count_of_1s++;
+	}
+
+	return count_of_1s;
+}
+
+static void mash_main(deark *c, struct lzh_ctx *cctx)
+{
+	while(1) {
+		UI count_of_1s;
+		UI offset = 0;
+		UI matchlen = 0;
+		UI unc_run_len = 0;
+		u8 hflag1, hflag2;
+
+		if(cctx->err_flag) goto done;
+		if(lzh_have_enough_output(cctx)) goto done;
+
+		// Read/decode the length of an uncompressed run (may be 0)
+
+		count_of_1s = mash_read_run_of_1_bits(cctx, 6);
+		if(count_of_1s<=5) {
+			unc_run_len = count_of_1s;
+		}
+		else {
+			count_of_1s = mash_read_run_of_1_bits(cctx, 16);
+			if(count_of_1s>=16) {
+				// TODO: Verify this limit
+				cctx->err_flag = 1;
+				goto done;
+			}
+
+			unc_run_len = mash_getbits(cctx, count_of_1s+1);
+			unc_run_len += (2<<count_of_1s) + 4;
+		}
+
+		// Copy the literal bytes
+
+		if(unc_run_len>0) {
+			UI ii;
+			u8 ib;
+
+			for(ii=0; ii<unc_run_len; ii++) {
+				if(cctx->bitrd.curpos >= cctx->bitrd.endpos) {
+					cctx->err_flag = 1;
+					return;
+				}
+				ib = dbuf_getbyte_p(cctx->dcmpri->f, &cctx->bitrd.curpos);
+				if(c->debug_level>=4) {
+					de_dbg(c, "lit %u", (UI)ib);
+				}
+				de_lz77buffer_add_literal_byte(cctx->ringbuf, ib);
+				if(cctx->err_flag) goto done;
+			}
+		}
+
+		if(lzh_have_enough_output(cctx)) goto done;
+
+		// Read/decode a matchlen and offset
+
+		hflag1 = mash_getbit(cctx);
+		if(hflag1==0) {
+			hflag2 = mash_getbit(cctx);
+			if(hflag2==0) {
+				matchlen = 2;
+				offset = mash_getbits(cctx, 9);
+			}
+			else {
+				matchlen = 3;
+				offset = mash_read_offset(cctx);
+			}
+		}
+		else {
+			count_of_1s = mash_read_run_of_1_bits(cctx, 15);
+			if(count_of_1s>=15) {
+				// TODO: Verify this limit
+				cctx->err_flag = 1;
+				goto done;
+			}
+
+			matchlen = mash_getbits(cctx, count_of_1s+1);
+			matchlen += (2<<count_of_1s) + 2;
+			offset = mash_read_offset(cctx);
+		}
+
+		if(c->debug_level>=4) {
+			de_dbg(c, "match d=%u l=%u", offset, matchlen);
+		}
+
+		if(offset==0) {
+			// TODO: Figure out what offset 0 means
+			// (seen at end of final chunk).
+			goto done;
+		}
+
+		if((i64)offset > cctx->nbytes_written) {
+			de_dfilter_set_errorf(c, cctx->dres, cctx->modname, "Bad offset");
+			cctx->err_flag = 1;
+			goto done;
+		}
+
+		de_lz77buffer_copy_from_hist(cctx->ringbuf,
+			(UI)(cctx->ringbuf->curpos-offset), matchlen);
+	}
+
+done:
+	;
+}
+
+static void mash_lz77buf_writebytecb(struct de_lz77buffer *rb, u8 n)
+{
+	struct lzh_ctx *cctx = (struct lzh_ctx*)rb->userdata;
+
+	if(lzh_have_enough_output(cctx)) {
+		// This ought to be an error, but in practice there's often an extra
+		// decompressed byte that we have to ignore.
+		return;
+	}
+	dbuf_writebyte(cctx->dcmpro->f, n);
+	cctx->nbytes_written++;
+}
+
+void fmtutil_xpkMASH_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params)
+{
+	struct lzh_ctx *cctx = NULL;
+
+	cctx = de_malloc(c, sizeof(struct lzh_ctx));
+	cctx->modname = "mash";
+	cctx->c = c;
+	cctx->dcmpri = dcmpri;
+	cctx->dcmpro = dcmpro;
+	cctx->dres = dres;
+	cctx->bitrd.bbll.is_lsb = 0;
+	cctx->bitrd.f = dcmpri->f;
+	cctx->bitrd.curpos = dcmpri->pos;
+	cctx->bitrd.endpos = dcmpri->pos + dcmpri->len;
+
+	cctx->ringbuf = de_lz77buffer_create(c, 32768);
+	cctx->ringbuf->userdata = (void*)cctx;
+	cctx->ringbuf->writebyte_cb = mash_lz77buf_writebytecb;
+
+	mash_main(c, cctx);
+
+	if(cctx->err_flag) {
+		de_dfilter_set_generic_error(c, dres, cctx->modname);
+	}
+	destroy_lzh_ctx(cctx);
+}
+
+///////////////////// InstaCompOne
+
+static UI read_run_of_1_bits(struct lzh_ctx *cctx, UI max1bits)
+{
+	UI count_of_1s = 0;
+
+	while(1) {
+		u8 b;
+
+		if(count_of_1s>=max1bits) break;
+		b = (u8)lzh_getbits(cctx, 1);
+		if(b==0) break;
+		count_of_1s++;
+	}
+
+	return count_of_1s;
+}
+
+static void make_ic1_trees(deark *c, struct lzh_ctx *cctx)
+{
+#define ICL_NUM_ML_CODES 14
+	static const u8 codes[ICL_NUM_ML_CODES] = {
+		0,1,4,0xa,0xb,0x18,0x19,0x34,0x35,0x36,0x37,0x0e,0x1e,0x1f };
+	static const u8 lengths[ICL_NUM_ML_CODES] = {
+		2,2,3,4,  4,  5,   5,   6,   6,   6,   6,   4,   5,   5 };
+	static const u8 vals[ICL_NUM_ML_CODES] = {
+		0,1,2,3,  4,  5,   6,   7,   8,   9,   10,  103, 104, 105 };
+	size_t k;
+
+	cctx->matchlengths_tree.ht = fmtutil_huffman_create_decoder(c, ICL_NUM_ML_CODES, 0);
+
+	for(k=0; k<ICL_NUM_ML_CODES; k++) {
+		fmtutil_huffman_add_code(c, cctx->matchlengths_tree.ht->bk,
+			codes[k], lengths[k], vals[k]);
+	}
+}
+
+static UI ic1_read_matchlencode(struct lzh_ctx *cctx)
+{
+	fmtutil_huffman_valtype n = 0;
+	UI v = 0;
+	UI num_extra_bits = 0;
+
+	n = read_next_code_using_tree(cctx, &cctx->matchlengths_tree);
+
+	if(n<100) {
+		v = (UI)n;
+	}
+	else if(n==103) {
+		v = 11;
+		num_extra_bits = 3;
+	}
+	else if(n==104) {
+		v = 19;
+		num_extra_bits = 3;
+	}
+	else { // 105
+		UI count_of_1s;
+
+		// Note that we've already read 5 1s.
+		count_of_1s = (UI)read_run_of_1_bits(cctx, 5);
+		count_of_1s += 5;
+		num_extra_bits = count_of_1s;
+		v = (1U<<count_of_1s) - 5;
+	}
+
+	if(num_extra_bits>0) {
+		UI x;
+
+		x = (UI)lzh_getbits(cctx, num_extra_bits);
+		v += x;
+	}
+
+	return v;
+}
+
+static UI ic1_read_litlen(struct lzh_ctx *cctx)
+{
+	UI n1;
+	UI x;
+	UI v = 0;
+
+	n1 = read_run_of_1_bits(cctx, 5);
+	v = 1<<n1;
+	x = (UI)lzh_getbits(cctx, n1);
+	v = v | x;
+	return v;
+}
+
+static UI ic1_read_distance(struct lzh_ctx *cctx)
+{
+	i64 bd;
+	UI v = 0;
+	UI n_extra_bits1 = 0;
+	UI n_extra_bits2 = 0;
+	UI bias1 = 0;
+	UI code11_param = 0;
+	u8 b0, b1;
+	deark *c = cctx->c;
+
+	bd = cctx->nbytes_written;
+
+	if(bd<=10) {
+		n_extra_bits1 = 2;
+	}
+	else if(bd<=20) {
+		n_extra_bits1 = 3;
+	}
+	else if(bd<=40) {
+		n_extra_bits1 = 4;
+	}
+	else if(bd<=80) {
+		n_extra_bits1 = 5;
+	}
+	else if(bd<=160) {
+		n_extra_bits1 = 6;
+	}
+	else if(bd<=672) {
+		n_extra_bits1 = 7;
+	}
+	else if(bd<=1000) {
+		// ??? Obviously this should be 1344, not 1000. But 1344 doesn't work.
+		n_extra_bits1 = 8;
+	}
+	else if(bd<=2688) {
+		n_extra_bits1 = 9;
+	}
+	else if(bd<=5376) {
+		n_extra_bits1 = 10;
+	}
+	else if(bd<=10752) {
+		n_extra_bits1 = 11;
+	}
+	else if(bd<=21504) {
+		n_extra_bits1 = 12;
+	}
+	else {
+		n_extra_bits1 = 13;
+	}
+
+	if(n_extra_bits1>=2) {
+		n_extra_bits2 = n_extra_bits1 - 2;
+		bias1 = 1U<<n_extra_bits2;
+	}
+	if(n_extra_bits1>=3) {
+		code11_param = (1U<<(n_extra_bits1-3))*10;
+	}
+
+	b0 = (u8)lzh_getbits(cctx, 1);
+	if(b0==1) {
+		b1 = (u8)lzh_getbits(cctx, 1);
+		if(b1==0) { // addressing mode "10"
+			v = (UI)lzh_getbits(cctx, n_extra_bits1);
+			v += bias1;
+		}
+		else { // addressing mode "11"
+			if(code11_param) {
+				UI n_extra_bits3;
+
+				if(bd>=1644 && bd<=1664) {
+					// FIXME: This hack shouldn't be here, and this range may be wrong.
+					// 1642 needs to NOT be included
+					// 1643 unknown
+					// 1644 needs to be included
+					n_extra_bits3 = 11;
+				}
+				else if(bd>43008) {
+					n_extra_bits3 = 15;
+				}
+				else {
+					n_extra_bits3 = (UI)de_log2_rounded_up((i64)bd-(i64)code11_param);
+				}
+
+				v = (UI)lzh_getbits(cctx, n_extra_bits3);
+				v += code11_param;
+			}
+			else {
+				cctx->err_flag = 1;
+				de_dfilter_set_errorf(c, cctx->dres, cctx->modname,
+					"TODO code11 bd=%"I64_FMT, bd);
+				goto done;
+			}
+		}
+	}
+	else { // addressing mode "0"
+		v = (UI)lzh_getbits(cctx, n_extra_bits2);
+	}
+
+done:
+	return v;
+}
+
+static void ic1_internal(struct lzh_ctx *cctx)
+{
+	deark *c = cctx->c;
+	u8 mode = 1;
+	UI k;
+	i64 next_special_outpos = 65536;
+	char descr[32];
+
+	(void)lzh_getbits(cctx, 32); // unused?
+
+	make_ic1_trees(c, cctx);
+
+	// TODO: Use the correct buffer size (32768?)
+	cctx->ringbuf = de_lz77buffer_create(cctx->c, 65536*2);
+	cctx->ringbuf->userdata = (void*)cctx;
+	cctx->ringbuf->writebyte_cb = lzh_lz77buf_writebytecb_flagerrors;
+
+	while(1) {
+		UI matchlencode;
+
+		if(c->debug_level>=4) {
+			de_bitreader_describe_curpos(&cctx->bitrd, descr, sizeof(descr));
+			de_dbg(c, "at %s mode=%u", descr, (UI)mode);
+		}
+
+		if(cctx->bitrd.eof_flag || cctx->err_flag) {
+			goto done;
+		}
+		if(lzh_have_enough_output(cctx)) {
+			goto done;
+		}
+
+		if(cctx->nbytes_written >= next_special_outpos) {
+			de_bitreader_skip_to_byte_boundary(&cctx->bitrd);
+			if(c->debug_level>=3) {
+				de_dbg(c, "segment data at at %"I64_FMT, cctx->bitrd.curpos);
+			}
+			(void)lzh_getbits(cctx, 32);
+			next_special_outpos += 65536;
+			mode = 1; // TODO: Is this right?
+		}
+
+		matchlencode = ic1_read_matchlencode(cctx);
+		if(cctx->err_flag) goto done;
+
+		if(matchlencode>0 || mode==0) {
+			UI matchlen;
+			UI offset;
+
+			matchlen = matchlencode + 2;
+			if(mode==0) {
+				matchlen++;
+			}
+
+			offset = ic1_read_distance(cctx);
+			if(cctx->err_flag) goto done;
+			if(offset > cctx->nbytes_written) {
+				cctx->err_flag = 1;
+				de_dfilter_set_errorf(cctx->c, cctx->dres, cctx->modname,
+					"Bad distance code: %u", offset);
+				goto done;
+			}
+
+			if(c->debug_level>=4) {
+				de_dbg(c, "op=%"I64_FMT" match d=%u l=%u", cctx->nbytes_written,
+					offset+1, matchlen);
+			}
+
+			de_lz77buffer_copy_from_hist(cctx->ringbuf,
+				(UI)(cctx->ringbuf->curpos-offset-1), matchlen);
+
+			mode = 1;
+		}
+		else { // run of literals
+			UI litlen;
+
+			litlen = ic1_read_litlen(cctx);
+			if(c->debug_level>=4) {
+				de_dbg(c, "op=%"I64_FMT" lit_run %u", cctx->nbytes_written,
+					litlen);
+			}
+			for(k=0; k<litlen; k++) {
+				u8 b;
+				b = (u8)lzh_getbits(cctx, 8);
+				de_lz77buffer_add_literal_byte(cctx->ringbuf, b);
+			}
+
+			mode = (litlen < 63) ? 0 : 1;
+		}
+	}
+
+done:
+	;
+}
+
+void fmtutil_ic1_codectype1(deark *c, struct de_dfilter_in_params *dcmpri,
+	struct de_dfilter_out_params *dcmpro, struct de_dfilter_results *dres,
+	void *codec_private_params)
+{
+	struct lzh_ctx *cctx = NULL;
+
+	cctx = de_malloc(c, sizeof(struct lzh_ctx));
+	cctx->modname = "instacomp1";
+	cctx->c = c;
+	cctx->dcmpri = dcmpri;
+	cctx->dcmpro = dcmpro;
+	cctx->dres = dres;
+
+	cctx->bitrd.bbll.is_lsb = 0;
+	cctx->bitrd.f = dcmpri->f;
+	cctx->bitrd.curpos = dcmpri->pos;
+	cctx->bitrd.endpos = dcmpri->pos + dcmpri->len;
+
+	ic1_internal(cctx);
+
+	if(cctx->err_flag) {
+		// A default error message
+		de_dfilter_set_errorf(c, dres, cctx->modname, "Decoding error");
+		goto done;
+	}
+
+	de_bitreader_skip_to_byte_boundary(&cctx->bitrd);
+	cctx->dres->bytes_consumed = cctx->bitrd.curpos - cctx->dcmpri->pos;
+	if(cctx->dres->bytes_consumed<0) {
+		cctx->dres->bytes_consumed = 0;
+	}
+	cctx->dres->bytes_consumed_valid = 1;
+
+done:
+	destroy_lzh_ctx(cctx);
+}
