@@ -5,7 +5,6 @@
 // This file is for format-specific functions that are used by multiple modules.
 
 #define DE_NOT_IN_MODULE
-#include "deark-config.h"
 #include "deark-private.h"
 #include "deark-fmtutil.h"
 
@@ -509,6 +508,7 @@ static int do_box(deark *c, struct de_boxesctx *bctx, i64 pos, i64 len,
 		}
 		header_len = 16;
 		size64 = dbuf_geti64be(bctx->f, pos+8);
+		de_sanitize_length(&size64);
 		if(size64<16) goto done;
 		payload_len = size64-16;
 	}
@@ -994,28 +994,82 @@ const char *fmtutil_get_windows_cb_data_type_name(unsigned int ty)
 	return name;
 }
 
+#define CODE_PK36 0x504b0306U // RESOF
+#define CODE_PK56 0x504b0506U
+#define CODE_PK67 0x504b0607U
+
+// We're don't really want to validate the EOCD, we just want to figure out
+// if this probably *is* an EOCD, and not a false positive.
+// Unfortunately, this has turned out to be messy to do.
+// TODO: Maybe this should accept a byte array, instead of a dbuf
+// (at this time, we really need both).
+static int is_sane_zip_eocd(dbuf *f, i64 pos)
+{
+	i64 this_disk_num;
+	i64 disk_num_with_central_dir_start;
+	i64 central_dir_num_entries;
+	i64 central_dir_byte_size;
+	i64 central_dir_offset;
+
+	// Validating Zip64 would take more work.
+	if(dbuf_getu32be(f, pos-20) == CODE_PK67) return 1;
+
+	this_disk_num = dbuf_getu16le(f, pos+4);
+	disk_num_with_central_dir_start = dbuf_getu16le(f, pos+6);
+
+	if(this_disk_num > 100) return 0;
+	if(disk_num_with_central_dir_start > this_disk_num) return 0;
+	if(disk_num_with_central_dir_start < this_disk_num-1) return 0;
+
+	central_dir_num_entries = dbuf_getu16le(f, pos+10);
+	central_dir_byte_size = dbuf_getu32le(f, pos+12);
+
+	if(central_dir_byte_size < 46*central_dir_num_entries) return 0;
+
+	if(this_disk_num==disk_num_with_central_dir_start) {
+		if(central_dir_byte_size > f->len) return 0;
+		central_dir_offset = dbuf_getu32le(f, pos+16);
+		if(central_dir_offset > f->len) return 0;
+	}
+
+	return 1;
+}
+
 // Search for the ZIP "end of central directory" object.
 // Also useful for detecting hybrid ZIP files, such as self-extracting EXE.
-int fmtutil_find_zip_eocd(deark *c, dbuf *f, i64 *foundpos)
+// flags:
+//   0x1 - Only do minimal "fast" tests
+int fmtutil_find_zip_eocd(deark *c, dbuf *f, UI flags, i64 *foundpos)
 {
+	u32 bof_sig;
 	u32 sig;
+	int skip_sanity_check = 0;
 	u8 *buf = NULL;
 	int retval = 0;
 	i64 buf_offset;
 	i64 buf_size;
+	i64 pos;
 	i64 i;
 
 	*foundpos = 0;
 	if(f->len < 22) goto done;
 
+	bof_sig = (u32)dbuf_getu32be(f, 0);
+	if(bof_sig==CODE_PK36) {
+		skip_sanity_check = 1;
+	}
+
 	// End-of-central-dir record usually starts 22 bytes from EOF. Try that first.
-	sig = (u32)dbuf_getu32le(f, f->len - 22);
-	if(sig == 0x06054b50U) {
-		*foundpos = f->len - 22;
+	pos = f->len - 22;
+	sig = (u32)dbuf_getu32be(f, pos);
+	if(sig==CODE_PK56 && (skip_sanity_check || is_sane_zip_eocd(f, pos))) {
+		*foundpos = pos;
 		retval = 1;
 		goto done;
 	}
 
+	if(flags & 0x1) goto done;
+
 	// Search for the signature.
 	// The end-of-central-directory record could theoretically appear anywhere
 	// in the file. We'll follow Info-Zip/UnZip's lead and search the last 66000
@@ -1029,7 +1083,9 @@ int fmtutil_find_zip_eocd(deark *c, dbuf *f, i64 *foundpos)
 	dbuf_read(f, buf, buf_offset, buf_size);
 
 	for(i=buf_size-22; i>=0; i--) {
-		if(buf[i]=='P' && buf[i+1]=='K' && buf[i+2]==5 && buf[i+3]==6) {
+		if(buf[i]=='P' && buf[i+1]=='K' && buf[i+2]==5 && buf[i+3]==6 &&
+			(skip_sanity_check || is_sane_zip_eocd(f, buf_offset+i)))
+		{
 			*foundpos = buf_offset + i;
 			retval = 1;
 			goto done;
@@ -1458,639 +1514,432 @@ i64 fmtutil_hlp_get_csl_p(dbuf *f, i64 *ppos)
 	return x1;
 }
 
-struct execomp_ctx {
-	u8 devmode;
-	char shortname[40];
-	char verstr[40];
-};
+#define ARJ_MIN_BASIC_HEADER_SIZE 30
+#define ARJ_MAX_BASIC_HEADER_SIZE 2600 // From the ARJ TECHNOTE file
+#define ARJ_MIN_FILE_SIZE 84
 
-static void calc_entrypoint_crc(deark *c, struct execomp_ctx *ectx, struct fmtutil_exe_info *ei)
+// Caller supplies an initialized crcobj to use.
+static int is_arj_data_at1(dbuf *f, i64 pos1, struct de_crcobj *crco)
+{
+	i64 basic_hdr_size;
+	int retval = 0;
+	u32 crc_calc, crc_reported;
+
+	basic_hdr_size = dbuf_getu16le(f, pos1+2);
+	if(basic_hdr_size>ARJ_MAX_BASIC_HEADER_SIZE || basic_hdr_size<ARJ_MIN_BASIC_HEADER_SIZE) goto done;
+	if(dbuf_getbyte(f, pos1+10) != 2) goto done; // "file type"
+	de_crcobj_addslice(crco, f, pos1+4, basic_hdr_size);
+	crc_calc = de_crcobj_getval(crco);
+	crc_reported = (u32)dbuf_getu32le(f, pos1+4+basic_hdr_size);
+	if(crc_calc!=crc_reported) goto done;
+	retval = 1;
+
+done:
+	return retval;
+}
+
+// This is a strict ARJ search, which checks the CRC.
+// It may not always be appropriate.
+int fmtutil_scan_for_arj_data(dbuf *f, i64 startpos, i64 max_skip,
+	UI flags, i64 *pfoundpos)
 {
 	struct de_crcobj *crco = NULL;
-	u32 crc1, crc2;
+	i64 curpos, endpos;
+	int retval = 0;
 
-	if(ei->entrypoint_crcs!=0) return;
+	curpos = startpos;
+	endpos = startpos + max_skip + 2; // Search space includes the 2-byte signature
+	if(endpos-2 > f->len-ARJ_MIN_FILE_SIZE) endpos = (f->len-ARJ_MIN_FILE_SIZE)+2;
 
-	// Sniff some bytes, starting at the code entry point.
-	crco = de_crcobj_create(c, DE_CRCOBJ_CRC32_IEEE);
-	de_crcobj_addslice(crco, ei->f, ei->entry_point, 32);
-	crc1 = de_crcobj_getval(crco);
-	de_crcobj_reset(crco);
-	de_crcobj_addslice(crco, ei->f, ei->entry_point+32, 32);
-	crc2 = de_crcobj_getval(crco);
-	ei->entrypoint_crcs = ((u64)crc1 << 32) | crc2;
-	if(ectx->devmode) {
-		de_dbg(c, "execomp crc: %016"U64_FMTx, ei->entrypoint_crcs);
-	}
+	while(curpos<=(endpos-2)) {
+		int ret;
 
-	de_crcobj_destroy(crco);
-}
+		ret = dbuf_search(f, (const u8*)"\x60\xea", 2, curpos, endpos-curpos,
+			pfoundpos);
+		if(!ret) goto done;
 
-static void detect_execomp_pklite(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	u8 maybe_pklite = 0; // Any format with the usual EXE header field values
-	u8 maybe_beta = 0;
-	u8 maybe_megalite = 0;
-	u8 cb[12]; // Bytes at entry point
-
-	if(ei->regIP==256 && ei->regCS==(-16) && ei->num_relocs<=2 &&
-		ei->entry_point==ei->start_of_dos_code)
-	{
-		maybe_pklite = 1;
-	}
-	else if(ei->regIP==256 && ei->regCS>(-16) && ei->num_relocs==0 &&
-		ei->entry_point>ei->start_of_dos_code)
-	{
-		maybe_beta = 1;
-	}
-	else if(ei->regCS==0 && ei->regIP==0 && ei->num_relocs==1 && ei->reloc_table_pos==28 &&
-		ei->entry_point==ei->start_of_dos_code)
-	{
-		maybe_megalite = 1;
-	}
-	else {
-		goto done;
-	}
-
-	dbuf_read(ei->f, cb, ei->entry_point, sizeof(cb));
-
-	if(maybe_pklite) {
-		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x8c\xdb\x03\xd8\x3b", 11, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v1.00-1.05, etc.
-		}
-		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x05\x00\x00\x3b", 10, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v1.12-1.15, etc.
-		}
-		if(de_memmatch(cb, (const u8*)"\x50\xb8??\xba??\x05\x00\x00\x3b", 11, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // v2.01, etc.
-		}
-		if(de_memmatch(cb, (const u8*)"\x9c\xba?\x00\x2d?\x00\x81\xe1?\x00\x81", 12, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // Patched by UN^2PACK v2.0?
-		}
-		goto done;
-	}
-
-	if(maybe_beta) {
-		if(de_memmatch(cb, (const u8*)"\x2e\x8c\x1e??\x8b\x1e\x02\x00\x8c\xda\x81", 12, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // Normal beta
-		}
-		if(de_memmatch(cb, (const u8*)"\x2e\x8c\x1e??\xfc\x8c\xc8\x2e\x2b\x06", 11, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE; // beta/loadhigh
-		}
-		goto done;
-	}
-
-	if(maybe_megalite) {
-		if(de_memmatch(cb, (const u8*)"\xb8??\xba??\x05\x00\x00\x3b\x2d", 11, '?', 0)) {
-			edd->detected_fmt = DE_SPECIALEXEFMT_PKLITE;
-		}
-		goto done;
-	}
-
-done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_PKLITE) {
-		de_strlcpy(ectx->shortname, "PKLITE", sizeof(ectx->shortname));
-		edd->modname = "pklite";
-	}
-}
-
-static void detect_execomp_lzexe(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	const char *vs = NULL;
-
-	if(ei->entrypoint_crcs==0x4b6802c9cf419437LLU) {
-		edd->detected_subfmt = 1;
-		vs = "0.90";
-	}
-	else if(ei->entrypoint_crcs==0x246655c50ae99574LLU) {
-		edd->detected_subfmt = 2;
-		vs = "0.91";
-	}
-	else if(ei->entrypoint_crcs==0xd8a60f138f680f0cLLU) {
-		edd->detected_subfmt = 3;
-		vs = "0.91e";
-	}
-
-	if(vs) {
-		edd->detected_fmt = DE_SPECIALEXEFMT_LZEXE;
-		de_strlcpy(ectx->shortname, "LZEXE", sizeof(ectx->shortname));
-		de_strlcpy(ectx->verstr, vs, sizeof(ectx->verstr));
-		edd->modname = "lzexe";
-	}
-}
-
-static void detect_execomp_exepack(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	u8 x, x2;
-	int has_RB = 0;
-
-	if(ei->num_relocs!=0) goto done;
-	if(ei->regIP!=16 && ei->regIP!=18) goto done;
-
-	if(dbuf_getu16be(ei->f, ei->entry_point-2) == 0x5242) {
-		has_RB = 1;
-	}
-
-	if(ei->entrypoint_crcs==0xa6ea214e6c16ee72LLU) {
-		edd->detected_subfmt = 5;
-		goto done;
-	}
-	else if(ei->entrypoint_crcs==0x4e04abaac5d3b465LLU) {
-		edd->detected_subfmt = 4;
-		goto done;
-	}
-	else if(ei->entrypoint_crcs==0x1f449ca73852e197LLU) {
-		x = dbuf_getbyte(ei->f, ei->entry_point+73);
-		x2 = dbuf_getbyte(ei->f, ei->entry_point+137);
-		if(x==0x0a && x2==0x27) {
-			edd->detected_subfmt = 2;
-			goto done;
-		}
-		else if(x==0x0a) {
-			edd->detected_subfmt = 1;
-			goto done;
-		}
-		else if(x==0x09) {
-			edd->detected_subfmt = 3;
-			goto done;
-		}
-	}
-	else if(ei->entrypoint_crcs==0x1da67457b559a299LLU) {
-		edd->detected_subfmt = 10; // David Fifield's exepack v1.3.0
-	}
-	else if(ei->entrypoint_crcs==0x629aca0c6520250aLLU) {
-		// EXPAKFIX-patched file. Original variant undetermined (TODO?).
-		edd->detected_subfmt = 11;
-	}
-
-	// TODO: Is SP always 128?
-	if(ei->regSP==128 && has_RB) {
-		edd->detected_fmt = DE_SPECIALEXEFMT_EXEPACK;
-		goto done;
-	}
-
-done:
-	if(edd->detected_subfmt!=0) {
-		edd->detected_fmt = DE_SPECIALEXEFMT_EXEPACK;
-	}
-
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_EXEPACK) {
-		if(ectx->devmode) {
-			de_dbg(c, "epvar: %u", (UI)edd->detected_subfmt);
-		}
-		de_strlcpy(ectx->shortname, "EXEPACK", sizeof(ectx->shortname));
-		edd->modname = "exepack";
-	}
-}
-
-static void detect_execomp_tinyprog(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	i64 pos;
-	i64 j;
-	u8 x;
-
-	if(ei->num_relocs!=0) goto done;
-	pos = ei->entry_point;
-	x = dbuf_getbyte(ei->f, pos);
-	if(x!=0xe9) goto done;
-	j = dbuf_getu16le(ei->f, pos+1);
-	pos += 3+j; // Jump over user data
-
-	x = dbuf_getbyte(ei->f, pos);
-	if(x != 0xeb) goto done;
-	j = dbuf_geti8(ei->f, pos+1);
-	pos += 2+j; // Jump over (some sort of) data
-
-	// This part seems consistent, but I'm really just guessing. Need to test more versions.
-	if(dbuf_memcmp(ei->f, pos,
-		(const void*)"\x83\xec\x10\x83\xe4\xe0\x8b\xec\x50\xbe\x05\x01\x03\x36\x01\x01", 16))
-	{
-		goto done;
-	}
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_TINYPROG;
-	de_strlcpy(ectx->shortname, "TINYPROG", sizeof(ectx->shortname));
-done:
-	;
-}
-
-static int execomp_diet_check_fingerprint(dbuf *f, i64 pos)
-{
-	u8 x;
-
-	if(dbuf_memcmp(f, pos,
-		(const u8*)"\x8e\xdb\x8e\xc0\x33\xf6\x33\xff\xb9\x08\x00\xf3\xa5\x4b\x48\x4a", 16))
-	{
-		return 0;
-	}
-
-	// Attempt to screen out LGLZ-compressed files (x==0x95).
-	x = dbuf_getbyte(f, pos+26);
-	if(x==0x8b) return 1;
-	return 0;
-}
-
-static void detect_execomp_diet(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	static const u8 offsets[] = {20, 40, 45};
-	i64 foundpos = 0;
-	size_t i;
-
-	if(ei->regCS!=0) goto done;
-	if(ei->regIP!=0 && ei->regIP!=3) goto done;
-	if(ei->num_relocs>1) goto done;
-
-	// Haven't figured out a good way to detect DIET. More research needed.
-	for(i=0; i<DE_ARRAYCOUNT(offsets); i++) {
-		if(execomp_diet_check_fingerprint(ei->f, ei->entry_point+(i64)offsets[i])) {
-			foundpos = (i64)offsets[i];
-			break;
-		}
-	}
-	if(foundpos==0) goto done;
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_DIET;
-	de_strlcpy(ectx->shortname, "DIET", sizeof(ectx->shortname));
-done:
-	;
-}
-
-static void detect_execomp_wwpack(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	u8 cb[11]; // Bytes at entry point
-
-	if(ei->num_relocs!=0) goto done;
-	if(ei->start_of_dos_code!=32) goto done;
-	if(ei->entry_point==ei->start_of_dos_code) goto done;
-
-	dbuf_read(ei->f, cb, ei->entry_point, sizeof(cb));
-	if(!de_memmatch(cb, (const u8*)"\xb8??\x8c\xca\x03\xd0\x8c\xc9\x81\xc1", 11, '?', 0)) {
-		goto done;
-	}
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_EXECOMP;
-	de_strlcpy(ectx->shortname, "WWPACK", sizeof(ectx->shortname));
-done:
-	;
-}
-
-// Caller initializes ei (to zeroes).
-// Records some basic information about an EXE file, to be used by routines that
-// detect special EXE formats.
-// The input file (f) must stay open after this. The detection routines will need
-// to read more of it.
-void fmtutil_collect_exe_info(deark *c, dbuf *f, struct fmtutil_exe_info *ei)
-{
-	i64 hdrsize; // in 16-byte units
-	i64 lfb, nblocks;
-
-	ei->f = f;
-	lfb = dbuf_getu16le(f, 2);
-	nblocks = dbuf_getu16le(f, 4);
-	ei->num_relocs = dbuf_getu16le(f, 6);
-	hdrsize = dbuf_getu16le(f, 8);
-	ei->start_of_dos_code = hdrsize*16;
-	ei->regSS = dbuf_geti16le(f, 14);
-	ei->regSP = dbuf_getu16le(f, 16);
-	ei->regIP = dbuf_getu16le(f, 20);
-	ei->regCS = dbuf_geti16le(f, 22);
-	ei->reloc_table_pos = dbuf_getu16le(f, 24);
-	ei->entry_point = (hdrsize + ei->regCS)*16 + ei->regIP;
-
-	ei->end_of_dos_code = nblocks*512;
-	if(lfb>=1 && lfb<=511) {
-		ei->end_of_dos_code = ei->end_of_dos_code - 512 + lfb;
-	}
-}
-
-// Caller supplies ei -- must call fmtutil_collect_exe_info() first.
-// Caller initializes edd, to receive the results.
-// If success, sets edd->detected_fmt to nonzero.
-// Always sets edd->detected_fmt_name to something, even if "unknown".
-// If we think we can decompress the format, sets edd->modname.
-void fmtutil_detect_execomp(deark *c, struct fmtutil_exe_info *ei,
-	struct fmtutil_specialexe_detection_data *edd)
-{
-	struct execomp_ctx ectx;
-
-	de_zeromem(&ectx, sizeof(struct execomp_ctx));
-	edd->detected_fmt = 0;
-	edd->detected_subfmt = 0;
-
-	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_PKLITE) {
-		detect_execomp_pklite(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_TINYPROG) {
-		detect_execomp_tinyprog(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_DIET) {
-		detect_execomp_diet(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-	if(edd->restrict_to_fmt==0) {
-		detect_execomp_wwpack(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-	calc_entrypoint_crc(c, &ectx, ei);
-
-	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_LZEXE) {
-		detect_execomp_lzexe(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-	if(edd->restrict_to_fmt==0 || edd->restrict_to_fmt==DE_SPECIALEXEFMT_EXEPACK) {
-		detect_execomp_exepack(c, &ectx, ei, edd);
-		if(edd->detected_fmt!=0) goto done;
-	}
-
-done:
-	if(ectx.shortname[0] && ectx.verstr[0]) {
-		de_snprintf(edd->detected_fmt_name, sizeof(edd->detected_fmt_name), "%s %s",
-			ectx.shortname, ectx.verstr);
-	}
-	else {
-		de_strlcpy(edd->detected_fmt_name, (ectx.shortname[0]?ectx.shortname:"unknown"),
-			sizeof(edd->detected_fmt_name));
-	}
-}
-
-// If found, writes a copy of pos to *pfoundpos.
-static int is_lhalike_data_at(struct fmtutil_exe_info *ei, i64 pos, u8 h_or_z, i64 *pfoundpos)
-{
-	u8 b2[8];
-
-	if(pos+21 > ei->f->len) return 0;
-	dbuf_read(ei->f, b2, pos, sizeof(b2));
-	if(b2[2]!='-' || b2[6]!='-') return 0;
-	if(b2[3]=='l' && b2[4]==h_or_z) {
-		*pfoundpos = pos;
-		return 1;
-	}
-	return 0;
-}
-
-static int is_lha_data_at(struct fmtutil_exe_info *ei, i64 pos, i64 *pfoundpos)
-{
-	return is_lhalike_data_at(ei, pos, 'h', pfoundpos);
-}
-
-// Detect LHA/LHarc self-extracting DOS EXE formats.
-// TODO: This is a work in progress.
-static void detect_exesfx_lha(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	u8 b[16];
-	int found;
-	i64 foundpos = 0;
-
-	if(ei->regSS != -16) goto done;
-	if(ei->regSP != 256) goto done;
-	if(ei->regCS != -16) goto done;
-	if(ei->regIP != 256) goto done;
-
-	dbuf_read(ei->f, b, 28, sizeof(b));
-	if(b[4]!=0xeb && b[4]!=0xe9) goto done;
-	if(b[8]=='L' && b[9]=='H') {
-		;
-	}
-	else if(b[9]=='L' && b[10]=='H') {
-		;
-	}
-	else if(b[10]=='L' && b[11]=='H') {
-		;
-	}
-	else if(b[10]=='S' && b[11]=='F') { // v1.00
-		;
-	}
-	else {
-		goto done;
-	}
-
-	found =
-		is_lha_data_at(ei, ei->end_of_dos_code, &foundpos) ||
-		is_lha_data_at(ei, ei->end_of_dos_code+1, &foundpos) ||
-		is_lha_data_at(ei, ei->end_of_dos_code+3, &foundpos) ||
-		is_lha_data_at(ei, ei->entry_point + 1292-32, &foundpos) ||
-		is_lha_data_at(ei, ei->entry_point + 1295-32, &foundpos) ||
-		is_lha_data_at(ei, ei->entry_point + 1322-32, &foundpos);
-	if(!found) goto done;
-
-	edd->payload_pos = foundpos;
-	edd->payload_len = ei->f->len - edd->payload_pos;
-	if(edd->payload_len<21) goto done;
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
-	edd->payload_valid = 1;
-	edd->payload_file_ext = "lha";
-
-done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
-		de_strlcpy(ectx->shortname, "LHA", sizeof(ectx->shortname));
-	}
-}
-
-static void detect_exesfx_larc(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	int found;
-	i64 foundpos = 0;
-
-	calc_entrypoint_crc(c, ectx, ei);
-	if(ei->entrypoint_crcs!=0x81681852d3882b18ULL) {
-		goto done;
-	}
-
-	found = is_lhalike_data_at(ei, ei->entry_point+525, 'z', &foundpos);
-	if(!found) goto done;
-
-	edd->payload_pos = foundpos;
-	edd->payload_len = ei->f->len - edd->payload_pos;
-	if(edd->payload_len<21) goto done;
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
-	edd->payload_valid = 1;
-	edd->payload_file_ext = "lzs";
-
-done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
-		de_strlcpy(ectx->shortname, "LArc", sizeof(ectx->shortname));
-	}
-}
-
-static int is_arc_data_at(struct fmtutil_exe_info *ei, i64 pos)
-{
-	u8 b[2];
-
-	dbuf_read(ei->f, b, pos, 2);
-	if(b[0]!=0x1a) return 0;
-	if(b[1]>30) return 0;
-	return 1;
-}
-
-// Detect some ARC self-extracting DOS EXE formats.
-// TODO: This is pretty fragile. It only detects files made by known versions of
-// MKSARC (from the ARC distribution).
-static void detect_exesfx_arc(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	int found = 0;
-	i64 foundpos = 0;
-
-	calc_entrypoint_crc(c, ectx, ei);
-	if(ei->entrypoint_crcs==0x057db6d8be3c4895ULL) { // MKSARC v1.00 from ARC v6.01
-		found = 1;
-		foundpos = ei->end_of_dos_code;
-	}
-	else if(ei->entrypoint_crcs==0x8542182a98613fe0ULL ||
-		ei->entrypoint_crcs==0x6bd653c8edd98eedULL)
-	{
-		// MKSARC v1.01 from ARC v6.02
-		// (Found two different versions with the same version number.)
-		// This version of MKSARC has a bug. Compared to the v1.00, the start of DOS
-		// code was reduced by 480, from 512 to 32. But the *end* of DOS code was not
-		// adjusted accordingly, leaving it 480 higher than it should be. This is
-		// important, because the end of DOS code is where we expect the ARC data to
-		// start.
-		found = 1;
-		foundpos = ei->end_of_dos_code - 480;
-		if(!is_arc_data_at(ei, foundpos)) {
-			foundpos = ei->end_of_dos_code; // In case there's a version without the bug
-		}
-	}
-	else if(ei->entrypoint_crcs==0x3230b4d5fca84644ULL) { // MKSARC v7.12, from ARC v7.12
-		found = 1;
-		foundpos = ei->end_of_dos_code;
-	}
-	// TODO: Detect MKSARC v7.12 with the /P option (OS/2 protected mode).
-	// Extraction would work, if we could detect it.
-
-	if(!found) goto done;
-	if(!is_arc_data_at(ei, foundpos)) {
-		goto done;
-	}
-
-	edd->payload_pos = foundpos;
-	edd->payload_len = ei->f->len - edd->payload_pos;
-	if(edd->payload_len<2) goto done;
-	// TODO: It would be nice to strip any padding from the end of the extracted ARC
-	// file, but that could be more trouble than it's worth.
-
-	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
-	edd->payload_valid = 1;
-	edd->payload_file_ext = "arc";
-
-done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
-		de_strlcpy(ectx->shortname, "ARC", sizeof(ectx->shortname));
-	}
-}
-
-// Also uses/updates edd->zip_eocd_* settings.
-static void detect_exesfx_zip(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
-{
-	int ret;
-	i64 foundpos = 0;
-
-	if(!edd->zip_eocd_looked_for) {
-		if(c->infile==ei->f && c->detection_data && c->detection_data->zip_eocd_looked_for) {
-			edd->zip_eocd_looked_for = 1;
-			edd->zip_eocd_found = c->detection_data->zip_eocd_found;
-			edd->zip_eocd_pos = c->detection_data->zip_eocd_pos;
+		if(crco) {
+			de_crcobj_reset(crco);
 		}
 		else {
-			edd->zip_eocd_found = (u8)fmtutil_find_zip_eocd(c, c->infile, &edd->zip_eocd_pos);
-			edd->zip_eocd_looked_for = 1;
+			crco = de_crcobj_create(f->c, DE_CRCOBJ_CRC32_IEEE);
+		}
+		if(is_arj_data_at1(f, *pfoundpos, crco)) {
+			retval = 1;
+			goto done;
+		}
+		curpos = *pfoundpos+2; // Continue the search here
+	}
+
+done:
+	de_crcobj_destroy(crco);
+	return retval;
+}
+
+static const u8 example_dqt_data0[] = {
+	0x10,0x0b,0x0c,0x0e,0x0c,0x0a,0x10,0x0e,
+	0x0d,0x0e,0x12,0x11,0x10,0x13,0x18,0x28,
+	0x1a,0x18,0x16,0x16,0x18,0x31,0x23,0x25,
+	0x1d,0x28,0x3a,0x33,0x3d,0x3c,0x39,0x33,
+	0x38,0x37,0x40,0x48,0x5c,0x4e,0x40,0x44,
+	0x57,0x45,0x37,0x38,0x50,0x6d,0x51,0x57,
+	0x5f,0x62,0x67,0x68,0x67,0x3e,0x4d,0x71,
+	0x79,0x70,0x64,0x78,0x5c,0x65,0x67,0x63
+};
+
+static const u8 example_dqt_data1[] = {
+	0x11,0x12,0x12,0x18,0x15,0x18,0x2f,0x1a,
+	0x1a,0x2f,0x63,0x42,0x38,0x42,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63,
+	0x63,0x63,0x63,0x63,0x63,0x63,0x63,0x63
+};
+
+static const u8 jpegtbl_dht0[] = {
+	// counts:
+	0x00,0x01,0x05,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,0x00,0x00,
+	// 12 data bytes:
+	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b
+};
+
+static const u8 jpegtbl_dht1[] = {
+	// counts:
+	0x00,0x02,0x01,0x03,0x03,0x02,0x04,0x03,0x05,0x05,0x04,0x04,0x00,0x00,0x01,0x7d,
+	// 162 data bytes:
+	0x01,0x02,0x03,0x00,0x04,0x11,0x05,0x12,0x21,0x31,0x41,0x06,0x13,0x51,0x61,0x07,
+	0x22,0x71,0x14,0x32,0x81,0x91,0xa1,0x08,0x23,0x42,0xb1,0xc1,0x15,0x52,0xd1,0xf0,
+	0x24,0x33,0x62,0x72,0x82,0x09,0x0a,0x16,0x17,0x18,0x19,0x1a,0x25,0x26,0x27,0x28,
+	0x29,0x2a,0x34,0x35,0x36,0x37,0x38,0x39,0x3a,0x43,0x44,0x45,0x46,0x47,0x48,0x49,
+	0x4a,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x63,0x64,0x65,0x66,0x67,0x68,0x69,
+	0x6a,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a,0x83,0x84,0x85,0x86,0x87,0x88,0x89,
+	0x8a,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0xa2,0xa3,0xa4,0xa5,0xa6,0xa7,
+	0xa8,0xa9,0xaa,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xc2,0xc3,0xc4,0xc5,
+	0xc6,0xc7,0xc8,0xc9,0xca,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,0xe1,0xe2,
+	0xe3,0xe4,0xe5,0xe6,0xe7,0xe8,0xe9,0xea,0xf1,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,
+	0xf9,0xfa
+};
+
+static const u8 jpegtbl_dht2[] = {
+	// counts:
+	0x00,0x03,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x01,0x00,0x00,0x00,0x00,0x00,
+	// 12 data bytes:
+	0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b
+};
+
+static const u8 jpegtbl_dht3[] = {
+	// counts:
+	0x00,0x02,0x01,0x02,0x04,0x04,0x03,0x04,0x07,0x05,0x04,0x04,0x00,0x01,0x02,0x77,
+	// 162 data bytes:
+	0x00,0x01,0x02,0x03,0x11,0x04,0x05,0x21,0x31,0x06,0x12,0x41,0x51,0x07,0x61,0x71,
+	0x13,0x22,0x32,0x81,0x08,0x14,0x42,0x91,0xa1,0xb1,0xc1,0x09,0x23,0x33,0x52,0xf0,
+	0x15,0x62,0x72,0xd1,0x0a,0x16,0x24,0x34,0xe1,0x25,0xf1,0x17,0x18,0x19,0x1a,0x26,
+	0x27,0x28,0x29,0x2a,0x35,0x36,0x37,0x38,0x39,0x3a,0x43,0x44,0x45,0x46,0x47,0x48,
+	0x49,0x4a,0x53,0x54,0x55,0x56,0x57,0x58,0x59,0x5a,0x63,0x64,0x65,0x66,0x67,0x68,
+	0x69,0x6a,0x73,0x74,0x75,0x76,0x77,0x78,0x79,0x7a,0x82,0x83,0x84,0x85,0x86,0x87,
+	0x88,0x89,0x8a,0x92,0x93,0x94,0x95,0x96,0x97,0x98,0x99,0x9a,0xa2,0xa3,0xa4,0xa5,
+	0xa6,0xa7,0xa8,0xa9,0xaa,0xb2,0xb3,0xb4,0xb5,0xb6,0xb7,0xb8,0xb9,0xba,0xc2,0xc3,
+	0xc4,0xc5,0xc6,0xc7,0xc8,0xc9,0xca,0xd2,0xd3,0xd4,0xd5,0xd6,0xd7,0xd8,0xd9,0xda,
+	0xe2,0xe3,0xe4,0xe5,0xe6,0xe7,0xe8,0xe9,0xea,0xf2,0xf3,0xf4,0xf5,0xf6,0xf7,0xf8,
+	0xf9,0xfa
+};
+
+void fmtutil_get_std_jpeg_qtable(UI tbl_id, u8 tbl[64])
+{
+	const u8 *src_tbl;
+
+	switch(tbl_id) {
+	case 0:
+		src_tbl = example_dqt_data0;
+		break;
+	case 1:
+		src_tbl = example_dqt_data1;
+		break;
+	default:
+		de_zeromem(tbl, 64);
+		return;
+	}
+	de_memcpy(tbl, src_tbl, 64);
+}
+
+void fmtutil_write_std_jpeg_dht(dbuf *outf, UI tbl_id)
+{
+	const u8 *tbl_data;
+	UI tbl_len;
+
+	switch(tbl_id) {
+	case 0:
+		tbl_data = jpegtbl_dht0;
+		tbl_len = 16+12;
+		break;
+	case 1:
+		tbl_data = jpegtbl_dht1;
+		tbl_len = 16+162;
+		break;
+	case 2:
+		tbl_data = jpegtbl_dht2;
+		tbl_len = 16+12;
+		break;
+	case 3:
+		tbl_data = jpegtbl_dht3;
+		tbl_len = 16+162;
+		break;
+	default:
+		return;
+	}
+
+	dbuf_write(outf, tbl_data, tbl_len);
+}
+
+// A simplified character graphics (or screen dump) API that should suffice for most
+// single-image PC formats.
+// Caller must create/free charctx (de_create_charctx/de_free_charctx)
+// and csctx (de_malloc).
+// Caller must set some fields in csctx, and optionally may set some fields
+// in charctx.
+void fmtutil_char_simple_run(deark *c, struct fmtutil_char_simplectx *csctx,
+	struct de_char_context *charctx)
+{
+	i64 i, j;
+	i64 inf_endpos;
+	i64 fg_stride;
+	i64 attr_offset;
+	u8 ccode, acode;
+	u8 fgcol, bgcol;
+	struct de_char_screen *screen;
+	de_encoding encoding;
+	struct de_encconv_state es;
+
+	inf_endpos = csctx->inf_pos + csctx->inf_len;
+	if(csctx->use_default_pal) {
+		de_copy_std_palette(DE_PALID_PC16, 0, 0, charctx->pal, 16, 0);
+	}
+	charctx->nscreens = 1;
+	charctx->screens = de_mallocarray(c, charctx->nscreens, sizeof(struct de_char_screen*));
+	charctx->screens[0] = de_malloc(c, sizeof(struct de_char_screen));
+	screen = charctx->screens[0];
+	screen->width = csctx->width_in_chars;
+	screen->height = csctx->height_in_chars;
+	screen->cell_rows = de_mallocarray(c, csctx->height_in_chars, sizeof(struct de_char_cell*));
+
+	if(csctx->input_encoding==DE_ENCODING_UNKNOWN) {
+		encoding = DE_ENCODING_CP437;
+	}
+	else {
+		encoding = csctx->input_encoding;
+	}
+	de_encconv_init(&es, DE_EXTENC_MAKE(encoding, DE_ENCSUBTYPE_PRINTABLE));
+
+	if(csctx->fg_stride) {
+		fg_stride = csctx->fg_stride;
+		attr_offset = csctx->attr_offset;
+	}
+	else {
+		fg_stride = 2;
+		attr_offset = 1;
+	}
+
+	for(j=0; j<csctx->height_in_chars; j++) {
+		screen->cell_rows[j] = de_mallocarray(c, csctx->width_in_chars, sizeof(struct de_char_cell));
+
+		for(i=0; i<csctx->width_in_chars; i++) {
+			i64 pos;
+
+			pos = csctx->inf_pos + j*csctx->width_in_chars*fg_stride + i*fg_stride;
+			if(pos < inf_endpos)
+				ccode = dbuf_getbyte(csctx->inf, pos);
+			else
+				ccode = 0;
+			if(pos < inf_endpos)
+				acode = dbuf_getbyte(csctx->inf, pos+attr_offset);
+			else
+				acode = 0;
+
+			if((acode&0x80) && !csctx->nonblink) {
+				screen->cell_rows[j][i].blink = 1;
+				acode -= 0x80;
+			}
+
+			fgcol = (acode & 0x0f);
+			bgcol = (acode & 0xf0) >> 4;
+
+			screen->cell_rows[j][i].fgcol = (u32)fgcol;
+			screen->cell_rows[j][i].bgcol = (u32)bgcol;
+			screen->cell_rows[j][i].codepoint = (i32)ccode;
+			screen->cell_rows[j][i].codepoint_unicode = de_char_to_unicode_ex((i32)ccode, &es);
 		}
 	}
 
-	if(!edd->zip_eocd_found) goto done;
-
-	// Look for a ZIP file in the overlay (approximately)
-	if(ei->f->len - ei->end_of_dos_code < 10) goto done; // Overlay too small
-
-	ret = dbuf_search(ei->f, (const u8*)"PK\x03\x04", 4, ei->end_of_dos_code-10, 20,
-		&foundpos);
-	if(!ret) goto done;
-	edd->detected_fmt = DE_SPECIALEXEFMT_ZIPSFX;
-
-done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_ZIPSFX) {
-		de_strlcpy(ectx->shortname, "ZIP", sizeof(ectx->shortname));
-	}
+	de_char_output_to_file(c, charctx);
 }
 
-static void detect_exesfx_zoo(deark *c, struct execomp_ctx *ectx,
-	struct fmtutil_exe_info *ei, struct fmtutil_specialexe_detection_data *edd)
+// **************************************************************************
+
+static int fmtid_is_dll(deark *c, struct fmtutil_fmtid_ctx *idctx)
 {
-	i64 overlay_len;
+	int retval = 0;
+	dbuf *tmpf = NULL;
+	struct fmtutil_exe_info *ei = NULL;
 
-	overlay_len = ei->f->len - ei->end_of_dos_code;
-	if(overlay_len < 24) goto done;
-	if((UI)dbuf_getu32le(ei->f, ei->end_of_dos_code+20) != 0xfdc4a7dc) goto done;
+	if(!idctx->inf) goto done;
+	if(de_getu32le_direct(&idctx->bof64bytes[60]) == 0) goto done;
 
-	calc_entrypoint_crc(c, ectx, ei);
-	switch((UI)(ei->entrypoint_crcs>>32)) {
-	case 0x6d384fa1U: // SEZ 2.00
-	case 0xec5138deU: // SEZ 2.30
-		// TODO: More Zoo SFX variants
-		break;
-	default:
+	tmpf = dbuf_open_input_subfile(idctx->inf, idctx->inf_pos, idctx->inf_len);
+	ei = de_malloc(c, sizeof(struct fmtutil_exe_info));
+	fmtutil_collect_exe_info(c, tmpf, ei);
+	if(ei->is_dll) {
+		retval = 1;
+	}
+
+done:
+	de_free(c, ei);
+	dbuf_close(tmpf);
+	return retval;
+}
+
+// Caller allocs idctx, and sets some fields.
+// [Do not use the same idctx more than once, without clearing it.]
+// Return value is ->fmtid and ->ext_sz.
+void fmtutil_fmtid(deark *c, struct fmtutil_fmtid_ctx *idctx)
+{
+	const char *ext = NULL;
+	UI m0;
+	u8 img_only = 0;
+
+	if(!idctx->have_bof64bytes) {
+		if(idctx->inf) {
+			dbuf_read(idctx->inf, idctx->bof64bytes, idctx->inf_pos,
+				de_min_int(64, idctx->inf_len));
+			idctx->have_bof64bytes = 1;
+		}
+	}
+
+	if(!idctx->have_bof64bytes) {
 		goto done;
 	}
 
-	edd->payload_pos = ei->end_of_dos_code;
-	edd->payload_len = overlay_len;
-	edd->payload_valid = 1;
-	edd->detected_fmt = DE_SPECIALEXEFMT_SFX;
-	edd->payload_file_ext = "zoo";
+	if(idctx->mode==FMTUTIL_FMTIDMODE_ALL_IMG) {
+		img_only = 1;
+	}
+
+	m0 = (UI)de_getu32be_direct(&idctx->bof64bytes[0]);
+
+#define MAGIC_PK34     0x504b0304U
+#define MAGIC_JPEG     0xffd8ff00U
+#define MAGIC_PNG      0x89504e47U
+#define MAGIC_BMP      0x424d0000U
+#define MAGIC_GIF      0x47494638U
+#define MAGIC_MZ       0x4d5a0000U
+#define MAGIC_ISH_Z    0x135d658cU
+#define MAGIC_ISH_IA   0x2aab79d8U
+#define MAGIC_ISH_INS1 0xffff0c00U
+#define MAGIC_ISH_INS2 0xb8c90c00U
+#define MAGIC_ISH_INI1 0x5b537461U
+#define MAGIC_ISH_PKG  0x4aa30000U
+#define MAGIC_TIFF1    0x49492a00U
+#define MAGIC_TIFF2    0x4d4d002aU
+#define MAGIC_8BPS     0x38425053U
+#define MAGIC_PDF      0x25504446U
+
+	if((m0&0xffffff00)==MAGIC_JPEG) {
+		idctx->fmtid = FMTUTIL_FMTID_JPEG;
+		ext = "jpg";
+		goto done;
+	}
+
+	if(m0==MAGIC_PNG) {
+		idctx->fmtid = FMTUTIL_FMTID_PNG;
+		ext = "png";
+		goto done;
+	}
+
+	if(m0==MAGIC_GIF) {
+		idctx->fmtid = FMTUTIL_FMTID_GIF;
+		ext = "gif";
+		goto done;
+	}
+
+	if(m0==MAGIC_TIFF1 || m0==MAGIC_TIFF2) {
+		idctx->fmtid = FMTUTIL_FMTID_TIFF;
+		ext = "tif";
+		goto done;
+	}
+
+	if(m0==MAGIC_8BPS) {
+		if(idctx->bof64bytes[5]==0x01) {
+			ext = "psd";
+			goto done;
+		}
+		if(idctx->bof64bytes[5]==0x02) {
+			ext = "psb";
+			goto done;
+		}
+	}
+
+	if(!img_only && m0==MAGIC_PK34) {
+		idctx->fmtid = FMTUTIL_FMTID_ZIP;
+		ext = "zip";
+		goto done;
+	}
+
+	if(!img_only && m0==MAGIC_PDF) {
+		ext = "pdf";
+		goto done;
+	}
+
+	if((m0&0xffff0000U)==MAGIC_BMP && idctx->bof64bytes[15]==0) {
+		idctx->fmtid = FMTUTIL_FMTID_BMP;
+		ext = "bmp";
+		goto done;
+	}
+
+	if(!img_only && (m0&0xffff0000U)==MAGIC_MZ) {
+		if(fmtid_is_dll(c, idctx)) {
+			ext = "dll";
+		}
+		else {
+			ext = "exe";
+		}
+		goto done;
+	}
+
+	if(!img_only && m0==MAGIC_ISH_Z) {
+		ext = "z";
+		goto done;
+	}
+
+	if(!img_only && m0==MAGIC_ISH_IA) {
+		ext = "ex_";
+		goto done;
+	}
+
+	if(idctx->mode==FMTUTIL_FMTIDMODE_ISH_SFX) {
+		if(m0==MAGIC_ISH_INS1 || m0==MAGIC_ISH_INS2) {
+			ext = "ins";
+			goto done;
+		}
+		if((m0&0xffff0000)==MAGIC_ISH_PKG) {
+			ext = "pkg";
+			goto done;
+		}
+		if(m0==MAGIC_ISH_INI1) {
+			ext = "ini";
+			goto done;
+		}
+	}
 
 done:
-	if(edd->detected_fmt==DE_SPECIALEXEFMT_SFX) {
-		de_strlcpy(ectx->shortname, "Zoo", sizeof(ectx->shortname));
-	}
-}
-
-void fmtutil_detect_exesfx(deark *c, struct fmtutil_exe_info *ei,
-	struct fmtutil_specialexe_detection_data *edd)
-{
-	struct execomp_ctx ectx;
-
-	de_zeromem(&ectx, sizeof(struct execomp_ctx));
-
-	detect_exesfx_zip(c, &ectx, ei, edd);
-	if(edd->detected_fmt) goto done;
-
-	detect_exesfx_lha(c, &ectx, ei, edd);
-	if(edd->detected_fmt) goto done;
-
-	detect_exesfx_arc(c, &ectx, ei, edd);
-	if(edd->detected_fmt) goto done;
-
-	detect_exesfx_larc(c, &ectx, ei, edd);
-	if(edd->detected_fmt) goto done;
-
-	detect_exesfx_zoo(c, &ectx, ei, edd);
-
-done:
-	if(ectx.shortname[0] && ectx.verstr[0]) {
-		de_snprintf(edd->detected_fmt_name, sizeof(edd->detected_fmt_name), "%s %s",
-			ectx.shortname, ectx.verstr);
-	}
-	else {
-		de_strlcpy(edd->detected_fmt_name, (ectx.shortname[0]?ectx.shortname:"unknown"),
-			sizeof(edd->detected_fmt_name));
+	if(ext && (idctx->fmtid==0)) {
+		idctx->fmtid = FMTUTIL_FMTID_OTHER;
 	}
+
+	if(!ext) {
+		ext = idctx->default_ext;
+	}
+	if(!ext) {
+		ext = "bin";
+	}
+	de_strlcpy(idctx->ext_sz, ext, sizeof(idctx->ext_sz));
 }
